/**
 * \file    doc/building.h
 * \brief   Documentation: How to build WOSH Framework
 ****************************************************************************
 * @author  Alessandro Polo
 * @version $Id: building.h 1927 2010-02-23 10:07:01Z alex $
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

/*! \page page_build Building
 *
 * This page explains how to compile and build WOSH on your system.
 *
 *
 * \section page_build_toc Table of Contents:
 *
 *  - \ref page_build_overview
 *  - \ref page_build_req
 *  - \ref page_build_common
 *  - \ref page_build_3rdparty
 *
 *  - \ref page_build_compile_posix
 *  - \ref page_build_compile_win32
 *  - \ref page_build_compile_wince
 *  - \ref page_build_compile_macos
 *
 *  - \ref page_build_doc
 *
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_build_overview Overview
 *
 * WOSH is a modular framework, building a WOSH application means satisfying
 * requirements at different levels:
 *  - WOSH Core - Core library (Kernel, Core Modules) is always included 
 *    in any WOSH application, so all of its requirements must be fulfilled.
 *  - WOSH Framework - Depending on the application and included services
 *    a part of (or the whole) framework library must be built.
 *  - WOSH Application - Finally application-scope explicit references
 *    to external libraries must be linked.
 *
 * Almost all (but not wosh::Thread, actually) WOSH Core library (source in
 * the /src/core folder) is written in pure ANSI C++ (also using STL)
 * and does NOT require any external library.
 *
 * The WOSH Framework is actually based on QT4 or grater.
 *
 * WOSH links only to stable, state-of-the-art libraries.
 *
 *
 * Actually, WOSH Core and Framework are compiled within same built, that means
 * you have to supply all required libraries, see next section for listing.
 * In short, for a standard built, you will need QT4 or greater.
 *
 *
 * \note I'm wondering to make WOSH more scalable without linking to the QT library.
 *       Actually there are few (but important) topics where QT comes in help:
 *       Threading (this may be implemented from scratch for each platform),
 *       Networking (should find a multiplatform TCP, UDP socket library),
 *       XML serialization (it needs a stable library compatible to WinXX and POSIX).
 *
 * Finally, note that built-in services adds requirements to the application.
 * For example, adding wosh::services::GTalkBotBundle means
 * that application will link to \c libgloox which is available for POSIX only,
 * so that built/configuration of the application will be platform-specific.
 *
 * In short, requirements of a WOSH application are the sum of its components'
 * requirements. See \ref page_debug_libs for more information about linking 
 * and shared libraries, the section prompts also an example of how many libs
 * are required by a sample \c woshsrv application built for Linux Debian,
 * don't be scared by the number of libraries, most of them are included in
 * the Linux distributions by default, others may be easily installed.
 *
 *
 * \note
 *  It is suggested to have a different working copy for each platform, this
 *  will avoid many issues about generated output and temporary files.
 *
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_build_req Platforms, Compilers and Required tools
 *
 * WOSH is written in ANSI C++ (ISO/IEC 14882:2003), (partially) based
 * on QT framework and compatible with following platforms:
 *  - POSIX [native] (Linux/UNIX systems)
 *  - Windows [native] (2000, XP, CE6.0, Mobile 6.0)
 *  - Windows [mingw] (some core files must be updated, actually not supported)
 *  - Window Vista (not tested! (but will work))
 *  - MAC OS (not tested! (but should work))
 *
 * \par Compilers and Libraries
 * Any ANSI C++ compatible compiler should work (GCC, MSC, MinGW GCC, ..),
 * STL (Standard Template Library) must be available as well (and it is, don't care).
 *
 * QT libraries and headers must be properly installed and configured.
 * WOSH Core and Framework require QT 4.1 or later, some applications (such as
 * WoshShop and KiosK) require QT 4.5 or later.
 * You may find some instructions and tips further within each OS-specific section.
 *
 * WOSH has been developed and tested on following OS:
 *
 *  - \ref page_build_compile_posix "POSIX (Linux)"
 *    - Linux Debian Etch (4.0) - GCC 4:4.1.1-15
 *    - Linux Debian Lenny (5.0) - GCC 4.3.2-1.1
 *    - Linux Ubuntu 9.x - GCC 4:4.3.3-1
 *
 *  - \ref page_build_compile_win32 "Windows"
 *    - Windows 2000 Professional (SP2) - Microsoft Visual Studio C++ 2008
 *    - Windows XP (SP3) - Microsoft Visual Studio C++ 2008 [Microsoft 32-bit C/C++ Optimizing Compiler Version 15.00.30729.01 for 80x86]
 *
 *    - \ref page_build_compile_wince "Windows Mobile" 6.1 - Microsoft Visual Studio C++ 2008 with Windows Mobile 6 Standard SDK [cross-compiled ARMV4I]
 *
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_build_common Common Approach and Configuration
 * 
 * Build process is organized using QT Project makefiles (.pro), \c qmake tool will generate
 * the appropriate makefile on each platform. There is a project file in each source directory.
 * You may find more information about QMake on QT documentation.
 * 
 *
 * Main project files are placed in <a href="../../src">/src</a> folder:
 *  - <a href="../../src/woshsrv.pro">/src/woshsrv.pro</a> - WOSH (Console) Server [MultiPlatform]
 *  - <a href="../../src/woshshop.pro">/src/woshshop.pro</a> - WOSH WorkShop [MultiPlatform]
 *  - <a href="../../src/woshremote.pro">/src/woshremote.pro</a> - WOSH Remote [MultiPlatform]
 *  - <a href="../../src/woshcesrv.pro">/src/woshcesrv.pro</a> - WOSH CE Server [for Windows Mobile/CE]
 *  - <a href="../../src/woshlib.pro">/src/woshlib.pro</a> - WOSH Static Library
 *
 * The build is organized hiercally, project files are included and always refers to /src folder.
 * Usually, all source folders contain a \c .pro  or \c .prf project file, which may be included
 * by its parent directory (depending on settings).
 *
 * All WOSH applications' project files (listed before) share many common directives
 * and source files, because of that they all include <a href="../../src/common.pri">common.pri</a> project file.
 *
 * This shared configuration forces many settings, such as:
 *  - include WOSH Core
 *  - Makefile are generated in <a href="../../src">/src</a> folder,
 *  - binaries are written in <a href="../../bin">/bin</a> folder,
 *  - temporary files are saved in \c tmp fodler (and sub-folders).
 *
 * Moreover, it would include also the WOSH Framework when the parent project file
 * defined namespaces (adding keywords to 'CONFIG').
 * Project files are commented, read them for more information.
 *
 * \c Network and \c DataBase modules of the WOSH framework are built-in by default
 * because of they are required for distributed computing.
 *
 *
 *
 *
 * \warning 
 *  Before generating Makefile you need to configure the built-in bundles.
 *  Bundles are not deployed as dynamic/shared library yet, they are built-in with WOSH Framework.
 *  Because of that, you must select Bundles and have their required libraries installed on the system.
 *  Each bundle's Makefile (project file) will add required libraries and directives to the build.
 *
 *
 * You have to create/modify following files:
 *  - /src/bundles/services/services.pri
 *    (you may copy from <a href="../../src/bundles/services/services.pri.default">services.pri.default</a>
 *     or <a href="../../src/bundles/services/services.all.pri">services.all.pri</a>)
 *  - /src/bundles/devices/devices.pri
 *    (you may copy from <a href="../../src/bundles/devices/devices.pri.default">devices.pri.default</a>
 *     or <a href="../../src/bundles/devices/devices.all.pri">devices.all.pri</a>)
 *
 *
 *
 * To code required to include a bundle is very simple and compact:
 * \verbatim
!include( bundles/services/BuildingManager/BuildingManager.prf ) {
	message( "*****  No bundles/services/BuildingManager/BuildingManager.prf file found. SKIPPED. *****" )
} 
\endverbatim
 *
 * This will include selected \c BuildingManager into the built.
 * By the QT Qmake point of view, commands will be directly included (so paths must
 * refer to the /src folder).
 *
 * Now you are ready to generate Makefile for your IDE. Go to your platform's section.
 *
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_build_3rdparty 3rd-party Libraries
 * 
 * Some part of the WOSH Framework (or built-in services) may reference to
 * external (3rd-party) libraries. WOSH Core doesn't require any of these libs.
 * Whenever it is possible and reasonable, these libraries are provided with
 * source/full distribution.
 * 
 * There external libraries are placed in <a href="../../lib/3rdparty">/lib/3rdparty</a>
 * folder. It is organized as following:
 *  - /lib/3rdparty contains \c compiled libraries (.lib, .a files)
 *  - each library has its own sub-folder. It may contain a single README file,
 *    but also source code (i.e. when it was locally updated) and custom data
 *
 *
 * [..]
 *
 *
 * 
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_build_compile_posix Build on Linux (POSIX)
 *
 * Indeed you need built-tools such as GCC (required 3.x, suggested 4.x), and STL
 * as supported by <i>GNU Standard C++ Library v3 (libstdc++-v3)</i>.
 *
 * On Debian-based distributions, such as Ubuntu, you may install them executing:
 * \verbatim
alex@linbox:/$ sudo apt-get build-essential
\endverbatim
 *
 * QT is available in most Linux distributions, on Debian-based systems you may install it executing:
 * \verbatim
alex@linbox:/$ sudo apt-get install libqt4-dev libqt4-network
\endverbatim
 * these packages require some more packages (accept all of them),
 * as developer you may install also \c libqt4-dbg module.
 *
 * To ensure the requirements are fulfilled, check response of following commands:
 * \verbatim
alex@linbox:/$ which qmake
  /usr/bin/qmake

alex@linbox:/$ qmake --version
  QMake version 2.01a
  Using Qt version 4.4.3 in /usr/lib
\endverbatim
 *
 * Once ready, in order to create Makefile, you need to execute in the project base directory :
 * \verbatim
alex@linbox:/home/alex/WOSH/src$ qmake <PROJECT>.pro
\endverbatim
 * You probably want to build \c woshsrv.pro
 * This operation is required only once, even after updating qmake project files.
 *
 * On my main WOSH server (Debian Lenny) it looks like <a href="../../var/log/qmake.woshsrv.log">/var/log/qmake.woshsrv.log</a>.
 *  
 * The Makefile has been created and you may build the software as usual, executing:
 * \verbatim
alex@linbox:/home/alex/WOSH/src$ make
\endverbatim
 *
 * Any raised error is probably related to 3rd-party libraries (or their paths)
 * which may be mis-configured or missing on your system, such libraries may be
 * required by included bundles, as said in \c overview section.
 * Don't worry, WOSH compiles properly once environment is ready.
 *
 * Example of required libraries by \b woshsrv (with many services ) running on Debian Lenny is
 * <a href="../../bin/woshsrv.posix.ldd">/bin/woshsrv.posix.ldd</a> ( \c ldd output).
 *
 * You may build other applications (such as \ref page_applications_woshshop "woshshop",
 * \ref page_applications_woshremote "woshremote" ) in the same way.
 *
 * Output will be generated in <a href="../../bin">/bin</a> folder (such as 'woshsrv').
 *
 *
 * Built!? Now you may read \ref page_run page.
 *
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_build_compile_win32 Build on Windows (32/64)
 *
 * WOSH may be compiled on Windows systems in two ways: \b native, \b ported (mingw).
 *
 * Since mingw-building is the most common option for QT-win and it's almost like
 * building software on Linux, we will focus on Windows-native built.
 *
 * On Windows, QT and WOSH support Microsoft Visual Studio 2005-2008.
 *
 *
 * \warning
 *  Download QT source code and build it following the instructions. Take care when selecting the
 *  archive on QT website, it may be confusing: you need QT's source code. Actually the package name
 *  is: <a href="http://qt.nokia.com/downloads/windows-cpp-vs2008">Qt for Open Source C++ development on Windows (VS2008)</a>
 *  (\c qt-win-opensource-4.6.1-vs2008.exe )
 *  It's a very good thing to avoid spaces in the QT's installation path.
 *  You need to install (and compile) QT for Windows. Just follow instructions, it's very easy but takes about 2 hours.
 *  define QTDIR_WIN32, don't define PATH when you istall more (version of) QT
 *
 * [..]
 *
 *
 * Some scripts (BATCH) are provided in order to automate the creation of Visual Studio project (solution)
 * but you have to customize installation path of QT and (in some cases) MSVC. 
 *  - QTDIR_WIN32 set to the root path of QT - native (locally built with MSVC)
 *  - QTDIR_WINGW set the the root path of QT - MinGW (just installed)
 *
 * Provided scripts are:
 *  - <a href="../../build/mscv_make_apps.bat">/build/mscv_make_apps.bat</a>
 *    Create project files and/or build solution of WOSH Applications (server, woshshop, remote, kiosk)
 *
 *  - <a href="../../build/mscv_make_lib.bat">/build/mscv_make_lib.bat</a>
 *    Create project files and/or build solution of WOSH Library
 *
 *  - <a href="../../build/msvc_make_3rdparty.bat">/build/msvc_make_3rdparty.bat</a>
 *    Build 3rd-party libraries
 *
 * You are probably interested into WOSH Applications (double-click mscv_make_apps.bat).
 *
 *
 * Manual generation of \c Makefile looks like:
 * \verbatim
C:\WOSH\src> qmake woshsrv.pro -t vcapp
\endverbatim
 *
 * Once the project has been created (as woshsrv.vcproj, woshshop.vcproj, woshremote.vcproj in \c /src folder),
 * you may open Microsoft Visual Studio and build the project (the solution file will be generated by VS itself).
 *
 *
 * Built!? Now you may read \ref page_run page.
 *
 *
Open your project properties, go to "C++| Code Generation" and set "Runtime Library" to "Multithreaded (Debug) DLL"
	Find the linker tab
	Open its last page where it says "command line"
	/FORCE:multiple
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_build_compile_wince Build on Windows CE/Mobile
 *
 * WOSH Core and most of the framework is compabile with Windows CE/Mobile 6.0
 * The goal of \c woshcesrv is to provide some Mobile-services to the WOSH network,
 * such as SMS, call monitoring. The device must be connected to the local network,
 * I run WOSH CE Server on my Samsung smartphone, USB connected to Linux (RNDIS).
 *
 * WOSH CE Server is a minimal server, intended to run the wosh::devices::WindowsMobileBundle
 *
 *
 * \warning
 *  You should be familiar with Windows CE/Mobile devices and development of their applications.
 *  You need to install and configure Windows Mobile 6.0 SDK, Microsoft ActiveSynch.
 *  You must be able (using ActiveSynch) to browser Device's filesystem and a PAN Network
 *  must be enabled. If you are not familiar with Mobile/CE devices, search for a tutorial
 *  and test a simple application before building and running WOSH.
 *
 *
 * Requirements of Windows CE/Mobile built are few:
 *  - Microsoft Visual Studio 2005-2008 (8.0,9.0)
 *  - Microsoft Windows Mobile 6.0 SDK
 *  - QT 4.1 (or later) built (locally) for Windows CE/Mobile
 *
 * You may download the Windows Mobile 6.0 SDK freely from Microsoft website.
 *
 *
 * \warning
 *  You need to install (and compile) QT SDK for Windows CE, it's very easy but takes about 2 hours.
 *  Download QT source code and build it following the instructions. Take care when selecting the
 *  archive on QT website, it may be confusing: you need QT's source code. Actually the package name
 *  is: <a href="http://qt.nokia.com/downloads/win-ce-cpp">Qt libraries 4.6.1 for Windows CE (160 MB)</a>
 *  (\c qt-everywhere-opensource-src-4.6.1.zip  )
 *  It's a very good thing to avoid spaces in the QT's installation path.
 *  You need to install (and compile) QT for Windows CE, it's very easy but takes about 2 hours.
 *
 * My own QT installation's commands are:
 * \code
C:\dev\Qt\wince.4.6.1> set PATH=C:\dev\Qt\wince.4.6.1;%PATH%
C:\dev\Qt\wince.4.6.1> configure -platform win32-msvc2008 -xplatform wincewm60standard-msvc2008
C:\dev\Qt\wince.4.6.1> setcepaths wincewm60standard-msvc2008
C:\dev\Qt\wince.4.6.1> nmake
 * \endcode
 *
 * A (BATCH) script is provided in order to automate the creation of Visual Studio project (solution)
 * but you have to customize installation path of QT. 
 *  - QTDIR_WINCE set to the root path of QT - native (locally built with MSVC)
 *
 *
 * Manual generation of \c Makefile looks like:
 * \verbatim
C:\WOSH\src> qmake woshcesrv.pro -t vcapp
\endverbatim
 *
 *
 * Once the project has been created (woshcesrv.vcproj in \c src folder),
 * you may open Microsoft Visual Studio and build the project (the solution file will be generated by VS itself).
 *
 *
 * Built!? Now you may read \ref page_run page.
 *
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_build_compile_macos Build on MAC OS
 *
 * \warning WOSH has NEVER been compiled neither tested on Mac OS.
 *
 * Would help?
 * Send results to contact _AT_ alessandropolo.name or on SourceForge website.
 *
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_build_doc Building Documentation
 *
 * The tool Doxygen (<a href="http://www.doxygen.org/index.html">http://www.doxygen.org</a>)
 * is required to build the documentation. 
 *
 * The complete WOSH documentation is integrated into source code and written in
 * doxygen format. This document (as others) is generated by DoxyGen.
 *
 *
 * The current configuration also requires Dot/Graphwiz (http://www.graphviz.org/);
 * change <tt>HAVE_DOT = YES|NO</tt> in <a href="../../doc/Doxyfile">/doc/Doxyfile</a>.
 *
 * On Debian-based system, you may install required packages executing:
 * \verbatim
alex@linbox:/$ sudo apt-get install doxygen graphviz
\endverbatim
 *
 * To build documentation manually change to <a href="../../doc">/doc</a> folder and execute:
 * \verbatim
alex@linbox:/home/alex/WOSH/doc$ doxygen Doxyfile
\endverbatim
 *
 * Doxygen will generate documentation in sub-folders, by default HTML and XML outputs
 * are enabled (writing in \c /doc/html and \c /doc/xml folders).
 *
 * You can give look to \c /doc/html/index.html.
 * 
 * On Windows system you may install Doxygen and open configuration file with \c doxywizard
 *
 *
 *
 */

