/**
 * @file    main.h
 * @brief   Documentation: Main documentation page of WOSH Framework
 ****************************************************************************
 * @author  Alessandro Polo
 * @version $Id: main.h 1818 2010-02-10 08:14:38Z alex $
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

/*! \mainpage WOSH Framework
 * 
 * WOSH (Wide Open Smart Home) is a software designed to control (automate) a Smart Home.
 * WOSH is not a monolithic application, it's rather a modular library.
 *
 * \note This framework is still \c experimental, currently released as preview for developers and expert users.
 *
 *
 * \section page_fundamentals_toc Table of Contents:
 *
 *  - \ref mainpage_overview "WOSH Overview"
 *  - \ref mainpage_readme
 *  - \ref mainpage_developers
 *
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section mainpage_overview Overview
 *
 * WOSH is an open multi-platform framework (message oriented middleware) written in ANSI C++.
 *
 * WOSH is a Service Oriented Framework (SOA) providing a (Network) OS independent infrastructure
 * for developing component based software (services, aka bundles).
 *
 *
 * WOSH ships with core & framework library, many included bundles and some end-user applications:
 *  - \ref page_applications_woshsrv
 *  - \ref page_applications_woshcesrv
 *  - \ref page_applications_woshshop
 *  - \ref page_applications_woshremote
 *
 *
 * Some reasons to use and join the WOSH projects are:
 *  - It's free, open source, alive!
 *  - It's modular and growing
 *  - It's easy to plug-in new features (C++ required)
 *  - It has been though to be efficient and intuitive
 *  - It contains state-of-the-art libraries (QT4, gloox, festival, gstreamer010, ..)
 *  - It comes with many built-in services (networking, communication, entertainment, remote control, ..)
 *
 *
 * Some examples of what WOSH offers are explained in section \ref page_using "Using WOSH".
 * Many features depends on hardware devices (such as X10 control), media playback,
 * telephony. Most important features for end-user are:
 *
 *  - Built-in end-user \c applications: console and graphical both.
 *
 *  - \c Distributed computing, zero-configuration networking.
 *
 *  - \b Multi-user, role based access.
 *
 *  - \b Remote \b control using Instant Messaging (on POSIX, using \c libgloox; compatible with GTalk),
 *    SMS (send/receive) and call monitoring (on Windows-Mobile, RNDIS connected smart phone)
 *
 *  - \b Appliances and \b sensors (X10 devices) monitor/control (on POSIX, based on \c Heyu)
 *
 *  - \b Entertaiment, multi-zone media playback (using \c GStreamer on POSIX or QT \c Phonon on
 *    Windows, \c VLC). Media-Director service provides a high abstraction layer, 
 *    the recovery/guess multimedia status, hardware/software shortcuts and more
 *
 *  - Centralized \b Communication system (selecting best communication channel), using
 *    also interpreters (such as Festival for Text2Speech)
 *
 *  - Building \b abstract-representation of the home and its devices (floor, rooms, lights, audio-box, ..)
 *
 *  - Big-Brother service, which recognizes users and their location
 *
 *  - Butler service, providing common services in a smarter way than event-based automations.
 *
 *  - Cron and \b Automation services, providing support for basic every-day tasks
 *
 *  - many not listed and more to come..
 *
 *
 * WOSH is distributed as \b Open \b Source. \ref page_license "Creative Commons BY-NC-SA/3.0"
 *
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section mainpage_readme Introduction for Users
 *
 * This section provides links to the basic knowledge of WOSH project for end-users.
 *
 *  - \ref page_distribution "Get and Install WOSH on your platform(s)"
 *
 *  - \ref page_applications "WOSH Applications"
 *
 *  - \ref page_config "Configure the WOSH system"
 *  - \ref page_run "Running WOSH for first time"
 *  - \ref page_using "Using WOSH.. some use-cases"
 *
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section mainpage_developers Introduction for Developers
 *
 * This information is intended primarily for WOSH developers (framework and extensions both).
 *
 * WOSH is architected to scale from tiny applications all the way up
 * to massive applications consisting of hundreds of components.
 *
 *
 * That means, the framework helps developers to build software systems consisting of modules which are communicating,
 * via clearly defined service interfaces, with other modules of the system.
 *
 *
 *  - \ref page_specifications "Specifications of WOSH"
 *
 *  - \ref page_core "WOSH Kernel and Core Modules"
 *  - \ref page_framework "WOSH Framework"
 *  - \ref page_bundles "Bundle (generic service)"
 *
 *  - \ref page_build "Compiling and Building WOSH"
 *
 * You may take a look to \ref woshsrv.cpp "woshsrv's source code".
 *
 * New developers are welcome to:
 *  - Contribute to the core library
 *  - Write bundles for WOSH
 *  - Write applications using WOSH
 *  - Contribute to the documentation
 *  - Build test cases
 *
 *
 * Before coding, discuss your plans and design with other people by posting detailed messages.
 * By all means, plan everything before you do it, and share information.
 * Everyone working on this project has some level of expertise and previous coding experiences
 * which can help us all from falling on our faces more than necessary.
 * When you come across a problem don't bang your head against the wall for too long.
 * Ask for help: perhaps someone has faced a similar problem in the past and can offer an immediate solution.
 *
 *
 * \par Wish to Join?
 *  Join online project (SourceForge, GoogleCode) or mail me at contact _AT_ alessandropolo.name
 *  If you are interested in taking on a development task, first make sure that
 *  everyone involved in the development effort knows your interest and intention.
 *  This is appropriately done with a request to the developers,
 *  either by asking specifically to work on one of the sub-projects or tasks,
 *  or by describing your new, proposed task.
 *  Once agreement is reached about the requirements, goals,
 *  the project administrators should then assign this task to you.
 *
 *
 * \par Reporting a Bug
 *  Sometimes the problem you encounter may look like a bug rather than a problem
 *  with the documentation or a generic request for help. If this is the case,
 *  please use the bug reporting service offered by SourceForge.
 *  First check the documentation to make sure you are using WOSH as it is expected to be used.
 *  Then check to see if anyone else has submitted the same bug (or possibly a tech support request)
 *  that may affect your problem. If not, please submit the bug and give as much detail as possible
 *  so that the person reading this bug will be able to recreate the situation.
 *  Please specify the command line options you are using, 
 *  as well as provide any source code which may be useful to reproduce the bug.
 *
 *
 * Also comments and tips are very welcome.
 *
 *
 *
 ****************************************************************************
 *
 */
