/** @file    JabberGloox09.cpp
 *  @author  Alessandro Polo
 *  @version $Id: JabberGloox09.cpp 1873 2010-02-20 13:23:43Z alex $
 *  @brief
 * File containing methods for the JabberGloox09 class.
 * The header for this class can be found in JabberGloox09.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "JabberGloox09.h"
 #include <core/Message.h>

 #include <client.h>
 #include <messagesession.h>
 #include <rostermanager.h>

 using namespace wosh;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

JabberGloox09::JabberGloox09( wosh::BundleGeneric& bundle ) : JabberGlooxImpl(bundle) {
 }

JabberGloox09::~JabberGloox09() {
	Log(LOG_VERBOSE, ":~JabberGloox09(): Destroying.." );
	if ( isThreadRunning() ) {
		quitThread();
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////// MESSAGING

bool JabberGloox09::sendMessage( const std::string& user, const std::string& message )
 {
	if ( !isThreadRunning() ) return false;
	if ( this->xmppJclient == NULL ) return false;
	Log(LOG_VERBOSE, ":sendMessage(): sending '%s' to %s..", message.c_str(), user.c_str() );

	gloox::Stanza *s = gloox::Stanza::createMessageStanza( user, message );
	this->xmppJclient->send( s );
	// well, I can only HOPE that message was sent/received!
	return true;
 }

/////////////////////////////////////////////////////////////////////////////////////////////////// MESSAGING
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////// MessageHandler INTERFACE

void JabberGloox09::handleMessage( gloox::Stanza* stanza, gloox::MessageSession* )
 {
	if ( stanza == NULL ) return;
	if ( Log.getLevel() == LOG_VERBOSE )
		Log(LOG_VERBOSE, ":handleMessage(): %s by %s", stanza->body().c_str(), stanza->from().bare().c_str() );

	if ( this->xmppJclient == NULL ) {
		Log(LOG_CRITICAL, ":handleMessage(): xmppJclient is NULL!" );
		return;
	 }

	if ( stanza->error() != gloox::StanzaErrorUndefined ) {
		Log(LOG_WARNING, ":handleMessage(): Stanza Error #%i ''", (int)stanza->error(), stanza->errorText().c_str() );
		return;
	 }
	if ( this->jabbListener == NULL )
		return;

	this->jabbListener->jabbbot_Message( stanza->from().bare(), stanza->body(), this );
 }

//////////////////////////////////////////////////////////////////////////////////// MessageHandler INTERFACE
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////// PresenceHandler INTERFACE

void JabberGloox09::handlePresence( gloox::Stanza* stanza )
 {
	if ( stanza == NULL ) return;
	Log(LOG_VERBOSE, ":handlePresence(): status of %s is '%s' !", stanza->from().full().c_str(), stanza->status().c_str() );

	// check if it refers to myself. this is periodic on most servers
	if ( this->xmppAccountName == stanza->from().bare() ) {
		// mmh.. great i hope so!
		// we don't notify that..
		return;
	 }
	if ( this->jabbListener == NULL )
		return;

	// eval and forward the state of the user
	int status = -1;
	switch( stanza->presence() ) {
		case gloox::PresenceUnavailable:	status = 0; break;
		case gloox::PresenceUnknown: 		status = 0; break;
		case gloox::PresenceXa: 			status = 0; break;
		default:							status = 1; break;
	 }
	this->jabbListener->jabbbot_UserStatus( stanza->from().bare(), status, this );
 }

/////////////////////////////////////////////////////////////////////////////////// PresenceHandler INTERFACE
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////// SubscriptionHandler INTERFACE

void JabberGloox09::handleSubscription( gloox::Stanza* stanza )
 {
	if ( stanza == NULL ) return;
	int status = -1;
	switch ( stanza->subtype() ) {
		case gloox::StanzaS10nSubscribe: {
				status = _jabbbot_Subscribe;
				Log(LOG_INFO, ":handleSubscription(): User %s requests to Subscribe!", stanza->from().full().c_str() );
				break;
			 }
		case gloox::StanzaS10nSubscribed:  {
				status = _jabbbot_Subscribed;
				Log(LOG_INFO, ":handleSubscription(): User %s Subscribed!", stanza->from().full().c_str() );
				break;
			 }
		case gloox::StanzaS10nUnsubscribe: {
				status = _jabbbot_UnSubscribe;
				Log(LOG_INFO, ":handleSubscription(): User %s requests Unsubscribe!", stanza->from().full().c_str() );
				break;
			 }
		case gloox::StanzaS10nUnsubscribed: {
				status = _jabbbot_UnSubscribed;
				Log(LOG_INFO, ":handleSubscription(): User %s Unsubscribed!", stanza->from().full().c_str() );
				break;
			 }

		default: {
				Log(LOG_WARNING, ":handleSubscription(): User %s sent invalid subscription request!", stanza->from().full().c_str() );
				break;
			 }
	 }

	if ( this->jabbListener == NULL )
		return;

	short ackRequest = this->jabbListener->jabbbot_SubscriptionChanged( stanza->from().full(), status, this );
	if ( ackRequest == -1 ) // ignore
		return;
	if ( this->xmppJRM == NULL ) {
		Log(LOG_CRITICAL, ":handleSubscription(%s, %i): NULL RosterManager!", stanza->from().full().c_str(), (int)stanza->subtype() );
		return;
	 }
	Log(LOG_INFO, ":handleSubscription(%s, %i): Replying ACK(%i) to subscription", stanza->from().full().c_str(), (int)stanza->subtype(), ackRequest );
	this->xmppJRM->ackSubscriptionRequest(stanza->from().full(), ackRequest);

 }

/////////////////////////////////////////////////////////////////////////////// SubscriptionHandler INTERFACE
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool JabberGloox09::setPresence( bool value ) {
	this->xmppAvailable = value;
	if ( isThreadRunning() ) {
		if ( this->xmppAvailable )
			this->xmppJclient->setPresence(gloox::PresenceAvailable, this->xmppPriority, this->xmppPresenceStatus);
		else
			this->xmppJclient->setPresence(gloox::PresenceUnavailable, this->xmppPriority, this->xmppPresenceStatus);
	 }
	return true;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
