/** @file    JabberGlooxImpl.cpp
 *  @author  Alessandro Polo
 *  @version $Id: JabberGlooxImpl.cpp 1873 2010-02-20 13:23:43Z alex $
 *  @brief
 * File containing methods for the JabberGlooxImpl class.
 * The header for this class can be found in JabberGlooxImpl.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "JabberGlooxImpl.h"

 #include <framework/network/NetworkUtilities.h>
 
 #include <client.h>
 #include <disco.h>
 #include <logsink.h>
 #include <connectionhttpproxy.h>
 #include <connectiontcpclient.h>
 #include <rostermanager.h>


using namespace wosh;
// gloox bug: its not included?
namespace gloox {
 const std::string EmptyString = "";
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

JabberGlooxImpl::JabberGlooxImpl( wosh::BundleGeneric& bundle ) : wosh::BundleGenericWorker(bundle)
 {
	this->jabbListener = NULL;

	this->xmppJclient = NULL;
	this->xmppConnClient = NULL;
	this->xmppConnProxy = NULL;
	this->xmppJRM = NULL;

	this->xmppServerAddress = "";
	this->xmppServerPort = 0;
	this->xmppServerProxy = "";

	this->xmppAccountName = "";
	this->xmppAccountPwd = "";

	this->xmppAvailable = true;
	this->xmppPresenceStatus = "wosh";
	this->xmppPriority = 5;
 }


JabberGlooxImpl::~JabberGlooxImpl()
 {
	Log(LOG_VERBOSE, ":~JabberGlooxImpl(): Destroying.." );
	if ( isThreadRunning() ) {
		quitThread();
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool JabberGlooxImpl::connect()
 {
	Log(LOG_VERBOSE, ":connect(): Connecting to XMPP Server [%s port %i]" , this->xmppServerAddress.c_str(),this->xmppServerPort );

	// connect SYNCH
	bool connected = this->xmppJclient->connect( false );
	if ( !connected ) {
		Log(LOG_CRITICAL, ":connect(): Can't connect to XMPP Server! Exiting (FAILURE)" );
		return false;
	 }
	Log(LOG_INFO, ":connect(): Connected to XMPP Server [%s port %i]" , this->xmppServerAddress.c_str(),this->xmppServerPort );
	return true;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////// RUN THREAD

bool JabberGlooxImpl::initializingThread()
 {
	Log(LOG_VERBOSE, ":initializingThread(): STARTING" );
	// setup account, this fields must be set BEFORE thread is started
	// and cannot be changed while running
	gloox::JID jid( this->xmppAccountName );
	this->xmppJclient = new gloox::Client( jid, this->xmppAccountPwd, this->xmppServerPort );
	this->xmppJclient->setServer( this->xmppServerAddress );
	this->xmppJclient->setPort( this->xmppServerPort ); //clientbase

	// map log for debugging
	this->xmppJclient->logInstance().registerLogHandler( gloox::LogLevelDebug, gloox::LogAreaAll, this ); //clientbase

	// setup common (optional) config
	this->xmppJclient->disco()->setVersion( "wosh::devices::JabberGloox", "1.20" ); //clientbase
	this->xmppJclient->disco()->setIdentity( "client", "wosh::devices::JabberGloox" ); //clientbase
	setPresence(true);

	// register callbacks: connection interrupts; incoming messages; presence detector, subscription requests
	this->xmppJclient->registerConnectionListener( this );
	this->xmppJclient->registerMessageHandler( this );
	this->xmppJclient->registerPresenceHandler( this );
	this->xmppJclient->registerSubscriptionHandler( this );

	if ( !this->xmppServerProxy.empty() ) {
		std::string proxyAddress = wosh::network::NetworkUtilities::getAddressFromDestination(this->xmppServerProxy);
		int proxyPort = wosh::network::NetworkUtilities::getPortFromDestination(this->xmppServerProxy);
		Log(LOG_INFO, ":initializingThread(): Setting up Proxy: %s port %i..", proxyAddress.c_str(), proxyPort );
		this->xmppConnClient = new gloox::ConnectionTCPClient(this->xmppJclient->logInstance(), proxyAddress, proxyPort);
		this->xmppConnProxy = new gloox::ConnectionHTTPProxy(this->xmppJclient, this->xmppConnClient, this->xmppJclient->logInstance(), this->xmppServerAddress, this->xmppServerPort );
		this->xmppJclient->setConnectionImpl( this->xmppConnProxy );
	 }

	// alloc RosterManager, for subscription management
	this->xmppJRM = new gloox::RosterManager(this->xmppJclient);

	return connect();
 }

void JabberGlooxImpl::runThread()
 {
	gloox::ConnectionError ce = gloox::ConnNoError;
	setThreadAlive();
	Log(LOG_VERBOSE, ":runThread(): Entering recv Loop.." );
	do {
		setThreadAlive();
		ce = this->xmppJclient->recv( 2 );
		if ( !this->running )
			break;
	 }
	while( ce == gloox::ConnNoError );
 }

void JabberGlooxImpl::exitingThread()
 {
	setPresence(false);
	Log(LOG_INFO, ":exitingThread() : Disconneting.." );
	this->xmppJclient->disconnect();

	Log(LOG_VERBOSE, ":exitingThread() : Destroying Jabber Client.." );
	delete this->xmppJclient; this->xmppJclient = NULL; this->xmppJRM = NULL;

	if ( this->xmppConnClient != NULL ) {
		Log(LOG_VERBOSE, ":exitingThread() : Destroying ConnectionTCPClient.." );
		delete this->xmppConnClient; this->xmppConnClient = NULL;
	 }

	if ( this->xmppConnProxy != NULL ) {
		Log(LOG_VERBOSE, ":exitingThread() : Destroying ConnectionHTTPProxy.." );
		delete this->xmppConnProxy; this->xmppConnProxy = NULL;
	 }
 }

////////////////////////////////////////////////////////////////////////////////////////////////// RUN THREAD
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////// LogHandler INTERFACE

void JabberGlooxImpl::handleLog( gloox::LogLevel level, gloox::LogArea area, const std::string& message ) {
	if ( Log.getLevel() != LOG_VERBOSE ) return;
	Log(LOG_VERBOSE, ":handleLog(): %d, area: %d, %s", level, area, message.c_str() );
 }

//////////////////////////////////////////////////////////////////////////////////////// LogHandler INTERFACE
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// ConnectionListener INTERFACE

void JabberGlooxImpl::onConnect() {
	Log(LOG_VERBOSE, ":onConnect(): Connection estabilished!" );
 }

bool JabberGlooxImpl::onTLSConnect( const gloox::CertInfo& ) {
	Log(LOG_VERBOSE, ":onTLSConnect(): Secure Connection estabilished!" );
	return true;
 }

void JabberGlooxImpl::onDisconnect( gloox::ConnectionError error )
 {
	std::string message = "unknown error";
	switch(error) {
		case gloox::ConnAuthenticationFailed:
			message = "Authentication failed. Username/password wrong or account does not exist. Maybe account was blocked.";
			break;
		case gloox::ConnUserDisconnected:
			message = "The user (or higher-level protocol) requested a disconnect.";
			break;
		case gloox::ConnNotConnected:
			message = "There is no active connection.";
			break;
		case gloox::ConnCompressionFailed:
			message = "Negotiating/initializing compression failed.";
			break;
		case gloox::ConnTlsFailed:
			message = "The server's certificate could not be verified or the TLS handshake did not complete successfully.";
			break;
		case gloox::ConnTlsNotAvailable:
			message = "The server didn't offer TLS while it was set to be required or TLS was not compiled in.";
			break;
		case gloox::ConnNoSupportedAuth:
			message = "The auth mechanisms the server offers are not supported or the server offered no auth mechanisms at all.";
			break;
		case gloox::ConnOutOfMemory:
			message = "Out of memory.";
			break;
		case gloox::ConnDnsError:
			message = "Resolving the server's hostname failed.";
			break;
		case gloox::ConnConnectionRefused:
			message = "The connection was refused by the server (on the socket level).";
			break;
		case gloox::ConnParseError:
			message = "An XML parse error occurred.";
			break;
		case gloox::ConnIoError:
			message = "An I/O error occured.";
			break;
		case gloox::ConnProxyNoSupportedAuth:
			message = "The HTTP/SOCKS5 proxy requires an unsupported auth mechanism.";
			break;
		case gloox::ConnProxyAuthFailed:
			message = "HTTP/SOCKS5 proxy authentication failed.";
			break;
		case gloox::ConnProxyAuthRequired:
			message = "The HTTP/SOCKS5 proxy requires authentication.";
			break;
		case gloox::ConnStreamClosed:
			message = "The stream has been closed (by the server).";
			break;
		case gloox::ConnStreamVersionError:
			message = "The incoming stream's version is not supported";
			break;
		case gloox::ConnStreamError:
			message = "A stream error occured. The stream has been closed.";
			break;
		case gloox::ConnNoError:
			message = "Disconnected from server without errors";
			break;
		default:
			break;
	 }
	if ( error == gloox::ConnUserDisconnected ) {
		Log(LOG_INFO, ":onDisconnect(): Disconnected (on request)" );
		return;
	 }

	if ( error != gloox::ConnNoError ) {
		Log(LOG_WARNING, ":onDisconnect(): Disconnected [error#%i: %s]!", (int)error, message.c_str() );
	 }
	if ( this->xmppJclient == NULL ) {
		Log(LOG_CRITICAL, ":onDisconnect(): NULL client!" );
		return;
	 }

	if ( this->running == false ) {
		return;
	 }
	Log(LOG_VERBOSE, ":onDisconnect(): forcing disconnect.." );
	this->xmppJclient->disconnect();

	if ( this->xmppConnClient != NULL )
		this->xmppConnClient->cleanup();
	if ( this->xmppConnProxy != NULL )
		this->xmppConnProxy->cleanup();

	Thread::sleepForMSec( 1000 );
	Log(LOG_VERBOSE, ":onDisconnect(): forcing (re)connect.." );
	connect();
 }

//////////////////////////////////////////////////////////////////////////////// ConnectionListener INTERFACE
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT JabberGlooxImpl::subscribe( const std::string& account_name, const std::string& opt_message )
 {
	if ( account_name.size() < 3 ) return WRET_ERR_PARAM;
	if ( !isThreadRunning() ) {
		Log(LOG_CRITICAL, ":subscribe(%s): Thread not running!", account_name.c_str() );
		return WRET_ERR_ILLEGAL_USE;
	 }
	if ( this->xmppJRM == NULL ) {
		Log(LOG_CRITICAL, ":subscribe(%s): NULL RosterManager!", account_name.c_str() );
		return WRET_ERR_ILLEGAL_USE;
	 }
	this->xmppJRM->subscribe( gloox::JID(account_name), "", gloox::StringList(), opt_message );
	//this->xmppJRM->synchronize();
	return WRET_OK;
 }

WRESULT JabberGlooxImpl::unSubscribe( const std::string& account_name, const std::string& opt_message )
 {
	if ( account_name.size() < 3 ) return WRET_ERR_PARAM;
	if ( !isThreadRunning() ) {
		Log(LOG_CRITICAL, ":unSubscribe(%s): Thread not running!", account_name.c_str() );
		return WRET_ERR_ILLEGAL_USE;
	 }
	if ( this->xmppJRM == NULL ) {
		Log(LOG_CRITICAL, ":unSubscribe(%s): NULL RosterManager!", account_name.c_str() );
		return WRET_ERR_ILLEGAL_USE;
	 }
	this->xmppJRM->unsubscribe( gloox::JID(account_name), opt_message );
	//this->xmppJRM->synchronize();
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
