/** @file    MailBoxBundle.cpp
 *  @author  Alessandro Polo
 *  @version $Id: MailBoxBundle.cpp 2142 2010-03-16 12:45:50Z alex $
 *  @brief
 * File containing methods for the wosh::devices::MailBoxBundle class.
 * The header for this class can be found in MailBoxBundle.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "MailBoxBundle.h"
 #include "MailBoxImpl.h"

 #include <core/MethodsDef.h>
 #include <core/MessageFilter.h>
 #include <core/Fact.h>
 #include <core/Table.h>
 #include <core/SecurityManager.h>
 #include <core/UserManager.h>
 #include <core/DeviceManager.h>

 #include <framework/communication/TextMessage.h>

 #include <iostream>


using namespace std;
namespace wosh {
 namespace devices {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

MailBoxBundle::MailBoxBundle() : BundleGeneric()
 {
	BundleGeneric::setName( _MailBox_NAME, false );
	Properties.update( _KEY_Version, _MailBox_VERSION );

//Log.setLevel( LOG_VERBOSE );

	Log(LOG_VERBOSE, " Configuring MailBoxImpl worker.." );
	this->mboxWorker = new MailBoxImpl(*this);
	this->mboxWorker->setServer( "", 0 );
	this->mboxWorker->setAccount( "", "" );
	this->mboxWorker->setMailBoxListener(this);

	Log(LOG_VERBOSE, " Setting default properties and permissions.." );
	Properties.set( _CommunicatorService_KEY_ChannelType,	(int)Notification::TYPE_TEXTUAL + (int)Notification::TYPE_DATA, Permission(Permission::Read) );
	Properties.set( _CommunicatorService_KEY_ChannelLink,	(int)Communicator::LINK_ASYNCH, Permission(Permission::Read) );
	Properties.set( _CommunicatorService_KEY_Cost,			0.0,Permission( Permission::RW, Permission::Read, Permission::Read) );
	Properties.set( _CommunicatorService_KEY_Priority,		15,	Permission( Permission::RW, Permission::RW, Permission::Read) );
	Properties.set( _CommunicatorService_KEY_Reliability,	20,	Permission( Permission::RW, Permission::RW, Permission::Read) );

	Properties.set( _MailBox_KEY_AutoCheckFrequency,0,	Permission( Permission::RW, Permission::RW, Permission::Read) );

	Properties.set( _MailBox_KEY_SmtpHostName,		"",	Permission( Permission::RW, Permission::RW, Permission::Read) );
	Properties.set( _MailBox_KEY_SmtpUserName,		"",	Permission( Permission::RW, Permission::RW, Permission::Read) );
	Properties.set( _MailBox_KEY_SmtpUserPassword,	"",	Permission( Permission::RW, Permission::RW, Permission::Read) );
	Properties.set( _MailBox_KEY_Pop3HostName,		"",	Permission( Permission::RW, Permission::RW, Permission::Read) );
	Properties.set( _MailBox_KEY_Pop3UserName,		"",	Permission( Permission::RW, Permission::RW, Permission::Read) );
	Properties.set( _MailBox_KEY_Pop3UserPassword,	"",	Permission( Permission::RW, Permission::RW, Permission::Read) );
	Properties.monitor(_MailBox_KEY_Pop3HostName);
	Properties.monitor(_MailBox_KEY_Pop3UserName);
	Properties.monitor(_MailBox_KEY_Pop3UserPassword);
	Properties.monitor(_MailBox_KEY_AutoCheckFrequency);
	Properties.monitor(_MailBox_KEY_SmtpHostName);
	Properties.monitor(_MailBox_KEY_SmtpUserName);
	Properties.monitor(_MailBox_KEY_SmtpUserPassword);

	Log(LOG_VERBOSE, " Registering methods.." );
	MethodMessageResponse* mmSendMessageToUser = Methods.create<MethodMessageResponse>( _CommunicatorService_METHOD_sendmessage, "send message to (wosh) username" );
	mmSendMessageToUser->setMethod( this, (MethodMessageResponsePtr)&MailBoxBundle::mmDoSendMessageToUser );
	mmSendMessageToUser->getPermission().setMask( Permission::RX, Permission::RX, Permission::RX );

	MethodMessageResponse* mmSendMail = Methods.create<MethodMessageResponse>( _MailBox_METHOD_sendmail, "send mail" );
	mmSendMail->setMethod( this, (MethodMessageResponsePtr)&MailBoxBundle::mmDoSendMail );
	mmSendMail->getPermission().setMask( Permission::RX, Permission::RX, Permission::RX );

	MethodRequest* mmListContacts = Methods.create<MethodRequest>( "list", "List contacts" );
	mmListContacts->setMethod( this, (MethodRequestPtr)&MailBoxBundle::mmDoListContacts );
	mmListContacts->getPermission().setMask( Permission::RX, Permission::RX, Permission::RX );

	BusDevices.setMessageHandler(this);
	WRESULT bus_connected = BusDevices.connect( _Bus_Devices );
	if ( WFAILED(bus_connected) )
		Log(LOG_CRITICAL, ":MailBoxBundle() : FAILED#%i Connecting Bus (wosh.Bus.Devices)..", bus_connected );

	setBundleState(Bundle::STATE_CREATED, false);
 }

MailBoxBundle::~MailBoxBundle()
 {
	Log(LOG_VERBOSE, " Destroying.." );

	if ( isBundleRunning() ) {
		Log(LOG_WARNING, "~MailBoxBundle() : Destroying while RUNNING! Trying to stop.." );
		bundleStop();
	 }

	delete this->mboxWorker; this->mboxWorker = NULL;
	Log(LOG_VERBOSE, ":~MailBoxBundle() : Destroyed." );
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE CONTROL

WRESULT MailBoxBundle::bundleStart()
 {
	if ( !BundleGeneric::bundleValidate_StartStop(Bundle::STATE_STARTING) )
		return WRET_ERR_WRONG_STATE;

	WRESULT ret = BundleGeneric::start_SynchThread( this->mboxWorker );

	Response* response = new Response( _METHOD_ListProperties, Properties.toTable(false) );
	Message* message = new Message(response);
	message->setSource(this);
	message->getDestination().fromString("*:Communication");
	message->setSecurityToken( SecurityManager::generateToken(this->getPermission().getOwner()) );
	BusCore.postMessage(message);

	// BUNDLE-STATE (STARTED) will be updated async by WORKER, through call: thread_event()
	return ret;
 }

WRESULT MailBoxBundle::bundleStop()
 {
	if ( !BundleGeneric::bundleValidate_StartStop(Bundle::STATE_STOPPING) )
		return WRET_ERR_WRONG_STATE;

	WRESULT ret = BundleGeneric::stop_SynchThread( this->mboxWorker );

	// BUNDLE-STATE (STOPPED) will be updated async by WORKER, through call: thread_event()
	return ret;
 }

////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE CONTROL
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////// PROCESS BUS MESSAGES

void MailBoxBundle::busMessage( const Message& message, const Bus* source )
 {
	if ( message.isEmpty() ) return;

	if ( message.getContent()->isNotification() ) {
		if ( message.getDestination().isBroadcast() )
			return;
		if ( !MessageFilter::isMessageTarget(message, this) )
			return;
		const Notification* notif = message.getContent()->asNotification();
		WRESULT sent = notify( notif );
		Message* reply = message.replyTo( new Response( _CommunicatorService_RESPONSE_notif_sent, new Variant(notif->getID()), sent ) );
		BusCore.postMessage(reply);
		return;
	 }

	DeviceManager::busMessageForBundleDevices( message, source, this, this->Methods, this->Log, this->BusCore, this->BusDevices );
 }

//////////////////////////////////////////////////////////////////////////////////////// PROCESS BUS MESSAGES
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////// GENERATED EVENTS

void MailBoxBundle::raiseEvent( MessageContent* content, SecurityToken* security )
 {
	Message* msg_event = new Message();
	msg_event->setSource( this );
	msg_event->setContent( content );
	msg_event->setDestinationBroadcast(true);
	msg_event->setSecurityToken(security);
	BusCore.postMessage(msg_event);
 }

//////////////////////////////////////////////////////////////////////////////////////////// GENERATED EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENTS

bool MailBoxBundle::updatingProperty( bool& do_update, const Data& value_proposed, Property& property_current, const PropertiesProvider* source )
 {
	if ( property_current.getKey() == _MailBox_KEY_Pop3HostName ||
		 property_current.getKey() == _MailBox_KEY_Pop3UserName ||
		 property_current.getKey() == _MailBox_KEY_Pop3UserPassword ) {
		if ( this->mboxWorker->isThreadRunning() )
			return false;
		bool ret = false;

		if ( property_current.getKey() == _MailBox_KEY_Pop3HostName )
			ret = this->mboxWorker->setServer( value_proposed.getData() );

		else if ( property_current.getKey() == _MailBox_KEY_Pop3UserName )
			ret = this->mboxWorker->setAccountUserName( value_proposed.getData() );

		else if ( property_current.getKey() == _MailBox_KEY_Pop3UserPassword )
			ret = this->mboxWorker->setAccountUserPassword( value_proposed.getData() );

		return ret;
	 }
	else if ( property_current.getKey() == _MailBox_KEY_AutoCheckFrequency ) {
		this->mboxWorker->setAutoCheckFrequency( value_proposed.toLong() );
	 }
	else if ( property_current.getKey() == _MailBox_KEY_SmtpHostName ) { }
	else if ( property_current.getKey() == _MailBox_KEY_SmtpUserName ) { }
	else if ( property_current.getKey() == _MailBox_KEY_SmtpUserPassword ) { }

	else // forward processing to base class
		return BundleGeneric::updatingProperty(do_update, value_proposed, property_current, source);
 }


///////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////// THREAD EVENTS
/*
void MailBoxBundle::thread_event( Thread::THREAD_STATE thread_event, Thread* thread_source )
 {
	long id = 0;
	if ( thread != NULL ) id = thread->getThreadID();
	Log(LOG_VERBOSE, ":thread_event(%i) : %s", id, Thread::getThreadStateAsString(event).c_str() );
	switch(event) {
		case Thread::STATE_RUNNING: {
				setBundleState( Bundle::STATE_STARTED );
				break;
			 }
		case Thread::STATE_STOPPED: {
				setBundleState( Bundle::STATE_STOPPED );
				break;
			 }
		default: break;
	 }
 }
*/
/////////////////////////////////////////////////////////////////////////////////////////////// THREAD EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////// GTALKBOT EVENTS

void MailBoxBundle::mailbox_Message( MailBoxImpl* , const string& sender_mail, const string& message )
 {
	// recognize the user
	string username = getUsernameFromEmail(sender_mail);
	if ( username == "" ) {
		Log(LOG_WARNING, ":mailbox_Message() : Email %s not recognized", sender_mail.c_str() );
		return;
	 }
	Log(LOG_VERBOSE, ":mailbox_Message() : Recognized %s as %s", sender_mail.c_str(), username.c_str() );

	///@todo EVAL USER/SOURCE vs. SECURITY RULES here

	this->contacts.set(sender_mail, Utilities::std_time()); //update contacts' timestamp 

	// inject TEXT_MESSAGE into WOSH
	TextMessage* txtmsg = new TextMessage();
	txtmsg->setSender_ID( sender_mail );
	txtmsg->setSender_User( username );
	txtmsg->setRecipent_ID( this->mboxWorker->getAccountName() );
	txtmsg->setRecipent_User( "wosh" );
	txtmsg->setPlain( message );

	raiseEvent( txtmsg );
 }

///////////////////////////////////////////////////////////////////////////////////////////// GTALKBOT EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

std::string MailBoxBundle::getUsernameFromEmail( const string& email_address )
 {
	std::vector<std::string> users = UserManager::findUserByProperty( "eMail", Data(email_address) );
	if ( users.size() == 0 ) {
		Log(LOG_WARNING, ":getUsernameFromEmail() : User '%s' not recognized!", email_address.c_str() );
		return "";
	 }
	else if ( users.size() > 1 ) {
		Log(LOG_WARNING, ":getUsernameFromEmail() : User '%s' not recognized!", email_address.c_str() );
		return "";
	 }
	return users[0];
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

std::string MailBoxBundle::getEmailFromUsername( const string& username )
 {
	bool found = UserManager::isUser(username);
	if ( !found ) {
		return "";
	 }
	return UserManager::getUserProperty(username, "eMail").getData();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

const std::string& MailBoxBundle::getServerAddress() const {
	return this->mboxWorker->getServerAddress();
 }
int MailBoxBundle::getServerPort() const {
	return this->mboxWorker->getServerPort();
 }
const std::string& MailBoxBundle::getAccountName() const {
	return this->mboxWorker->getAccountName();
 }

bool MailBoxBundle::setServer( const std::string& address, int port ) {
	return this->mboxWorker->setServer(address, port);
 }
bool MailBoxBundle::setAccount( const std::string& username, const std::string& password ) {
	return this->mboxWorker->setAccount(username, password);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool MailBoxBundle::isUserReachable( const string& username )
 {
	std::string email_address = getEmailFromUsername(username);
	if ( email_address == "" )
		return false;

	return true;
 }

bool MailBoxBundle::isUserRegistered( const string& username )
 {
	std::string email_address = getEmailFromUsername(username);
	if ( email_address == "" )
		return false;

	if ( !this->contacts.existsKey_(email_address) )
		return false;

	return true;
 }

long MailBoxBundle::getUserLastSeen( const string& username )
 {
	std::string email_address = getEmailFromUsername(username);
	if ( email_address == "" )
		return 0;

	MutexLockerRead mL(&this->contacts.mutex());
	if ( this->contacts.existsKey_(email_address) )
		return 0;

	if ( this->contacts.key_(email_address) < 0 ) // negative means he is offline from abs(.)
		return -this->contacts.key_(email_address);

	return this->contacts.key_(email_address);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT MailBoxBundle::sendMessageToUser( const string& wosh_username, const string& message )
 {
	std::string email_address = getEmailFromUsername(wosh_username);
	return sendMessageTo( email_address, message );
 }

WRESULT MailBoxBundle::sendMessageTo( const string& mail_address, const string& message )
 {
	if ( mail_address == "" ) return WRET_ERR_PARAM;
	WRESULT ret = MailBoxImpl::sendMail(	Properties.getValueOf(_MailBox_KEY_SmtpHostName).getData(),
											Properties.getValueOf(_MailBox_KEY_SmtpUserName).getData(),
											Properties.getValueOf(_MailBox_KEY_SmtpUserPassword).getData(),
											mail_address, "Message from WOSH", message );
	if ( ret != WRET_OK ) {
		Log(LOG_CRITICAL, ":sendMessageTo(%s) ERROR#%i sending message", mail_address.c_str(), ret );
		return WRET_ERR_INTERNAL;
	 }
	return WRET_OK;
 }

WRESULT MailBoxBundle::notify( const Notification* notification )
 {
	if ( notification == NULL ) return WRET_ERR_PARAM;
	WRESULT ret = WRET_OK;

	if ( notification->isForAnyone() || notification->getRecipent_User().size() == 0 ) {

		// should iterate when group-select or/and ?
		ret = WRET_ERR_INTERNAL;
	 }
	else
		return sendMessageToUser( notification->getRecipent_User(), notification->getPlain() );

	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS

Response* MailBoxBundle::mmDoListContacts( const Request* request )
 {
	if ( request == NULL ) return NULL;
	string option = request->getArgument().getData();
	Log(LOG_VERBOSE, "mmDoListContacts(%s)", option.c_str() );

	if ( option == "s" || option == "short" ) {
		DataList* objs = new DataList();
		this->contacts.mutex().lockForRead();
		std::map<std::string,long>::const_iterator it;
		std::map<std::string,long>::const_iterator it_end = this->contacts.stdEnd();
		for ( it=this->contacts.stdBegin(); it!=it_end; it++ ) {
			objs->add( new Data(it->first) );
		}
		this->contacts.mutex().unLock();
		return request->replyResponse(WRET_OK, objs );
	 }
	else {
		this->contacts.mutex().lockForRead();
		Table* tbl = new Table( this->contacts.size_(), 3, true );
		tbl->setTableName("Contacts");
		tbl->setHeaderHorizontal( new Data("Name"), 0);
		tbl->setHeaderHorizontal( new Data("Username"), 1);
		tbl->setHeaderHorizontal( new Data("Status"), 2);
		unsigned int iRow = 0;
		std::map<std::string,long>::const_iterator it;
		std::map<std::string,long>::const_iterator it_end = this->contacts.stdEnd();
		for ( it=this->contacts.stdBegin(); it!=it_end; it++ ) {

			tbl->set( new Data(it->first), iRow, 0);
			tbl->set( new Data(getUsernameFromEmail(it->first)), iRow, 1);
			if ( it->second > 0 )
				tbl->set( new Data("ONLINE at " + Utilities::getTimeStampUTF(it->second)), iRow, 2);
			else if ( it->second < 0 )
				tbl->set( new Data("OFFLINE from " + Utilities::getTimeStampUTF(-it->second)), iRow, 2);
			else
				tbl->set( new Data("UNKONWN!"), iRow, 2);

			++iRow;
		}
		this->contacts.mutex().unLock();
		return request->replyResponse(WRET_OK, tbl );
	 }
 }

Response* MailBoxBundle::mmDoSendMessageToUser( const Message* requestMessage )
 {
	if ( requestMessage == NULL || requestMessage->isEmpty() ) return NULL;
	if ( !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();

	std::string username = "";
	std::string message = "";
	if ( request->getData()->isClass(_DataList_TYPE) ) {
		const DataList* dl = dynamic_cast<const DataList*>(request->getData());
		if ( dl->size() < 2 ) {
			return request->replyResponse(WRET_ERR_PARAM, "Invalid arguments [username, message]" );
		 }
		username = dl->get(0).getData();
		dl->join(message, " ", 1);
	 }

	std::string sender = "";
	if ( requestMessage->getSecurityToken() != NULL )
		sender = requestMessage->getSecurityToken()->getUser();

	Log(LOG_VERBOSE, "mmDoSendMessageToUser(%s) : %s", username.c_str(), message.c_str() );

	if ( sender != username && sender != "wosh ")
		message += " [by " + sender + "]";

	WRESULT ret = sendMessageToUser( username, message );

	if ( request->doOmitResponse() )
		return NULL;
	if ( ret == WRET_OK )
		return request->replyResponse(ret, "Sent!" );
	return request->replyResponse(ret, "Internal Error!" );
 }

Response* MailBoxBundle::mmDoSendMail( const Message* requestMessage )
 {
	if ( requestMessage == NULL || requestMessage->isEmpty() ) return NULL;
	if ( !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();

	std::string email_address = "";
	std::string message = "";
	if ( request->getData()->isClass(_DataList_TYPE) ) {
		const DataList* dl = dynamic_cast<const DataList*>(request->getData());
		if ( dl->size() < 2 ) {
			return request->replyResponse(WRET_ERR_PARAM, "Invalid arguments [address, message]" );
		 }
		email_address = dl->get(0).getData();
		dl->join(message, " ", 1);
	 }

	std::string sender = "";
	if ( requestMessage->getSecurityToken() != NULL )
		sender = requestMessage->getSecurityToken()->getUser();

	Log(LOG_VERBOSE, "mmDoSendMail(%s) : %s", email_address.c_str(), message.c_str() );

	if ( sender != email_address && sender != "wosh ")
		message += " [by " + sender + "]";

	WRESULT ret = sendMessageTo( email_address, message );

	if ( request->doOmitResponse() )
		return NULL;
	if ( ret == WRET_OK )
		return request->replyResponse(ret, "Sent!" );
	return request->replyResponse(ret, "Internal Error!" );
 }

///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace devices
}; // namespace wosh
