/** @file    MailBoxImpl.cpp
 *  @author  Alessandro Polo
 *  @version $Id: MailBoxImpl.cpp 2082 2010-03-11 06:26:42Z alex $
 *  @brief
 * File containing methods for the MailBoxImpl class.
 * The header for this class can be found in MailBoxImpl.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "MailBoxImpl.h"

 #include <framework/network/SocketTcp.h>
 
 #include "3rdparty/SendEmail.h"


using namespace std;
using namespace wosh;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

MailBoxImpl::MailBoxImpl( BundleGeneric& bundle ) : BundleGenericWorker(bundle)
 {
	this->mboxListener = NULL;

	this->tcpSocket = NULL;
	this->ready = false;
	this->handshaked = false;

	this->readOnly = true;

	this->autoRefreshState = 0;
	this->messagesCount = -1;
	this->messagesSize = 0;
	this->statsTimestamp = 0;

	this->pop3ServerAddress = "";
	this->pop3ServerPort = 110;

	this->pop3AccountName = "";
	this->pop3AccountPwd = "";

	this->state = MailBoxImpl::NOTCONNECTED;

	this->msgBufferInSize = 0;
	this->msgBufferIn = new char[16384];
	this->msgBufferOut = new char[16384];
 }


MailBoxImpl::~MailBoxImpl() {
	Log(LOG_VERBOSE, ":~MailBoxImpl(): Destroying.." );

	if ( isThreadRunning() ) {
		quitThread(10000);
	 }

	if ( this->msgBufferOut != NULL )
		delete [] this->msgBufferOut;
	this->msgBufferOut = NULL;

	if ( this->msgBufferIn != NULL )
		delete [] this->msgBufferIn;
	this->msgBufferIn = NULL;
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////// RUN THREAD

bool MailBoxImpl::initializingThread()
 {
	Log(LOG_VERBOSE, ":runThread(): STARTING" );

	this->tcpSocket = new SocketTcp();
//	this->tcpSocket->setParent(this);
	this->tcpSocket->setListener(this);

	return true;
 }

void MailBoxImpl::runThread()
 {
	Log(LOG_VERBOSE, ":runThread(): RUNNING" );

	long counter = this->autoRefreshState - 30;
	this->ready = true;
	while(this->running) {
		Thread::sleepForSec(1);
		setThreadAlive();

		if ( autoRefreshState == 0 )
			continue;

		counter++;
		if ( counter < this->autoRefreshState )
			continue;
		counter = 0;
		WRESULT ret = WRET_OK;

		//check connection
		if ( this->tcpSocket->isConnected() ) {
			this->tcpSocket->disconnect();
		}

		Log(LOG_VERBOSE, "runThread() : Connecting.." );
		ret += connectServer(pop3ServerAddress, pop3ServerPort);
		if ( ret != WRET_OK ) {
			Log(LOG_WARNING, "runThread() : Error Connecting!" );
			continue;
		 }
		ret += login(pop3AccountName, pop3AccountPwd);
		if ( ret != WRET_OK ) {
			Log(LOG_WARNING, "runThread() : Error Logging in!" );
			continue;
		 }

		Log(LOG_VERBOSE, "runThread() : Synchronizing Mail.." );

		readMail();

		Thread::sleepForSec(1);
		ret += disconnectServer();

		Thread::sleepForSec(2);
	 }
 }

void MailBoxImpl::exitingThread()
 {
	Log(LOG_VERBOSE, ":runThread(): EXITING" );

	disconnectServer();

	delete this->tcpSocket; this->tcpSocket = NULL;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////


WRESULT MailBoxImpl::checkMail()
 {
	WRESULT stats = this->doGetStatus( this->messagesCount, this->messagesSize);

	if ( stats == WRET_OK ) {
		this->statsTimestamp = time(NULL);
		//notifyEvent_CheckedMail( this->messagesCount, true );
		return WRET_OK;
	 }

	return stats;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT MailBoxImpl::readMail()
 {
	MutexLockerWrite locker(&this->mails.mutex());

	vector<int> toDown;
	// compare old ULIST with new ULIST
	map<int,string> id_uid_list;
	WRESULT ret_uid = doGetMsgUIdList( id_uid_list );
	if ( ret_uid != WRET_OK ) {
		return WRET_ERR_INTERNAL;
	 }
	Log(LOG_VERBOSE, "readMail() : Comparing Messages %i <=> %i)", id_uid_list.size(), this->mails.size_() );

	map<int,string>::const_iterator it;
	for ( it=id_uid_list.begin(); it != id_uid_list.end(); it++ ) {
		bool mail_found = false;
		vector<Mail*>::const_iterator it_mails;
		for ( it_mails=this->mails.stdBegin(); it_mails != this->mails.stdEnd(); it_mails++ ) {
			Mail* mail = (*it_mails);
			if ( mail == NULL) continue;

			if ( mail->getUID() == (*it).second ) {
				mail_found = true;
				break;
			 }
		 }
		if ( !mail_found ) { // new mail !!
			toDown.push_back( (*it).first );
		 }
	 }

	bool getTop = false;

	Log(LOG_VERBOSE, "readMail() : Reading %i new messages", toDown.size() );

	vector<int>::const_iterator it_new;
	for ( it_new=toDown.begin(); it_new != toDown.end(); it_new++ ) {
		Mail* mail = new Mail();
		WRESULT mail_ok = doGetMessage( *it_new, mail, getTop );
	 }

	//notifyEvent_CheckedMail( this->mails.size_() );

	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////// POP3 METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
WRESULT MailBoxImpl::connectServer( const string& host, unsigned int port )
 {
	if ( this->tcpSocket == NULL ) return WRET_ERR_ILLEGAL_USE;

	WRESULT ret = this->tcpSocket->connectTo(host,port);
	
	if ( ret != WRET_OK ) {
		Log(LOG_CRITICAL, "connectServer() FAILED : Could not connect to %s port %i", host.c_str(), port );
		return WRET_ERR_PARAM;
	}

	long size = 0;
	bool read_ok = this->tcpSocket->waitResponse( this->msgBufferIn, 16384, size, 15000 );
	memset(this->msgBufferIn+size, 0, 1);
	string response(this->msgBufferIn);
	if ( !isResponseOK(response) ) {
		Log(LOG_CRITICAL, "connectServer() FAILED : Invalid Response %s", response.c_str() );
		return WRET_ERR_INTERNAL;
	 }
	Log(LOG_VERBOSE, "connectServer() : Connected %s %i", host.c_str(), port );

	this->state = MailBoxImpl::AUTHORIZATION;
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT MailBoxImpl::disconnectServer()
 {
	if ( this->tcpSocket == NULL ) return WRET_ERR_ILLEGAL_USE;

	if ( !this->tcpSocket->isConnected() ) {
		return WRET_ERR_ILLEGAL_USE;
	 }

//	if ( this->readOnly )
//		doResetMarks();

	if ( this->state != MailBoxImpl::AUTHORIZATION ) {
		Log(LOG_VERBOSE, "disconnectServer() : Sending QUIT" );
		string response = doCommand("QUIT",false);
		if ( !isResponseOK(response) ) {
			Log(LOG_WARNING,  "disconnectServer() : Error sending QUIT, forcing disconnection: " );
			this->tcpSocket->disconnect();
			return WRET_ERR_INTERNAL;
		 }
	}

	WRESULT ret = this->tcpSocket->disconnect();
	if ( ret != WRET_OK ) {
		Log(LOG_WARNING,  "disconnectServer() : TcpSocket disconnect error" );
		return WRET_ERR_INTERNAL;
	}
	Log(LOG_VERBOSE, "disconnectServer() : Disconnected." );

	this->state = MailBoxImpl::NOTCONNECTED;
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void MailBoxImpl::tcpSocketError( int error_code, const char* description, SocketTcp* socket )
 {
	(void)socket;
	if ( description == NULL ) {
		Log(LOG_WARNING, ":tcpSocketError(%i) Unknown Error description", error_code );
	 }
	else Log(LOG_WARNING, ":tcpSocketError(%i) : %s", error_code, description );

	switch(error_code) {
		default: {
			break;
		 }
	 }
 }

void MailBoxImpl::tcpDataAvailable( long data_len, SocketTcp* socket )
 {
	Log(LOG_VERBOSE, ":tcpDataAvailable(%i) ..", data_len );

	data_len = _GET_Min(data_len, 16383);

	memset(this->msgBufferIn+data_len, 0, 1);

	data_len = socket->read(this->msgBufferIn, data_len);


 }

////////////////////////////////////////////////////////////////////////////////////////////////// RUN THREAD
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//Sends a command to the server and returns the response
string MailBoxImpl::doCommand( const string& command, bool isMultiline)
 {
	if ( this->tcpSocket == NULL ) return "";

	string commandr = command + "\r\n";
	qint64 writeResult = this->tcpSocket->sendDatagram( commandr.c_str(), commandr.size() );
	if ( writeResult != commandr.size() ) {
		Log(LOG_WARNING, ":doCommand(%i bytes,M=%i) ERROR, Written %i", command.size(), (short)isMultiline, writeResult );
		return "";
	 }

	long size = 0;
	bool read_ok = this->tcpSocket->waitResponse( this->msgBufferIn, this->msgBufferInSize, size, 15000 );
	memset(this->msgBufferIn+size, 0, 1);

	Log(LOG_VERBOSE, ":doCommand(%i bytes,M=%i) Written&Got %i bytes", command.size(), (short)isMultiline, size );

	return string(this->msgBufferIn);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT MailBoxImpl::login( const string& user, const string& pwd )
 {
	Log(LOG_VERBOSE, ":login(%s,*) : Logging..", user.c_str() );

	string user_resp = doCommand( "USER " + user, false );
	if ( !isResponseOK(user_resp) ) {
		Log(LOG_CRITICAL, ":login(%s,*) : Error waiting USER!", user.c_str() );
		return WRET_ERR_INTERNAL;
	 }

	string pwd_resp = doCommand( "PASS " + pwd, false );
	if ( !isResponseOK(pwd_resp) ) {
		Log(LOG_CRITICAL, ":login(%s,*) : Error waiting PASS!", user.c_str() );
		return WRET_ERR_INTERNAL;
	 }

	this->state = MailBoxImpl::TRANSACTION;
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT MailBoxImpl::doGetStatus(int& nbMessages, int& totalSize)
 {
	string res = doCommand( "STAT", false );
	if ( !isResponseOK(res) ) {
		Log(LOG_CRITICAL, ":doGetStatus() ERROR Response [%s]", res.c_str() );
		return WRET_ERR_INTERNAL;
	 }

	vector<string> infos;
	Utilities::splitString(res, " ", infos, false, true);
	if (infos.size() < 3) {
		Log(LOG_CRITICAL, ":doGetStatus() ERROR Parsing Response [%s]", res.c_str() );
		return WRET_ERR_INTERNAL;
	 }
	
	nbMessages = Utilities::fromString<int>(infos.at(1));
	totalSize = Utilities::fromString<int>(infos.at(2));

	Log(LOG_VERBOSE, ":doGetStatus() %i Message(s), %i bytes", nbMessages, totalSize );

	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT MailBoxImpl::doDeleteMark( int msgIndex )
 {
	string res = doCommand( "DELE " + Utilities::toString(msgIndex), false );
	if ( !isResponseOK(res) ) {
		Log(LOG_CRITICAL, ":doDeleteMark(%i) ERROR Response [%s]", msgIndex, res.c_str() );
		return WRET_ERR_INTERNAL;
	 }
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT MailBoxImpl::doGetMsgList( map<int,int>& id_size_list )
 {
	string res = doCommand( "LIST", true ); // MULTILINE
	if ( !isResponseOK(res) ) {
		Log(LOG_CRITICAL, ":doGetMsgList() ERROR Response [%s]", res.c_str() );
		return WRET_ERR_INTERNAL;
	 }

	int errors = 0;
	vector<string> fields;
	vector<string> msgs;
	Utilities::splitString(res, "\r\n", msgs, false, true);
	if ( msgs.size() > 0 ) // remove +OK
		msgs.erase(msgs.begin());

	vector<string>::const_iterator it;
	vector<string>::const_iterator it_field;
	for( it=msgs.begin(); it!=msgs.end(); it++ ) {
		fields.clear();
		Utilities::splitString(*it, " ", fields, false, true);
		if ( fields.size() < 2 ) {
			Log(LOG_WARNING, ":doGetMsgList() ERROR parsing Response [%s]", (*it).c_str() );
			++errors;
			continue;
		 }
		id_size_list[ Utilities::fromString<int>(fields.at(0)) ] = Utilities::fromString<int>(fields.at(1));
	 }

	Log(LOG_VERBOSE, ":doGetMsgList() Got %i Message(s), %i errors", id_size_list.size(), errors );

	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT MailBoxImpl::doGetMsgUIdList( map<int,string>& id_uid_list )
 {
	string res = doCommand( "UIDL", true ); // MULTILINE
	if ( !isResponseOK(res) ) {
		Log(LOG_CRITICAL, ":doGetMsgUIdList() ERROR Response [%i]", res.size() );
		return WRET_ERR_INTERNAL;
	 }

	int errors = 0;
	vector<string> fields;
	vector<string> msgs;
	Utilities::splitString(res, "\r\n", msgs, false, true);
	if ( msgs.size() > 0 ) // remove +OK (first line)
		msgs.erase(msgs.begin());
	if ( msgs.back() == "." ) // remove . (last line)
		msgs.pop_back();
	else
		Log(LOG_WARNING, ":doGetMsgUIdList() ERROR Response does not end with '.'" );

	vector<string>::const_iterator it;
	vector<string>::const_iterator it_field;
	for( it=msgs.begin(); it!=msgs.end(); it++ ) {
		fields.clear();
		Utilities::splitString(*it, " ", fields, false, true);
		if ( fields.size() < 2 ) {
			Log(LOG_WARNING, ":doGetMsgUIdList() ERROR parsing Response [%s]", (*it).c_str() );
			++errors;
			continue;
		 }
		id_uid_list.insert ( pair<int,string>(Utilities::fromString<int>(fields.at(0)),fields.at(1)) );
	 }

	Log(LOG_VERBOSE, ":doGetMsgUIdList() Got %i Message(s), %i errors", id_uid_list.size(), errors );

	return WRET_OK;
 }

WRESULT MailBoxImpl::doGetMsgUId( int msgIndex, string& unique_id )
 {
	unique_id = "";
	string res = doCommand( "UIDL " + Utilities::toString<int>(msgIndex), false );
	if ( !isResponseOK(res) ) {
		Log(LOG_CRITICAL, ":doGetMsgUId() ERROR Response [%i bytes]", res.size() );
		return WRET_ERR_INTERNAL;
	 }
	Log(LOG_VERBOSE, ":doGetMessageTop(%i) Got Message [%s]", msgIndex, res.size()-4 );

	if ( res.size() > 4 )
		unique_id = res.substr(4);

	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT MailBoxImpl::doGetMessageTop( int msgIndex, string& msgTop, unsigned int nbLines )
 {
	msgTop = "";
	string res = doCommand( "TOP " + Utilities::toString<int>(msgIndex) +
							" " + Utilities::toString<int>(nbLines), true ); // MULTILINE
	if ( !isResponseOK(res) ) {
		Log(LOG_CRITICAL, ":doGetMessageTop(%i) ERROR Response [%s]", msgIndex, res.c_str() );
		return WRET_ERR_INTERNAL;
	 }
	Log(LOG_VERBOSE, ":doGetMessageTop(%i) Got Message [%s]", msgIndex, res.size()-4 );

	if ( res.size() > 4 )
		msgTop = res.substr(4);
	return WRET_OK;
 }

WRESULT MailBoxImpl::doGetMessage( int msgIndex, string& msg )
 {
	msg = "";
	string res = doCommand( "RETR " + Utilities::toString<int>(msgIndex), true ); // MULTILINE

	if (res.size() == 0) {
		Log(LOG_CRITICAL, ":doGetMessage(%i) ERROR Request [%s]", msgIndex, res.c_str() );
		return WRET_ERR_INTERNAL;
	 }
	if ( !isResponseOK(res) ) {
		Log(LOG_CRITICAL, ":doGetMessage(%i) ERROR Response [%s]", msgIndex, res.c_str() );
		return WRET_ERR_INTERNAL;
	 }

	Log(LOG_VERBOSE, ":doGetMessage(%i) Got Message [%s]", msgIndex, res.size()-4 );

	if ( res.size() > 4 )
		msg = res.substr(4);
	return WRET_OK;
 }

WRESULT MailBoxImpl::doGetMessage( int msgIndex, Mail* mail, short how )
 {
	(void)how;
	if ( mail == NULL )
		return WRET_ERR_PARAM;

	string res = doCommand( "RETR " + Utilities::toString<int>(msgIndex), true );
	if (res.size() == 0) {
		Log(LOG_CRITICAL, ":doGetMessage(%i) ERROR Request [empty]", msgIndex );
		return WRET_ERR_INTERNAL;
	 }
	if ( !isResponseOK(res) ) {
		Log(LOG_CRITICAL, ":doGetMessage(%i) ERROR Response [%i bytes]", msgIndex, res.size() );
		return WRET_ERR_INTERNAL;
	 }

	mail->clear();
	WRESULT ret = mail->loadFromPlain(res);
	if ( ret != WRET_OK ) {
		Log(LOG_WARNING, ": doGetMessage(%i) : ERROR Loading Message [%i bytes]", msgIndex, res.size() );
		return WRET_ERR_INTERNAL;
	 }
	Log(LOG_VERBOSE, ": doGetMessage(%i) : Loaded Message [%i bytes]", msgIndex, res.size() );

	mail->setID(msgIndex);
	string unique_id = "";
	ret = doGetMsgUId( msgIndex, unique_id );
	mail->setUID(unique_id);

	return ret;
 }
 
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool MailBoxImpl::isResponseOK( const string& data )
 {
	return ( data.size() > 2 && data.substr(0,3) == "+OK" );
 }

bool MailBoxImpl::doResetMarks()
{
	string res = doCommand( "RSET", false );
	if ( !isResponseOK(res) ) {
		Log(LOG_WARNING, "doResetMarks() : FAILED" );
		return false;
	 }
	return true;
}

bool MailBoxImpl::doNoOperation()
{
	string res = doCommand( "NOOP", false );
	if ( !isResponseOK(res) ) {
		Log(LOG_WARNING, "doNoOperation() : FAILED" );
		return false;
	 }
	return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void MailBoxImpl::setMailBoxListener( IMailBoxListener* listener ) {
	this->mboxListener = listener;
 }

bool MailBoxImpl::setServer( const std::string& address, int port ) {
	if ( isThreadRunning() ) return false;
	this->pop3ServerAddress = address;
	this->pop3ServerPort = port;
	return true;
 }
bool MailBoxImpl::setServerAddress( const std::string& address ) {
	if ( isThreadRunning() ) return false;
	this->pop3ServerAddress = address;
	return true;
 }
bool MailBoxImpl::setServerPort( int port ) {
	if ( isThreadRunning() ) return false;
	this->pop3ServerPort = port;
	return true;
 }

bool MailBoxImpl::setAccount( const std::string& name, const std::string& pwd ) {
	if ( isThreadRunning() ) return false;
	this->pop3AccountName = name;
	this->pop3AccountPwd = pwd;
	return true;
 }
bool MailBoxImpl::setAccountUserName( const std::string& name ) {
	if ( isThreadRunning() ) return false;
	this->pop3AccountName = name;
	return true;
 }
bool MailBoxImpl::setAccountUserPassword( const std::string& pwd ) {
	if ( isThreadRunning() ) return false;
	this->pop3AccountPwd = pwd;
	return true;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////// STATIC

WRESULT MailBoxImpl::sendMail( const string& hostname, const string& username, const string& password, const string& recipent_address, const string& subject, const string& body )
 {
	bool sent = SendEmailSimple(	hostname.c_str(), recipent_address.c_str(),
									subject.c_str(), body.c_str(),
									username.c_str(), password.c_str() );
	if ( sent )
		return WRET_OK;
	return WRET_ERR_INTERNAL;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
