/**
 * @class   wosh::devices::ModemDslBundle
 * @brief   Device/Bundle implementation providing ModemDsl abstraction
 *          for POSIX (linux) systems, designed for PPPoE driver.
 *
 *          Works as Bundle and register itself as Device.
 *          Offers live (auto-refreshed) properties (stats).
 *
 *          Real implementation is class wosh::devices::ModemDsl, most
 *          methods are just a proxy to that internal object.
 *
 * @ingroup Devices
 * @see     wosh::devices::ModemDsl
 ****************************************************************************
 * @version $Id: ModemDslBundle.h 2074 2010-03-10 02:38:16Z alex $
 * @author  Alessandro Polo
 * @date    Released on 2009-06-04
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Bundles_Devices_ModemDsl_H__
 #define __WOSH_Bundles_Devices_ModemDsl_H__

 #include <core/BundleGeneric.h>
 #include <core/Thread.h>

 #include <interfaces/Device.h>
 #include <interfaces/devices/Modem.h>
 #include <interfaces/devices/ModemDsl.h>

 #include "IModemDslListener.h"

class ModemDslImpl;

namespace wosh {
 namespace devices {

 #define _ModemDsl_NAME							"ModemDsl"
 #define _ModemDsl_VERSION						1.01

 #define _ModemDsl_DEFAULT_pppInterface			"ppp0"
 #define _ModemDsl_DEFAULT_PingCheckAddress		"google.com"

 #define _ModemDsl_DEFAULT_Script				"$CONFIG/scripts/dslmodem.sh"


class ModemDslBundle :	public BundleGeneric,
						public virtual wosh::interfaces::Device,
						public virtual wosh::interfaces::devices::Modem,
						public virtual wosh::interfaces::devices::ModemDsl,
						public virtual IModemDslListener
 {
	WOSH_BUNDLE(wosh::devices::ModemDslBundle)
	WOSH_BUNDLE_INTERFACE("wosh::interfaces::devices::ModemDsl")

	public:

		/**
		 * @brief   Empty constructor, init bundle and vars, Properties and Methods.
		 */
		ModemDslBundle( const std::string& name = "" );

		/**
		 * @brief   Empty destructor, may (synch) stop bundle and internal threads when still running
		 */
		virtual ~ModemDslBundle();


/** @name Bundle Interface
 * @{
 ******************************************************************************/
	public:

		/**
		 * @brief   Start the internal thread (used for refreshing stats). Also register bundle as Device
		 * @see    BundleGeneric::bundleStart()
		 */
		virtual WRESULT bundleStart();

		/**
		 * @brief   Stop the internal thread (used for refreshing stats). Also unregister bundle as Device
		 * @see    BundleGeneric::bundleStop()
		 */
		virtual WRESULT bundleStop();

		virtual WRESULT bundleDiagnostic();

		virtual bool updatingProperty( bool& do_update, const Variant& value_proposed, Property& property_current, const PropertiesProvider* source );

		virtual void busMessage( const Message& message, const Bus* source );

//@}
/** @name Device Interface
 * @{
 ******************************************************************************/
	public:

		virtual bool isDeviceLinked() const					{ return (this->devState == wosh::interfaces::Device::LINKED); }
		virtual bool isDeviceUnLinked() const				{ return (this->devState == wosh::interfaces::Device::UNLINKED); }
		virtual bool isDeviceBroken() const					{ return (this->devState == wosh::interfaces::Device::BROKEN); }

		virtual wosh::interfaces::Device::DEVICE_STATE getDeviceState() const
															{ return this->devState; }

//@}
/** @name ModemDsl Interface
 * @{
 ******************************************************************************/
	public:

		virtual bool isModemConnected() const				{ return (this->connState == wosh::interfaces::devices::Modem::CONNECTED); }

		virtual bool isModemDisconnected() const			{ return (this->connState == wosh::interfaces::devices::Modem::DISCONNECTED); }

		virtual wosh::interfaces::devices::Modem::CONNECTION_STATE getConnectionState() const
															{ return this->connState; }


		/**
		 * @brief   Connect modem. Raise Fact when connected.
		 * @param params Connection parameters (when required)
		 * @return  WRET_ERR_INTERNAL if something went wrong; WRET_OK is modem has been disconnected.
		 */
		virtual WRESULT connect( std::string params = "" );

		/**
		 * @brief   Disconnect modem. Raise Fact when disconnected.
		 * @return  WRET_ERR_INTERNAL if something went wrong; WRET_OK is modem has been disconnected.
		 */
		virtual WRESULT disconnect();

		/**
		 * @brief   Reconnect modem. Raise Fact when connected.
		 * @return  WRET_ERR_INTERNAL if something went wrong; WRET_OK is modem has been disconnected.
		 */
		virtual WRESULT reconnect();

		/**
		 * @brief   Force connection check. Raise Fact when connection state changed.
		 * @return  WRET_ERR_INTERNAL if something went wrong; WRET_OK if check was successful.
		 */
		virtual WRESULT checkConnected();

		/**
		 * @brief   Force stats reset.
		 * @return  WRET_NOT_IMPLEMENTED or WRET_ERR_ILLEGAL_USE if feature is not available;
		 *          WRET_ERR_INTERNAL if something went wrong; WRET_OK if check was successful.
		 */
		virtual WRESULT resetStats();

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:

		/**
		 * @brief   Get period of refresh loop
		 * @return  refresh period in seconds
		 */
		long getRefreshFrequency() const;

		/**
		 * @brief   Get address used while checking connection through ping
		 * @return  IPv4 or hostname address (example: www.google.com)
		 */
		std::string getPingCheckAddress() const;

		/**
		 * @brief   Get full path of support script, used to control the modem
		 * @return  full path of script (example: bin/scripts/dslmodem.sh)
		 */
		std::string getCtrlScript() const;

		/**
		 * @brief   Get POSIX interface associated with connection
		 * @return  interface name (example: ppp0)
		 */
		std::string getPppInterface() const;

//@}
/** @name Modem Events
 * @{
 ******************************************************************************/
 	public:
		virtual void modem_ConnectionState( ModemDslImpl* modem, wosh::interfaces::devices::Modem::CONNECTION_STATE state );
		virtual void modem_StatsUpdate( ModemDslImpl* modem, const pppStats& stats );

//@}
/** @name Thread Events
 * @{
 ******************************************************************************/
 	public:
		virtual void thread_event( Thread::THREAD_STATE thread_event, Thread* thread_source );
//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:

		/**
		 * @brief   Connect method, accept connection parameters as (first) argument
		 * @return  standard WRESULT return value and connection state as response's data
		 */
		Response* mmDoConnect( const Request* request );

		/**
		 * @brief   Force-State method, forces current Modem-Connection state to passed value (CONNECTION_STATE as string)
		 * @return  standard WRESULT return value and connection state as response's data
		 */
		Response* mmDoForceState( const Request* request );

		/**
		 * @brief   Check-Connected method, force connection-check (using script)
		 * @return  standard WRESULT return value and connection state as response's data
		 * @note    method calls directly the bash support script, by default it pings an address
		 * @see     _ModemDsl_KEY_PingCheckAddress
		 */
		Response* mmDoCheckConnected( const Request* request );

		/**
		 * @brief   Raise a standard Device event (fact): Create Message and inject in BusDevices (wosh.Bus.Devices)
		 * @param fact [in] the fact (instance) to include in (event) Message, instance will be deleted by Bus
		 */
		void raiseEvent( Fact* fact );

		/**
		 * @brief   Update Connection State of Modem, internal var and Property. Also may raise Device event
		 * @param state [in] the new connection's state
		 * @param raiseEvent [in] pick if/how raise Device Event. '0' won't raise event; '2' will always raise event;
		 *                         '1' will raise event only if connection's state changed. Event message (Fact) contains
		 *                         CONNECTION_STATE as string.
		 */
		void setConnectionState( wosh::interfaces::devices::Modem::CONNECTION_STATE state, int raiseEvent = 1 );

//@}

	protected:

		BusConnector BusDevices;					///< register myself & work as Device too

		ModemDslImpl* modemWorker;					///< real implementation

		wosh::interfaces::devices::Modem::CONNECTION_STATE connState;	///< current state of the modem
		wosh::interfaces::Device::DEVICE_STATE devState;		///< current state of device


}; // class def

 }; // namespace devices
}; // namespace wosh

#endif //__WOSH_Bundles_Devices_ModemDsl_H__
