/**
 * @class   wosh::services::BigBrotherBundle
 * @brief   A network-singleton service designed to recognize users
 *          and their location.
 *
 *
 *
 * ---------------------------------------------------------------
 * \verbatim

Specifications:
  - Monitor Bus: Devices, Building
  - Manage wosh::building::Location (load (create), update)
  - recognize/guess user vs. locations
  - Update User<>::Location<...>

An inhabitant (user) may be recognized in a wosh::building::Location \b L at time \b T
with probability \b P
This is a record of wosh::UserInfoLocation collection.

Location is defined by:
  - a wosh::building::Room
  - a zone defined by a Device ????

Location holds information about:


User Recognition:
  ? out of home? how? (forced?, my communicator info?)
  > at home: assuming: system works with (only) 1 user @ home

  - device interaction with authorization and location info (ex. DesktopNotifier)
  - follows user within the home, using (motion) sensors
  

Whole-System Effects:
  - updating user's location, any services which may/require this information may read it..
  - communication: since comminicator devices may have a reference location, the Communicator
    Manager may select the nearest to the current user's position (according to many other factors)
  - privacy, the network and WOSH itself should be secure to prevent hacking

\endverbatim
 * ---------------------------------------------------------------
 *
 *
 *
 *
 *
 *
 * @ingroup Services
 ****************************************************************************
 * @version $Id: BigBrotherBundle.h 2074 2010-03-10 02:38:16Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Bundles_Services_BigBrother_H__
 #define __WOSH_Bundles_Services_BigBrother_H__

 #include <core/BundleGeneric.h>
 #include <core/Thread.h>

 #include <core/Communicator.h>
 #include <interfaces/Service.h>
 #include <interfaces/services/BigBrother.h>

 #include "IBigBrotherListener.h"

class BigBrotherParser;

using namespace std;
namespace wosh {
 namespace services {

 #define _BigBrother_VERSION			0.01



class BigBrotherBundle :	public BundleGeneric,
							public virtual wosh::interfaces::services::BigBrother,
							public virtual IBigBrotherListener
 {
	WOSH_BUNDLE(wosh::services::BigBrotherBundle)
	WOSH_BUNDLE_INTERFACE("wosh::interfaces::services::BigBrother")

	public:

		BigBrotherBundle();

		virtual ~BigBrotherBundle();


/** @name Bundle Interface
 * @{
 ******************************************************************************/
	public:
		virtual WRESULT bundleStart();
		virtual WRESULT bundleStop();

		virtual void busMessage( const Message& message, const Bus* source );


/** @name BigBrother Interface
 * @{
 ******************************************************************************/
	public:

		WRESULT updateHomeLocations();


//@}
/** @name Getters
 * @{
 ******************************************************************************/
 	public:


//@}
/** @name Interface for Communication callbacks
 * @{
 ******************************************************************************/
	public:

		void evalResponse( const Message& message );

		virtual void updateObject( MessageContent* content, const string& uri, BigBrotherImpl* );

//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:
		Response* mmDoListHomeLocations( const Request* request );

		Response* mmDoForceUserIn( const Message* requestMessage );

//@}


	protected:

		BusConnector BusBuilding;
		BusConnector BusDevices;
		BusConnector BusSecurity;

		BigBrotherImpl* butlerWorker;
		BigBrotherParser* butlerParser;


}; // class def

 }; // namespace services
}; // namespace wosh

#endif //__WOSH_Bundles_Services_BigBrother_H__
