/**
 * @class   wosh::services::BuildingManagerBundle
 * @brief   Manager of Building Objects (such as Floor, Room and other
 *          types from wosh::building namespace).
 *
 *
 locations planning:
------------------------------------------------------------------
 goals:
 - used to reference users' position
 - someway mapped to BuildingObject
 - hierarchy, cross-relations (access-to, child-of)
 - geometry and distance
 - 

 system-relations
 - WRITTEN: by BuildingManager only
 - READ: heavily by BigBrother (predict, undestand where users are)
 - READ: my Butler.. to predict and choose users' actions
 - REFERRED: saved in Users' data (through UserManager)

------------------------------------------------------------------
 *
 *
 * @ingroup Services
 ****************************************************************************
 * @version $Id: BuildingManagerBundle.h 2129 2010-03-14 18:15:59Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Bundles_Services_BuildingManager_H__
 #define __WOSH_Bundles_Services_BuildingManager_H__

 #include <core/BundleGeneric.h>
 #include <core/Containers.h>

 #include <framework/building/BuildingObject.h>
 #include <framework/building/Location.h>

 #include <interfaces/Service.h>
 #include <interfaces/services/BuildingManager.h>


using namespace std;
using namespace wosh::building;
namespace wosh {
 namespace services {

 #define _BuildingManager_VERSION				0.85


class BuildingManagerBundle :	public BundleGeneric,
								public virtual wosh::interfaces::services::BuildingManager
 {
	WOSH_BUNDLE(wosh::services::BuildingManagerBundle)
	WOSH_BUNDLE_INTERFACE("wosh::interfaces::services::BuildingManager")

	typedef Map22<std::string, BuildingObject*> tStrBuildingObjectMap;
	typedef Map22<std::string, Location*> tStrLocationMap;


	public:

		BuildingManagerBundle();

		virtual ~BuildingManagerBundle();


/** @name Bundle Interface
 * @{
 ******************************************************************************/
	public:
		virtual WRESULT bundleStart();
		virtual WRESULT bundleStop();

		virtual void busMessage( const Message& message, const Bus* source );

//@}
/** @name BuildingManager Interface
 * @{
 ******************************************************************************/
	public:
		virtual WRESULT loadBuildingObjects();
		virtual WRESULT saveBuildingObjects();

//@}
/** @name IPropertiesProviderListener interface
 * @{
 ******************************************************************************/
 	public:
//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:
		Response* mmDoListBObj( const Request* request );
		Response* mmDoReloadBObj( const Request* request );
		Response* mmDoFakeBuildingObjEvent( const Request* request );
		Response* mmDoGetPropertiesOf( const Message* requestMessage );

	protected:
		WRESULT addObject_( BuildingObject* obj );
		WRESULT removeObject_( BuildingObject* obj );

		WRESULT fixReferences_( BuildingObject* bobj );
		void applyNested_( BuildingObject* bobj );

		WRESULT fixReferencesBy( const std::string& deviceName );

//@}
	protected:
		BusConnector BusBuilding;

		tStrBuildingObjectMap objects;
		tStrLocationMap locations;


}; // class def

 }; // namespace services
}; // namespace wosh

#endif //__WOSH_Bundles_Services_BuildingManager_H__
