/** @file    FreeLingImpl.cpp
 *  @author  Alessandro Polo
 *  @version $Id: FreeLingImpl.cpp 2082 2010-03-11 06:26:42Z alex $
 *  @brief
 * File containing methods for the FreeLingImpl class.
 * The header for this class can be found in FreeLingImpl.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "FreeLingImpl.h"

 #include <core/Message.h>
 #include <core/FileSystem.h>
 #include <core/SecurityManager.h>


 #include <freeling.h>


using namespace std;
using namespace wosh;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

FreeLingImpl::FreeLingImpl( BundleGeneric& bundle ) : BundleGenericWorker(bundle)
 {
	this->flTokenizer = NULL;
	this->flSplitter = NULL;
	this->flMacoOptions = NULL;
	this->flMacoMorfo = NULL;
	this->flTagger = NULL;
	this->flParser = NULL;
	this->flTxala = NULL;


 }


FreeLingImpl::~FreeLingImpl() {
	if ( isThreadRunning() ) {
		quitThread(10000);
	 }

 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////// RUN THREAD

void FreeLingImpl::runThread()
 {
	//------------------------------------------------------------------------------
	// INIT

	string path = "/usr/share/FreeLing/en/";
	// create analyzers
	this->flTokenizer = new tokenizer( path + "tokenizer.dat" );
	this->flSplitter = new splitter( path + "splitter.dat" );

	// morphological analysis has a lot of options, and for simplicity they are packed up
	// in a maco_options object. First, create the maco_options object with default values.
	this->flMacoOptions = new maco_options("en");  

	// then, set required options on/off  
	configure( this->flMacoOptions, path );

	// create the analyzer with the just build set of maco_options
	this->flMacoMorfo = new maco( *this->flMacoOptions );

	// create a hmm tagger for spanish (with retokenization ability, and forced 
	// to choose only one tag per word)
	this->flTagger = new hmm_tagger( "en", path + "tagger.dat", true, true ); 

	// create chunker
	this->flParser = new chart_parser( path + "grammar-dep.dat" );

	// create dependency parser 
	this->flTxala = new dep_txala( path + "dep/dependences.dat", this->flParser->get_start_symbol() );

	// INIT
	//------------------------------------------------------------------------------

	while( this->running )
	 {
		Thread::sleepForMSec(100);
		setThreadAlive();

		this->jobs.mutex().lockForWrite();
		if ( this->jobs.size_() == 0 ) {
			Thread::sleepForMSec(100);
			this->jobs.mutex().unLock();
			continue;
		 }

		FreeLingJob* job = this->jobs.stdFront();
		this->jobs.remove_(job);
		this->jobs.mutex().unLock();

		processJob(job);

		if ( job != NULL )
			delete job;
	 }

	//------------------------------------------------------------------------------
	// release instances
	delete this->flTxala; this->flTxala = NULL;

	delete this->flParser; this->flParser = NULL;

	delete this->flTagger; this->flTagger = NULL;

	delete this->flMacoMorfo; this->flMacoMorfo = NULL;

	delete this->flMacoOptions; this->flMacoOptions = NULL;

	delete this->flSplitter; this->flSplitter = NULL;
	delete this->flTokenizer; this->flTokenizer = NULL;
	//------------------------------------------------------------------------------


	Log(LOG_VERBOSE, ":runThread(): EXITING" );
 }

////////////////////////////////////////////////////////////////////////////////////////////////// RUN THREAD

void FreeLingImpl::processJob( FreeLingJob* job )
 {
	if ( job == NULL ) return;
	if ( job->conversion == NULL ) return;
	if ( job->conversion->getOriginal() == NULL ) return;
	if ( job->conversion->getOriginal()->getPlain().empty() ) {
		Log(LOG_CRITICAL, ":processJob(): %i is EMPTY!", job->conversion->getOriginal()->getID() );
		return;
	 }

	Log(LOG_INFO, ":processJob(): Converting %i", job->conversion->getOriginal()->getID() );

	TextMessage* aMsg = new TextMessage();
	aMsg->fromNotification( job->conversion->getOriginal() );

	//------------------------------------------------------------------------------
	// do CONVERT
	list<sentence> ls;
	analyzeText( job->conversion->getOriginal()->getPlain(), ls );
	processSentences( ls );
	// do CONVERT
	//------------------------------------------------------------------------------



	Message* message = new Message( aMsg );
	message->setSource(this->getOwner()->getURI());
	if ( job->requestSource.isEmpty() )
		message->getDestination().fromString("*:Communication");
	else
		message->setDestination( job->requestSource );

	message->setSecurityToken( SecurityManager::generateToken(job->conversion->getOriginal()->getSender_User()) );

	Log(LOG_VERBOSE, ":processJob(): Replying %i to %s", job->conversion->getOriginal()->getID(), job->requestSource.toString().c_str() );

	BusCore.postMessage(message);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FreeLingImpl::enqueue( FreeLingJob* job )
 {
	if ( job == NULL ) return;
	if ( job->conversion == NULL ) return;
	if ( job->conversion->getOriginal() == NULL ) return;
	if ( job->conversion->getOriginal()->getPlain().empty() ) {
		Log(LOG_CRITICAL, ":processJob(): %i is EMPTY!", job->conversion->getOriginal()->getID() );
		return;
	 }

	Log(LOG_INFO, ":enqueue(): Enqueued %i", job->conversion->getOriginal()->getID() );

	this->jobs.add(job);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FreeLingImpl::configure( maco_options* opt, const string& path )
 {
	opt->QuantitiesDetection = false;  //deactivate ratio/currency/magnitudes detection 
	opt->AffixAnalysis = true; opt->MultiwordsDetection = true; opt->NumbersDetection = true; 
	opt->PunctuationDetection = true; opt->DatesDetection = true; opt->QuantitiesDetection = false; 
	opt->DictionarySearch = true; opt->ProbabilityAssignment = true; opt->NERecognition = NER_BASIC;   
	// alternatively, you can set active modules in a single call:
	//     opt->set_active_modules(true, true, true, true, true, false, true, true, NER_BASIC);

	// and provide files for morphological submodules. Note that it is not necessary
	// to set opt->QuantitiesFile, since Quantities module was deactivated.
	opt->LocutionsFile = path + "locucions.dat";
	opt->AffixFile = path + "afixos.dat";
	opt->ProbabilityFile = path + "probabilitats.dat";
	opt->DictionaryFile = path + "maco.db";
	opt->NPdataFile = path + "np.dat";
	opt->PunctuationFile = path + "../common/punct.dat"; 
	// alternatively, you can set the files in a single call:
	//  opt.set_data_files(path+"locucions.dat", "", path+"afixos.dat", 
	//                     path+"probabilitats.dat", path+"maco.db", 
	//                     path+"np.dat", path+"../common/punct.dat");
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FreeLingImpl::analyzeText( const string& text, list<sentence>& ls )
 {
	list<word> lw;
	// tokenize input line into a list of words
	lw = this->flTokenizer->tokenize(text);

	// accumulate list of words in splitter buffer, returning a list of sentences.
	// The resulting list of sentences may be empty if the splitter has still not 
	// enough evidence to decide that a complete sentence has been found. The list
	// may contain more than one sentence (since a single input line may consist 
	// of several complete sentences).
	ls = this->flSplitter->split(lw, false);

	// perform and output morphosyntactic analysis and disambiguation
	this->flMacoMorfo->analyze(ls);
	this->flTagger->analyze(ls);

	lw.clear(); 
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FreeLingImpl::processSentences( const list<sentence>& ls )
 {
	list<sentence>::const_iterator s;
	word::const_iterator a;   //iterator over all analysis of a word
	sentence::const_iterator w;

	// for each sentence in list
	for(s=ls.begin(); s!=ls.end(); s++) {
		// print sentence XML tag
		cout<<"<SENT>"<<endl;

		// for each word in sentence
		for(w=s->begin(); w!=s->end(); w++) {
			// print word form, with PoS and lemma chosen by the tagger
			cout<<"  <WORD form=\""<<w->get_form();
			cout<<"\" lemma=\""<<w->get_lemma();
			cout<<"\" pos=\""<<w->get_parole();
			cout<<"\">"<<endl;

			// for each possible analysis in word, output lemma, parole and probability
			for(a=w->analysis_begin(); a!=w->analysis_end(); ++a) {
				// print analysis info
				cout<<"    <ANALYSIS lemma=\""<<a->get_lemma();
				cout<<"\" pos=\""<<a->get_parole();
				cout<<"\" prob=\""<<a->get_prob();
				cout<<"\"/>"<<endl;
			 }

			// close word XML tag after list of analysis
			cout<<"</WORD>"<<endl;
		 }

		// close sentence XML tag
		cout<<"</SENT>"<<endl;
	 }

 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

