/** @file    SessionProtocolHuman.cpp
 *  @author  Alessandro Polo
 *  @version $Id: SessionProtocolHuman.cpp 2030 2010-03-04 09:41:53Z alex $
 *  @brief
 * File containing methods for the SessionProtocolHuman class.
 * The header for this class can be found in SessionProtocolHuman.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "SessionProtocolHuman.h"
 #include "HumanInterpreterBundle.h"
 
 #include <core/Bundle.h>
 #include <core/DataList.h>
 #include <core/Request.h>
 #include <core/Response.h>
 #include <core/Property.h>
 #include <core/Notification.h>

 #include <core/BundleManager.h>
 #include <core/Utilities.h>
 #include <core/InterfacesProvider.h> // cos of method def
 #include <core/MethodsProvider.h> // cos of method def
 #include <core/PropertiesProvider.h> // cos of method def
 #include <core/WoshKernel.h> // cos of method def
  
 #include <QtDebug>

 
using namespace std;
using namespace wosh;
using namespace wosh::services;


int tokenize_message_in_sentences( std::vector<string>& sentences, const string& message );
int build_response( string& output, const vector<string>& responses, int preferred_max_size );


 #define _ENDLINE	"\r\n"
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

SessionProtocolHuman::SessionProtocolHuman() : SessionProtocol()
 {
	this->hiParent = NULL;
 }

SessionProtocolHuman::~SessionProtocolHuman()
 {

 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool SessionProtocolHuman::getLoginData( wosh::communication::Session& session, string& user, string& pwd )
 {
	bool sentok = true;

	session.printMessage("Username: ");
//	session.istream() >> user;

	session.printMessage("Password: ");
//	session.istream() >> pwd;

	return sentok;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SessionProtocolHuman::parse_command( wosh::communication::Session& session, const string& dataline )
 {
	ShellCommand curr_cmd;

	curr_cmd.fromString( dataline );

	bool found = internal_command( session, &curr_cmd );
	if ( found )
		return;
	if ( this->hiParent == NULL ) {
		session.printMessage("  Error: can't access HumanInterpreter (NULL)"_ENDLINE);
		return;
	 }

	string response = "";
	WRESULT ret = evalAnswerHumanMessage( session, dataline, response );
	if ( WFAILED(ret) ) {
		session.printMessage( Utilities::format("  Error: HumanInterpreter Eval-Error #%i"_ENDLINE, ret) );
		return;
	 }
	session.printMessage(response+_ENDLINE);
 }


bool SessionProtocolHuman::internal_command( wosh::communication::Session& session, ShellCommand* command )
 {
	if ( command == NULL )
		return false;
	if ( command->getMainCommand() == "exit" || command->getMainCommand() == "quit" ) {
		session.sessionStop(); // ByUser
		return true;
	 }
	else if ( command->getMainCommand() == "help" ) {
		session.printMessage(	_ENDLINE "WOSH Shell Protocol - Commands:" _ENDLINE \
								"\t help : \t print this screen" _ENDLINE \
								"\t proto : \t change session protocol" _ENDLINE \
								"\t exit : \t close session" _ENDLINE _ENDLINE
							);
		return true;
	 }
	else if ( command->getMainCommand() == "proto" ) {
		if ( command->getArgCount() == 0 )
			session.printMessage( Utilities::format("  SessionProtocol :\t%s [%f]"_ENDLINE, this->getProtocolName().c_str(), this->getProtocolVersion()) );
		else if ( this->parent == NULL )
			session.printMessage("  SessionProtocol :\tINTERNAL ERROR"_ENDLINE);
		else {
			this->parent->changeSessionProtocol( &session, command->getArg(0) );
//			session.ostream() << "  SessionProtocol :\t" << this->getProtocolName() << " " << this->getProtocolVersion() << _ENDLINE << std::flush;
		 }
		return true;
	 }




	return false;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT SessionProtocolHuman::evalAnswerHumanMessage( wosh::communication::Session& session, const string& message, string& response, int preferred_max_size )
 {
	vector<string> sentences;
	vector<string> responses;

	// tokenize message in sentences using punctuation
	int count = tokenize_message_in_sentences( sentences, message );

	if ( count == 0 ) {
		response = "Can't tokenize sentences";
//		Log(LOG_VERBOSE, ":evalAnswerHumanMessage(%s) ERROR : %s", message.c_str(), response.c_str() );
		return WRET_ERR_PARAM;
	 }

	// iterate and gen action->response for each sentence
	std::vector<std::string>::const_iterator it;
	for ( it=sentences.begin(); it!=sentences.end(); it++ ) {
		response = "";
		WRESULT ret_s = evalAnswerHumanRequest( session, *it, response, preferred_max_size/count );
		responses.push_back(response);
	 }

	 // implode responses
	build_response( response, responses, preferred_max_size );

//	Log(LOG_VERBOSE, ":evalAnswerHumanMessage(%s) : %s", message.c_str(), response.c_str() );

	return WRET_OK;
 }


WRESULT SessionProtocolHuman::evalAnswerHumanRequest( wosh::communication::Session& session, const string& sentence, string& response, int preferred_max_size )
 {
	if ( this->parent == NULL ) {
		session.printMessage("  Error: can't post messages (NULL parent)"_ENDLINE);
		return WRET_ERR_INTERNAL;
	 }
//	Log(LOG_VERBOSE, ":evalAnswerHumanRequest(%s) : %s", sentence.c_str(), response.c_str() );

	// understand message (grammar)
	// METHOD (verb) [ARGUMENTS(words)] => TARGET (object)

	// find the target
	///@bug assume target is last word:
	std::vector<std::string> words;
	string uri_target = "";
	string method = "";
	int words_c = Utilities::splitString(sentence, " ", words, false, true );
	if ( words_c == 0 ) {
		return WRET_ERR_PARAM;
	 }
	else if ( words_c == 1 ) {
		// load/assume last target of current session
		
// uri_target = session->lastTargeT!
	 }
	else {
		uri_target = "*:**/~" + words.back();
		words.pop_back();
	 }

	method = words.front();
	words.erase(words.begin());

	DataList* arguments = NULL;
	if ( words.size() > 0 ) {
		arguments = new DataList();
		arguments->fromVector( words );
	 }

	Request* request = new Request(method, arguments);

	Message* message = new Message();
	message->setSource( &session );
	message->getDestination().fromString(uri_target);
	message->setDestinationBus( "*" );
	message->setContent( request );

	bool sent = this->parent->session_post_message( &session, message );
	if ( !sent )
		session.printMessage("  Error: can't post messages (unknown)"_ENDLINE);
	return WRET_OK;
 }

WRESULT SessionProtocolHuman::translateMessageToHumanLanguage( const Message& message, string& res_output, int preferred_max_size )
 {
	if ( message.getContent()->isResponse() ) {
		const Response* response = message.getContent()->asResponse();

		if ( response->getReturnValue() != WRET_OK ) {
			res_output = Utilities::format("ERROR#%i of %s", response->getReturnValue(), response->getMethod().c_str() );
			if ( response->hasData() )
				res_output += ": " + response->getData()->toString();
			else
				res_output += ".";
		 }
		else {
			res_output = "OK(" + response->getMethod();
			if ( response->hasData() )
				res_output += "): " + response->getData()->toString();
			else
				res_output += ").";
		 }
	 }
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SessionProtocolHuman::evalMessage( wosh::communication::Session& session, const Message* message )
 {
	if ( message == NULL || message->isEmpty() ) return;

	string response = "";
	WRESULT ret = translateMessageToHumanLanguage( *message, response );
	if ( WFAILED(ret) ) {
		session.printMessage( Utilities::format("  Error: HumanInterpreter Translate Error #%i"_ENDLINE, ret) );
		return;
	 }
	session.printMessage(response+_ENDLINE);
 }


void SessionProtocolHuman::printDataList( wosh::communication::Session& session, const DataList* list, const string& header )
 {
	if ( list == NULL || list->isEmpty() ) {
		session.printMessage(header + " List is Empty!" + _ENDLINE);
		return;
	 }

	string buffer = "";
	vector<string> keys;
	list->toVector( keys );
	std::vector<string>::const_iterator it;
	for ( it=keys.begin(); it!=keys.end(); it++ ) {
		buffer += "\t" + *it + "\r\n";
	 }
	session.printMessage(header + "\r\n" + buffer + _ENDLINE);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SessionProtocolHuman::printPrompt( wosh::communication::Session&  ) const
 {
	;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

int tokenize_message_in_sentences( vector<string>& sentences, const string& input )
 {
	int count = input.size();
	string token = "";
	int found = 0;
	for (int i=0; i<count; i++) {
		if ( input.at(i) == '.' || input.at(i) == ';' || input.at(i) == ',' ) {
			if ( token.size() > 0 ) {
				Utilities::trimSpaces(token,2);
				sentences.push_back( token );
				++found;
			 }
			token = "";
		 }
		token.append( 1, input.at(i) );
	 }
	if ( token.size() > 0 ) {
		Utilities::trimSpaces(token,2);
		sentences.push_back( token );
		++found;
	 }
	return found;
 }

int build_response( string& output, const vector<string>& responses, int preferred_max_size )
 {
	std::vector<std::string>::const_iterator it;
	for ( it=responses.begin(); it!=responses.end(); it++ ) {
		output += "." + *it;
	 }
	return output.size();
 }
