/** @file    OrganizerBundle.cpp
 *  @author  Alessandro Polo
 *  @version $Id: OrganizerBundle.cpp 2142 2010-03-16 12:45:50Z alex $
 *  @brief
 * File containing methods for the wosh::services::OrganizerBundle class.
 * The header for this class can be found in OrganizerBundle.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "OrganizerBundle.h"
 #include "OrganizerImpl.h"

 #include <core/Notification.h>
 #include <core/MethodsDef.h>
 #include <core/MessageFilter.h>
 #include <core/MessageProcessingCommon.h>
 #include <core/Fact.h>
 #include <core/NEW_List.h>
 #include <core/Table.h>
 #include <core/SecurityManager.h>
 #include <core/DataBaseManager.h>
 #include <core/FileSystem.h>
 #include <core/SystemError.h>

 #include <framework/communication/TextMessage.h>



using namespace std;
namespace wosh {
 using namespace communication;
 namespace services {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

OrganizerBundle::OrganizerBundle() : BundleGeneric()
 {
	BundleGeneric::setName( _Organizer_NAME, false );
	Properties.update( _KEY_Version, _Organizer_VERSION );

//Log.setLevel( LOG_VERBOSE );

	Log(LOG_VERBOSE, " Configuring Organizer worker.." );
	this->orgWorker = new OrganizerImpl(*this);
	this->orgWorker->setOrganizerListener(this);
	
	Log(LOG_VERBOSE, " Setting default properties and permissions.." );
//	Properties.set( _BuildingManager_KEY_count,	0,	Permission(Permission::Read) );
	Properties.set( _Organizer_KEY_DB,		"	$DATABASE/organizer/organizer.xml", Permission(Permission::Read) );
	Properties.set( _Organizer_KEY_DB_shadow,	"$DATABASE/organizer/organizer.shadow", Permission(Permission::Read) );

	Log(LOG_VERBOSE, " Registering methods.." );
	MethodRequest* mmListToDos = Methods.create<MethodRequest>( _METHOD_List, "List ToDos(s)" );
	mmListToDos->setMethod( this, (MethodRequestPtr)&OrganizerBundle::mmDoListToDos );
	mmListToDos->getPermission().setMask( Permission::RX, Permission::RX, Permission::RX );

	MethodWRESULT* mmLoad = Methods.create<MethodWRESULT>( _Organizer_METHOD_load, "Load items from DB" );
	mmLoad->setMethod( this, (MethodWRESULTPtr)&OrganizerBundle::load );
	mmLoad->getPermission().setMask( Permission::RX, Permission::RX, Permission::Read );
	MethodWRESULT* mmSave = Methods.create<MethodWRESULT>( _Organizer_METHOD_save, "Save items to DB" );
	mmSave->setMethod( this, (MethodWRESULTPtr)&OrganizerBundle::save );
	mmSave->getPermission().setMask( Permission::RX, Permission::RX, Permission::Read );

	MethodMessageResponse* mmNewToDo = Methods.create<MethodMessageResponse>( _Organizer_METHOD_todo, "Create new ToDo" );
	mmNewToDo->setMethod( this, (MethodMessageResponsePtr)&OrganizerBundle::mmDoNewToDo );
	mmNewToDo->getPermission().setMask( Permission::RX, Permission::RX, Permission::Read );

	MethodMessageResponse* mmNotifyToDo = Methods.create<MethodMessageResponse>( _Organizer_METHOD_todo, "Notify ToDo" );
	mmNotifyToDo->setMethod( this, (MethodMessageResponsePtr)&OrganizerBundle::mmDoNotifyToDo );
	mmNotifyToDo->getPermission().setMask( Permission::RX, Permission::RX, Permission::Read );

	setBundleState(Bundle::STATE_CREATED, false);
 }

OrganizerBundle::~OrganizerBundle()
 {
	Log(LOG_INFO, " Destroying.." );

	if ( isBundleRunning() ) {
		Log(LOG_WARNING, "~OrganizerBundle() : Destroying while RUNNING! Trying to stop.." );
		bundleStop();
	 }

	delete this->orgWorker; this->orgWorker = NULL;
	Log(LOG_VERBOSE, ":~OrganizerBundle() : Destroyed." );
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE CONTROL

WRESULT OrganizerBundle::bundleStart()
 {
	if ( !BundleGeneric::bundleValidate_StartStop(Bundle::STATE_STARTING) )
		return WRET_ERR_WRONG_STATE;

	//WRESULT ret = 
	load();

	WRESULT ret = BundleGeneric::start_SynchThread( this->orgWorker );

	// BUNDLE-STATE (STARTED) will be updated async by WORKER, through call: thread_event()
	return ret;
 }

WRESULT OrganizerBundle::bundleStop()
 {
	if ( !BundleGeneric::bundleValidate_StartStop(Bundle::STATE_STOPPING) )
		return WRET_ERR_WRONG_STATE;

	WRESULT ret = BundleGeneric::stop_SynchThread( this->orgWorker );

	//WRESULT ret = 
	save();

	// BUNDLE-STATE (STOPPED) will be updated async by WORKER, through call: thread_event()
	return ret;
 }

////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE CONTROL
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////// PROCESS BUS MESSAGES

void OrganizerBundle::busMessage( const Message& message, const Bus* source )
 {
	if ( message.isEmpty() ) return;
	BundleGeneric::busMessage(message, source);

	// monitor user presence to report todos
	if ( !message.getContent()->isFact() ) {
	 }
	else if ( !message.getContent()->isNotification() ) {
	 }

	if ( !message.getDestination().isChildOf(this->getURI(), true) )
		return;

	string todo_name = message.getDestination().getName();

	// map (proxy) methods and properties [REQUESTs]
	Log(LOG_VERBOSE, ":busMessage() = Message-Request for Child '%s'", todo_name.c_str() );

	OrganizerToDo* todo = NULL;

	this->orgWorker->getToDos().mutex().lockForRead();

	std::vector<OrganizerToDo*>::iterator it;
	std::vector<OrganizerToDo*>::iterator it_end = this->orgWorker->getToDos().stdEnd();
	for ( it=this->orgWorker->getToDos().stdBegin(); it!=it_end; it++ ) {
		if ( *it == NULL ) continue;
		if ( (*it)->getName() != todo_name ) continue;
		todo = *it;
		break;
	 }
	if ( todo == NULL ) {
		this->orgWorker->getToDos().mutex().unLock();
		if ( !message.getDestination().isPathAny() ) // let's not flood
			Log(LOG_WARNING, ":busMessage() = Request for UNKNOWN Child '%s'", todo_name.c_str() );
		return;
	 }

	Message* reply = busMessageTodo( message, todo );//no mutex

	this->orgWorker->getToDos().mutex().unLock();

	if ( reply == NULL ) return;
	BusCore.postMessage(reply);
 }


//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Message* OrganizerBundle::busMessageTodo( const Message& message, OrganizerToDo* todo )//no mutex
 {
	if ( todo == NULL ) return NULL;
	if ( message.isEmpty() ) return NULL;
	if ( !message.getContent()->isRequest() ) return NULL;
	const Request* request = message.getContent()->asRequest();
	Response* response = NULL;

	// map (proxy) methods and properties [REQUESTs]
	Log(LOG_VERBOSE, ":busMessageCronJob() = Message-Request for Virtual-Child '%s'", todo->getName().c_str() );

	// check permission
	const Permission* jobPerm = &((const OrganizerToDo*)todo)->getPermission();

	bool allowRead = SecurityManager::evalAuthorization( message.getSecurityToken(), jobPerm, Permission::Read );
	if ( !allowRead ) {
		Log(LOG_WARNING, ":busMessage() : Not allowed to read todo '%s' ", todo->getName().c_str() );
		return NULL;
	 }
/*
	if ( request->getMethod() == _METHOD_GetProperty )
		response = MPC_mmDoPropertyGet( &message, *todo, *jobPerm );

	else if ( request->getMethod() == _METHOD_SetProperty )
		response = MPC_mmDoPropertySet( &message, *todo, *jobPerm, Log );

	else if ( request->getMethod() == _METHOD_ListProperties )
		response = MPC_mmDoPropertyList( &message, *todo, *jobPerm );
*/
	// ------------------------------------------------------------------------------------------------------- LIST METHODS
	if ( request->getMethod() == _METHOD_ListMethods ) {
		NEW_List* methods = new NEW_List();
		methods->add( _METHOD_GetProperty );
		methods->add( _METHOD_SetProperty );
		methods->add( _METHOD_ListProperties );
		methods->add( _METHOD_ListMethods );
	//	methods->add( "execute" );
		response = new Response( request->getMethod(), methods );
	 }
	// ------------------------------------------------------------------------------------------------------- method: 

	// -------------------------------------------------------------------------------------------------------
	else {
		Log(LOG_WARNING, ":busMessage() FAILED : Method '%s' not found", request->getMethod().c_str() );
	 }

	if ( response == NULL )
		return NULL;

	Message* reply = message.replyTo( response );

	return reply;
 }

//////////////////////////////////////////////////////////////////////////////////////// PROCESS BUS MESSAGES
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENT

void OrganizerBundle::organizer_raise( OrganizerToDo* todo, OrganizerImpl* )
 {
	if ( todo == NULL ) return;

	TextMessage* notif = new TextMessage();
	notif->setRecipent_User(todo->getUsername());
	notif->setPlain( todo->getMessage() );
	Message* message = new Message( notif );
	BusCore.postMessage(message);
 }

////////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENT
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT OrganizerBundle::load()
 {
	return loadToDos();
 }

WRESULT OrganizerBundle::save()
 {
	return saveToDos();
 }

WRESULT OrganizerBundle::loadToDos()
 {
	// may also be a folder of .xml files
	string xmlfile = Properties.getValueOf(_Organizer_KEY_DB).getData();
	Log(LOG_VERBOSE, ":loadToDos() File %s ..", xmlfile.c_str() );
	std::vector<ISerializable*> objects;
	WRESULT ret = DataBaseManager::loadObjects( objects, "wosh::OrganizerToDo", xmlfile, "xml" );

	std::vector<ISerializable*>::iterator it;
	for ( it=objects.begin(); it!=objects.end(); it++ ) {
		OrganizerToDo* currObj = dynamic_cast<OrganizerToDo*>(*it);
		if ( currObj == NULL ) continue;
		this->orgWorker->registerToDo( currObj );
	 }

	return ret;
 }

WRESULT OrganizerBundle::saveToDos()
 {
	string xmlfile = Properties.getValueOf(_Organizer_KEY_DB_shadow).getData();
	Log(LOG_VERBOSE, ":saveToDos() File %s ..", xmlfile.c_str() );
	std::vector<ISerializable*> objects;
	this->orgWorker->getToDos().mutex().lockForRead();
	std::vector<OrganizerToDo*>::const_iterator it;
	std::vector<OrganizerToDo*>::const_iterator it_end = this->orgWorker->getToDos().stdEnd();
	for ( it=this->orgWorker->getToDos().stdBegin(); it!=it_end; it++ ) {
		if ( *it == NULL ) continue;
		objects.push_back( *it );
	 }

	WRESULT ret = DataBaseManager::saveObjects( objects, xmlfile, "xml" );
	this->orgWorker->getToDos().mutex().unLock();

	if ( WFAILED(ret) ) {
		SystemError::raise(this, Log, SystemError::CRITICAL, 0, "saveToDos", "FAILED#%i saving Database %s..", ret, xmlfile.c_str() );
	 }
	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS

Response* OrganizerBundle::mmDoNotifyToDo( const Message* requestMessage )
 {
	if ( requestMessage == NULL || requestMessage->isEmpty() ) return NULL;
	if ( !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();

	std::string owner = "";
	std::string group = "";
	if ( requestMessage->getSecurityToken() != NULL ) {
		owner = requestMessage->getSecurityToken()->getUser();
		group = "";
	 }

	// retrieve options
	string todo_name = request->getArgument().getData();
	OrganizerToDo* todo = NULL;

	this->orgWorker->getToDos().mutex().lockForRead();

	std::vector<OrganizerToDo*>::const_iterator it;
	std::vector<OrganizerToDo*>::const_iterator it_end = this->orgWorker->getToDos().stdEnd();
	for ( it=this->orgWorker->getToDos().stdBegin(); it!=it_end; it++ ) {
		if ( *it == NULL ) continue;
		if ( (*it)->getName() != todo_name ) continue;
		todo = *it;
		break;
	 }

	if ( todo == NULL ) {
		this->orgWorker->getToDos().mutex().unLock();
		return request->replyResponse(WRET_ERR_PARAM, "Todo not found!");
	 }
	WRESULT ret = WRET_OK;
	organizer_raise( todo, NULL ); // fake event

	this->orgWorker->getToDos().mutex().unLock();

	return request->replyResponse(ret, "Notification sent!" );
 }

Response* OrganizerBundle::mmDoNewToDo( const Message* requestMessage )
 {
	if ( requestMessage == NULL || requestMessage->isEmpty() ) return NULL;
	if ( !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();

	std::string owner = "";
	std::string group = "";
	if ( requestMessage->getSecurityToken() != NULL ) {
		owner = requestMessage->getSecurityToken()->getUser();
		group = "";
	 }

	std::string subject = "";
	std::string message = "";
	if ( request->getData()->isClass(_NEW_List_TYPE) ) {
		const NEW_List* dl = dynamic_cast<const NEW_List*>(request->getData());
		if ( dl->size() < 2 ) {
			if ( request->doOmitResponse() )
				return NULL;
			return request->replyResponse(WRET_ERR_PARAM, "Invalid arguments [subject, message]" );
		 }
		
		subject = dl->get(0).getData();
		dl->join(message, " ", 1);
	 }
	
	///@ bug check registered jobs
	bool found = false;//this->cronWorker->exists(name);
	if ( found )
		return request->replyResponse(WRET_ERR_PARAM, "OrganizerToDo already exists!" );

	OrganizerToDo* todo = new OrganizerToDo();
	todo->setPermission( Permission(owner, group, Permission::RWX, Permission::RWX, Permission::Read) );
	todo->setName( subject );
	todo->setUsername( owner );
	todo->setSubject( subject );
	todo->setMessage( message );

	WRESULT ret = this->orgWorker->registerToDo( todo );
	if ( ret != WRET_OK )
		return request->replyResponse(ret, "Error adding ToDo!" );

	return request->replyResponse(ret, "Added!" );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* OrganizerBundle::mmDoListToDos( const Request* request )
 {
	if ( request == NULL ) return NULL;
	string option = request->getArgument().getData();
	Log(LOG_VERBOSE, ":mmDoListToDos(%s)", option.c_str() );

	if ( option == "s" || option == "short" ) {
		NEW_List* objs = new NEW_List();
		this->orgWorker->getToDos().mutex().lockForRead();
		std::vector<OrganizerToDo*>::const_iterator it;
		std::vector<OrganizerToDo*>::const_iterator it_end = this->orgWorker->getToDos().stdEnd();
		for ( it=this->orgWorker->getToDos().stdBegin(); it!=it_end; it++ ) {
			if ( *it == NULL ) continue;
			objs->add( new Data((*it)->getName()) );
		}
		this->orgWorker->getToDos().mutex().unLock();
		return request->replyResponse( objs );
	 }
	else {
		this->orgWorker->getToDos().mutex().lockForRead();
		Table* tbl = new Table( this->orgWorker->getToDos().size_(), 3, true );
		tbl->setTableName("ToDos");
		tbl->setHeaderHorizontal( new Data("Name"), 0);
		tbl->setHeaderHorizontal( new Data("Username"), 1);
		tbl->setHeaderHorizontal( new Data("Subject"), 2);
		unsigned int iRow = 0;
		std::vector<OrganizerToDo*>::const_iterator it;
		std::vector<OrganizerToDo*>::const_iterator it_end = this->orgWorker->getToDos().stdEnd();
		for ( it=this->orgWorker->getToDos().stdBegin(); it!=it_end; it++ ) {
			if ( *it == NULL ) continue;
			const OrganizerToDo* cToDo = *it;
			tbl->set( new Data(cToDo->getName()), iRow, 0);
			tbl->set( new Data(cToDo->getUsername()), iRow, 1);
			tbl->set( new Data(cToDo->getSubject()), iRow, 2);
			++iRow;
		}
		this->orgWorker->getToDos().mutex().unLock();
		return request->replyResponse( tbl );
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace services
}; // namespace wosh
