/** @file    SessionServerBundle.cpp
 *  @author  Alessandro Polo
 *  @version $Id: SessionServerBundle.cpp 2198 2010-03-23 12:03:07Z alex $
 *  @brief
 * File containing methods for the wosh::services::SessionServerBundle class.
 * The header for this class can be found in SessionServerBundle.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "SessionServerBundle.h"
 
 #include "SessionServerImpl.h"

 #include <core/ObjectAllocator.h>
 #include <core/MessageFilter.h>
 #include <core/MethodsDef.h>
 #include <core/MessageProcessingCommon.h>
 #include <core/Notification.h>
 #include <core/Fact.h>
 #include <core/Table.h>
 #include <core/WoshKernel.h>
 #include <core/BusManager.h>
 #include <core/UserManager.h>

 #include <core/Communicator.h>

 #include <interfaces/services/CommunicatorService.h>
 #include <framework/communication/SessionIM.h>
 #include <framework/communication/SessionShell.h>

 #include <framework/communication/TextMessage.h>


using namespace std;
namespace wosh {
 namespace services {

 WOSH_REGISTER(wosh::services::SessionServerBundle, "wosh::interfaces::services::SessionServer", _SessionServer_VERSION, _static_SessionServerBundle1 )
 WOSH_REGISTER(wosh::services::SessionServerBundle, "wosh::interfaces::services::CommunicatorService", _SessionServer_VERSION, _static_SessionServerBundle2 )

 WOSH_REGISTER(wosh::services::SessionServerBundle, "wosh::BundleGeneric", _SessionServer_VERSION, _static_SessionServerBundle3 )

SessionServerBundle* SessionServerBundle::ms_SessionServer_instance = NULL;
SessionServerBundle* SessionServerBundle::getInstance() { return SessionServerBundle::ms_SessionServer_instance; }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

SessionServerBundle::SessionServerBundle() : BundleGeneric()
 {
	BundleGeneric::setName( _SessionServer_NAME, false );
	Object::getURI().setKernelLocal();
	Object::getURI().setName( "Sessions" );
	Object::getURI().registerGlobal(); // so BundleGeneric won't overwrite my uri
	Properties.update( _KEY_Version, _SessionServer_VERSION );

//Log.setLevel( VERBOSE );

	this->autoAllocSessionIM = true;

	Log(LOG_VERBOSE, " Configuring SessionServer worker.." );
	this->ssrvWorker = new SessionServerImpl(BusCore, Log, this);

	Log(LOG_VERBOSE, " Setting default properties and permissions.." );
	Properties.set( _CommunicatorService_KEY_ChannelType,	(int)Notification::TYPE_TEXTUAL, Permission(Permission::Read) );
	Properties.set( _CommunicatorService_KEY_ChannelLink,	(int)Communicator::LINK_ASYNCH, Permission(Permission::Read) );
	Properties.set( _CommunicatorService_KEY_Protocol,		(int)Notification::PROTOCOL_SHELL,	Permission( Permission::RW, Permission::RW, Permission::Read) );
	Properties.set( _CommunicatorService_KEY_Cost,			0.0,Permission( Permission::RW, Permission::Read, Permission::Read) );
	Properties.set( _CommunicatorService_KEY_Priority,		15,	Permission( Permission::RW, Permission::RW, Permission::Read) );
	Properties.set( _CommunicatorService_KEY_Reliability,	10,	Permission( Permission::RW, Permission::RW, Permission::Read) );

	Properties.set( _KEY_Version, _SessionServer_VERSION, Permission(Permission::Read) );
	Properties.set( _SessionServer_KEY_WelcomeString, Utilities::format("Welcome WOSH SessionServer v.%g", _SessionServer_VERSION), Permission(Permission::RW, Permission::RW, Permission::Read) );
	Properties.set( _SessionServer_KEY_AutoAllocSessionIM, true, Permission(Permission::RW, Permission::Read, Permission::Read) );
	Properties.monitor(_SessionServer_KEY_AutoAllocSessionIM);


	Log(LOG_VERBOSE, " Registering methods.." );
	MethodMessageResponse* mmGetSessions = Methods.create<MethodMessageResponse>( _METHOD_List, "Get Sessions" );
	mmGetSessions->setMethod( this, (MethodMessageResponsePtr)&SessionServerBundle::mmDoGetSessions );
	mmGetSessions->getPermission().setMask( Permission::RX, Permission::RX, Permission::RX );


	setBundleState(Bundle::STATE_CREATED, false);

	SessionServerBundle::ms_SessionServer_instance = this;
 }

SessionServerBundle::~SessionServerBundle()
 {
	Log(LOG_VERBOSE, " Destroying.." );

	if ( isBundleRunning() ) {
		Log(LOG_WARNING, "~SessionServerBundle() : Destroying while RUNNING! Trying to stop.." );
		bundleStop();
	 }

	delete this->ssrvWorker; this->ssrvWorker = NULL;

	Log(LOG_VERBOSE, ":~SessionServerBundle() : Destroyed." );
	SessionServerBundle::ms_SessionServer_instance = NULL;
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE CONTROL

WRESULT SessionServerBundle::bundleStart()
 {
	if ( !BundleGeneric::bundleValidate_StartStop(Bundle::STATE_STARTING) )
		return WRET_ERR_WRONG_STATE;

	setBundleState( Bundle::STATE_STARTED );

	Response* response = new Response( _METHOD_ListProperties, Properties.toTable(false) );
	Message* message = new Message(response);
	message->setSource(this);
	message->getDestination().fromString("*:Communication");
	message->setSecurityToken( SecurityManager::generateToken(this->getPermission().getOwner()) );
	BusCore.postMessage(message);

	return WRET_OK;
 }

WRESULT SessionServerBundle::bundleStop()
 {
	if ( !BundleGeneric::bundleValidate_StartStop(Bundle::STATE_STOPPING) )
		return WRET_ERR_WRONG_STATE;

	setBundleState( Bundle::STATE_STOPPED );
	return WRET_OK;
 }

////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE CONTROL
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////// PROCESS BUS MESSAGES

void SessionServerBundle::busMessage( const Message& message, const Bus* source )
 {
	if ( message.isEmpty() ) return;

	if ( message.getContent()->isNotification() ) {
		if ( message.getDestination().isBroadcast() )
			return;
		if ( !MessageFilter::isMessageTarget(message, this) )
			return;
		const Notification* notif = message.getContent()->asNotification();
		WRESULT sent = notify( notif, message );
		Message* reply = message.replyTo( new Response( _CommunicatorService_RESPONSE_notif_sent, new Variant(notif->getID()), sent ) );
		reply->setSecurityToken( SecurityManager::generateToken(this->getPermission().getOwner()) );
		BusCore.postMessage(reply);
		if ( WSUCCEEDED(sent) ) {
			Message* reply2 = message.replyTo( new Response( _CommunicatorService_RESPONSE_notif_read, new Variant(notif->getID()), WRET_OK ) );
			reply2->setSecurityToken( SecurityManager::generateToken(this->getPermission().getOwner()) );
			BusCore.postMessage(reply2);
		 }
		return;
	 }
	BundleGeneric::busMessage(message, source);

	long id = 0;
	if ( message.getDestination().isChildOf(this->getURI(), true) ) {
		string id_str = message.getDestination().getName();
		if ( id_str.find("Session#") == 0 && id_str.size() > 8 )
			id_str = id_str.substr(8);
		id = Utilities::fromString<long>(id_str);
	 }
	else // target isn't a session..
		return;

	Log(LOG_VERBOSE, ":busMessage() : Queueing message for Session %i..", id );
	this->ssrvWorker->forwardMessage_( message, id );
 }

WRESULT SessionServerBundle::notify( const Notification* notif, const Message& message )
 {
	if ( notif == NULL ) return WRET_ERR_PARAM;

	if ( notif->getProtocol() != Notification::PROTOCOL_SHELL ) {
		Log(LOG_VERBOSE, ":notify() : INVALID Protocol [%i] (Resuming..)", (int)notif->getProtocol() );
	 }

	if ( notif->isForAnyone() || notif->getRecipent_User().size() == 0 ) {
		//any session
		Log(LOG_VERBOSE, ":notify() : Queueing message for Any Session.." );
		this->ssrvWorker->forwardMessage_( message, 0 );
		return WRET_OK;
	 }
	if ( notif->getRecipent_User() != "wosh" ) {
		// a notification for a custom user.. probably..
		// shoulnd't be parsed by me, since i'm a communicator FOR the WOSH system itself,
		// the only output to the user is the LOCAL STD SHELL (if was loaded)

		this->ssrvWorker->getSessions().transactionBeginRead();
		Session* session_found = this->ssrvWorker->getSessionByUser_(notif->getRecipent_User());
		if ( session_found == NULL ) { // user isn't logged
			this->ssrvWorker->getSessions().transactionEnd();
			return WRET_ERR_PARAM;
		 }
		Log(LOG_WARNING, ":notify() : Acting as a Communicator!, prompting to Session#%i", session_found->getID() );

		session_found->appendMessage( message.clone() );
		this->ssrvWorker->getSessions().transactionEnd();
		return WRET_OK;
	 }

	// target user is WOSH.. we use an old/new SESSION and its PROTOCOL to interpret
	// the message and produce a Response.
	// session type is SessionIM, which will reply generating a Notification message for logged user


	this->ssrvWorker->getSessions().transactionBeginWrite();

	SessionIM* sessionIM = NULL;
	Session* session_found = this->ssrvWorker->getSessionByUser_(notif->getSender_User());
	if ( session_found == NULL ) { // user isn't logged

		if ( !this->autoAllocSessionIM ) {
			this->ssrvWorker->getSessions().transactionEnd();
			Log(LOG_CRITICAL, ":busMessage() SKIPPED [Not Allocating New Sessions!] (%s)", notif->getSender_User().c_str() );
			return WRET_ERR_INTERNAL;
		 }

		sessionIM = new SessionIM();
		sessionIM->setCommunicator( message.getSource() );
		sessionIM->setSessionServer(this);
		sessionIM->setUsername(notif->getSender_User());
		sessionIM->getURI().setParent( this->getURI() );
		sessionIM->getURI().setName( sessionIM->getName() );
		sessionIM->getURI().registerGlobal();
		this->ssrvWorker->registerSession_( sessionIM );

		Message* msg_logged = new Message( new Fact( _User_EVENT_UserLogged, new Variant(notif->getSender_User())) );
		msg_logged->setSource(sessionIM);
		msg_logged->setDestinationBroadcast(true);
		WoshKernel::getInstance()->busses().postMessage( msg_logged, _Bus_Security );

		sessionIM->setUsername( notif->getSender_User() );
		sessionIM->setWelcomeMessage( "hey Alex!" );
		sessionIM->sessionStart();
		Log(LOG_INFO, ":busMessage() New SessionIM for %s #%i", notif->getSender_User().c_str(), sessionIM->getID() );
	 }
	else {
		sessionIM = dynamic_cast<SessionIM*>( session_found );
		if ( sessionIM == NULL ) {
			this->ssrvWorker->getSessions().transactionEnd();
			Log(LOG_CRITICAL, ":busMessage() Internal get-Session ERROR (#%i not SessionIM!), User: %s", session_found->getID(), notif->getSender_User().c_str() );
			return WRET_ERR_INTERNAL;
		 }
		Log(LOG_VERBOSE, ":busMessage() Using SessionIM for %s #%i", notif->getSender_User().c_str(), session_found->getID() );
	 }
	if ( sessionIM == NULL ) {
		this->ssrvWorker->getSessions().transactionEnd();
		Log(LOG_CRITICAL, ":busMessage() Internal get-Session ERROR, User: %s", notif->getSender_User().c_str() );
		return WRET_ERR_INTERNAL;
	 }

	sessionIM->evalNotificationInput(notif, message);
	this->ssrvWorker->getSessions().transactionEnd();
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////// PROCESS BUS MESSAGES
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENT

bool SessionServerBundle::updatingProperty( bool& do_update, const Variant& value_proposed, Property& property_current, const PropertiesProvider* source )
 {
	if ( property_current.getKey() == _SessionServer_KEY_AutoAllocSessionIM ) {
		this->autoAllocSessionIM = value_proposed.toBoolean(this->autoAllocSessionIM);
		return true;
	 }
	else // maybe is a (common) BundleGeneric property, let base implementation process it
		return BundleGeneric::updatingProperty(do_update, value_proposed, property_current, source);
 }

////////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENT
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT SessionServerBundle::registerSession( Session* session, bool start_session )
 {
	if ( session == NULL ) return WRET_ERR_PARAM;
	if ( SessionServerBundle::getInstance() == NULL ) return WRET_ERR_INTERNAL;
	WRESULT ret = WRET_OK;

	session->setSessionServer( SessionServerBundle::getInstance() );

	session->getURI().setParent( SessionServerBundle::getInstance()->getURI() );
	session->getURI().setName( session->getName() );
	session->getURI().registerGlobal();

	SessionServerBundle::getInstance()->ssrvWorker->getSessions().transactionBeginWrite();
	ret += SessionServerBundle::getInstance()->ssrvWorker->registerSession_( session );

	if ( start_session )
		ret += session->sessionStart();

	SessionServerBundle::getInstance()->ssrvWorker->getSessions().transactionEnd();
	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT SessionServerBundle::unRegisterSession( Session* session )
 {
	if ( session == NULL ) return WRET_ERR_PARAM;
	if ( SessionServerBundle::getInstance() == NULL ) return WRET_ERR_INTERNAL;
	WRESULT ret = WRET_OK;

	session->getURI().unRegisterGlobal();

	SessionServerBundle::getInstance()->ssrvWorker->getSessions().transactionBeginWrite();
	ret += session->sessionStop();
	ret += SessionServerBundle::getInstance()->ssrvWorker->unRegisterSession_( session );
	SessionServerBundle::getInstance()->ssrvWorker->getSessions().transactionEnd();

	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

long SessionServerBundle::createLocalShellSession( const string& username, bool skipLogin, bool autoStart )
 {
	(void)skipLogin;

	SessionShell* session = new SessionShell();
	session->setSessionServer(this);

	session->setStreamStdIO();
	session->setSkipWelcome(false);
	session->setUsername( username );
	session->setWelcomeMessage( "Welcome to Local Shell Session!" );

	long id = session->getID();
	this->registerSession(session, autoStart);
	return id;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool SessionServerBundle::session_is_auth( Session* session, const string& user, const string& pwd ) {
	return this->ssrvWorker->session_is_auth(session, user, pwd);
 }

bool SessionServerBundle::session_post_message( Session* session, Message* message ) {
	return this->ssrvWorker->session_post_message(session, message);
 }

WRESULT SessionServerBundle::changeSessionProtocol( Session* session, const string& protocol_name ) {
	return this->ssrvWorker->changeSessionProtocol(session, protocol_name);
 }

bool SessionServerBundle::isUserLogged( const std::string& username ) const {
	return this->ssrvWorker->isUserLogged(username);
 }

long SessionServerBundle::getSessionIDByUser( const std::string& username ) const {
	return this->ssrvWorker->getSessionIDByUser(username);
 }

WRESULT SessionServerBundle::registerProtocol( SessionProtocol* session_proto ) {
	return this->ssrvWorker->registerProtocol(session_proto);
 }

WRESULT SessionServerBundle::unRegisterProtocol( SessionProtocol* session_proto ) {
	return this->ssrvWorker->unRegisterProtocol(session_proto);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT SessionServerBundle::sendMessageToUser( const std::string& userWosh, const std::string& message ) {
	// BUG
	return WRET_OK;
 }

WRESULT SessionServerBundle::sendMessageTo( const std::string& userID, const std::string& message ) {
	// BUG
	return WRET_OK;
 }

WRESULT SessionServerBundle::notify( const Notification* notif ) {
	if ( notif == NULL ) return WRET_ERR_PARAM;
	if ( notif->getProtocol() != Notification::PROTOCOL_SHELL ) {
		Log(LOG_VERBOSE, ":notify() : INVALID Protocol [%i] (Resuming..)", (int)notif->getProtocol() );
	 }
	// BUG
	return WRET_OK;
 }
 
bool SessionServerBundle::isUserReachable( const string& username ) {
	this->ssrvWorker->getSessions().transactionBeginRead();
	Session* session_found = this->ssrvWorker->getSessionByUser_(username);
	if ( session_found == NULL ) { // user isn't logged
		this->ssrvWorker->getSessions().transactionEnd();
		return false;
	 }
	this->ssrvWorker->getSessions().transactionEnd();
	return true;
 }
long SessionServerBundle::getUserLastSeen( const string& username ) {
	long ts = 0;
	this->ssrvWorker->getSessions().transactionBeginRead();
	Session* session_found = this->ssrvWorker->getSessionByUser_(username);
	if ( session_found == NULL ) { // user isn't logged
		this->ssrvWorker->getSessions().transactionEnd();
		return ts;
	 }
	ts = session_found->getLastInputTS();
	this->ssrvWorker->getSessions().transactionEnd();
	return ts; 
 }

bool SessionServerBundle::isUserRegistered( const string& username ) {
	return UserManager::isUser(username);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* SessionServerBundle::mmDoGetSessions( const Message* requestMessage )
 {
	if ( requestMessage == NULL || requestMessage->isEmpty() ) return NULL;
	if ( !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();
	if ( request == NULL ) return NULL;

	// retrieve options
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, ":mmDoGetSessions(%s)", option.c_str() );

	this->ssrvWorker->getSessions().transactionBeginRead();
	wosh::Table* tbl = new wosh::Table( this->ssrvWorker->getSessions().size(), 5, true );
	tbl->setTableName("Sessions");
	tbl->setHeaderHorizontal("Name", 0);
	tbl->setHeaderHorizontal("Type", 1);
	tbl->setHeaderHorizontal("Username", 2);
	tbl->setHeaderHorizontal("State", 3);
	tbl->setHeaderHorizontal("TimeStamp", 4);
	unsigned int iRow = 0;
	tSessionList::ConstIterator it;
	const tSessionList::ConstIterator it_end = this->ssrvWorker->getSessions().end();
	for ( it=this->ssrvWorker->getSessions().begin(); it!=it_end; ++it, ++iRow ) {
		Session* currS = *it;
		if ( currS == NULL ) continue;
		tbl->set<Variant>(currS->getName(), iRow, 0);
		tbl->set<Variant>(currS->getClassName(), iRow, 1);
		tbl->set<Variant>(currS->getUsername(), iRow, 2);
		tbl->set<Variant>(currS->getStateAsString(), iRow, 3);
		tbl->set( new Variant(currS->getLastInputTS(), Variant::DATETIME), iRow, 4);
	 }
	this->ssrvWorker->getSessions().transactionEnd();

	return request->replyResponse(WRET_OK, tbl);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace services
}; // namespace wosh
