/** @file    SessionServerImpl.cpp
 *  @author  Alessandro Polo
 *  @version $Id: SessionServerImpl.cpp 2214 2010-03-26 16:03:31Z alex $
 *  @brief
 * File containing methods for the SessionServerImpl class.
 * The header for this class can be found in SessionServerImpl.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "SessionServerImpl.h"

 #include <framework/communication/SessionProtocolShell.h>
 #include <core/Message.h>
 #include <core/Request.h>
 #include <core/UserManager.h>
 #include <core/SecurityManager.h>
 #include <core/Functional.h>


using namespace std;
using namespace wosh;
//using namespace wosh::services;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

SessionServerImpl::SessionServerImpl( BusConnector& connector, LogContext& logger, Object* owner )
 : BusCore(connector), Log(logger), ownerObj(owner)
 {

	this->protocolDefault = new SessionProtocolShell();
	this->protocolDefault->setSessionServer(this);
	this->protocols.push_back( this->protocolDefault );

 }


SessionServerImpl::~SessionServerImpl() {
	this->sessions.transactionBeginWrite();
	tSessionList::ConstIterator it;
	const tSessionList::ConstIterator it_end = this->sessions.end();
	for ( it=this->sessions.begin(); it!=it_end; ++it ) {
		if ( *it == NULL ) continue;
		(*it)->sessionStop(); //synch
	 }
	this->sessions.transactionEnd();

	if ( this->sessions.count() != 0 )
		Log(LOG_VERBOSE, ":~SessionServerImpl() : Freeing [%d] Sessions..", this->sessions.count() );
	this->sessions.clear();

	Log(LOG_VERBOSE, ":~SessionServerImpl() : Freeing protocolDefault.." );
	delete this->protocolDefault; this->protocolDefault = NULL;

 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT SessionServerImpl::registerSession_( Session* session )
 {
	if ( session == NULL ) return WRET_ERR_PARAM;

	Log(LOG_INFO, ":*registerSession_(%s)", session->getName().c_str() );


	session->setSessionProtocol( this->protocolDefault );

	session->setParentObject( this->getOwner() );
	session->getURI().setParent( this->getOwner()->getURI() );
	session->getURI().registerGlobal();

	this->sessions.push_back(session);

	return WRET_OK;
 }

WRESULT SessionServerImpl::unRegisterSession_( Session* session )
 {
	if ( session == NULL ) return WRET_ERR_PARAM;

	Log(LOG_INFO, ":*unRegisterSession(%s)", session->getName().c_str() );

	session->setSessionProtocol(NULL);
	this->sessions.pop(session);

	return WRET_OK;
 }

Session* SessionServerImpl::getSessionByID_( long id )
 {
	tSessionList::ConstIterator it = std::find_if(this->sessions.begin(), this->sessions.end(),
													Functor_Validator<Session, long>(id, &Session::getID ) );
	if ( it == this->sessions.end() )
		return NULL;
	return *it;
 }

Session* SessionServerImpl::getSessionByUser_( const std::string& username )
 {
	tSessionList::ConstIterator it = std::find_if(this->sessions.begin(), this->sessions.end(),
													Functor_Validator<Session, const std::string&>(username, &Session::getUsername ) );
	if ( it == this->sessions.end() )
		return NULL;
	return *it;
 }

long SessionServerImpl::getSessionIDByUser( const std::string& username ) const
 {
	MutexLockerRead mL(this->sessions.getMutex());
	tSessionList::ConstIterator it = std::find_if(this->sessions.begin(), this->sessions.end(),
													Functor_Validator<Session, const std::string&>(username, &Session::getUsername ) );
	if ( it == this->sessions.end() )
		return 0;
	return (*it)->getID();
 }

WRESULT SessionServerImpl::changeSessionProtocol( Session* session, const string& protocol_name )
 {
	if ( session == NULL ) return WRET_ERR_PARAM;

	MutexLockerRead mL(this->protocols.getMutex());

	tSessionProtocolList::Iterator it = std::find_if(this->protocols.begin(), this->protocols.end(),
													Functor_Validator<SessionProtocol, std::string>(protocol_name, &SessionProtocol::getProtocolName ) );
	if ( it == this->protocols.end() || *it == NULL ) {
		Log(LOG_WARNING, ":*changeSessionProtocol(): Protocol Not found '%s'", protocol_name.c_str() );
		return WRET_ERR_PARAM;
	 }
	Log(LOG_INFO, ":*changeSessionProtocol(): #%i Switching to Protocol '%s'", session->getID(), protocol_name.c_str() );

	session->setSessionProtocol( *it );
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT SessionServerImpl::registerProtocol( SessionProtocol* session_proto )
 {
	if ( session_proto == NULL ) return WRET_ERR_PARAM;

	std::vector<SessionProtocol*>::iterator it;
	for ( it=this->protocols.begin(); it!=this->protocols.end(); it++ ) {
		if ( *it == NULL ) continue;
		if ( *it == session_proto )
			return WRET_ERR_PARAM;
	 }
	Log(LOG_INFO, ":*registerProtocol(%s)", session_proto->getProtocolName().c_str() );
	this->protocols.push_back(session_proto);
	session_proto->setSessionServer(this);
	return WRET_OK;
 }

WRESULT SessionServerImpl::unRegisterProtocol( SessionProtocol* session_proto )
 {
	if ( session_proto == NULL ) return WRET_ERR_PARAM;

	Log(LOG_INFO, ":*unRegisterProtocol(%s)", session_proto->getProtocolName().c_str() );
	bool found = this->protocols.pop(session_proto);
	if ( !found )
		return WRET_ERR_PARAM;
	return WRET_OK;
 }


//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool SessionServerImpl::session_is_auth( Session* session, const string& user, const string& pwd )
 {
	if ( session == NULL ) return false;

	bool auth = UserManager::isAuthorized(user, pwd);
	return auth;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SessionServerImpl::forwardMessage_( const Message& message, long id )
 {
	tSessionList::ConstIterator it;
	const tSessionList::ConstIterator it_end = this->sessions.end();
	for ( it=this->sessions.begin(); it!=it_end; ++it ) {
		if ( *it == NULL ) continue;
		if ( id != 0 && id != (*it)->getID() ) continue;
		(*it)->appendMessage( message.clone() );
	 }
 }

bool SessionServerImpl::session_post_message( Session* session, Message* message )
 {
	if ( session == NULL )
		return false;

	SecurityToken* msec = SecurityManager::generateToken( session->getUsername() );
	message->setSecurityToken(msec);

	BusCore.postMessage( message );
	return true;
 }


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
