/** @file    TaskManagerBundle.cpp
 *  @author  Alessandro Polo
 *  @version $Id: TaskManagerBundle.cpp 2051 2010-03-07 08:27:48Z alex $
 *  @brief
 * File containing methods for the wosh::services::TaskManagerBundle class.
 * The header for this class can be found in TaskManagerBundle.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "TaskManagerBundle.h"

 #include "TaskManagerImpl.h"

 #include <core/ObjectAllocator.h>
 #include <core/MethodsDef.h>
 #include <core/MessageProcessingCommon.h>
 #include <core/MessageFilter.h>
 #include <core/Fact.h>
 #include <core/NEW_List.h>
 #include <core/Table.h>
 #include <core/DataBaseManager.h>
 #include <core/SecurityManager.h>
 #include <core/FileSystem.h>
 #include <core/SystemError.h>

 #include <stdlib.h>
 #include <time.h>
 #include <algorithm>

using namespace std;
namespace wosh {
 namespace services {

 WOSH_REGISTER(wosh::services::TaskManagerBundle, "wosh::interfaces::services::TaskManager", _TaskManager_VERSION, _static_TaskManagerBundle1 )
 WOSH_REGISTER(wosh::services::TaskManagerBundle, "wosh::BundleGeneric", _TaskManager_VERSION, _static_TaskManagerBundle2 )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

TaskManagerBundle::TaskManagerBundle() : BundleGeneric()
 {
	BundleGeneric::setName( _TaskManager_NAME, false );
	Object::getURI().setKernelLocal();
	Object::getURI().clearPathName();
	Object::getURI().setName( "Tasks" );
	Object::getURI().registerGlobal(); // so BundleGeneric won't overwrite my uri
	Properties.update( _KEY_Version, _TaskManager_VERSION );

//Log.setLevel(LOG_VERBOSE);

	Log(LOG_VERBOSE, " Configuring TaskManager workers.." );
	this->taskWorker = new TaskManagerImpl(*this);

	Log(LOG_VERBOSE, " Setting default properties and permissions.." );
	Properties.set( _TaskManager_KEY_DB,	"$DATABASE/automations/", Permission::RW_RW_R__() );

//	Properties.set( "SaveOnExit", true, Permission( Permission::RX, Permission::RX, Permission::Read ) );
//save archived
//save on exit

	Log(LOG_VERBOSE, " Registering methods.." );
	MethodRequest* mmListTasks = Methods.create<MethodRequest>( _METHOD_List, "List Tasks", Permission(Permission::RX) );
	mmListTasks->setMethod( this, (MethodRequestPtr)&TaskManagerBundle::mmDoListTasks );

	// BUG, SECURITY, use MessageResponse
	MethodRequest* mmExecTask = Methods.create<MethodRequest>( _TaskManager_METHOD_exec, "exec Tasks", Permission::R_XR_XR__() );
	mmExecTask->setMethod( this, (MethodRequestPtr)&TaskManagerBundle::mmDoExecTask );


	setBundleState(Bundle::STATE_CREATED, false);
 }

TaskManagerBundle::~TaskManagerBundle() {
	Log(LOG_INFO, " Destroying.." );

	if ( isBundleRunning() ) {
		Log(LOG_WARNING, "~TaskManagerBundle() : Destroying while RUNNING! Trying to stop.." );
		bundleStop();
	 }

	delete this->taskWorker; this->taskWorker = NULL;
	Log(LOG_VERBOSE, ":~TaskManagerBundle() : Destroyed." );
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE CONTROL

WRESULT TaskManagerBundle::bundleStart()
 {
	if ( !BundleGeneric::bundleValidate_StartStop(Bundle::STATE_STARTING) )
		return WRET_ERR_WRONG_STATE;

	WRESULT ret = WRET_OK;
	ret += BundleGeneric::start_SynchThread( this->taskWorker );
	// BUNDLE-STATE (STARTED) will be updated async by WORKER, through call: thread_event()

	loadAutomations();

	return ret;
 }

WRESULT TaskManagerBundle::bundleStop()
 {
	if ( !BundleGeneric::bundleValidate_StartStop(Bundle::STATE_STOPPING) )
		return WRET_ERR_WRONG_STATE;

	saveAutomations();

	WRESULT ret = WRET_OK;
	ret += BundleGeneric::stop_SynchThread( this->taskWorker );
	// BUNDLE-STATE (STOPPED) will be updated async by WORKER, through call: thread_event()
	return ret;
 }

////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE CONTROL
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////// PROCESS BUS MESSAGES

void TaskManagerBundle::busMessage( const Message& message, const Bus* source )
 {
	if ( message.isEmpty() ) return;

	if ( isBundleRunning() )
		this->taskWorker->busMessage( message, source);

	if ( !MessageFilter::isMessageTarget(message, this) )
		return;

	if ( message.getContent()->isRequest() ) {
		Message* reply = MPC_busMessageRequest( message, this->Methods, this->Log );
		if ( reply == NULL )
			return;
		reply->setSource( this );
		BusCore.postMessage(reply);
		return;
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////// PROCESS BUS MESSAGES
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENT

bool TaskManagerBundle::updatingProperty( bool& do_update, const Variant& value_proposed, Property& property_current, const PropertiesProvider* source )
 {

//	else // maybe is a (common) BundleGeneric property, let base implementation process it
		return BundleGeneric::updatingProperty(do_update, value_proposed, property_current, source);
 }

////////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENT
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT TaskManagerBundle::addAutomation( Automation* automation )
 {
	if ( automation == NULL ) return WRET_ERR_PARAM;

	automation->getURI().setParent( this->getURI() );
	automation->getURI().setName( automation->getName() );
	automation->getURI().registerGlobal();

	WRESULT ret = WRET_OK;
	this->taskWorker->getAutomations().append( automation );
	if ( WSUCCEEDED(ret) ) {
	//	automation->setAutomationManager(this);
		Log(LOG_INFO, ":addAutomation(%s) Added", automation->getName().c_str() );
	 }
	else
		Log(LOG_CRITICAL, ":addAutomation(%s) = %i", automation->getName().c_str(), ret );

	return ret;
 }

WRESULT TaskManagerBundle::loadAutomations()
 {
	FilePath xmlFile;
	if ( !xmlFile.set( Properties.getValueOf(_TaskManager_KEY_DB).toString() ) ) {
		SystemError::raise(this, Log, SystemError::CRITICAL, 0, "loadObjects", "FAILED#1 Invalid Archive File [%s]", xmlFile.getPath().c_str() );
		return WRET_ERR_PARAM;
	 }
	if ( !xmlFile.exists() ) {
		SystemError::raise(this, Log, SystemError::CRITICAL, 0, "loadObjects", "FAILED#1 Archive File doesn't exists %s [%s]!", xmlFile.getPath().c_str(), xmlFile.getPathLocal().c_str() );
		return WRET_ERR_PARAM;
	 }
	// may also be a folder of .xml files
	Log(LOG_VERBOSE, ":loadObjects() File %s ..", xmlFile.getPathLocal().c_str() );

	std::vector<ISerializable*> objects;
	WRESULT ret = DataBaseManager::loadObjects( objects, "", xmlFile.getPathLocal(), "xml" );

	if ( WFAILED(ret) ) {
		SystemError::raise(this, Log, SystemError::CRITICAL, 0, "loadAutomations", "FAILED#%i loading Database %s..", ret, xmlFile.getPathLocal().c_str() );
		return ret;
	 }
	if ( objects.size() == 0 ) {
		Log(LOG_WARNING, ":loadAutomations() : No Items found in database.." );
		return WRET_OK;
	 }

	// register objects
	std::vector<ISerializable*>::iterator it;
	for ( it=objects.begin(); it!=objects.end(); it++ ) {
		Automation* currObj = dynamic_cast<Automation*>(*it);
		if ( currObj == NULL ) continue;
		addAutomation( currObj );
	 }

	return ret;
 }

WRESULT TaskManagerBundle::saveAutomations()
 {
	FilePath xmlFile;
	if ( !xmlFile.set( Properties.getValueOf(_TaskManager_KEY_DB).toString() ) ) {
		SystemError::raise(this, Log, SystemError::CRITICAL, 0, "saveAutomations", "FAILED#1 Invalid Archive File [%s]", xmlFile.getPath().c_str() );
		return WRET_ERR_PARAM;
	 }
	Log(LOG_VERBOSE, ":saveAutomations() File %s ..", xmlFile.getPathLocal().c_str() );
	std::vector<ISerializable*> objects;

	this->taskWorker->getAutomations().transactionBeginRead();
	tAutomationList::ConstIterator it;
	const tAutomationList::ConstIterator it_end = this->taskWorker->getAutomations().end();
	for ( it=this->taskWorker->getAutomations().begin(); it!=it_end; ++it ) {
		if ( *it == NULL ) continue;
		objects.push_back( *it );
	 }

	WRESULT ret = DataBaseManager::saveObjects( objects, "../var/database/automations/automations2.bak", "xml" );
	this->taskWorker->getAutomations().transactionEnd();

	if ( WFAILED(ret) ) {
		SystemError::raise(this, Log, SystemError::CRITICAL, 0, "saveAutomations", "FAILED#%i saving Database %s..", ret, xmlFile.getPathLocal().c_str() );
	 }
	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS

Response* TaskManagerBundle::mmDoExecTask( const Request* request )
 {
	if ( request == NULL ) return NULL;
	string taskid = request->getArgument().toString();
	Log(LOG_VERBOSE, ":mmDoListTasks(%s)", taskid.c_str() );

	WRESULT ret = WRET_ERR_PARAM;
	this->taskWorker->getAutomations().transactionBeginRead();
	tAutomationList::ConstIterator it;
	const tAutomationList::ConstIterator it_end = this->taskWorker->getAutomations().end();
	for ( it=this->taskWorker->getAutomations().begin(); it!=it_end; ++it ) {
		if ( *it == NULL ) continue;
		if ( (*it)->getName() == taskid ) {
			ScheduledTask* st = new ScheduledTask(NULL, Utilities::std_time());
			//st->priority
			st->addTask(*it);
			this->taskWorker->scheduleTask(st);
			ret = WRET_OK;
			break;
		 }
	}

	this->taskWorker->getAutomations().transactionEnd();
	return request->replyResponse(ret);

 }

Response* TaskManagerBundle::mmDoListTasks( const Request* request )
 {
	if ( request == NULL ) return NULL;
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, ":mmDoListTasks(%s)", option.c_str() );

	if ( option == "s" || option == "short" ) {
		this->taskWorker->getAutomations().transactionBeginRead();
		NEW_List* objs =  new NEW_List(this->taskWorker->getAutomations().size());
		tAutomationList::ConstIterator it;
		const tAutomationList::ConstIterator it_end = this->taskWorker->getAutomations().end();
		for ( it=this->taskWorker->getAutomations().begin(); it!=it_end; ++it ) {
			if ( *it == NULL ) continue;
			objs->push_back<Variant>( (*it)->getID() );
		 }
		this->taskWorker->getAutomations().transactionEnd();
		return request->replyResponse(WRET_OK, objs);
	 }
	else {
		this->taskWorker->getAutomations().transactionBeginRead();
		Table* tbl = new Table( this->taskWorker->getAutomations().size(), 6, true );
		tbl->setTableName("Automations");
		tbl->setHeaderHorizontal("ID", 0);
		tbl->setHeaderHorizontal("Name", 1);
		tbl->setHeaderHorizontal("Enabled", 2);
		tbl->setHeaderHorizontal("Type", 3);
		tbl->setHeaderHorizontal("Perms", 4);
		tbl->setHeaderHorizontal("Last TS", 5);
		unsigned int iRow = 0;
		tAutomationList::ConstIterator it;
		const tAutomationList::ConstIterator it_end = this->taskWorker->getAutomations().end();
		for ( it=this->taskWorker->getAutomations().begin(); it!=it_end; ++it, ++iRow ) {
			if ( *it == NULL ) continue;
			const Automation* cAuto = *it;
			tbl->set<Variant>( cAuto->getID(), iRow, 0);
			tbl->set<Variant>( cAuto->getName(), iRow, 1);
			tbl->set( new Variant(cAuto->isEnabled()), iRow, 2);
			tbl->set<Variant>( cAuto->getClassName(), iRow, 3);
			tbl->set( cAuto->getPermission().clone(), iRow, 4);
			tbl->set( new Variant(cAuto->getLastExecutedTime(), Variant::DATETIME), iRow, 5);
		 }
		this->taskWorker->getAutomations().transactionEnd();
		return request->replyResponse(WRET_OK, tbl);
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
 }; // namespace services
}; // namespace wosh
