/** @file    TelnetSession.cpp
 *  @author  Alessandro Polo
 *  @version $Id: TelnetSession.cpp 2147 2010-03-16 20:56:25Z alex $
 *  @brief
 * File containing methods for the wosh::TelnetSession class.
 * The header for this class can be found in TelnetSession.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "TelnetSession.h"
 
 #include <framework/communication/SessionProtocol.h>

// #include <bundles/services/SessionServer/SessionServer.h>
 #include <core/Utilities.h>
 #include <core/WoshKernel.h>
 #include <core/Request.h>

 #define _ENDLINE	"\r\n"

 
using namespace std;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

TelnetSession::TelnetSession() : Session(), ISocketTcpListener(), Thread(NULL, "wosh.TelnetSession.Thread")
 {
	this->socket = NULL;

	setWelcomeMessage("Welcome WOSH TelnetSession v." _ENDLINE );

	this->msgBufferIn = new char[16384];
	this->msgBufferOut = new char[16384];

 }

TelnetSession::~TelnetSession()
 {
	sessionStop();

	if ( this->msgBufferOut != NULL )
		delete [] this->msgBufferOut;
	this->msgBufferOut = NULL;

	if ( this->msgBufferIn != NULL )
		delete [] this->msgBufferIn;
	this->msgBufferIn = NULL;
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT TelnetSession::sessionStart( unsigned long timeout ) {
	return startThread(timeout);
 }

WRESULT TelnetSession::sessionStop( unsigned long timeout ) {
	return quitThread(timeout);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void TelnetSession::runThread()
 {
	if ( this->socket == NULL ) {
	
		return;
	 }
	ASSERT( this->protocol != NULL )

	setSessionState(Session::CONNECTED);

	// welcome text
	if ( !this->skipWelcome ) {
		this->socket->sendDatagram(this->msgWelcome.c_str(), this->msgWelcome.size() );
		bool sentok = true;
		if ( !sentok ) {
	//		Log(LOG_CRITICAL, ":runThread() : Cant write to Output!" );
		 }
	 }

	username = "root";

	setSessionState(Session::OPENED);

	this->protocol->printPrompt( *this );

	this->running = true;
	string buf_out = "";
	while( this->running ) {
		setThreadAlive();

		//this->outputStream >> buf_out;
		buf_out = this->outputStream.str();
		this->outputStream.str("");
		if ( buf_out.size() > 0 ) {
//qDebug() << "LOOP.2" << buf_out.c_str();
			this->socket->sendDatagram( buf_out.c_str(), buf_out.size() );
		 }

		Thread::sleepForMSec(10);
		if ( this->incomingMsg.isEmpty() )
			continue;

//qDebug() << "LOOP.eval msg " << this->incomingMsg.size();
		Message* message = this->incomingMsg.pop();
		this->protocol->evalMessage( *this, message );
		delete message;

		Thread::sleepForMSec(20);
	 }
	setSessionState(Session::CLOSED);
	this->socket->sendDatagram("Logged Out." _ENDLINE, 11+2);

 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void TelnetSession::tcpDataAvailable( int data_len, SocketTcp* socket )
 {
//	Log(LOG_VERBOSE, ":tcpDataAvailable(%i) ..", data_len );
	(void)socket;

	data_len = _GET_Min(data_len, 16383);

	memset(this->msgBufferIn+data_len, 0, 1);

	this->socket->read(this->msgBufferIn, data_len);

	if ( this->msgBufferIn[data_len-1] == 10 || this->msgBufferIn[data_len-1] == 13 ) {
		this->msgBufferIn[--data_len] = 0;
	 }
	if ( this->msgBufferIn[data_len-1] == 10 || this->msgBufferIn[data_len-1] == 13 ) {
		this->msgBufferIn[--data_len] = 0;
	 }

	setLastInputNow();
	if ( data_len == 0 ) {

		return;
	 }

	string inp(this->msgBufferIn);
	this->protocol->parse_command( *this, inp );
	saveHistory(inp);

 }

void TelnetSession::tcpSocketError( int error_code, const char* description, SocketTcp* socket )
 {
(void)error_code;
(void)description;
(void)socket;
/*
(void)socket;
if ( description != NULL )
log->log( LOG_VERBOSE, ":tcpSocketError(%i) Unknown Error description", error_code );
else
log->log( LOG_VERBOSE, ":tcpSocketError(%i) : %s", error_code, description );
*/
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT TelnetSession::disconnect()
 {
	if ( this->socket == NULL ) {
		return WRET_ERR_ILLEGAL_USE;
	 }

	if ( !this->socket->isConnected() ) {
		return WRET_ERR_PARAM;
	 }

	WRESULT ret = this->socket->disconnect();
	if ( WSUCCEEDED(ret) ) {
		delete this->socket; this->socket = NULL;
	 }

	return ret;
 }

bool TelnetSession::isConnected() const
 {
	return ( this->socket != NULL && this->socket->isConnected() );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
