/** @file    WeatherBundle.cpp
 *  @author  Alessandro Polo
 *  @version $Id: WeatherBundle.cpp 2142 2010-03-16 12:45:50Z alex $
 *  @brief
 * File containing methods for the wosh::services::WeatherBundle class.
 * The header for this class can be found in WeatherBundle.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "WeatherBundle.h"

 #include "WeatherSourceIGoogleAPI.h"

 #include <core/ObjectAllocator.h>
 #include <core/MethodsDef.h>
 #include <core/MessageProcessingCommon.h>
 #include <core/Fact.h>
 #include <core/Table.h>
 #include <core/WoshKernel.h>


using namespace std;
namespace wosh {
 namespace services {

 WOSH_REGISTER(wosh::services::WeatherBundle, "wosh::interfaces::services::Weather", _Weather_VERSION, _static_WeatherBundle1 )
 WOSH_REGISTER(wosh::services::WeatherBundle, "wosh::BundleGeneric", _Weather_VERSION, _static_WeatherBundle2 )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

WeatherBundle::WeatherBundle() : BundleGeneric()
 {
	BundleGeneric::setName( _Weather_NAME, false );
	Properties.update( _KEY_Version, _Weather_VERSION );

Log.setLevel(LOG_VERBOSE);

	Log(LOG_VERBOSE, " Configuring Weather worker.." );
	this->myWorker = new WeatherWorker(*this);
	this->myWorker->setListener(this);
	this->myWorker->addSource( new WeatherSourceIGoogleAPI() );

	Log(LOG_VERBOSE, " Setting default properties and permissions.." );

	Log(LOG_VERBOSE, " Registering methods.." );

	MethodRequest* mmGetWeather = Methods.create<MethodRequest>( _Weather_METHOD_get_weather, "Get Current Weather" );
	mmGetWeather->setMethod( this, (MethodRequestPtr)&WeatherBundle::mmDoGetWeather );
	mmGetWeather->getPermission().setMask( Permission::RX, Permission::Read, Permission::Read );

	MethodRequest* mmGetForeCast = Methods.create<MethodRequest>( _Weather_METHOD_get_forecast, "Get Forecast" );
	mmGetForeCast->setMethod( this, (MethodRequestPtr)&WeatherBundle::mmDoGetForeCast );
	mmGetForeCast->getPermission().setMask( Permission::RX, Permission::Read, Permission::Read );

	MethodRequest* mmGetForeCastAsTable = Methods.create<MethodRequest>( _Weather_METHOD_get_forecast_tbl, "Get Forecast Table" );
	mmGetForeCastAsTable->setMethod( this, (MethodRequestPtr)&WeatherBundle::mmDoGetForeCastAsTable );
	mmGetForeCastAsTable->getPermission().setMask( Permission::RX, Permission::Read, Permission::Read );


	MethodRequest* mmProcessDebug = Methods.create<MethodRequest>( "asdasdasd", "Test processing" );
	mmProcessDebug->setMethod( this, (MethodRequestPtr)&WeatherBundle::mmDoProcessDebug );
	mmProcessDebug->getPermission().setMask( Permission::RX, Permission::Read, Permission::Read );

	setBundleState(Bundle::STATE_CREATED, false);
 }


WeatherBundle::~WeatherBundle()
 {
	Log(LOG_INFO, " Destroying.." );

	if ( isBundleRunning() ) {
		Log(LOG_WARNING, "~WeatherBundle() : Destroying while Bundle is running! Trying to stop.." );
		bundleStop();
	 }

	delete this->myWorker; this->myWorker = NULL;

	Log(LOG_VERBOSE, ":~WeatherBundle() : Destroyed." );
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE CONTROL

WRESULT WeatherBundle::bundleStart()
 {
	if ( !BundleGeneric::bundleValidate_StartStop(Bundle::STATE_STARTING) )
		return WRET_ERR_WRONG_STATE;

	WRESULT ret = BundleGeneric::start_SynchThread( this->myWorker );

	// BUNDLE-STATE (STARTED) will be updated async by WORKER, through call: thread_event()
	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT WeatherBundle::bundleStop()
 {
	if ( !BundleGeneric::bundleValidate_StartStop(Bundle::STATE_STOPPING) )
		return WRET_ERR_WRONG_STATE;

	WRESULT ret = BundleGeneric::stop_SynchThread( this->myWorker );

	// BUNDLE-STATE (STOPPED) will be updated async by WORKER, through call: thread_event()
	return ret;
 }

////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE CONTROL
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void WeatherBundle::busMessage( const Message& message, const Bus* source )
 {
	if ( message.isEmpty() ) return;
	BundleGeneric::busMessage(message, source);

 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENT


////////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENT
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void WeatherBundle::updatedWeather( const WeatherInfo* newInfo, WeatherWorker* )
 {
	if ( newInfo == NULL )
		return;

	Properties.update( _Weather_KEY_CurrentCondition, newInfo->getConditionSummary() );
	Properties.update( _Weather_KEY_CurrentWeatherTs, Variant(newInfo->getTimestamp(), Variant::DATETIME) );

 }

void WeatherBundle::updatedForecast( const std::vector<WeatherInfo*>& , WeatherWorker* )
 {
	Properties.update( "ForeCastUpdate", Variant(Utilities::std_time(), Variant::DATETIME) );

 }


///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS

Response* WeatherBundle::mmDoProcessDebug( const Request* request )
 {
	if ( request == NULL ) return NULL;

	Log(LOG_INFO, "mmDoProcessDebug()" );

//	this->myWorker->doasd();
	return NULL;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* WeatherBundle::mmDoGetWeather( const Request* request )
 {
	if ( request == NULL ) return NULL;
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, ":mmDoGetWeather(%s)", option.c_str() );

	this->myWorker->getWeatherMux().lockForRead();
	WeatherInfo* wInfo_copy = this->myWorker->getWeather().clone();
	this->myWorker->getWeatherMux().unLock();

	return request->replyResponse(WRET_OK, wInfo_copy);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* WeatherBundle::mmDoGetForeCast( const Request* request )
 {
	if ( request == NULL ) return NULL;
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, ":mmDoGetForeCast(%s)", option.c_str() );

	this->myWorker->getForeCast().transactionBeginRead();


	this->myWorker->getForeCast().transactionEnd();

	return NULL;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* WeatherBundle::mmDoGetForeCastAsTable( const Request* request )
 {
	if ( request == NULL ) return NULL;
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, ":mmDoGetForeCastAsTable(%s)", option.c_str() );

	this->myWorker->getForeCast().transactionBeginRead();
	Table* tbl = new Table( 4, this->myWorker->getForeCast().size(), true, true );
	tbl->setTableName("ForeCast");
	tbl->setHeaderVertical("Temp Low", 0);
	tbl->setHeaderVertical("Temp High", 1);
	tbl->setHeaderVertical("Summary", 2);
	tbl->setHeaderVertical("Key", 3);
	unsigned int iCol = 0;
	tWeatherInfoList::ConstIterator it;
	const tWeatherInfoList::ConstIterator it_end = this->myWorker->getForeCast().end();
	for ( it=this->myWorker->getForeCast().begin(); it!=it_end; ++it ) {
		if ( *it == NULL ) continue;
		tbl->setHeaderHorizontal((*it)->getPeriodName(), iCol);
		tbl->set<Variant>( (*it)->getTemperatureLowest(), 0, iCol);
		tbl->set<Variant>( (*it)->getTemperatureHighest(), 1, iCol);
		tbl->set<Variant>( (*it)->getConditionSummary(), 2, iCol);
		tbl->set<Variant>( (*it)->getConditionKey(), 3, iCol);
		++iCol;
	 }
	this->myWorker->getForeCast().transactionEnd();
	return request->replyResponse(WRET_OK, tbl);
 }

///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace services
}; // namespace wosh
