/** @file    WeatherSourceIGoogleAPI.cpp
 *  @author  Alessandro Polo
 *  @version $Id: WeatherSourceIGoogleAPI.cpp 1759 2010-02-02 03:18:18Z alex $
 *  @brief
 * File containing methods for the WeatherSourceIGoogleAPI class.
 * The header for this class can be found in WeatherSourceIGoogleAPI.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "WeatherSourceIGoogleAPI.h"

 #include <core/Utilities.h>

 #include <QFile>
 #include <QtXml>


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

WeatherSourceIGoogleAPI::WeatherSourceIGoogleAPI() : WeatherSource()
 {
	Object::setName("WeatherSourceIGoogleAPI");

	Log.setContext( "WeatherSourceIGoogleAPI" );

	this->httpClient = new ClientHttp();
	this->httpClient->setListener(this);
 }

WeatherSourceIGoogleAPI::~WeatherSourceIGoogleAPI()
 {
	delete this->httpClient; this->httpClient = NULL;
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT WeatherSourceIGoogleAPI::updateData()
 {
	string url = "http://www.google.com/ig/api?weather=Rovereto&hl=en";
	string filename = "$TEMP/api.xml";

	WRESULT ret = this->httpClient->downloadFile( url, filename );
	return ret;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////


WRESULT WeatherSourceIGoogleAPI::loadForecast( WeatherInfo* &wCurrentInfo, vector<WeatherInfo*>& forecast )
 {
	FilePath srcFile;
	srcFile.set( "$TEMP/api.xml" );
	string filename = srcFile.toPathLocal();

	wCurrentInfo = NULL;

	QDomDocument doc("none");
	QFile file(filename.c_str());
	if ( !file.open(QIODevice::ReadOnly) ) {
		Log(LOG_CRITICAL, ":loadForecast() FAILED : Can't open file: %s", filename.c_str() );
		return WRET_ERR_PARAM;
	 }
	if ( !doc.setContent(&file) ) {
		file.close();
		Log(LOG_CRITICAL, ":loadForecast() FAILED : Can't set contents from: %s", filename.c_str() );
		return WRET_ERR_PARAM;
	 }
	file.close();

	QDomElement api_reply_Node = doc.documentElement();
	if ( api_reply_Node.isNull() ) {
		Log(LOG_WARNING, ":loadForecast() FAILED : No 'xml_api_reply' node in %s", filename.c_str() );
		return WRET_ERR_INTERNAL;
	 }
	QDomElement weatherNode = api_reply_Node.namedItem("weather").toElement();
	if ( weatherNode.isNull() ) {
		Log(LOG_WARNING, ":loadForecast() FAILED : No 'weather' node in %s", filename.c_str() );
		return WRET_ERR_INTERNAL;
	 }

	long dataTimestamp = Utilities::std_time();


/*
	<forecast_information>
		<city data="Rovereto, Trentino-Alto Adige"/>
		<postal_code data="Rovereto"/>
		<latitude_e6 data=""/>
		<longitude_e6 data=""/>
		<forecast_date data="2010-02-02"/>
		<current_date_time data="2010-02-02 03:15:16 +0000"/>
		<unit_system data="SI"/>
	</forecast_information>
*/

	QDomElement currConditionNode = weatherNode.namedItem("current_conditions").toElement();
	if ( !currConditionNode.isNull() ) {
		wCurrentInfo = new WeatherInfo();
		wCurrentInfo->setTimestamp(dataTimestamp);

		
		QDomElement TempLowNode = currConditionNode.namedItem("temp_f").toElement();
		if ( !TempLowNode.isNull() && TempLowNode.hasAttribute("data") )
			wCurrentInfo->setTemperatureLowest( TempLowNode.attribute("data").toInt() );

		QDomElement TempHighNode = currConditionNode.namedItem("temp_c").toElement();
		if ( !TempHighNode.isNull() && TempHighNode.hasAttribute("data") )
			wCurrentInfo->setTemperatureHighest( TempHighNode.attribute("data").toInt() );

		QDomElement conditionNode = currConditionNode.namedItem("condition").toElement();
		if ( !conditionNode.isNull() && conditionNode.hasAttribute("data") )
			wCurrentInfo->setConditionSummary( conditionNode.attribute("data").toLatin1().data() );

	 }
/*
	<current_conditions>
		<condition data="Chiaro"/>
		<temp_f data="23"/><temp_c data="-5"/>
		<humidity data="Umidit: 91%"/>
		<icon data="/ig/images/weather/sunny.gif"/>
		<wind_condition data="Vento: N a 0 km/h"/>
	</current_conditions>
*/

	QDomNodeList forecastList = weatherNode.elementsByTagName( "forecast_conditions" );
	for(int j=0; j<forecastList.size(); j++) {
		QDomElement cForecastNode = forecastList.item(j).toElement();
		WeatherInfo* wInfo = new WeatherInfo();
		wInfo->setTimestamp(dataTimestamp);

		QDomElement dayOfWeekNode = cForecastNode.namedItem("day_of_week").toElement();
		if ( !dayOfWeekNode.isNull() && dayOfWeekNode.hasAttribute("data") ) {

			wInfo->setPeriodName( dayOfWeekNode.attribute("data").toLatin1().data() );
		 }

		QDomElement TempLowNode = cForecastNode.namedItem("low").toElement();
		if ( !TempLowNode.isNull() && TempLowNode.hasAttribute("data") )
			wInfo->setTemperatureLowest( TempLowNode.attribute("data").toInt() );

		QDomElement TempHighNode = cForecastNode.namedItem("high").toElement();
		if ( !TempHighNode.isNull() && TempHighNode.hasAttribute("data") )
			wInfo->setTemperatureHighest( TempHighNode.attribute("data").toInt() );

		QDomElement conditionNode = cForecastNode.namedItem("condition").toElement();
		if ( !conditionNode.isNull() && conditionNode.hasAttribute("data") )
			wInfo->setConditionSummary( conditionNode.attribute("data").toLatin1().data() );

		QDomElement iconNode = cForecastNode.namedItem("icon").toElement();
		if ( !iconNode.isNull() && iconNode.hasAttribute("data") ) {
			QString icon_path = iconNode.attribute("data");
			icon_path = icon_path.mid(19);
			icon_path.chop(4);
 			wInfo->setConditionKey( icon_path.toLatin1().data() );
		 }

		forecast.push_back(wInfo);
	 }

	return WRET_OK;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void WeatherSourceIGoogleAPI::httpRequestInitiated( ClientHttp* ) {
 }

void WeatherSourceIGoogleAPI::httpRequestCompleted( ClientHttp* ) {
 }

void WeatherSourceIGoogleAPI::httpRequestError( ClientHttp* ) {
 }

void WeatherSourceIGoogleAPI::httpDataReadProgress( long bytesRead, long totalBytes, ClientHttp* ) {
	(void)bytesRead; (void)totalBytes;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
