/** @file    WeatherWorker.cpp
 *  @author  Alessandro Polo
 *  @version $Id: WeatherWorker.cpp 1759 2010-02-02 03:18:18Z alex $
 *  @brief
 * File containing methods for the WeatherWorker class.
 * The header for this class can be found in WeatherWorker.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "WeatherWorker.h"


using namespace std;
using namespace wosh;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

WeatherWorker::WeatherWorker( BundleGeneric& bundle ) : BundleGenericWorker(bundle)
 {
	this->listener = NULL;

	this->source = NULL;
 }


WeatherWorker::~WeatherWorker() {
	if ( isThreadRunning() ) {
		quitThread(10000);
	 }

	if ( this->source != NULL )
		delete this->source; this->source = NULL;

 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////// RUN THREAD

void WeatherWorker::runThread()
 {
	long period_sec = 30;

	long counter = 0;
	bool get_gather = true;
	while( this->running )
	 {
		Thread::sleepForMSec(500);
		setThreadAlive();
		Thread::sleepForMSec(500);
		setThreadAlive();

		++counter;
		if ( counter >= period_sec ) {

			if ( get_gather )
				updateInfo(this->source);
			else
				gatherInfo(this->source);

			counter = 0;
		 }
		
		setThreadAlive();
	 }

 }

////////////////////////////////////////////////////////////////////////////////////////////////// RUN THREAD
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT WeatherWorker::addSource( WeatherSource* Source )
 {
	this->source = Source;
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT WeatherWorker::updateInfo( WeatherSource* wSource )
 {
	if ( wSource == NULL ) return WRET_ERR_PARAM;
	Log(LOG_VERBOSE, ":updateInfo() : Forcing update of %s", wSource->getName().c_str() );
	return wSource->updateData();
 }

WRESULT WeatherWorker::gatherInfo( WeatherSource* wSource )
 {
	if ( wSource == NULL ) return WRET_ERR_PARAM;

	WeatherInfo* newInfo = NULL;
	vector<WeatherInfo*> newForecast;
	WRESULT ret = wSource->loadForecast( newInfo, newForecast );

	WRESULT ret_w = dataFusion_CurrentWeather( newInfo, wSource );
	WRESULT ret_f = dataFusion_Forecast( newForecast, wSource );

	if ( newInfo != NULL ) {
		delete newInfo;
	 }

	vector<WeatherInfo*>::iterator it;
	for (it=newForecast.begin(); it!=newForecast.end(); it++) {
		if ( *it != NULL )
			delete *it;
	 }

	// callbacks
	if ( this->listener != NULL ) {
		if ( WSUCCEEDED(ret_w) )
			this->listener->updatedWeather( &this->wCurrentInfo,  this);
		if ( WSUCCEEDED(ret_f) )
			this->listener->updatedForecast( this->forecast.getVector(), this);
	 }

	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT WeatherWorker::dataFusion_CurrentWeather( const WeatherInfo* newInfo, WeatherSource* )
 {
	if ( newInfo == NULL ) {
		Log(LOG_VERBOSE, ":dataFusion_CurrentWeather() NULL info provided!" );
		return WRET_ERR_PARAM;
	 }
	Log(LOG_VERBOSE, ":dataFusion_CurrentWeather() : Merging data.." );

	// dummy implementation
	this->wCurrentInfoMux.lockForWrite();
	this->wCurrentInfo = *newInfo;
	this->wCurrentInfoMux.unLock();

	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT WeatherWorker::dataFusion_Forecast( const vector<WeatherInfo*>& newForecast, WeatherSource* )
 {
	 Log(LOG_VERBOSE, ":dataFusion_Forecast() : Merging %i data..", newForecast.size() );
	// dummy implementation

	this->forecast.transactionBeginWrite();

	this->forecast.free();
	vector<WeatherInfo*>::const_iterator it;
	for (it=newForecast.begin(); it!=newForecast.end(); it++) {
		if ( *it == NULL ) continue;
		this->forecast.push_back( (*it)->clone() );
	 }

	this->forecast.transactionEnd();

	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
