/** @file    BundleManager.cpp
 *  @author  Alessandro Polo
 *  @version $Id: BundleManager.cpp 2214 2010-03-26 16:03:31Z alex $
 *  @brief
 * File containing methods for the wosh::BundleManager class.
 * The header for this class can be found in BundleManager.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <core/BundleManager.h>

 #include <core/ObjectAllocator.h>
 #include <core/Settings.h>
 #include <core/WoshKernel.h>
 #include <core/Thread.h>
 #include <core/MethodsDef.h>
 #include <core/MessageFilter.h>
 #include <core/Fact.h>
 #include <core/SystemError.h>
 #include <core/NEW_List.h>
 #include <core/Table.h>
 #include <core/Functional.h>
 #include <core/SecurityManager.h>
 #include <core/ObjectFactory.h>
 #include <core/BundleRequirements.h>
 #include <core/PropertiesProvider.h>
 #include <core/InterfacesProvider.h>
 #include <core/MethodsProvider.h>
/*
 #include <core/MessageProcessingCommon.h>
 #include <core/FileSystem.h>

*/

using namespace std;
namespace wosh {

// Gamma singleton! static reference is allocated and maintained by WoshKernel
extern "C" {
	BundleManager* BundleManager::getInstance() { 
		if ( WoshKernel::getInstance() == NULL ) return NULL;
		return &WoshKernel::getInstance()->bundles();
	}
}
 // register as abstract because it is a singleton:
 WOSH_REGISTER_INTERFACE(wosh::BundleManager, "wosh::WoshModule", 0.01, _static_BundleManager )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

BundleManager::BundleManager( const WoshKernel* parent ) : WoshModule(parent)
 {
	Object::setName( _BundleManager_NAME );
	Object::getURI().fromString( _BundleManager_URI );
	Object::getURI().registerGlobal();

//Log.setLevel( LOG_VERBOSE );

	Log(LOG_VERBOSE, " Initializing.." );
	Properties.set( _BundleManager_KEY_Count, 0, Permission( Permission::Read ) );

	Properties.set( _BundleManager_KEY_AutoLoadBundles,	false, Permission( Permission::RW, Permission::Read, Permission::Read ) );
	Properties.set( _BundleManager_KEY_AutoStartBundles,false, Permission( Permission::RW, Permission::Read, Permission::Read ) );
	
	Properties.set( _BundleManager_KEY_BundleConfigFolder, "$CONFIG/bundles-enabled", Permission( Permission::RW, Permission::Read, Permission::Read ) );
	Properties.set( _BundleManager_KEY_BundleLoadFolder, "$CONFIG/bundles-enabled", Permission( Permission::RW, Permission::Read, Permission::Read ) );
	Properties.set( _BundleManager_KEY_BundleLibFolder, "$ROOT/lib", Permission( Permission::RW, Permission::Read, Permission::Read ) );

	Log(LOG_VERBOSE, " Registering Methods.." );
	MethodRequest* mmListBundles = Methods.create<MethodRequest>( _METHOD_List, "List installed Bundles", Permission(Permission::RX) );
	mmListBundles->setMethod( this, (MethodRequestPtr)&BundleManager::mmDoListBundles );

	MethodMessageResponse* mmLoadBundle = Methods.create<MethodMessageResponse>( _BundleManager_METHOD_LoadBundle, "Load a Bundle", Permission::RW_RW_R__() );
	mmLoadBundle->setMethod( this, (MethodMessageResponsePtr)&BundleManager::mmDoLoadBundle );

	MethodMessageResponse* mmControlBundle = Methods.create<MethodMessageResponse>( _BundleManager_METHOD_control_bundle, "control a Bundle", Permission::RW_RW_R__() );
	mmControlBundle->setMethod( this, (MethodMessageResponsePtr)&BundleManager::mmDoControlBundle );
 }

BundleManager::~BundleManager()
 {
	Log(LOG_INFO, " Destroying.." );

	Log(LOG_VERBOSE, ":~BundleManager() : Stopping Bundles.." );
	stopBundles();

	// delete remaining bundles (such the ones are created inline by user)
	if ( this->bundles.size() != 0 ) {
		Log(LOG_VERBOSE, ":~BundleManager() : Freeing Bundles [%d]..", this->bundles.size() );
	 }
	this->bundles.clear();

	Log(LOG_VERBOSE, ":~BundleManager() Destroyed." );
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////// KERNEL EVENTS

WRESULT BundleManager::onKernelInitializing() {
	WRESULT ret = WoshModule::onKernelInitializing();

	bool configured = false;
	if ( Properties.exists(_BundleManager_KEY_BundleConfigFolder) ) {
		string cfgPath = Properties.getValueOf(_BundleManager_KEY_BundleConfigFolder).toString();
		ret += WoshKernel::getInstance()->settings().loadConfigurationFolder( cfgPath );
		if ( WFAILED(ret) )
			Log(LOG_CRITICAL, ":init() FAILED#%d Loading Config Folder!", ret );
		else
			configured = true;
	 }
	bool autoLoad = Properties.getValueOf(_BundleManager_KEY_AutoLoadBundles).toBoolean(false);
	if ( autoLoad && !configured ) {
		Log(LOG_WARNING, ":init() AutoLoading Bundles is enabled  but had Errors Loading Config Folder!" );
		autoLoad = false;
		ret += WRET_ERR_PARAM;
	 }

	bool autostart = Properties.getValueOf(_BundleManager_KEY_AutoStartBundles).toBoolean(false);
	if ( autostart && !autoLoad ) {
		Log(LOG_WARNING, ":init() AutoStarting Bundles is enabled but had Errors Loading Config Folder/Loading Bundles!" );
		ret += WRET_ERR_PARAM;
	 }

	return ret;
 }
WRESULT BundleManager::onKernelInitialized() {
	WRESULT ret = WoshModule::onKernelInitialized();
	bool autoLoad = Properties.getValueOf(_BundleManager_KEY_AutoLoadBundles).toBoolean(false);
	if ( !autoLoad )
		return ret;

	Log(LOG_VERBOSE, ":doAutoLoadBundles() : Selecting Property "_BundleManager_KEY_BundleLoadFolder );
	string local_folder = Properties.getValueOf(_BundleManager_KEY_BundleLoadFolder).toString();
	ret = doAutoLoadBundles(local_folder);
	// apply settings before starting!!
	ret = applySettingsOfBundles( &WoshKernel::getInstance()->settings() );
	return ret;
 }
WRESULT BundleManager::onKernelRunning() {
	WRESULT ret = WoshModule::onKernelRunning();
	bool autostart = Properties.getValueOf(_BundleManager_KEY_AutoStartBundles).toBoolean(false);
	if ( !autostart )
		return ret;

	ret = this->loader.executeActions( BundleAction::ACTION_START );
	if ( WFAILED(ret) ) {
		Log(LOG_CRITICAL, ":doAutoStart() FAILED#%d Appling LOAD Actions!", ret );
	 }
	return ret;
 }
WRESULT BundleManager::onKernelStopping() {
	WRESULT ret = WoshModule::onKernelStopping();

	ret = stopBundles();

	return ret;
 }
WRESULT BundleManager::onKernelStopped() {
	WRESULT ret = WoshModule::onKernelStopped();

	ret = unloadBundles();

	return ret;
 }

/////////////////////////////////////////////////////////////////////////////////////////////// KERNEL EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT BundleManager::doAutoLoadBundles( const string& cfgPath )
 {
	Log(LOG_INFO, ":doAutoLoadBundles(%s) : AutoLoading Bundles..", cfgPath.c_str() );
	WRESULT bundles_confs = this->loader.loadActionsFolder( cfgPath );
	if ( bundles_confs != WRET_OK ) {
		Log(LOG_CRITICAL, ":doAutoLoadBundles() FAILED#%d Loading Load Folder %s!", bundles_confs, cfgPath.c_str() );
	 }
	// create&register Bundles
	WRESULT bundles_exec = this->loader.executeActions( BundleAction::ACTION_LOAD );
	if ( bundles_exec != WRET_OK ) {
		Log(LOG_CRITICAL, ":doAutoLoadBundles() FAILED#%d Appling LOAD Actions!", bundles_exec );
	 }
	return bundles_exec;
 }

WRESULT BundleManager::applySettingsOfBundles( Settings* settings ) {
	if ( settings == NULL ) return WRET_ERR_PARAM;
	WRESULT ret = WRET_OK;
	this->bundles.transactionBeginRead();
	tBundleList::Iterator it;
	const tBundleList::ConstIterator it_end = this->bundles.end();
	for ( it=this->bundles.begin(); it!=it_end; ++it ) {
		if ( *it == NULL ) continue;
		ret += BundleAction::applySettingTo( *it );
	 }
	this->bundles.transactionEnd();
	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT BundleManager::installBundle( Bundle* bundle, bool start_bundle, bool applySettings, unsigned long timeout ) {
	if ( bundle == NULL ) return WRET_ERR_PARAM;
	WRESULT ret = WRET_OK;
	Log(LOG_INFO, ":installBundle(%s) : Installing..", bundle->getName().c_str() );
	// register bundle to BundleManager
	this->bundles.transactionBeginWrite();
	ret = registerBundle_( bundle );
	if ( WFAILED(ret) ) {
		this->bundles.transactionEnd();
		Log(LOG_CRITICAL, ":installBundle(%s) : FAILED#%d Registering!", bundle->getName().c_str(), ret );
		return ret;
	 }
	// apply related default settings if any
	if ( applySettings )
		ret = BundleAction::applySettingTo( bundle );

	if ( !start_bundle ) {
		this->bundles.transactionEnd();
		return ret;
	 }
	Log(LOG_VERBOSE, ":installBundle(%s) : Starting..", bundle->getName().c_str() );
	BundleActionStart* action = new BundleActionStart(Log);
	action->bundle_name = bundle->getName();
	this->bundles.transactionEnd();

	action->execute();
	ret = action->result_ret;
	if ( WFAILED(ret) )
		Log(LOG_VERBOSE, ":installBundle(%s) : ERROR#%d Starting!", bundle->getName().c_str(), action->result_ret );
	delete action;

	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT BundleManager::loadBundle( const string& bundle_type, const string& bundle_name, bool applySettings ) {
	WRESULT ret = WRET_OK;
	Log(LOG_INFO, ":loadBundle(%s,%s) : Installing..", bundle_type.c_str(), bundle_name.c_str() );

	BundleActionLoad* action = new BundleActionLoad(Log);
	action->bundle_type.name = bundle_type;
	action->bundle_name = bundle_name;

	action->execute();
	ret = action->result_ret;
	if ( WFAILED(ret) )
		Log(LOG_VERBOSE, ":loadBundle(%s,%s) : ERROR#%d Starting!", bundle_type.c_str(), bundle_name.c_str(), action->result_ret );
	delete action;


	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT BundleManager::registerBundle( Bundle* bundle ) {
	MutexLockerWrite mL(this->bundles.getMutex());
	return registerBundle_(bundle);
}

WRESULT BundleManager::registerBundle_( Bundle* bundle ) {
	if ( bundle == NULL ) return WRET_ERR_PARAM;
	if ( this->bundles.exists(bundle) ) {
		this->bundles.transactionEnd();
		SystemError::raise(this, Log, SystemError::WARNING, 0, "registerBundle", "FAILED: Bundle '%s' already registered", bundle->getName().c_str() );
		return WRET_ERR_ILLEGAL_USE;
	 }
	tBundleList::ConstIterator it_f = std::find_if(this->bundles.begin(), this->bundles.end(),
		Functor_Validator<Bundle, const std::string&>(bundle->getName(), &Bundle::getName ) );
	if ( it_f != this->bundles.end() ) {
		SystemError::raise(this, Log, SystemError::WARNING, 0, "registerBundle", "WARNING: Bundle '%s' already registered", bundle->getName().c_str() );
		return WRET_ERR_PARAM;
	 }
	bundle->setParentObject(this);
	bundle->setPermission( Object::getPermission() );
	if ( !bundle->getURI().isGlobal() ) {
		bundle->getURI().setKernelLocal();
		bundle->getURI().setParent( this->getURI() );
		bundle->getURI().setName( bundle->getName() );
		bundle->getURI().registerGlobal();
	}
	Log(LOG_INFO, ":registerBundle(%s) : Adding to list..", bundle->getName().c_str() );
	this->bundles.push_back(bundle);
	Properties.update( _BundleManager_KEY_Count, this->bundles.size() );
	return WRET_OK;
 }

WRESULT BundleManager::unRegisterBundle( Bundle* bundle )
 {
	if ( bundle == NULL ) return WRET_ERR_PARAM;

	this->bundles.transactionBeginWrite();
	if ( !this->bundles.exists(bundle) ) {
		this->bundles.transactionEnd();
		SystemError::raise(this, Log, SystemError::WARNING, 0, "unRegisterBundle", "FAILED: Bundle '%s' not found", bundle->getName().c_str() );
		return WRET_ERR_PARAM;
	 }
	Log(LOG_INFO, ":unRegisterBundle(%s) : Removing from list..", bundle->getName().c_str() );
	if ( bundle->getURI().isGlobal() )
		bundle->getURI().unRegisterGlobal();
	this->bundles.pop(bundle);
	Properties.update( _BundleManager_KEY_Count, this->bundles.size() );
	this->bundles.transactionEnd();
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT BundleManager::startBundles()
 {
	Log(LOG_INFO, ":startBundles() : Starting.." );
	WRESULT ret = WRET_OK;
	tBundleActionList actions;
	ret = this->loader.createActions( actions, Bundle::STATE_STARTED );
	ret = this->loader.executeActions( actions );
	actions.free();
	return ret;
 }

WRESULT BundleManager::stopBundles()
 {
	Log(LOG_INFO, ":stopBundles() : Stopping.." );
	WRESULT ret = WRET_OK;
	tBundleActionList actions;
	ret = this->loader.createActions( actions, Bundle::STATE_STOPPED );
	ret = this->loader.executeActions( actions );
	actions.free();
	return ret;
 }

WRESULT BundleManager::unloadBundles()
 {
	Log(LOG_INFO, ":unloadBundles() : UnLoading.." );
	WRESULT ret = WRET_OK;
	tBundleActionList actions;
	ret = this->loader.createActions( actions, Bundle::STATE_DESTROYED );
	ret = this->loader.executeActions( actions );
	actions.free();
	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Bundle* BundleManager::getBundleByName_( const string& bundle_name ) const
 {
	tBundleList::ConstIterator it_f = std::find_if(this->bundles.begin(), this->bundles.end(),
		Functor_Validator<Bundle, const std::string&>(bundle_name, &Bundle::getName ) );
	if ( it_f == this->bundles.end() )
		return NULL;
	return *it_f;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE EVENTS

void BundleManager::bundle_event( Bundle::BUNDLE_STATE event, Bundle* bundle )
 {
	if ( bundle == NULL ) return;
	Log(LOG_VERBOSE, ":bundle_event() : Event #%d from %s", (int)event, bundle->getName().c_str() );
	switch(event) {
		case Bundle::STATE_CREATED: {	break; }
		case Bundle::STATE_STARTED: {
//				this->msgQueue.push( new Message( new Fact( "bundle_started", new Variant(bundle->getName()) ) ) );
				break;
			 }
		case Bundle::STATE_STOPPED: {
//				this->msgQueue.push( new Message( new Fact( "bundle_stopped", new Variant(bundle->getName()) ) ) );
				break;
			 }
		case Bundle::STATE_DESTROYED: {
//				this->unRegisterBundle( bundle );
				break;
			 }
		case Bundle::STATE_FAILURE: {
//				this->msgQueue.push( new Message( new Fact( "bundle_failure", new Variant(bundle->getName()) ) ) );
				break;
			 }
		default: { break; }
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS

Response* BundleManager::mmDoControlBundle( const Message* requestMessage )
 {
	if ( requestMessage == NULL || requestMessage->isEmpty() ) return NULL;
	if ( !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();
	// check rigths
	Variant dBundleName;
	Variant dAction;
	request->extractArguments(dBundleName, dAction);
	if ( dBundleName.isStringEmpty() )
		return request->replyResponse(WRET_ERR_PARAM, "invalid bus_name");
	this->bundles.transactionBeginRead();

	Bundle* bundle = getBundleByName_( dBundleName.asString() );
	if ( bundle == NULL ) {
		this->bundles.transactionEnd();
		return request->replyResponse(WRET_ERR_PARAM, "invalid bus_name");
	 }
	WRESULT ret = WRET_ERR_INTERNAL;
	if ( dAction == (int)Bundle::STATE_STARTED || dAction == "start" ) {
		if ( !bundle->isBundleRunning() )
			ret = bundle->bundleStart(); // asynch??
	 }
	else if ( dAction == (int)Bundle::STATE_STOPPED || dAction == "stop" ) {
		if ( bundle->isBundleRunning() )
			ret = bundle->bundleStop(); // asynch??
	 }
	else if ( dAction == "restart" ) {
		if ( bundle->isBundleRunning() )
			ret = bundle->bundleRestart(); // asynch??
	 }
	this->bundles.transactionEnd();
	return request->replyResponse(ret);
 }

Response* BundleManager::mmDoLoadBundle( const Message* requestMessage )
 {
	if ( requestMessage == NULL || requestMessage->isEmpty() ) return NULL;
	if ( !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();
	///@todo check rigths
	Variant dBundleType;
	Variant dBundleName;
	request->extractArguments(dBundleType, dBundleName);
	if ( dBundleType.isStringEmpty() )
		return request->replyResponse(WRET_ERR_PARAM, "invalid bundle type");

	BundleActionLoad load_bundle(Log);
	load_bundle.bundle_type.name = dBundleType.asString();
	if ( !dBundleName.isStringEmpty() )
		load_bundle.bundle_name = dBundleName.asString();

	Log(LOG_VERBOSE, ":mmDoLoadBundle(%s)", request->getArgument().toString().c_str() );
	load_bundle.execute();

	WRESULT ret = load_bundle.result_ret;
	return request->replyResponse(ret, load_bundle.bundle_name.c_str() );
 }











































template <class RUNNER_TYPE, typename RET_TYPE, class RET_PROXY>
class Functor_TableSetter_TY  {
	public:
		typedef RET_TYPE (RUNNER_TYPE::*MethodSetterSelector)(void) const;

		explicit Functor_TableSetter_TY( Table* output, bool vertical, int row_col, MethodSetterSelector fptr )
			: targetTable(output), orientationVertical(vertical), fSelectorPtr(fptr) {
			this->iRow = 0;
			this->iCol = 0;
			if ( orientationVertical )
				this->iCol = row_col;
			else
				this->iRow = row_col;
		 }

		bool operator() ( RUNNER_TYPE* object ) const {
			if ( object == NULL ) return false;
			// for each row/column iterate..
// wrong on g++			if ( this->orientationVertical )
// wrong on g++				this->targetTable->set<RET_PROXY>( (object->*fSelectorPtr)(), this->iRow++, this->iCol, true );
// wrong on g++			else
// wrong on g++				this->targetTable->set<RET_PROXY>( (object->*fSelectorPtr)(), this->iRow, this->iCol++, true );
			return true;
		 }

	private:
		mutable Table* targetTable;
		mutable int iRow;
		mutable int iCol;
		bool orientationVertical;
		MethodSetterSelector fSelectorPtr;

};

template <class RUNNER_TYPE>
class TableGenerator  {
	public:
		TableGenerator( Table* output, bool vertical = true )
			: targetTable(output), orientationVertical(vertical) {
		 }

//		template <typename RET_TYPE, class RET_PROXY>
//		void setSchema(int index, Functor_TableSetter_TY<RUNNER_TYPE,RET_TYPE,RET_PROXY>::MethodSetterSelector aa, ISerializable* header ) {
//		void setSchema(int index, Functor_TableSetter_TY::MethodSetterSelector fptr, ISerializable* header ) {
	//		if ( this->orientationVertical )
	//			this->targetTable->setHeaderHorizontal( new Variant("Name"), index);
	//		else
	//			this->targetTable->setHeaderVertical( new Variant("Name"), index);

//			Functor_TableSetter_TY<RUNNER_TYPE, RET_TYPE, RET_PROXY> MySetter(this->targetTable, orientationVertical, 0, fptr);
//		 }

		void generate() {
			if ( this->orientationVertical ) {
				const int cols = targetTable->getCols();
				for (int col=0; col<cols; ++col) {

//Functor_TableSetter_TY<Bundle, const std::string&, Variant> MySetter(tbl, true, 0, &Bundle::getName);
//for_each( this->bundles.begin(), this->bundles.end(), MySetter );


				 }
			 }
		 }

	private:
		Table* targetTable;
		bool orientationVertical;

};


Response* BundleManager::mmDoListBundles( const Request* request )
 {
	if ( request == NULL ) return NULL;
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, ":mmDoListBundle(%s)", option.c_str() );

	if ( option == "s" || option == "short" ) { 
		NEW_List* objs = new NEW_List();
		this->bundles.transactionBeginRead();
		for_each( this->bundles.begin(), this->bundles.end(),
					Functor_Appender<NEW_List, Bundle, const std::string&>(*objs, &Bundle::getName ) );
		this->bundles.transactionEnd();
		return request->replyResponse(WRET_OK, objs);
	 }
	else {
/*
Table* tbl = new Table( this->bundles.size(), 5, true );
TableGenerator<Bundle> tG(tbl);
setColumn(-1, void, "vertical-header-line" );
setColumn(0, &Bundle::getName, "Name" );
setColumn(1, &Bundle::getPermission, "Permission" );

*/

		this->bundles.transactionBeginRead();
		Table* tbl = new Table( this->bundles.size(), 5, true );

//OK tbl->set<Variant>( "asd", 0, 3, true );
//OK Functor_TableSetter_TY<Bundle, const std::string&, Variant> MySetter(tbl, true, 0, &Bundle::getName);
//OK for_each( this->bundles.begin(), this->bundles.end(), MySetter );

//TableGenerator<Bundle> tG(tbl);
//tG.setSchema<const std::string&, Variant>(0, &Bundle::getName, "Name" );


		tbl->setTableName("Bundles");
		tbl->setHeaderHorizontal("Name", 0);
		tbl->setHeaderHorizontal("Interfaces", 1);
		tbl->setHeaderHorizontal("State", 2);
		tbl->setHeaderHorizontal("URI", 3);
		tbl->setHeaderHorizontal("Permission", 4);
		unsigned int iRow = 0;
		tBundleList::ConstIterator it;
		const tBundleList::ConstIterator it_end = this->bundles.end();
		for ( it=this->bundles.begin(); it!=it_end; ++it, ++iRow ) {
			if ( *it == NULL ) continue;
			Bundle* currB = *it;
			tbl->set<Variant>( currB->getName(), iRow, 0);
			if ( currB->bundleInterfaces() != NULL )
				tbl->set<Variant>( currB->bundleInterfaces()->toString(), iRow, 1);
			tbl->set( currB->bundleInterfaces()->toList(), iRow, 1);
			tbl->set<Variant>( Bundle::getBundleStateAsString(currB->getBundleState()), iRow, 2);
			tbl->set( currB->getURI().clone(), iRow, 3);
			tbl->set( currB->getPermission().clone(), iRow, 4);
		 }
		this->bundles.transactionEnd();
		return request->replyResponse(WRET_OK, tbl);
	 }
 }

///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// STATIC MEMBERS

bool BundleManager::waitState( const Bundle* bundle, Bundle::BUNDLE_STATE state, unsigned long timeout )
 {
	if ( bundle == NULL ) return false;
	if ( timeout > 0 )
		timeout = (unsigned long)(timeout / 10) + 1;
	unsigned long i = 0;
	while( ++i < timeout ) {
		if ( bundle->getBundleState() == state )
			return true;
		if ( state != Bundle::STATE_FAILURE && bundle->getBundleState() == Bundle::STATE_FAILURE )
			return false;
		Thread::sleepForMSec(10);
	 }
	if ( timeout > 0 )
		SystemError::raise(BundleManager::getInstance(),SystemError::CRITICAL,_BundleManager_ERROR_waitstate,"waitState");
	return ( bundle->getBundleState() == state );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

}; // namespace wosh
