/** @file    BundleRequirements.cpp
 *  @author  Alessandro Polo
 *  @version $Id: BundleRequirements.cpp 1499 2009-12-31 16:39:34Z alex $
 *  @brief
 * File containing methods for the wosh::BundleRequirements class.
 * The header for this class can be found in BundleRequirements.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <core/BundleRequirements.h>

 #include <core/NEW_List.h>
 #include <core/Table.h>
 #include <core/Functional.h>


using namespace std;
namespace wosh {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

BundleRequirements::~BundleRequirements() {
	this->requirements.clear();
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT BundleRequirements::add( BundleRequirement* requirement ) {
	if ( requirement == NULL ) return WRET_ERR_PARAM;
	MutexLockerWrite mL(this->requirements.getMutex());
	tBundleRequirementList::ConstIterator it;
	const tBundleRequirementList::ConstIterator it_end = this->requirements.end();
	for ( it=this->requirements.begin(); it!=it_end; ++it ) {
		if ( *it == NULL ) continue;
		if ( *it == requirement ) return WRET_ERR_PARAM;
		if ( *(*it) == *requirement ) return WRET_ERR_PARAM;
	 }
	this->requirements.push_back(requirement);
	return WRET_OK;
 }

WRESULT BundleRequirements::add( const std::string& classType, Bundle::BUNDLE_STATE scope, double interfaceVersionMin ) {
	BundleRequirement* breq = new BundleRequirement();
	breq->setClassType( classType );
	breq->setClassVersionMin( interfaceVersionMin );
	breq->setScopes( scope );
	WRESULT ret = add(breq);
	if ( WFAILED(ret) )
		delete breq;
	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

int BundleRequirements::count( Bundle::BUNDLE_STATE scope ) const {
	MutexLockerRead mL(this->requirements.getMutex());
	int found = 0;
	BundleRequirements::tBundleRequirementList::ConstIterator it;
	const BundleRequirements::tBundleRequirementList::ConstIterator it_end = this->requirements.end();
	for ( it=this->requirements.begin(); it!=it_end; ++it ) {
		if ( *it == NULL ) continue;
		if ( (*it)->getScopes() & (int)scope )
			++found;
	 }
	return found;
 }

int BundleRequirements::getRequirementsOf( vector<BundleRequirement*>& req_list, Bundle::BUNDLE_STATE scope ) const {
	BundleRequirements::tBundleRequirementList::ConstIterator it;
	const BundleRequirements::tBundleRequirementList::ConstIterator it_end = this->requirements.end();
	for ( it=this->requirements.begin(); it!=it_end; ++it ) {
		if ( *it == NULL ) continue;
		if ( (*it)->getScopes() & (int)scope )
			req_list.push_back( new BundleRequirement(*(*it)) );
	 }
	return req_list.size();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

wosh::BundleRequirements& BundleRequirements::operator=(const wosh::BundleRequirements& m){
	if (this == &m) return *this; // same object?

	return *this;
 }

bool BundleRequirements::operator==(const wosh::BundleRequirements &other) const
 {
	if ( this->requirements.count() != other.requirements.count() )
		return false;

	return true;
 }
 
bool BundleRequirements::operator>(const wosh::BundleRequirements& other) const
 {
	if ( this->requirements.count() > other.requirements.count() )
		return true;

	return false;
 }

bool BundleRequirements::operator<(const wosh::BundleRequirements& other) const
 {
	if ( this->requirements.count() < other.requirements.count() )
		return true;

	return false;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

std::string BundleRequirements::toString() const
 {
	return "";
 }

WRESULT BundleRequirements::store( IEncoderWriter* serializer ) const {
	if ( serializer == NULL ) return WRET_ERR_PARAM;

	return WRET_OK;
 }
WRESULT BundleRequirements::retrieve( IEncoderReader* deserializer ) {
	if ( deserializer == NULL ) return WRET_ERR_PARAM;

	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

NEW_List* BundleRequirements::toList() const
 {
	this->requirements.transactionBeginRead();
	NEW_List* objs = new NEW_List(this->requirements.size());
	for_each( this->requirements.begin(), this->requirements.end(),
				Functor_Appender<NEW_List, BundleRequirement, const std::string&>(*objs, &BundleRequirement::getClassType ) );
	this->requirements.transactionEnd();
	return objs;
 }

Table* BundleRequirements::toTable() const
 {
	this->requirements.transactionBeginRead();
	Table* table = new Table( this->requirements.size(), 4, true );
	table->setTableName("Requirements");
	table->setHeaderHorizontal("Scopes", 0);
	table->setHeaderHorizontal("Type", 1);
	table->setHeaderHorizontal("Hops (Max)", 2);
	table->setHeaderHorizontal("Version (Min)", 3);
	unsigned int iRow = 0;
	tBundleRequirementList::ConstIterator it;
	const tBundleRequirementList::ConstIterator it_end = this->requirements.end();
	for ( it=this->requirements.begin(); it!=it_end; ++it, ++iRow ) {
		if ( *it == NULL ) continue;
		table->set<Variant>( (*it)->getScopes(), iRow, 0 );
		table->set<Variant>( (*it)->getClassType(), iRow, 1 );
		table->set<Variant>( (*it)->getClassVersionMin(), iRow, 2 );
		table->set<Variant>( (*it)->getLocationHopsMax(), iRow, 3 );
	 }
	this->requirements.transactionEnd();
	return table;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

}; // namespace wosh
