/** @file    DeviceManager.cpp
 *  @author  Alessandro Polo
 *  @version $Id: DeviceManager.cpp 2225 2010-03-26 21:50:21Z alex $
 *  @brief
 * File containing methods for the wosh::DeviceManager class.
 * The header for this class can be found in DeviceManager.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <core/DeviceManager.h>

 #include <core/ObjectAllocator.h>
 #include <core/BusRing.h>
 #include <core/Thread.h>
 #include <core/MessageFilter.h>
 #include <core/SecurityManager.h>
 #include <core/BusManager.h>
 #include <core/SystemError.h>
 #include <core/WoshKernel.h>

 #include <core/MethodsDef.h>
 #include <core/Fact.h>
 #include <core/NEW_List.h>
 #include <core/Table.h>
 #include <core/MessageProcessingCommon.h>

 #include <core/Bundle.h>

using namespace std;
namespace wosh {

// Gamma singleton! static reference is allocated and maintained by WoshKernel
extern "C" {
	DeviceManager* DeviceManager::getInstance() { 
		if ( WoshKernel::getInstance() == NULL ) return NULL;
		return &WoshKernel::getInstance()->devices();
	}
}
 // register as abstract because it is a singleton:
 WOSH_REGISTER_INTERFACE(wosh::DeviceManager, "wosh::WoshModule", 0.01, _static_DeviceManager )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

DeviceManager::DeviceManager( const WoshKernel* parent ) : WoshModule(parent)
 {
	Object::setName( _DeviceManager_NAME );
	Object::getURI().fromString( _DeviceManager_URI );
	Object::getURI().registerGlobal();

//Log.setLevel( VERBOSE );

	Log(LOG_VERBOSE, " Initializing.." );
	this->eventLogMaxSize = _DeviceManager_DEFAULT_EventLogMaxSize;
	this->eventLogLocallyOnly = true;

	Properties.set( _DeviceManager_KEY_EventLogMaxSize,		this->eventLogMaxSize, Permission::RW_RW_R__() );
	Properties.set( _DeviceManager_KEY_EventLogLocallyOnly,	true, Permission::RW_RW_R__() );
	Properties.monitor(_DeviceManager_KEY_EventLogMaxSize);
	Properties.monitor(_DeviceManager_KEY_EventLogLocallyOnly);

	Log(LOG_VERBOSE, " Registering Methods.." );
	MethodMessageResponse* mmListDevices = Methods.create<MethodMessageResponse>( _METHOD_List, "List Devices", Permission(Permission::RX) );
	mmListDevices->setMethod( this, (MethodMessageResponsePtr)&DeviceManager::mmDoListDevices );
	MethodMessageResponse* mmListDevicesByType = Methods.create<MethodMessageResponse>( _DeviceManager_METHOD_GetListByType, "List Devices of Type", Permission(Permission::RX) );
	mmListDevicesByType->setMethod( this, (MethodMessageResponsePtr)&DeviceManager::mmDoListDevicesByType );

	MethodRequest* mmRegisterDevice = Methods.create<MethodRequest>( _DeviceManager_METHOD_register, "register Device", Permission::R_XR__R__() );
	mmRegisterDevice->setMethod( this, (MethodRequestPtr)&DeviceManager::mmDoRegisterDevice );
	MethodRequest* mmUnRegisterDevice = Methods.create<MethodRequest>( _DeviceManager_METHOD_unregister, "unregister Device", Permission::R_XR__R__() );
	mmUnRegisterDevice->setMethod( this, (MethodRequestPtr)&DeviceManager::mmDoUnRegisterDevice );

	MethodRequest* mmFakeDeviceEvent = Methods.create<MethodRequest>( _DeviceManager_METHOD_fake_event, "fake Device event", Permission::R_XR__R__() );
	mmFakeDeviceEvent->setMethod( this, (MethodRequestPtr)&DeviceManager::mmDoFakeDeviceEvent );

	MethodRequest* mmEventsLog = Methods.create<MethodRequest>( _DeviceManager_METHOD_eventslog, "list Devices' events", Permission::R_XR__R__() );
	mmEventsLog->setMethod( this, (MethodRequestPtr)&DeviceManager::mmDoEventsLog );
	MethodWRESULT* mmClearLog = Methods.create<MethodWRESULT>( _DeviceManager_METHOD_clear_log, "clear events log", Permission::R_XR__R__() );
	mmClearLog->setMethod( this, (MethodWRESULTPtr)&DeviceManager::clearEventsLog );

 }

DeviceManager::~DeviceManager() {
	Log(LOG_VERBOSE, ":~DeviceManager() Destroying.." );

	if ( this->devices.count() != 0 )
		Log(LOG_VERBOSE, ":~DeviceManager() : Freeing Devices [%d]..", this->devices.count() );
	this->devices.clear();
	if ( this->eventsLog.count() != 0 )
		Log(LOG_VERBOSE, ":~DeviceManager() : Freeing EventsLog [%d]..", this->eventsLog.count() );
	this->eventsLog.clear();

	Log(LOG_INFO, " Destroyed." );
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////// KERNEL EVENTS

WRESULT DeviceManager::onKernelInitializing() {
	WRESULT ret = WoshModule::onKernelInitializing();

	Log(LOG_VERBOSE, " Initializing BusRing (wosh.Bus.Devices).." );
	Bus* devicesBus = new BusRing(_Bus_Devices);
	devicesBus->setParentObject(this);
	devicesBus->setSharedLocal(true);
	devicesBus->setSharedNetwork(true);
	WRESULT created_bus = WoshKernel::getInstance()->busses().registerBus(devicesBus);
	if ( WFAILED(created_bus) )
		Log(LOG_CRITICAL, " FAILED#%d Creating BusRing (wosh.Bus.Devices)", created_bus );
	else
		Log(LOG_INFO, " Created BusRing (wosh.Bus.Security)." );

	BusDevices.setMessageHandler(this);
	WRESULT bus_d_connected = BusDevices.connect( _Bus_Devices );
	if ( WFAILED(bus_d_connected) )
		SystemError::raise(this, Log, SystemError::CRITICAL, 0, "init", "FAILED#%d Connecting Bus (wosh.Bus.Devices)..", bus_d_connected );
	else
		Log(LOG_INFO, " Created BusRing (wosh.Bus.Devices)." );

	// setup DSM
	this->dsmDevices.setName(_DeviceManager_DSM_NAME);
	this->dsmDevices.setOwnerObject(this);
	//this->dsmDevices.setParentObject(this);
	WRESULT dsm_init = this->dsmDevices.init();
	if ( WFAILED(dsm_init) )
		SystemError::raise(this, Log, SystemError::CRITICAL, 0, "init", "FAILED#%d Initializing DSM ("_DeviceManager_DSM_NAME")..", dsm_init );
	else
		Log(LOG_INFO, " Initialized DSM ("_DeviceManager_DSM_NAME")" );

	return ret + created_bus + bus_d_connected;
 }

/////////////////////////////////////////////////////////////////////////////////////////////// KERNEL EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENTs

bool DeviceManager::updatingProperty( bool& do_update, const Variant& value_proposed, Property& property_current, const PropertiesProvider* source )
 {
	if ( property_current.getKey() == _DeviceManager_KEY_EventLogMaxSize ) {
		this->eventsLog.transactionBeginWrite();
		this->eventLogMaxSize = value_proposed.toLong(this->eventLogMaxSize);
		this->eventsLog.transactionEnd();
		return true;
	 }
	else if ( property_current.getKey() == _DeviceManager_KEY_EventLogLocallyOnly ) {
		this->eventsLog.transactionBeginWrite();
		this->eventLogLocallyOnly = value_proposed.toBoolean(this->eventLogLocallyOnly);
		this->eventsLog.transactionEnd();
		return true;
	 }
	// forward processing to base class
	return WoshModule::updatingProperty(do_update, value_proposed, property_current, source);
 }

///////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENTs
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////// COLLECTOR

bool DeviceManager::exists( const string& deviceName ) const {
	MutexLockerRead mL(this->devices.getMutex());
	return this->devices.exists(deviceName);
 }

WRESULT DeviceManager::registerDevice( Device* device ) {
	MutexLockerWrite mL(this->devices.getMutex());
	return registerDevice_(device, true);
 }

WRESULT DeviceManager::registerDevice_( Device* device, bool raise_event ) {
	if ( device == NULL ) return WRET_ERR_PARAM;
	if ( device->getName().size() <= 3 ) {
		Log(LOG_CRITICAL, ":registerDevice() : FAILED Registering '%s' [Empty name!]", device->getName().c_str() );
		return WRET_ERR_PARAM;
	 }
	// fix device uri
	if ( device->getURI().isGlobal() )
		device->getURI().unRegisterGlobal();
	device->getURI().setKernelLocal();
	device->getURI().setParent( this->getURI() );
	device->getURI().setName( device->getName() );
	device->getURI().registerGlobal();
	Fact* fact = NULL;
	if ( this->devices.exists(device->getName()) ) {
		Log(LOG_WARNING, ":registerDevice() : Updated '%s'", device->getName().c_str() );
		this->devices.remove(device->getName(), true);
		this->devices.set(device->getName(), device);
		if ( !raise_event )
			return WRET_OK;
		fact = new Fact( _DeviceManager_EVENT_DeviceUpdated, new Variant(device->getName()) );
	 }
	else {
		Log(LOG_INFO, ":registerDevice() : New Device %s", device->getName().c_str() );
		this->devices.set(device->getName(), device);
		if ( !raise_event )
			return WRET_OK;
		fact = new Fact( _DeviceManager_EVENT_DeviceAdded, new Variant(device->getName()) );
	 }
	Message* msg_event = new Message(fact);
	msg_event->setSource( this );
	msg_event->getDestination().setAny();
	BusCore.postMessage(msg_event);
	return WRET_OK;
 }

WRESULT DeviceManager::registerDevices( vector<Device*>& new_devices ) {
	if ( new_devices.empty() ) {
		Log(LOG_WARNING, ":registerDevices() : No device found" );
		return WRET_ERR_PARAM;
	 }
	WRESULT ret = WRET_OK;
	NEW_List* dList = new NEW_List(new_devices.size());
	std::vector<Device*>::iterator it;
	for ( it=new_devices.begin(); it!=new_devices.end(); ++it ) {
		if ( *it == NULL ) continue;
		ret = registerDevice_( *it, false );
		if ( WFAILED(ret) ) continue;
		dList->push_back<Variant>( (*it)->getName() );
	 }
	if ( dList->isEmpty() ) {
		delete dList;
		return WRET_ERR_PARAM;
	 }
	Log(LOG_INFO, ":registerDevices() : %d device registered/updated (of %d)", dList->size(), new_devices.size() );
	Fact* fact = new Fact( _DeviceManager_EVENT_DeviceAdded, dList );
	Message* msg_event = new Message(fact);
	msg_event->setSource( this );
	msg_event->getDestination().setAny();
	BusCore.postMessage(msg_event);
	return ret;
 }

WRESULT DeviceManager::unRegisterDevice( const std::string& devName ) {
	this->devices.transactionBeginWrite();
	if ( !this->devices.exists(devName) ) {
		this->devices.transactionEnd();
		Log(LOG_CRITICAL, ":unRegisterDevice() : FAILED UnRegistering %s [NOT Exists!]", devName.c_str() );
		return WRET_ERR_PARAM;
	 }
	Log(LOG_INFO, ":unRegisterDevice() : %s", devName.c_str() );
	WRESULT ret = this->devices.erase(devName, true);
	this->devices.transactionEnd();
	if ( WFAILED(ret) ) {
		Log(LOG_CRITICAL, ":unRegisterDevice(%s) : FAILED#%d UnRegistering ", devName.c_str(), ret );
		return ret;
	 }
	Message* msg_event = new Message( new Fact( _DeviceManager_EVENT_DeviceRemoved, new Variant(devName) ) );
	msg_event->setSource( this );
	msg_event->getDestination().setAny();
	BusCore.postMessage(msg_event);
	return WRET_OK;
 }

WRESULT DeviceManager::unRegisterDevicesOf( const std::string& ownerName ) {
	Log(LOG_VERBOSE, ":unRegisterDevicesOf(%s) : Looping..", ownerName.c_str() );
	this->devices.transactionBeginWrite();
	NEW_List* dList = new NEW_List();
	tStrDeviceMap::Iterator it;
	const tStrDeviceMap::ConstIterator it_end = this->devices.end();
	for ( it=this->devices.begin(); it!=it_end;) {
		if ( it->second == NULL ) { ++it; continue; }
		dList->push_back<Variant>( it->first );
		delete it->second; it->second = NULL;
		this->devices.getMap().erase(it);
	 }
	this->devices.transactionEnd();
	if ( dList->isEmpty() ) {
		Log(LOG_WARNING, ":unRegisterDevicesOf(%s) : No devices found", ownerName.c_str() );
		return WRET_ERR_PARAM;
	 }
	Log(LOG_INFO, ":unRegisterDevicesOf(%s) : %d devices removed", ownerName.c_str(), dList->size() );
	Message* msg_event = new Message( new Fact( _DeviceManager_EVENT_DeviceRemoved, dList ) );
	msg_event->setSource( this );
	msg_event->getDestination().setAny();
	BusCore.postMessage(msg_event);
	return WRET_OK;
 }

/////////////////////////////////////////////////////////////////////////////////////////////////// COLLECTOR
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT DeviceManager::getInterfacesOf( const std::string& devName, InterfacesProvider* interfaces ) {
	MutexLockerRead mL(this->devices.getMutex());
	Device* device = this->devices.find(devName);
	if ( device == NULL ) {
		Log(LOG_WARNING, ":getInterfacesOf(%s) Device not found.", devName.c_str() );
		return WRET_ERR_PARAM;
	 }
	Log(LOG_VERBOSE, ":getInterfacesOf(%s) [%d]", devName.c_str(), device->getInterfaces().count() );
	*interfaces = device->getInterfaces(); // copy
	return WRET_OK;
 }

/////////////////////////////////////////////////////////////////////////////////////////////////// COLLECTOR
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////// BUS EVENTS

void DeviceManager::busMessage( const Message& message, const Bus* source ) {
	if ( message.isEmpty() ) return;
	if ( message.getContent()->isFact() && message.getSourceBus() == _Bus_Devices ) {
		logEvent(message);
		return;
	 }
	if ( !message.getContent()->isRequest() ) return;
	// avoid message replica
	if ( !MessageFilter::filterReplica( message, &BusCore ) )
		return;
	WoshModule::busMessage( message, source );
 }

////////////////////////////////////////////////////////////////////////////////////////////////// BUS EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void DeviceManager::logEvent( const Message& message ) {
	if ( this->eventLogMaxSize == 0 ) return;
	if ( this->eventLogLocallyOnly  && !message.getSource().isKernelEmptyOrLocal() ) return;
	this->eventsLog.transactionBeginWrite();
	if ( this->eventsLog.size() >= this->eventLogMaxSize ) {
		this->eventsLog.erase( this->eventsLog.begin(), true );
	 }
	this->eventsLog.push_back( message.clone() );
	this->eventsLog.transactionEnd();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS

WRESULT DeviceManager::clearEventsLog() {
	Log(LOG_VERBOSE, "clearEventsLog()" );
	this->eventsLog.clear();
	return WRET_OK;
 }

Response* DeviceManager::mmDoRegisterDevice( const Request* request ) {
	if ( request == NULL ) return NULL;
	if ( !request->hasData() )
		return request->replyResponse(WRET_ERR_PARAM, "missing argument");
	WRESULT ret = WRET_ERR_INTERNAL;
	if ( request->getData()->isClass(_Device_TYPE) ) {
		const Device* device = request->getData()->as<Device>();
		if ( device == NULL )
			return request->replyResponse(WRET_ERR_PARAM, "invalid device argument");
		ret = registerDevice( device->clone() );
	 }
	if ( request->getData()->isClass(_NEW_List_TYPE) ) {
		const NEW_List* devices = request->getData()->as<NEW_List>();
		if ( devices == NULL )
			return request->replyResponse(WRET_ERR_PARAM, "invalid device argument");
		NEW_List* devices_cp = devices->clone();
		vector<Device*> dv = devices_cp->getVector<Device>();
		ret = registerDevices( dv );
		devices_cp->getVector().clear();
		delete devices_cp;
	 }
	return request->replyResponse(ret);
 }

Response* DeviceManager::mmDoUnRegisterDevice( const Request* request ) {
	if ( request == NULL ) return NULL;
	Variant dDeviceName = request->getArgument();
	if ( dDeviceName.isStringEmpty() )
		return request->replyResponse(WRET_ERR_PARAM, "invalid name");
	WRESULT ret = unRegisterDevice( dDeviceName.asString() );
	return request->replyResponse(ret);
 }

Response* DeviceManager::mmDoFakeDeviceEvent( const Request* request ) {
	if ( request == NULL ) return NULL;
	Variant dDeviceName;
	Variant dEventName;
	Variant dEventData;
	request->extractArguments(dDeviceName, dEventName, dEventData);
	if ( dDeviceName.isStringEmpty() || dEventName.isStringEmpty() ) {
		return request->replyResponse(WRET_ERR_PARAM, "Invalid arguments [device-name, event-name]");
	 }
	Log(LOG_VERBOSE, "mmDoFakeDeviceEvent(%s) Fact: %s [%s]", dDeviceName.asString().c_str(), dEventName.asString().c_str(), dEventData.toString().c_str() );

	Message* msg_event = new Message();
	msg_event->setSource( this );
	msg_event->getSource().setParent(this->getURI());
	msg_event->getSource().setName( dDeviceName.asString() );
	msg_event->setContent( new Fact(dEventName.asString(), new Variant(dEventData)) );
	msg_event->setDestinationBroadcast();
	BusDevices.postMessage(msg_event);
	return request->replyResponse(WRET_OK, "Device-Event (message) sent!" );
 }

Response* DeviceManager::mmDoListDevices( const Message* requestMessage ) {
	if ( requestMessage == NULL || requestMessage->isEmpty() ) return NULL;
	if ( !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, ":mmDoListDevices(%s)", option.c_str() );

	if ( option == "s" || option == "short" ) {
		this->devices.transactionBeginRead();
		NEW_List* objs = new NEW_List(this->devices.size());
		tStrDeviceMap::ConstIterator it;
		const tStrDeviceMap::ConstIterator it_end = this->devices.end();
		for ( it=this->devices.begin(); it!=it_end; ++it ) {
			if ( it->second == NULL ) continue;
			objs->push_back( it->second->getURI().clone() );
		}
		this->devices.transactionEnd();
		return request->replyResponse(WRET_OK, objs);
	 }
	else {
		this->devices.transactionBeginRead();
		Table* tbl = new Table( this->devices.size(), 5, true );
		tbl->setTableName("Devices");
		tbl->setHeaderHorizontal("URI", 0);
		tbl->setHeaderHorizontal("Type", 1);
		tbl->setHeaderHorizontal("BundleOwner", 2);
		tbl->setHeaderHorizontal("Perms", 3);
		tbl->setHeaderHorizontal("Interfaces", 4);
		unsigned int iRow = 0;
		tStrDeviceMap::ConstIterator it;
		const tStrDeviceMap::ConstIterator it_end = this->devices.end();
		for ( it=this->devices.begin(); it!=it_end; ++it, ++iRow ) {
			if ( it->second == NULL ) continue;
			const Device* cDev = it->second;
			tbl->set( cDev->getURI().clone(), iRow, 0);
			tbl->set<Variant>( cDev->getClassName(), iRow, 1);
			tbl->set<Variant>( cDev->getBundleOwner(), iRow, 2);
			tbl->set( cDev->getPermission().clone(), iRow, 3);
			tbl->set<Variant>( cDev->getInterfaces().toString(), iRow, 4);
		}
		this->devices.transactionEnd();
		return request->replyResponse(WRET_OK, tbl);
	 }
 }

Response* DeviceManager::mmDoListDevicesByType( const Message* requestMessage ) {
	if ( requestMessage == NULL || requestMessage->isEmpty() ) return NULL;
	if ( !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();
	string option;
	string device_type = request->getArgument().toString();
	Log(LOG_VERBOSE, ":mmDoListDevicesByType(%s)", option.c_str() );

	if ( option == "s" || option == "short" ) {
		this->devices.transactionBeginRead();
		NEW_List* objs = new NEW_List(this->devices.size());
		tStrDeviceMap::ConstIterator it;
		const tStrDeviceMap::ConstIterator it_end = this->devices.end();
		for ( it=this->devices.begin(); it!=it_end; ++it ) {
			if ( it->second == NULL ) continue;
			if ( !it->second->getInterfaces().exists(device_type) ) continue;
			objs->append( it->second->getURI().clone() );
		}
		this->devices.transactionEnd();
		return request->replyResponse(WRET_OK, objs);
	 }
	else {
		this->devices.transactionBeginRead();
		Table* tbl = new Table( this->devices.size(), 5, true );
		tbl->setTableName("Devices");
		tbl->setHeaderHorizontal("URI", 0);
		tbl->setHeaderHorizontal("Type", 1);
		tbl->setHeaderHorizontal("BundleOwner", 2);
		tbl->setHeaderHorizontal("Perms", 3);
		tbl->setHeaderHorizontal("Interfaces", 4);
		unsigned int iRow = 0;
		tStrDeviceMap::ConstIterator it;
		const tStrDeviceMap::ConstIterator it_end = this->devices.end();
		for ( it=this->devices.begin(); it!=it_end; ++it, ++iRow ) {
			if ( it->second == NULL ) continue;
			if ( !it->second->getInterfaces().exists(device_type) ) continue;
			const Device* cDev = it->second;
			tbl->set( cDev->getURI().clone(), iRow, 0);
			tbl->set<Variant>( cDev->getClassName(), iRow, 1);
			tbl->set<Variant>( cDev->getBundleOwner(), iRow, 2);
			tbl->set( cDev->getPermission().clone(), iRow, 3);
			tbl->set( cDev->getInterfaces().toList(), iRow, 4);
		}
		this->devices.transactionEnd();
		return request->replyResponse(WRET_OK, tbl);
	 }
 }

Response* DeviceManager::mmDoEventsLog( const Request* request ) {
	if ( request == NULL ) return NULL;
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, ":mmDoEventsLog(%s)", option.c_str() );

	if ( option == "s" || option == "short" ) {
		this->eventsLog.transactionBeginRead();
		NEW_List* objs = new NEW_List(this->eventsLog.size());
		tMessageList::ConstIterator it;
		const tMessageList::ConstIterator it_end = this->eventsLog.end();
		for ( it=this->eventsLog.begin(); it!=it_end; ++it ) {
			if ( *it == NULL ) continue;
			if ( (*it)->isEmpty() ) continue;
			if ( !(*it)->getContent()->isFact() ) continue;
			objs->push_back<Variant>( (*it)->getContent()->asFact()->getEvent() );
		}
		this->eventsLog.transactionEnd();
		return request->replyResponse(WRET_OK, objs);
	 }
	else {
		this->eventsLog.transactionBeginRead();
		Table* tbl = new Table( this->eventsLog.size(), 4, true );
		tbl->setTableName("EventLog");
		tbl->setHeaderHorizontal("TimeStamp", 0);
		tbl->setHeaderHorizontal("URI", 1);
		tbl->setHeaderHorizontal("Event", 2);
		tbl->setHeaderHorizontal("Data", 3);
		unsigned int iRow = 0;
		tMessageList::ConstIterator it;
		const tMessageList::ConstIterator it_end = this->eventsLog.end();
		for ( it=this->eventsLog.begin(); it!=it_end; ++it, ++iRow ) {
			if ( *it == NULL ) continue;
			if ( (*it)->isEmpty() ) continue;
			const Fact* fact = (*it)->getContent()->asFact();
			tbl->set( new Variant((*it)->getTimestamp(), Variant::DATETIME), iRow, 0);
			tbl->set( (*it)->getSource().clone(), iRow, 1);
			tbl->set<Variant>( fact->getEvent(), iRow, 2);
			if ( fact->hasData() )
				tbl->set( fact->getData()->clone(), iRow, 3);
			else
				tbl->set<Variant>( "NULL", iRow, 3);

		}
		this->eventsLog.transactionEnd();
		return request->replyResponse(WRET_OK, tbl);
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void DeviceManager::busMessageForBundleDevices( const Message& message, const Bus*, Bundle* bundle, MethodsProvider& Methods, LogContext& Log, BusConnector& BusCore, BusConnector& BusDevices )
 {
	if ( message.isEmpty() || !message.getContent()->isRequest() ) return;
	if ( message.getSourceUUID() == bundle->getEntityID() ) return; // avoid loopback

	// as Device, we listen on two busses (Core and Devices) with different URI
	// a custom filtering implementation is required:

	// avoid message replicas
	if ( !MessageFilter::filterReplica( message, &BusCore ) )
		return;

	if ( !URI::pregMatchKernel( message.getDestination(), bundle->getURI() ) )
		return;

	if ( message.getSourceBus() == BusDevices.getBusName() ) {
		// my URI in wosh.Bus.Devices is JUST MY NAME!
		if ( !message.getDestination().getPath().empty() ) {
			if ( !URI::pregMatchStep( message.getDestination().getPath().at(0), "Devices" ) )
				return;
		 }
	 }

	if ( message.getSourceBus() == BusCore.getBusName() ) {
		// my URIS in wosh.Bus.Core are Devices/<NAME> and Bundles/<NAME>
		if ( message.getDestination().getPath().size() != 1 )
			return;
		if ( !URI::pregMatchStep( message.getDestination().getPath().at(0), "Bundles" ) &&
			!URI::pregMatchStep( message.getDestination().getPath().at(0), "Devices" ) )
			return;
	 }

	if ( !URI::pregMatchStep(message.getDestination().getName(), bundle->getURI().getName() ) )
		return;

	if ( message.getDestination().hasExpression() ) {
		bool valid = MessageFilter::validateExpression( message.getDestination().getExpression(), bundle->bundleProperties(), bundle->bundleInterfaces(), message.getSecurityToken() );
		if ( !valid )
			return;
	 }

	Message* reply = MPC_busMessageRequest( message, Methods, Log );
	if ( reply == NULL )
		return;
	reply->setSource( bundle );

	if ( message.getSourceBus() == BusDevices.getBusName() ) {
		reply->getSource().setPath( "Devices" ); // override source URI cos we are injecting in Device bus!
		reply->getSource().setName( bundle->getName() );
		BusDevices.postMessage(reply);
	 }
	else { //if ( message.getSourceBus() == BusCore.getBusName() ) {
		BusCore.postMessage(reply);
	 }
 }

}; // namespace wosh
