/**
 * @class   wosh::DeviceManager
 * @brief   Core module managing devices (wosh::Device) of local host.
 *
 * \image html diagrams/DeviceManager.png "wosh::DeviceManager UML diagram"
 *
 *
 * @ingroup CoreModules
 ****************************************************************************
 * @version 0.8.499 $Id: DeviceManager.h 2225 2010-03-26 21:50:21Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/*! \page page_core
 * \htmlonly <hr/> \endhtmlonly
 * \section page_core_devices Devices
 *
 * wosh::DeviceManager is a distributed component hosting information about
 * devices available to the WOSH system. Information are hold by wosh::Device
 * object.
 *
 * [..]
 *
 *
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_DeviceManager_H__
 #define __WOSH_Core_DeviceManager_H__

 #include <woshDefs.h>
 #include <core/WoshModule.h>
 #include <core/Containers.h>
 #include <core/SharedMemory.h>
 #include <core/Device.h>


namespace wosh {

 #define _DeviceManager_NAME							"DeviceManager"
 #define _DeviceManager_TYPE							"wosh::DeviceManager"
 #define _DeviceManager_URI								":Devices"

 #define _DeviceManager_DSM_NAME						"Core_Devices"

 #define _DeviceManager_KEY_Devices						"Devices"
 #define _DeviceManager_KEY_EventLogMaxSize				"EventLogMaxSize"
 #define _DeviceManager_KEY_EventLogLocallyOnly			"EventLogLocallyOnly"

 #define _DeviceManager_DEFAULT_EventLogMaxSize			400


 #define _DeviceManager_METHOD_ListDevices				"list"
 #define _DeviceManager_METHOD_LoadDevice				"load"
 #define _DeviceManager_METHOD_ListSupportedDevices		"list_supported"

 #define _DeviceManager_METHOD_register					"register"
 #define _DeviceManager_METHOD_unregister				"unregister"

 #define _DeviceManager_METHOD_fake_event				"fake_event"
 #define _DeviceManager_METHOD_getdevice				"getdevice"
 #define _DeviceManager_METHOD_GetListByType			"list_by_type"

 #define _DeviceManager_METHOD_eventslog				"eventslog"
 #define _DeviceManager_METHOD_clear_log				"clear_log"

 #define _DeviceManager_EVENT_DeviceAdded				"device_added"
 #define _DeviceManager_EVENT_DeviceUpdated				"device_updated"
 #define _DeviceManager_EVENT_DeviceRemoved				"device_removed"


class Bundle;
class WoshKernel;

class DeviceManager : public WoshModule
 {
	WOSH_MODULE(wosh::DeviceManager)

	typedef Map22<std::string, wosh::Device*> tStrDeviceMap;
	typedef List22<wosh::Message*> tMessageList;


	protected:
		/**
		 * @brief  Default constructor. Call base constructor and initialize some properties and hosted methods.
		 * @note   Called by WoshKernel only. Static getter (gamma singleton) will return NULL until kernel is created.
		 */
		DeviceManager( const WoshKernel* parent = NULL );

		/**
		 * @brief  Call base class implementation and create the Device Bus (wosh.Bus.Devices).
		 *         DSM is initialized.
		 * @return result of WoshModule::onKernelInitializing() + bus-creation result
		 */
		WRESULT onKernelInitializing();

		friend class wosh::WoshKernel;

	public:
		/**
		 * @brief  Destructor. Devices and Events dataset are destroyed.
		 */
		virtual ~DeviceManager();

/** @name Public methods (bundles)
 * @{
 ******************************************************************************/
	public:
		/**
		 * @brief  Register a Device. If a Device with same name exists, it is updated. Reentrant.
		 * @param device [in] the device object to be registered, ownership is acquired
		 * @return WRET_ERR_PARAM if argument is NULL or its name is empty; WRET_OK else
		 * @see    registerDevice_()
		 */
		WRESULT registerDevice( Device* device );
		/**
		 * @brief  Register a set of Devices. Reentrant.
		 * @param devices [in] the device objects to be registered, ownership is acquired
		 * @return WRET_ERR_PARAM if no devices were registered; WRET_OK else
		 * @see    registerDevice()
		 */
		WRESULT registerDevices( std::vector<Device*>& devices );
		/**
		 * @brief  UnRegister a Device. Reentrant.
		 * @param devName [in] the name of the device to be removed
		 * @return WRET_ERR_PARAM if device was not found or name is empty; WRET_OK else
		 * @see    registerDevice_()
		 */
		WRESULT unRegisterDevice( const std::string& devName );
		/**
		 * @brief  UnRegister all Device owned by argument. Reentrant.
		 * @paramownerName [in] the name of the devices' owner, all device of this owner will be removed
		 * @return WRET_ERR_PARAM if no devices were found or name is empty; WRET_OK else
		 */
		WRESULT unRegisterDevicesOf( const std::string& ownerName );

		/**
		 * @brief Evaluate if a Device exists. Reentrant.
		 * @param device [in] the name of the device to be checked
		 * @return true if a Device with same name exists; false else
		 */
		bool exists( const std::string& deviceName ) const;
		/**
		 * @brief Get the Interface offered (supported) by a Device. Reentrant.
		 * @param interfaces [in] the name of the device to be evaluated
		 * @param interfaces [out] the interface implemented by the device
		 * @return WRET_ERR_PARAM if device was not found or name is empty; WRET_OK else
		 */
		WRESULT getInterfacesOf( const std::string& deviceName, InterfacesProvider* interfaces );

//@}
/** @name Public methods (loaders)
 * @{
 ******************************************************************************/
	public:
		///@todo TO BE REDESIGNED
		static void busMessageForBundleDevices( const Message& message, const Bus* source, Bundle* bundle, MethodsProvider& Methods, LogContext& Log, BusConnector& BusCore, BusConnector& BusDevices );

//@}
/** @name Message Events (incoming message)
 * @{
 ******************************************************************************/
	public:
		void busMessage( const Message& message, const Bus* source );
//@}
/** @name IPropertiesProviderListener interface
 * @{
 ******************************************************************************/
 	protected:
		bool updatingProperty( bool& do_update, const Variant& value_proposed, Property& property_current, const PropertiesProvider* source );
		friend class wosh::PropertiesProvider;
//@}
/** @name Internal Mehods and Events management
 * @{
 ******************************************************************************/
	protected:
		friend class wosh::Device;

	private:
		/**
		 * @brief  Access the SystemMonitor singleton. Gamma Pattern.
		 *         Static reference is allocated and detroyed by WoshKernel on constructor/destructor respectively
		 * @return pointer to DeviceManager singleton. May be NULL (if kernel has not been allocated yet)
		 */
		static DeviceManager* getInstance();

		/**
		 * @brief  Append a device-event to event-log cache. Reentrant.
		 * @param message [in] the message representing the devic-event (from Device bus, containing wosh::Fact)
		 * @see    eventsLog
		 */
		void logEvent( const Message& message );
		/**
		 * @brief  Register a Device. If a Device with same name exists, it is updated. Not Reentrant.
		 * @param device [in] the device object to be registered, ownership is acquired
		 * @param raise_event [in] when true, it will broadcast a Fact on wosh.Bus.Core
		 * @return WRET_ERR_PARAM if argument is NULL or its name is empty; WRET_OK else
		 * @see    registerDevice()
		 */
		WRESULT registerDevice_( Device* device, bool raise_event );
//@}
	private:
		Response* mmDoListDevices( const Message* requestMessage );
		Response* mmDoListDevicesByType( const Message* requestMessage );
		Response* mmDoRegisterDevice( const Request* request );
		Response* mmDoUnRegisterDevice( const Request* request );
		Response* mmDoFakeDeviceEvent( const Request* request );
		Response* mmDoEventsLog( const Request* request );
		WRESULT clearEventsLog();

	private:
		BusConnector BusDevices;	///< Bus of Devices, network-shared, created by myself on kernel-init

		SharedMemory dsmDevices;	///< shared memory agent for the Devices dataset
		tStrDeviceMap devices;		///< map of pairs [device-name, Device*] (Device* holds deviceInfo)

		tMessageList eventsLog;		///< cache of last events (Message.Fact) monitored on Device bus.
		long eventLogMaxSize;		///< size of the EventLog cache. 0 will disable logging
		bool eventLogLocallyOnly;	///< when enabled (default), filter only local events


}; // class def

}; // namespace wosh

#endif //__WOSH_Core_DeviceManager_H__
