/**
 * @class   wosh::Fact
 * @brief   Facts are the way any Event is broadcasted within WOSH system.
 *          They can be raised from Devices or Services, hardware or software.
 *          Usually it's Automation(manager) listening for Fact and generating
 *          effects, but also some GUI applications may listen them as notifiers.
 *          
 *          Some examples of facts:
 *           - Device turned on/off
 *
 *
 ****************************************************************************
 * @version $Id: Fact.h 2084 2010-03-11 10:49:02Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_Fact_H__
 #define __WOSH_Core_Fact_H__

 #include <core/MessageContentGeneric.h>

 
namespace wosh {

 #define _Fact_TYPE					"wosh::Fact"


class Fact : public MessageContentGeneric
 {
	WOSH_SERIALIZABLE(wosh::Fact)
	WOSH_COMPARE_RTTI(wosh::Fact)


	public:
		/**
		 * @brief  Empty constructor.
		 */
		Fact();
		/**
		 * @brief  Constructor with initializer(s), provided for convenience.
		 * @param [in] eventname the event string to be assigned
		 */
		explicit Fact( const std::string& eventname );
		/**
		 * @brief  Constructor with initializer(s), provided for convenience.
		 * @param [in] eventname the event string to be assigned
		 * @param [in] data the data object (wosh::ISerializable abstraction) associated with the event. May be NULL.
		 */
		explicit Fact( const std::string& eventname, wosh::ISerializable* data );
		/**
		 * @brief  Copy constructor.
		 */
		Fact( const wosh::Fact& m );
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~Fact() { }


/** @name Getters
 * @{
 ******************************************************************************/
	public:
		/**
		 * @brief  Get the event (string) associated with the event
		 * @return constant STL string reference (to internal field)
		 */
		inline const std::string& getEvent() const						{ return this->contentContext; }

		/**
		 * @brief  Check if Fact contains any Data object
		 * @return true when data object is set (not NULL), false else.
		 */
		inline bool hasData() const 									{ return (this->content != NULL); }

		/**
		 * @brief  Get the data object (wosh::ISerializable abstraction) associated with the event.
		 * @return constant pointer to the data object. May be NULL.
		 */
		inline const wosh::ISerializable* getData() const				{ return this->content; }
		/**
		 * @brief  Get the data object (wosh::ISerializable abstraction) associated with the event.
		 * @return pointer to the data object (editable). May be NULL.
		 */
		inline wosh::ISerializable* getData()							{ return this->content; }

		/**
		 * @brief  Get MessageContent::TYPE of this implementation: a FACT
		 * @return MessageContent::FACT
		 */
		virtual MessageContent::TYPE getType() const					{ return MessageContent::FACT; }

//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:
		/**
		 * @brief  Set the event (string) associated with the event
		 * @param [in] value the event string to be assigned
		 * @see    setContentContext()
		 */
		inline void setEvent( const std::string& value ) 				{ this->contentContext = value; }

		/**
		 * @brief  Set the data object (wosh::ISerializable abstraction) associated with the event.
		 * @param [in] data the data object associated with the event. May be NULL.
		 * @see    setContent()
		 */
		inline void setData( wosh::ISerializable* data )				{ setContent(data); }

//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:
		virtual wosh::Fact& operator=(const wosh::Fact& m);
		virtual bool operator==(const wosh::Fact &other) const;
		virtual bool operator!=(const wosh::Fact &other) const			{ return !(*this == other);  }
//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:

//@}


}; // class def

}; // namespace wosh

#endif //__WOSH_Core_Fact_H__
