/**
 * @class   wosh::FileSystem
 * @brief   
 * 
 * \image html diagrams/FileSystem.png "wosh::FileSystem UML diagram"
 *
 * manage files and folders as URI?
 * examples:
 * @code
 *  :/FileSystem/
 * 
 * 		WOSH/var/extras/ ..			=> local installation folder (of current bin)
 * 		alias/my_track.mp3			=> alias mapping to another FS URI
 * 		local/$C/My Documents ..	=> local filesystem uri
 * 		shared/music/track1.mp3		=> dynamic mapping ! requires a lower level file-sharing system (such as SAMBA, NFS, ..)
 * 
 * @endcode
 *
 * @todo see brief;
 *       http://www.gnu.org/s/libc/manual/html_node/Simple-Directory-Lister.html
 *
 *
 *
 * @ingroup CoreModules
 ****************************************************************************
 * @version 0.6.030 $Id: FileSystem.h 2200 2010-03-23 16:23:37Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/*! \page page_core
 * \htmlonly <hr/> \endhtmlonly
 * \section page_core_filesystem FileSystem
 *
 * [..]
 *
 * wosh::FileSystem
 *
 * wosh::FilePath
 *
 * wosh::ResourcePath
 *
 *
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR UserS;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_FileSystem_H__
 #define __WOSH_Core_FileSystem_H__

 #include <woshDefs.h>
 #include <core/WoshModule.h>
 #include <core/FilePath.h>
 #include <core/Containers.h>
 #include <iostream>


namespace wosh {

 #define _FileSystem_NAME					"FileSystem"
 #define _FileSystem_TYPE					"wosh::FileSystem"
 #define _FileSystem_URI					":FileSystem"

 #define _FileSystem_METHOD_set_alias		"set_alias"

 #define _FileSystem_KEY_WoshRoot			"WoshRoot"

 #define _FileSystem_KEY_Aliases			"Aliases"

 #define _FileSystem_KEY_LocalFolderTemp	"LocalFolderTemp"
 #define _FileSystem_KEY_LocalFolderRoot	"LocalFolderRoot"



class FileSystem : public WoshModule
 {
	WOSH_MODULE(wosh::FileSystem)

	typedef Map22<std::string, std::string> tStrStrMap;

	protected:
		FileSystem( const WoshKernel* parent = NULL );

		WRESULT onKernelInitializing();

		friend class wosh::WoshKernel;

	public:
		virtual ~FileSystem();

//@}
/** @name Public methods (not for bundles, usually hidden to most classes)
 * @{
 ******************************************************************************/
	public:
		virtual WRESULT setWoshRoot(  const std::string& root_path );

//@}
/** @name FileSystem core methods
 * @{
 ******************************************************************************/
	public:

		static WRESULT registerAlias( const std::string& alias, const std::string& full_path );
		static WRESULT unRegisterAlias( const std::string& alias );

		static std::string getAlias( const std::string& alias );

		static bool resolveAlias( std::string& path_step );
		static bool resolveLocalPath( const std::string& local_path, std::string& wosh_path );


		static WRESULT reserveTempFile( const std::string& filename, std::string& full_wosh_path );

		static WRESULT freeTempFile( const std::string& full_wosh_path );


//@}
/** @name Static filesystem methods
 * @{
 ******************************************************************************/
	public:

		static long getTimeStampOf( const std::string& filepath );

		static WRESULT deleteFile( const std::string& filepath );
		static WRESULT renameFile( const std::string& filepath, const std::string& new_filepath );

		static WRESULT saveTextAs( const std::string& filepath, const char* data, long size, bool append = false );

		static WRESULT getFilesOf( const std::string& dir, std::vector<std::string>& files, bool recursive = false, bool full_path = false, const std::string& pattern = "" );


		static std::string getCurrentWorkingDirectory();

//@}
	private:

		static FileSystem* getInstance();

	protected:
		Response* mmDoListAliases( const Message* requestMessage );
		Response* mmDoSetAliases( const Message* requestMessage );

/** @name Internal vars
 * @{
 ******************************************************************************/
	protected:
		std::string woshRootPath;		///< root path of WOSH FileSystem (containing bin, etc, var, ..)
		static tStrStrMap fsAliases;	///< alias->full local path

//@}

}; // class def

}; // namespace wosh

#endif //__WOSH_Core_FileSystem_H__
