/**
 * @class   wosh::InterfacesProvider
 * @brief   Class/interface for Objects which provide wosh::Property, some
 *          helper functions are provided for convenience.
 *
 *
 ****************************************************************************
 * @version $Id: InterfacesProvider.h 2084 2010-03-11 10:49:02Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_InterfacesProvider_H__
 #define __WOSH_Core_InterfacesProvider_H__

 #include <woshDefs.h>
 #include <core/Serialization.h>
 #include <core/Containers.h>


namespace wosh {

 #define _InterfaceInfo_TYPE			"wosh::InterfaceInfo"

class InterfaceInfo
 {
	public:
		/**
		 * @brief  Empty constructor.
		 */
		InterfaceInfo() : classVersion(0.0) { }

		InterfaceInfo( const InterfaceInfo& m ) {
			*this = m;
		 }
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~InterfaceInfo() { }

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:
		const std::string& getClassType() const			{ return this->classType; }
		double getClassVersion() const					{ return this->classVersion; }
//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:
		void setClassType( const std::string& value )	{ this->classType = value; }
		void setClassVersion( double value )			{ this->classVersion = value; }
//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:
		wosh::InterfaceInfo& operator=(const wosh::InterfaceInfo& m) {
			if (this == &m) return *this; // same object?
			this->classType = m.classType;
			this->classVersion = m.classVersion;
			return *this;
		 }
		bool operator==(const wosh::InterfaceInfo &other) const {
			if ( this->classType != other.classType ) return false;
			if ( this->classVersion != other.classVersion ) return false;
			return true;
		 }
		bool operator!=(const wosh::InterfaceInfo &other) const { return !(*this == other);  }
		// operator &

//@}
	protected: // tarrget identification
		std::string classType;		///< 
		double classVersion;		///< 

}; // class def


 #define _InterfacesProvider_TYPE			"wosh::InterfacesProvider"

class NEW_List;
class Table;

class InterfacesProvider
 {
	WOSH_SERIALIZABLE(wosh::InterfacesProvider)
	WOSH_COMPARE_RTTI(wosh::InterfacesProvider)

	typedef Map22<std::string, InterfaceInfo*> tStrInterfaceInfoMap;


	public:
		InterfacesProvider() { }
		InterfacesProvider( const InterfacesProvider& m ) {
			*this = m;
		 }

		virtual ~InterfacesProvider();

	public:
		virtual WRESULT add( InterfaceInfo* interface_info );

		WRESULT add( const std::string& classType, double interfaceVersionMin = 0.0 );


		WRESULT createFor( const IReflection* object );
		WRESULT createFor( const std::string& classname );

		bool exists( const std::string& classname, double min_version = 0.0 ) const;

		inline int count() const		{ return this->interfaces.count(); }

		WRESULT fromString( const std::string& data );


	public:
		NEW_List* toList() const;
		Table* toTable() const;

//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:
		wosh::InterfacesProvider& operator=(const wosh::InterfacesProvider& m);
		bool operator==(const wosh::InterfacesProvider &other) const;
		bool operator!=(const wosh::InterfacesProvider &other) const { return !(*this == other);  }
//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const;
		virtual WRESULT store( IEncoderWriter* serializer ) const;
		virtual WRESULT retrieve( IEncoderReader* deserializer );
//@}
	protected:
		tStrInterfaceInfoMap interfaces;


}; // class def

}; // namespace wosh

#endif //__WOSH_Core_InterfacesProvider_H__
