/**
 * @class   wosh::Interpreter
 * @brief   The class holds information about a generic interpreter service,
 *          defined by wosh::interfaces::services::InterpreterService
 *
 * @see     wosh::services::CommunicatorManagerBundle
 ****************************************************************************
 * @version $Id: Interpreter.h 2036 2010-03-04 16:09:59Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_Interpreter_H__
 #define __WOSH_Core_Interpreter_H__

 #include <woshDefs.h>
 #include <core/Object.h>
 #include <core/SharedObject.h>
 #include <core/InterfacesProvider.h>
 #include <core/Notification.h>
 #include <core/Communicator.h>

namespace wosh {

 #define _Interpreter_TYPE					"wosh::Interpreter"


class Interpreter :		public Object
 {
	WOSH_SERIALIZABLE(wosh::Interpreter)

	public:

		Interpreter() : Object()
		 {
			Object::setName( "Interpreter" );

			this->kernelHost = "";

			this->inputTypes = Notification::TYPE_UNKNOWN;
			this->outputTypes = Notification::TYPE_UNKNOWN;

			this->inputProtocols = Notification::PROTOCOL_UNKNOWN;
			this->outputProtocols = Notification::PROTOCOL_UNKNOWN;

			setSeen();
		 }

		/**
		 * @brief  Copy constructor.
		 */
		Interpreter( const Interpreter& m ) : Object(m) {
			*this = m;
		 }

		virtual ~Interpreter() { }


	public:

		virtual void setName( const std::string& value )
		 {
			Object::setName(value);
			bool glob = Object::getURI().isGlobal();
			if ( glob )
				Object::getURI().unRegisterGlobal();
			Object::getURI().setName( Object::getName() );
			if ( glob )
				Object::getURI().registerGlobal();
		 }

		using Object::setURI;

		void setKernelHost( const std::string& value )
		 {
			this->kernelHost = value;
			Object::getURI().setKernelName(this->kernelHost);
		 }

		void setInputTypes( int value )						{ this->inputTypes = value; }
		void setOutputTypes( int value )					{ this->outputTypes = value; }

		void setInputProtocols( int value )					{ this->inputProtocols = value; }
		void setOutputProtocols( int value )				{ this->outputProtocols = value; }

	public:

		int getInputTypes() const							{ return this->inputTypes; }
		int getOutputTypes() const							{ return this->outputTypes; }

		int getInputProtocols() const						{ return this->inputProtocols; }
		int getOutputProtocols() const						{ return this->outputProtocols; }

		const std::vector<std::string>& getInputLanguages() const	{ return this->inputLanguages; }
		std::vector<std::string>& getInputLanguages()				{ return this->inputLanguages; }

		const std::vector<std::string>& getOutputLanguages() const	{ return this->outputLanguages; }
		std::vector<std::string>& getOutputLanguages()				{ return this->outputLanguages; }

		void setSeen()										{ this->timestamp = Utilities::std_time(); }
		long getSeen() const								{ return this->timestamp; }

		const std::string& getKernelHost() const			{ return this->kernelHost; }
		std::string& getKernelHost()						{ return this->kernelHost; }

		const InterfacesProvider& getInterfaces() const		{ return this->Interfaces; }
		InterfacesProvider& getInterfaces()					{ return this->Interfaces; }

	protected:

		virtual Interpreter& operator=(const Interpreter& m)
		 {
			if (this == &m) return *this; // same object?

			this->timestamp = m.timestamp;

			this->kernelHost = m.kernelHost;
			this->Interfaces = m.Interfaces;

			this->inputTypes = m.inputTypes;
			this->outputTypes = m.outputTypes;

			this->inputProtocols = m.inputProtocols;
			this->outputProtocols = m.outputProtocols;

			this->inputLanguages = m.inputLanguages;
			this->outputLanguages = m.outputLanguages;

			return *this;
		 }

		virtual bool operator==(const Interpreter &other) const
		 {
(void)other;
			return false;
		 }

		virtual bool operator!=(const Interpreter &other) const	{ return !(*this == other);  }


//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const	{ return ""; }
		virtual WRESULT store( IEncoderWriter* serializer ) const {
			if ( serializer == NULL ) return WRET_ERR_PARAM;

			return WRET_OK;
		 }
		virtual WRESULT retrieve( IEncoderReader* deserializer ) {
			if ( deserializer == NULL ) return WRET_ERR_PARAM;

			return WRET_OK;
		 }

//@}
	protected:

		long timestamp;					///< last seen (epoch) of real object
		std::string kernelHost;			///< kernel which hosts the service
		InterfacesProvider Interfaces;	///< implemented interfaces

		int inputTypes;
		int outputTypes;
		
		int inputProtocols;
		int outputProtocols;

		std::vector<std::string> inputLanguages;
		std::vector<std::string> outputLanguages;

}; // class def

}; // namespace wosh

#endif //__WOSH_Core_Interpreter_H__
