/**
 * @class   wosh::LogHandlers
 * @brief   
 *          
 *
 *
 ****************************************************************************
 * @version $Id: LogHandlers.h 1938 2010-02-24 20:18:36Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_LogHandlers_H__
 #define __WOSH_Core_LogHandlers_H__

 #include <woshDefs.h>
 #include <core/Containers.h>
 #include <core/LogContext.h>
 #include <core/Reflection.h>

 #include <stdarg.h>
 #include <stdio.h> //plaintext handler

namespace wosh {
 class SystemLogger;

class LogHandler : public virtual IReflection
 {
	WOSH_CLASSNAME(wosh::LogHandler)
	WOSH_CLASS_ABSTRACT(true)

	public:
		LogHandler();
		virtual ~LogHandler();

		virtual long getID() const	{ return this->id; }

	public:
		virtual WRESULT open() = 0;
		virtual WRESULT close() = 0;

		virtual WRESULT flush() = 0;

		virtual bool isOpen() const = 0;

		virtual std::string getProtocol() const = 0;

		virtual void log( const char* context, LogLevel level, const char* format, va_list args ) = 0;

	protected:
		long id;
		void bufferTime( const char* format );
		char bufferDateTime[32];
		MutexRW configMux;
		Mutex writeMux;

		static List22<LogHandler*>& getInstances();

		friend class wosh::SystemLogger;

 };

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class LogHandlerPlainText : public LogHandler
 {
	WOSH_CLASSNAME(wosh::LogHandlerPlainText)
	WOSH_CLASS_ABSTRACT(false)


	public:
		LogHandlerPlainText()
			: LogHandler(), outFile(NULL), outFilename("default.log"), appendMode(true) { }

		virtual ~LogHandlerPlainText() {
			close();
		 }

	public:
		virtual WRESULT open();
		virtual WRESULT close();
		virtual WRESULT flush();
		virtual bool isOpen() const;

		virtual std::string getProtocol() const {
			MutexLockerRead mL(&this->configMux);
			return  "file://" + this->outFilename;
		 }

		// really-multithread safe
		virtual void log( const char* context, LogLevel level, const char* format, va_list args );

	public:
		void log( LogLevel level, const char* format, ... );

	public:
		WRESULT setFileName( const std::string& filename, bool append = true );

	public:
		inline std::string getFileName() const {
			MutexLockerRead mL(&this->configMux);
			return this->outFilename;
		 }

	protected:
		virtual WRESULT open_();
		virtual WRESULT close_();

	protected:
		std::FILE* outFile;
		std::string outFilename;
		bool appendMode;

 };

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class LogHandlerStdOut : public LogHandler
 {
	WOSH_CLASSNAME(wosh::LogHandlerStdOut)
	WOSH_CLASS_ABSTRACT(false)


	public:
		LogHandlerStdOut()
			: LogHandler(), outStd(stderr), printColoured(false) { }

		virtual ~LogHandlerStdOut() {
			close();
		 }

	public:
		static LogHandlerStdOut* getInstance();

	public:
		virtual WRESULT open();
		virtual WRESULT close()			{ return WRET_OK; }
		virtual WRESULT flush()			{ return WRET_OK; }
		virtual bool isOpen() const		{ return true; }

		virtual std::string getProtocol() const {
			MutexLockerRead mL(&this->configMux);
			if ( outStd == stderr ) return "std://stderr";
			if ( outStd == stdout ) return "std://stdout";
			return "std://";
		 }

		// really-multithread safe
		virtual void log( const char* context, LogLevel level, const char* format, va_list args );

	public:
		void setColoured( bool enabled = true );

	protected:
		std::FILE* outStd;
		bool printColoured;

 };

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <int COUNT>
class LogHandlerMultiplexer : public LogHandler
 {
	WOSH_CLASSNAME(wosh::LogHandlerMultiplexer)
	WOSH_CLASS_ABSTRACT(false)


	public:
		LogHandlerMultiplexer()
			: LogHandler() {
			for( int i=0; i<COUNT; ++i )
				this->handlers[i] = NULL;
		 }
		LogHandlerMultiplexer( LogHandler* handler0, LogHandler* handler1 = NULL, LogHandler* handler2 = NULL )
			: LogHandler() {
			for( int i=0; i<COUNT; ++i )
				this->handlers[i] = NULL;
			if ( COUNT > 0 )
				this->handlers[0] = handler0;
			if ( COUNT > 1 )
				this->handlers[1] = handler1;
			if ( COUNT > 2 )
				this->handlers[2] = handler2;
		 }
		virtual ~LogHandlerMultiplexer() {
			close();
		 }

	public:
		WRESULT addHandler( LogHandler* handler ) {
			MutexLockerWrite mL(&this->configMux);
			for( int i=0; i<COUNT; ++i ) {
				if ( this->handlers[i] == NULL ) continue;
				this->handlers[i] = handler;
				return WRET_OK;
			 }
			return WRET_ERR_ILLEGAL_USE;
		 }
		WRESULT deleteHandler( LogHandler* handler ) {
			MutexLockerWrite mL(&this->configMux);
			for( int i=0; i<COUNT; ++i ) {
				if ( this->handlers[i] == NULL ) continue;
				if ( this->handlers[i] != handler ) continue;
				this->handlers[i] = NULL;
				return WRET_OK;
			 }
			return WRET_ERR_PARAM;
		 }

	public:
		virtual WRESULT open() {
			WRESULT ret = WRET_OK; int found = 0;
			MutexLockerWrite mL(&this->configMux);
			for( int i=0; i<COUNT; ++i ) {
				if ( this->handlers[i] == NULL ) continue;
				++found;
				ret += this->handlers[i]->open();
			 }
			if ( ret != found )
				return WRET_ERR_INTERNAL;
			return ret;
		 }
		virtual WRESULT close() {
			WRESULT ret = WRET_OK; int found = 0;
			MutexLockerWrite mL(&this->configMux);
			for( int i=0; i<COUNT; ++i ) {
				if ( this->handlers[i] == NULL ) continue;
				++found;
				ret += this->handlers[i]->close();
			 }
			if ( ret != found )
				return WRET_ERR_INTERNAL;
			return ret;
		 }
		virtual WRESULT flush() {
			WRESULT ret = WRET_OK; int found = 0;
			MutexLockerWrite mL(&this->configMux);
			for( int i=0; i<COUNT; ++i ) {
				if ( this->handlers[i] == NULL ) continue;
				++found;
				ret += this->handlers[i]->flush();
			 }
			if ( ret != found )
				return WRET_ERR_INTERNAL;
			return ret;
		 }
		virtual bool isOpen() const {
			int ret = 0; int found = 0;
			MutexLockerWrite mL(&this->configMux);
			for( int i=0; i<COUNT; ++i ) {
				if ( this->handlers[i] == NULL ) continue;
				++found;
				if ( !this->handlers[i]->isOpen() ) continue;
				++ret;
			 }
			return (ret != 0 && ret == found );
		 }

		virtual std::string getProtocol() const {
			MutexLockerRead mL(&this->configMux);
			int found = 0;
			for( int i=0; i<COUNT; ++i ) {
				if ( this->handlers[i] == NULL ) continue;
				++found;
			 }
			return Utilities::format("multiplex[%d]://", found);
		 }

		virtual void log( const char* context, LogLevel level, const char* format, va_list args ) {
			this->configMux.lockForRead();
			this->writeMux.lock();
			for( int i=0; i<COUNT; ++i ) {
				if ( this->handlers[i] == NULL ) continue;
				this->handlers[i]->log( context, level, format, args );
			 }
			this->writeMux.unLock();
			this->configMux.unLock();
		 }

	protected:
		LogHandler* handlers[COUNT];

};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class DefaultSharedLogHandler : public LogHandlerMultiplexer<4>
 {
	WOSH_CLASSNAME(wosh::DefaultSharedLogHandler)
	WOSH_CLASS_ABSTRACT(false)


	public:
		static DefaultSharedLogHandler* getInstance(); // meyer singleton

	public:
		WRESULT setStdOut( bool enabled = true, bool coloured = true );
		WRESULT setFileName( const std::string& filename, bool append = true );

		std::string getFileName() const;

	private:
		DefaultSharedLogHandler()
			: LogHandlerMultiplexer<4>( new LogHandlerStdOut() ) {
			open();
		 }
		DefaultSharedLogHandler( const DefaultSharedLogHandler& ) : LogHandlerMultiplexer<4>() { }
		DefaultSharedLogHandler& operator=( const DefaultSharedLogHandler&  ) { return *this; }
	public:
		virtual ~DefaultSharedLogHandler() {
			this->configMux.lockForRead();
			this->writeMux.lock();
			for( int i=0; i<4; ++i ) {
				if ( this->handlers[i] == NULL ) continue;
				delete this->handlers[i]; this->handlers[i] = NULL;
			 }
			this->writeMux.unLock();
			this->configMux.unLock();
		 }
};


}; // namespace wosh

#endif //__WOSH_Core_LogHandlers_H__
