/** @file    MessageProcessingCommon.cpp
 *  @author  Alessandro Polo
 *  @version $Id: MessageProcessingCommon.cpp 2222 2010-03-26 20:49:35Z alex $
 *  @brief
 * File containing implementations of file MessageProcessingCommon.h
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <core/MessageProcessingCommon.h>

 #include <core/NEW_List.h>
 #include <core/Table.h>
 #include <core/SecurityManager.h>
 #include <core/UserManager.h>


using namespace std;
namespace wosh {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Message* MPC_busMessageRequest( const Message& requestMessage, MethodsProvider& Methods, LogContext& Log )
 {
	if ( requestMessage.isEmpty() || !requestMessage.getContent()->isRequest() ) return NULL;
	MutexLockerRead mL(Methods.getMutex());
	// lock invalidating operations (REMOVAL; List::Permission updates)
	bool allowread = SecurityManager::evalAuthorization( requestMessage.getSecurityToken(), &Methods.getListPermission(), Permission::Read );
	if ( !allowread ) // also validated missing security token
		return NULL; // silent
	const Request* request = requestMessage.getContent()->asRequest();
	Method* method = Methods.find( request->getMethod() );
	if ( method == NULL ) {
		Log(LOG_WARNING, ":busMessage() FAILED : Method '%s' not found", request->getMethod().c_str() );
		if ( request->doOmitResponse() ) return NULL;
		return requestMessage.replyTo( request->replyResponse(WRET_ERR_PARAM, Utilities::format("Method '%s' not found!", request->getMethod().c_str()).c_str()) );
	 }
	// read the permission, lock it for reading
	method->transactionBeginRead();
	bool allowRead = SecurityManager::evalAuthorization( requestMessage.getSecurityToken(), &method->getPermission(), Permission::Read );
	if ( !allowRead ) {
		method->transactionEnd();
		Log(LOG_WARNING, ":busMessage() : Not allowed to read method '%s' ", request->getMethod().c_str() );
		return NULL; // silent
	 }
	bool allowExec = SecurityManager::evalAuthorization( requestMessage.getSecurityToken(), &method->getPermission(), Permission::Execute );
	if ( !allowExec ) {
		method->transactionEnd();
		Log(LOG_WARNING, ":busMessage() : Not allowed to exec method '%s' ", request->getMethod().c_str() );
		if ( request->doOmitResponse() ) return NULL;
		return requestMessage.replyTo( request->replyResponse(WRET_ERR_ILLEGAL_USE, "Not allowed to exec method") );
	 }
	// call the message
	Log(LOG_VERBOSE, ":busMessage() : Executing '%s'", request->getMethod().c_str() );
	Message* reply = method->exec( &requestMessage );
	Log(LOG_VERBOSE, ":busMessage() : Executed '%s' (%d)", request->getMethod().c_str(), (int)(reply!=NULL) );
	// we are done
	method->transactionEnd();
	// return reply (if any)
	if ( reply != NULL && request->doOmitResponse() ) {
		delete reply; reply = NULL;
	 }
	return reply;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// SET-PROPERTY

Response* MPC_mmDoPropertySet( const Message* requestMessage, PropertiesProvider& Properties, LogContext& Log )
 {
	if ( requestMessage == NULL || requestMessage->isEmpty() || !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();
	if ( !request->hasData() )
		return request->replyResponse(WRET_ERR_PARAM, "Empty Data");

	MutexLockerRead mL(Properties.getMutex());
	// lock invalidating operations (REMOVAL; List::Permission updates)
	bool allowread = SecurityManager::evalAuthorization( requestMessage->getSecurityToken(), &Properties.getListPermission(), Permission::Read );
	if ( !allowread ) // also validated missing security token
		return NULL; // silent

	if ( request->getData()->isClass(_Property_TYPE) ) {
		const Property* property_source = dynamic_cast<const Property*>( request->getData() );
		Log(LOG_VERBOSE, ":MPC_mmDoPropertySet(%s) [Property]", property_source->getKey().c_str() );
		PropertyHolder* propHolder = Properties.find( property_source->getKey() );
		if ( propHolder == NULL ) {
			// validate insertion against list permission, user must be granted to write the list
			bool allowAdd = SecurityManager::evalAuthorization( requestMessage->getSecurityToken(), &Properties.getListPermission(), Permission::Write );
			if ( !allowAdd )
				return request->replyResponse(WRET_ERR_SECURITY, "Property not found, Not allowed to add new");
			// user may add the property, 
			///@todo should validate property permission..
			// add/update (non-locking, callbacks)
			WRESULT ret = Properties.set( property_source->getKey(), property_source->getValue(), property_source->getPermission(), Permission::Overwrite_All);
			// expensive, but useful to spread the updated property,
			// note that the whole operation is NOT atomic, following returned item MAY be != *property_source
			return request->replyResponse(ret, Properties.getCopyOf(property_source->getKey()));
		 }
		else {
			propHolder->transactionBeginWrite();
			if ( propHolder->data == NULL ) {
				propHolder->transactionEnd();
				return request->replyResponse(WRET_ERR_INTERNAL, "Internal error (misaligned)");
			 }
			// eval permission, overwrite value (we won't update permission)
			bool allowWrite = SecurityManager::evalAuthorization( requestMessage->getSecurityToken(), &propHolder->data->getPermission(), Permission::Write );
			if ( !allowWrite ) {
				propHolder->transactionEnd();
				return request->replyResponse(WRET_ERR_SECURITY, "Not allowed to edit property");
			 }
			// go on with the atomic operation, update value/permission fields, callbacks enabled
			WRESULT ret = Properties.updating( propHolder, property_source->getValue(), &property_source->getPermission(), Permission::Overwrite_All, true );
			propHolder->transactionEnd();
			// expensive, but useful to spread the updated property,
			// note that the whole operation is NOT atomic, following returned item MAY be != *property_source
			return request->replyResponse(ret, Properties.getCopyOf(property_source->getKey()));
		 }
	 }
	else if ( request->getDataClassName() == _NEW_List_TYPE ) {
		Variant dKey;
		Variant dValue;
		request->extractArguments(dKey, dValue);
		if ( dKey.isStringEmpty() )
			return request->replyResponse(WRET_ERR_PARAM, "Wrong DataType: NEW_List too short (key; !value)");
		Log(LOG_VERBOSE, ":MPC_mmDoPropertySet(%s) [NEW_List]", dKey.asString().c_str() );
		PropertyHolder* propHolder = Properties.find( dKey.asString() );
		if ( propHolder == NULL ) {
			// validate insertion against list permission, user must be granted to write the list
			bool allowAdd = SecurityManager::evalAuthorization( requestMessage->getSecurityToken(), &Properties.getListPermission(), Permission::Write );
			if ( !allowAdd )
				return request->replyResponse(WRET_ERR_SECURITY, "Property not found, Not allowed to add new");
			// user may add the property, 
			///@todo should validate property permission..
			// add/update (non-locking, callbacks)
			WRESULT ret = Properties.set( dKey.asString(), dValue, Permission(), Permission::Overwrite_None);
			// expensive, but useful to spread the updated property,
			// note that the whole operation is NOT atomic, following returned item MAY be != *property_source
			return request->replyResponse(ret, Properties.getCopyOf(dKey.asString()));
		 }
		else {
			propHolder->transactionBeginWrite();
			if ( propHolder->data == NULL ) {
				propHolder->transactionEnd();
				return request->replyResponse(WRET_ERR_INTERNAL, "Internal error (misaligned)");
			 }
			// eval permission, overwrite value (we won't update permission)
			bool allowWrite = SecurityManager::evalAuthorization( requestMessage->getSecurityToken(), &propHolder->data->getPermission(), Permission::Write );
			if ( !allowWrite ) {
				propHolder->transactionEnd();
				return request->replyResponse(WRET_ERR_SECURITY, "Not authorized to edit property");
			 }
			// go on with the atomic operation, update value/permission fields, callbacks enabled
			WRESULT ret = Properties.updating( propHolder, dValue, NULL, Permission::Overwrite_None, true );
			propHolder->transactionEnd();
			// expensive, but useful to spread the updated property,
			// note that the whole operation is NOT atomic, following returned item MAY be != *property_source
			return request->replyResponse(ret, Properties.getCopyOf(dKey.asString()));
		 }
	 }
	Log(LOG_WARNING, ":MPC_mmDoPropertySet(%s) [Invalid Type]", request->getDataClassName().c_str() );
	return request->replyResponse(WRET_ERR_PARAM, "Wrong DataType: not Property/NEW_List!");
 }

//////////////////////////////////////////////////////////////////////////////////////////////// SET-PROPERTY
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// GET-PROPERTY

Response* MPC_mmDoPropertyGet( const Message* requestMessage, const PropertiesProvider& Properties, LogContext& Log )
 {
	if ( requestMessage == NULL || requestMessage->isEmpty() || !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();
	bool allowread = SecurityManager::evalAuthorization( requestMessage->getSecurityToken(), &Properties.getListPermission(), Permission::Read );
	if ( !allowread ) // also validated missing security token
		return request->replyResponse(WRET_ERR_SECURITY, "Not authorized to read");
	std::string key = request->getArgument().toString();
	Log(LOG_VERBOSE, ":MPC_mmDoPropertyGet(%s)", key.c_str() );
	MutexLockerRead mL(Properties.getMutex());
	// lock invalidating operations (REMOVAL; List::Permission updates)
	const PropertyHolder* propHolder = Properties.find( key );
	if ( propHolder == NULL )
		return request->replyResponse(WRET_ERR_PARAM, "Property not found");
	propHolder->transactionBeginRead();
	if ( propHolder->data == NULL ) {
		propHolder->transactionEnd();
		return request->replyResponse(WRET_ERR_INTERNAL, "Internal error (misaligned)");
	 }
	// eval permission, overwrite value (we won't update permission)
	bool allowRead = SecurityManager::evalAuthorization( requestMessage->getSecurityToken(), &propHolder->data->getPermission(), Permission::Read );
	if ( !allowRead ) {
		propHolder->transactionEnd();
		return request->replyResponse(WRET_ERR_SECURITY, "Not authorized to read property");
	 }
	// go on with the atomic operation, check if listener allows reading
	bool accepted = Properties.reading( propHolder );
	if ( !accepted ) {
		propHolder->transactionEnd();
		return request->replyResponse(WRET_ERR_INTERNAL, "Internal error while reading");
	 }
	Property* prop = propHolder->data->clone();
	propHolder->transactionEnd();
	return request->replyResponse(WRET_OK, prop);
 }

//////////////////////////////////////////////////////////////////////////////////////////////// GET-PROPERTY
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* MPC_mmDoPropertiesSet( const Message* requestMessage, PropertiesProvider& Properties, LogContext& Log )
 {
	if ( requestMessage == NULL || requestMessage->isEmpty() || !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();
	Log(LOG_VERBOSE, ":MPC_mmDoPropertiesSet()" );
	if ( !request->hasData() )
		return request->replyResponse(WRET_ERR_PARAM, "Empty Data");

	// lock invalidating operations (REMOVAL; List::Permission updates)
	MutexLockerRead mL(Properties.getMutex());

	if ( request->getData()->isClass(_Table_TYPE) ) {
		const Table* table = dynamic_cast<const Table*>(request->getData());
		if ( table->isEmpty() )
			return request->replyResponse(WRET_ERR_PARAM, "Invalid Argument (empty table)");

		return request->replyResponse(WRET_ERR_INTERNAL);
	 }
	return request->replyResponse(WRET_ERR_PARAM, "Invalid Argument (not a NEW_List/Table)");
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* MPC_mmDoPropertiesGet( const Message* requestMessage, const PropertiesProvider& Properties, LogContext& Log )
 {
	if ( requestMessage == NULL || requestMessage->isEmpty() ) return NULL;
	if ( !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();
	bool allowread = SecurityManager::evalAuthorization( requestMessage->getSecurityToken(), &Properties.getListPermission(), Permission::Read );
	if ( !allowread ) // also validated missing security token
		return request->replyResponse(WRET_ERR_SECURITY, "Not authorized to read");
	std::vector<std::string> keys;
	if ( request->getData()->isClass(_Variant_TYPE) ) { // not optimized
		const Variant* vKeyRegex = request->getData()->as<Variant>();
		if ( vKeyRegex == NULL ) // should never happen
			return request->replyResponse(WRET_ERR_PARAM, "Invalid Variang Argument (internal failure!?)");
		std::string regex_key = vKeyRegex->toString();
		Properties.transactionBeginRead();
		PropertiesProvider::ConstIterator it;
		const PropertiesProvider::ConstIterator it_end = Properties.getMap().end();
		for( it=Properties.getMap().begin(); it!=it_end; ++it ) {
			if ( it->second == NULL ) continue;
			if ( Utilities::evalRegexWildChar( regex_key, it->first ) == 0 )
				keys.push_back(it->first);
		 }
		Properties.transactionEnd();
	 }
	else if ( request->getData()->isClass(_NEW_List_TYPE) ) {
		const NEW_List* keyList = request->getData()->as<NEW_List>();
		if ( keyList == NULL ) // should never happen
			return request->replyResponse(WRET_ERR_PARAM, "Invalid NEW_List Argument (internal failure!?)");
		keyList->toVectorString(keys);
	 }
	Log(LOG_VERBOSE, ":MPC_mmDoPropertiesGet() %d items", keys.size() );
	if ( keys.empty() )
		return request->replyResponse(WRET_ERR_PARAM, "Invalid Argument (empty NEW_List)");
	NEW_List* outList = new NEW_List( keys.size() );
	Properties.transactionBeginRead();
	std::vector<std::string>::const_iterator it;
	const std::vector<std::string>::const_iterator it_end = keys.end();
	for( it=keys.begin(); it!=it_end; ++it ) {
		const PropertyHolder* pH = Properties.find( *it );
		if( pH == NULL ) continue;
		pH->transactionBeginRead();
		if( pH->data == NULL ) {
			pH->transactionEnd(); continue;
		 }
		bool allowRead = SecurityManager::evalAuthorization( requestMessage->getSecurityToken(), &pH->data->getPermission(), Permission::Read );
		if ( !allowRead ) {
			pH->transactionEnd(); continue;
		 }
		allowRead = Properties.reading(pH);
		if ( !allowRead ) {
			pH->transactionEnd(); continue;
		 }
		Property* copy = pH->data->clone();
		outList->append( copy );
		pH->transactionEnd();
	 }
	Properties.transactionEnd();
	// for security reasons: don't send empty lists, it would provide confidential info on keys
	outList->cleanNULLs();
	if ( outList->isEmpty() ) {
		delete outList; outList = NULL;
	 }
	return request->replyResponse(WRET_OK, outList);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* MPC_mmDoPropertyList( const Message* requestMessage, const PropertiesProvider& Properties, LogContext& Log )
 {
	if ( requestMessage == NULL || requestMessage->isEmpty() ) return NULL;
	if ( !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, ":MPC_mmDoPropertyList(%s)", option.c_str() );

	bool allowList = SecurityManager::evalAuthorization( requestMessage->getSecurityToken(), &Properties.getListPermission(), Permission::Read );
	if ( !allowList )
		return request->replyResponse(WRET_ERR_SECURITY, "Not authorized to read");

	///@bug security bug
	if ( option == "s" || option == "short" )
		return request->replyResponse(WRET_OK, Properties.toList(0) );
	return request->replyResponse(WRET_OK, Properties.toTable(true) );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* MPC_mmDoMethodList( const Message* requestMessage, const MethodsProvider& Methods, LogContext& Log )
 {
	if ( requestMessage == NULL || requestMessage->isEmpty() ) return NULL;
	if ( !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, ":MPC_mmDoMethodList(%s)", option.c_str() );

	bool allowList = SecurityManager::evalAuthorization( requestMessage->getSecurityToken(), &Methods.getListPermission(), Permission::Read );
	if ( !allowList )
		return request->replyResponse(WRET_ERR_SECURITY, "Not authorized to read");

	///@bug security bug
	if ( option == "s" || option == "short" )
		return request->replyResponse(WRET_OK, Methods.toList(0) );
	return request->replyResponse(WRET_OK, Methods.toTable(true, true) );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* MPC_mmDoChMod_apply( Permission* targetPerm, const string& newMask, const Message* requestMessage ) {
	const Request* request = requestMessage->getContent()->asRequest();
	if ( targetPerm == NULL )
		return request->replyResponse(WRET_ERR_PARAM, "Invalid target");
	// recognize my dear god
	if ( requestMessage->getSecurityToken()->getUser() != "root" &&
		 targetPerm->getOwner() != requestMessage->getSecurityToken()->getUser() )
			return request->replyResponse(WRET_ERR_SECURITY, "User Not authorized");

	//if ( targetPerm->getMask(0) & Permission::Read == false )
	///@todo if ( perm.getMask(0) == Permission::Read ) SET enabled

	targetPerm->setMask( newMask );
	return request->replyResponse(WRET_OK, targetPerm->clone());
 }

Response* MPC_mmDoChMod( const Message* requestMessage, Object* object, PropertiesProvider* Properties, MethodsProvider* Methods, LogContext* Log )
 {
	if ( requestMessage == NULL || requestMessage->isEmpty() || !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();
	if ( !request->hasData() )
		return request->replyResponse(WRET_ERR_PARAM, "Empty Data");
	// it should exist but let's ensure:
	if ( !requestMessage->hasSecurityToken() ) return NULL; //silent
	Response* response = NULL;
	// detect target
	// verify authorization
	// change permission
	if ( request->getData()->isClass(_Variant_TYPE) ) {
		// so, for sure, it is referred to main object!
		if ( object == NULL )
			return request->replyResponse(WRET_ERR_INTERNAL, "Internal Error");
		///@warning not thread safe on 'object'
		Permission perm = ((const Object*)object)->getPermission();
		// recognize my dear god
		if ( requestMessage->getSecurityToken()->getUser() != "root" &&
			perm.getOwner() != requestMessage->getSecurityToken()->getUser() )
				return request->replyResponse(WRET_ERR_INTERNAL, "User Not authorized");
		if ( perm.getMask(0) == Permission::Read &&
			perm.getMask(1) == Permission::Read &&
			perm.getMask(2) == Permission::Read )
				return request->replyResponse(WRET_ERR_INTERNAL, "Rights Not authorized");
		string perms = request->getArgument().toString();
		if ( Log != NULL )
			Log->operator()(LOG_INFO, ":MPC_mmDoChMod(%s) %s", object->getName().c_str(), perms.c_str() );
		perm.setMask( perms );
		///@todo if ( perm.getMask(0) == Permission::Read ) SET enabled
		object->setPermission( perm );
		return request->replyResponse(WRET_OK, object->getPermission().clone());
	 }
	else if ( request->getData()->isClass(_NEW_List_TYPE) ) {
		Variant dTarget;
		Variant dPerms;
		request->extractArguments(dTarget, dPerms);
		if ( dTarget.isStringEmpty() || dPerms.isStringEmpty() )
			return request->replyResponse(WRET_ERR_INTERNAL, "Invalid input [empty NEW_List]");
		// is it a property?
		if ( Properties != NULL ) {
			Properties->transactionBeginRead();
			PropertyHolder* prop = Properties->find(dTarget.asString());
			if ( prop != NULL ) {
				prop->transactionBeginWrite();
				if ( prop->data == NULL ) {
					prop->transactionEnd();
					Properties->transactionEnd();
					return request->replyResponse(WRET_ERR_INTERNAL, "Internal Error");
				 }
				response = MPC_mmDoChMod_apply( &prop->data->getPermission(), dPerms.asString(), requestMessage );
				prop->transactionEnd();
				Properties->transactionEnd();
				if ( Log != NULL )
					Log->operator()(LOG_INFO, ":MPC_mmDoChMod(%s) [Property] %s [by %s]", dTarget.asString().c_str(), dPerms.asString().c_str(), requestMessage->getSecurityToken()->getUser().c_str() );
				return response;
			 }
			Properties->transactionEnd();
		 }
		// so.. is it a method?
		if ( Methods != NULL ) {
			Methods->transactionBeginRead();
			Method* method = Methods->find(dTarget.asString());
			if ( method != NULL ) {
				method->transactionBeginWrite();
				response = MPC_mmDoChMod_apply( &method->getPermission(), dPerms.asString(), requestMessage );
				method->transactionEnd();
				Methods->transactionEnd();
				if ( Log != NULL )
					Log->operator()(LOG_INFO, ":MPC_mmDoChMod(%s) [Method] %s [by %s]", dTarget.asString().c_str(), dPerms.asString().c_str(), requestMessage->getSecurityToken()->getUser().c_str() );
				return response;
			 }
			Methods->transactionEnd();
		 }
	 }
	else {
		if ( Log != NULL )
			Log->operator()(LOG_CRITICAL, ":MPC_mmDoChMod(%s) not implemented" );
	 }
	return request->replyResponse(WRET_ERR_INTERNAL, "not implemented");
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* MPC_mmDoChOwn_apply( Permission* targetPerm, const string& newOwner, const Message* requestMessage ) {
	const Request* request = requestMessage->getContent()->asRequest();
	if ( targetPerm == NULL )
		return request->replyResponse(WRET_ERR_PARAM, "Invalid target");
	// recognize my dear god
	if ( requestMessage->getSecurityToken()->getUser() != "root" &&
		 targetPerm->getOwner() != requestMessage->getSecurityToken()->getUser() )
			return request->replyResponse(WRET_ERR_SECURITY, "User Not authorized");

	if ( targetPerm->getMask(0) == Permission::Read && 
		 targetPerm->getMask(1) == Permission::Read && 
		 targetPerm->getMask(2) == Permission::Read )
			return request->replyResponse(WRET_ERR_SECURITY, "Rights Not authorized");

	targetPerm->setOwner( newOwner );
	return request->replyResponse(WRET_OK, targetPerm->clone());
 }

Response* MPC_mmDoChOwn( const Message* requestMessage, Object* object, PropertiesProvider* Properties, MethodsProvider* Methods, LogContext* Log )
 {
	if ( requestMessage == NULL || requestMessage->isEmpty() || !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();
	if ( !request->hasData() )
		return request->replyResponse(WRET_ERR_PARAM, "Empty Data");
	// it should exist but let's ensure:
	if ( !requestMessage->hasSecurityToken() ) return NULL; //silent
	Response* response = NULL;
	// we must recognize the target { Object, Property, Method }
	if ( request->getData()->isClass(_Variant_TYPE) ) {
		// single 'Data' argument surely refer to the object itself
		if ( object == NULL )
			return request->replyResponse(WRET_ERR_INTERNAL, "Internal Error");
		string newOwner = request->getArgument().toString();
		if ( !UserManager::isUser(newOwner) )
			return request->replyResponse(WRET_ERR_PARAM, "unknown User");
		///@warning not thread safe on 'object'
		Permission temp = ((const Object*)object)->getPermission();
		response = MPC_mmDoChOwn_apply( &temp, newOwner, requestMessage );
		object->setPermission( temp );
		if ( Log != NULL )
			Log->operator()(LOG_INFO, ":MPC_mmDoChOwn(%s) %s [by %s]", object->getName().c_str(), newOwner.c_str(), requestMessage->getSecurityToken()->getUser().c_str() );
		return response;
	 }
	else if ( request->getData()->isClass(_NEW_List_TYPE) ) {
		Variant dTarget;
		Variant dNewOwner;
		request->extractArguments(dTarget, dNewOwner);
		if ( dTarget.isStringEmpty() || dNewOwner.isStringEmpty() )
			return request->replyResponse(WRET_ERR_PARAM, "Invalid input [empty NEW_List]");
		if ( !UserManager::isUser(dNewOwner.asString()) )
			return request->replyResponse(WRET_ERR_PARAM, "unknown User");
		// is it a property?
		if ( Properties != NULL ) {
			Properties->transactionBeginRead();
			PropertyHolder* prop = Properties->find(dTarget.asString());
			if ( prop != NULL ) {
				prop->transactionBeginWrite();
				if ( prop->data == NULL ) {
					prop->transactionEnd();
					Properties->transactionEnd();
					return request->replyResponse(WRET_ERR_INTERNAL, "Internal Error");
				 }
				response = MPC_mmDoChOwn_apply( &prop->data->getPermission(), dNewOwner.asString(), requestMessage );
				prop->transactionEnd();
				Properties->transactionEnd();
				if ( Log != NULL )
					Log->operator()(LOG_INFO, ":MPC_mmDoChOwn(%s) [Property] %s [by %s]", dTarget.asString().c_str(), dNewOwner.asString().c_str(), requestMessage->getSecurityToken()->getUser().c_str() );
				return response;
			 }
			Properties->transactionEnd();
		 }
		// so.. is it a method?
		if ( Methods != NULL ) {
			Methods->transactionBeginRead();
			Method* method = Methods->find(dTarget.asString());
			if ( method != NULL ) {
				method->transactionBeginWrite();
				response = MPC_mmDoChOwn_apply( &method->getPermission(), dNewOwner.asString(), requestMessage );
				method->transactionEnd();
				Methods->transactionEnd();
				if ( Log != NULL )
					Log->operator()(LOG_INFO, ":MPC_mmDoChOwn(%s) [Method] %s [by %s]", dTarget.asString().c_str(), dNewOwner.asString().c_str(), requestMessage->getSecurityToken()->getUser().c_str() );
				return response;
			 }
			Methods->transactionEnd();
		 }
	 }
	return request->replyResponse(WRET_ERR_PARAM, "Target not Found");
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool MPC_PropertyParse_LogLevel( const Variant& value_proposed, LogContext& Log ) {
	LogLevel level = LOG_UNKNOWN;
	if ( value_proposed.isNumeric() )
		level = static_cast<LogLevel>( value_proposed.toInteger(0) );
	else if ( value_proposed.isString() )
		level = LogContext::getLogLevelFromString( value_proposed.asString().c_str() );
	if ( level <= LOG_UNKNOWN || level > LOG_ALL ) return false;
	Log(LOG_WARNING, " Changing Debug-Level to '%s'", value_proposed.toString().c_str() );
	Log.setLevel( level );
	return true;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

}; // namespace wosh
