/**
 * @file    MethodsDef.h
 * @brief   Defines some common Method implementations
 *
 ****************************************************************************
 * @version $Id: MethodsDef.h 2084 2010-03-11 10:49:02Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR MethodRequestS;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_MethodDefs_H__
 #define __WOSH_Core_MethodDefs_H__

 #include <woshDefs.h>
 #include <core/MethodT.h>

 #include <core/Message.h>
 #include <core/Request.h>
 #include <core/Response.h>


namespace wosh {

////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////

 #define _METHOD_List				"list"

 #define _METHOD_GetProperty		"prop_get"
 #define _METHOD_SetProperty		"prop_set"
 #define _METHOD_ListProperties		"prop_list"

 #define _METHOD_SetProperties		"props_set"
 #define _METHOD_GetProperties		"props_get"

 #define _METHOD_ChMod				"chmod"
 #define _METHOD_ChOwn				"chown"

 #define _METHOD_ListMethods		"method_list"

 #define _METHOD_ListInterfaces		"interface_list"

 #define _METHOD_Man				"man"

////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////
// Most Generic Method: Message* FUNCTION ( const Message* )

typedef MethodT<const Message*, MethodRunner, Message*>::MethodTPtr MethodMessagePtr;

class MethodMessage : public MethodT<const Message*, MethodRunner, Message*>
 {
	WOSH_CLASSNAME(wosh::MethodMessage)
	WOSH_CLASS_ABSTRACT(false)

	public:
		MethodMessage( const std::string& Name )
			: MethodT<const Message*, MethodRunner, Message*>(Name) { }

		MethodMessage( const std::string& Name, const std::string& Description, const Permission& Perm = Permission::R_XR_XR__() )
			: MethodT<const Message*, MethodRunner, Message*>(Name, Description, Perm) { }

		virtual ~MethodMessage() { }

	public:
		virtual Message* exec( const Message* requestMessage )
		 {
			if ( requestMessage == NULL ) return NULL;

			Message* result = execT( requestMessage );

			return result;
		 }

}; // class def


////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////
// Classic Method: Response* FUNCTION ( const Request* )

typedef MethodT<const Request*, MethodRunner, Response*>::MethodTPtr MethodRequestPtr;

class MethodRequest : public MethodT<const Request*, MethodRunner, Response*>
 {
	WOSH_CLASSNAME(wosh::MethodRequest)
	WOSH_CLASS_ABSTRACT(false)

	public:
		MethodRequest( const std::string& Name )
			: MethodT<const Request*, MethodRunner, Response*>(Name) { }

		MethodRequest( const std::string& Name, const std::string& Description, const Permission& Perm = Permission::R_XR_XR__() )
			: MethodT<const Request*, MethodRunner, Response*>(Name, Description, Perm) { }

		virtual ~MethodRequest() { }

	public:
		virtual Message* exec( const Message* requestMessage )
		 {
			if ( requestMessage == NULL || requestMessage->isEmpty() ) return NULL;
			if ( !requestMessage->getContent()->isRequest() ) return NULL;
			const Request* request = requestMessage->getContent()->asRequest();

			Response* result = execT( request );

			Message* reply = requestMessage->replyTo( result );
			return reply;
		 }

}; // class def


////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////
// Method which need Message context: Response* FUNCTION ( const Message* )

typedef MethodT<const Message*, MethodRunner, Response*>::MethodTPtr MethodMessageResponsePtr;

class MethodMessageResponse : public MethodT<const Message*, MethodRunner, Response*>
 {
	WOSH_CLASSNAME(wosh::MethodMessageResponse)
	WOSH_CLASS_ABSTRACT(false)

	public:

		MethodMessageResponse( const std::string& Name )
			: MethodT<const Message*, MethodRunner, Response*>(Name) { }

		MethodMessageResponse( const std::string& Name, const std::string& Description, const Permission& Perm = Permission::R_XR_XR__() )
			: MethodT<const Message*, MethodRunner, Response*>(Name, Description, Perm) { }

		virtual ~MethodMessageResponse() { }

	public:

		// explicit
		virtual void setMethod( MethodRunner* pare, MethodMessageResponsePtr fptr ) {
			this->parent = pare;
			this->smMethodTPointer = fptr;
			PROFILING_INIT(this, this->parent);
		 }

		virtual Message* exec( const Message* message )
		 {
			if ( message == NULL ) return NULL;

			Response* result = execT( message );

			Message* reply = message->replyTo( result );
			return reply;
		 }

}; // class def


////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////
// Simplest Method: WRESULT FUNCTION ()

typedef WRESULT (MethodRunner::*MethodWRESULTPtr)();

class MethodWRESULT : public Method
 {
	WOSH_CLASSNAME(wosh::MethodWRESULT)
	WOSH_CLASS_ABSTRACT(false)

	public:
		MethodWRESULT( const std::string& Name )
			: Method(Name) {
			this->parent = NULL;
			this->smMethodTPointer = NULL;
		 }

		MethodWRESULT( const std::string& Name, const std::string& Description, const Permission& Perm = Permission::R_XR_XR__() )
			: Method(Name, Description, Perm) {
			this->parent = NULL;
			this->smMethodTPointer = NULL;
		 }

		virtual ~MethodWRESULT() { }

//@}
/** @name Execution and Validation
 * @{
 ******************************************************************************/
	public:
		virtual Message* exec( const Message* requestMessage ) {
			if ( this->parent == NULL || this->smMethodTPointer == NULL )
				return NULL;

			PROFILING_ENTER(this, 0)

			WRESULT ret = (this->parent->*smMethodTPointer)();

			PROFILING_EXIT(this, 0)

			Message* reply = NULL;
			if ( requestMessage != NULL )
				reply = requestMessage->replyTo();
			else
				reply = new Message();
			reply->setContent( new Response(this->name, NULL, ret) );
			return reply;
		 }

//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:
		void setMethod( MethodRunner* pare, MethodWRESULTPtr fptr ) {
			this->parent = pare;
			this->smMethodTPointer = fptr;
			PROFILING_INIT(this, this->parent);
		 }
//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:
		virtual bool isLinkedFunction() const				{ return (this->smMethodTPointer != NULL); }
		virtual bool isLinkedParent() const					{ return (this->parent != NULL); }
//@}
	protected:
		MethodRunner* parent;				///< parent of the method (usually it's the object implementing it)
		MethodWRESULTPtr smMethodTPointer;	///< real function pointer

		virtual void clear() {
			Method::clear();
			this->parent = NULL;
			this->smMethodTPointer = NULL;
		 }

		virtual short compare(const Method& other) const {
			const MethodWRESULT& mt = dynamic_cast<const MethodWRESULT&>(other);
			if ( this->parent != mt.parent )
				return 1;
			if ( this->smMethodTPointer != mt.smMethodTPointer )
				return -1;
			return 0;
		 }

}; // class def

////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////
// Classic Method: Response* FUNCTION ( const Request* )

typedef MethodT<const SecurityToken*, MethodRunner, WRESULT>::MethodTPtr MethodWRESULTwSecurityTokenPtr;

class MethodWRESULTwSecurityToken : public MethodT<const SecurityToken*, MethodRunner, WRESULT>
 {
	WOSH_CLASSNAME(wosh::MethodWRESULTwSecurityToken)
	WOSH_CLASS_ABSTRACT(false)

	public:
		MethodWRESULTwSecurityToken( const std::string& Name )
			: MethodT<const SecurityToken*, MethodRunner, WRESULT>(Name) { }

		MethodWRESULTwSecurityToken( const std::string& Name, const std::string& Description, const Permission& Perm = Permission::R_XR_XR__() )
			: MethodT<const SecurityToken*, MethodRunner, WRESULT>(Name, Description, Perm) { }

		virtual ~MethodWRESULTwSecurityToken() { }

	public:
		virtual Message* exec( const Message* requestMessage )
		 {
			if ( requestMessage == NULL || requestMessage->isEmpty() ) return NULL;
			if ( !requestMessage->getContent()->isRequest() ) return NULL;

			WRESULT ret = execT( requestMessage->getSecurityToken() );

			Message* reply = requestMessage->replyTo();
			reply->setContent( new Response(this->name, NULL, ret) );
			return reply;
		 }

}; // class def

////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////

class MethodNULL : public MethodT<const Message*, MethodRunner, Response*>
 {
	WOSH_CLASSNAME(wosh::MethodNULL)
	WOSH_CLASS_ABSTRACT(false)

	public:
		MethodNULL( const std::string& Name )
			: MethodT<const Message*, MethodRunner, Response*>(Name) { }

		virtual ~MethodNULL() { }

	public:
		virtual Message* exec( const Message* ) {
			return NULL;
		 }

}; // class def



}; // namespace wosh

#endif //__WOSH_Core_MethodDefs_H__
