/**
 * @class   wosh::MethodsProvider
 * @brief   
 *
 *
 *Sorted Associative Container t
 Unique Associative Container,
 * - Pure STL
 * - requirement tree:
 *   - std::string, std::map
 *   - wosh::Method
 *   - wosh::Permission
 *
 * Container is std::map
 *
 *
 Class/interface for Objects which provide Methods, some
 *          helper functions are provided for convenience.
 *          Basically it's a sort of custom collector.
 *
 * @see     wosh::Method
 * @see     wosh::BundleGeneric
 *
 ****************************************************************************
 * @version $Id: MethodsProvider.h 2019 2010-03-03 01:19:43Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR MethodsProviderS;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_MethodsProvider_H__
 #define __WOSH_Core_MethodsProvider_H__

 #include <woshDefs.h>
 #include <core/Containers.h>
 #include <core/Method.h>
 #include <core/Permission.h>
 #include <core/Reflection.h>


namespace wosh {

 #define _MethodsProvider_TYPE						"wosh::MethodsProvider"

 #define _MethodsProvider_METHOD_methodlist			"methodlist"
 #define _MethodsProvider_METHOD_methodinfo			"methodinfo"
 #define _MethodsProvider_METHOD_methodNotFound		"method_not_found"
 #define _MethodsProvider_METHOD_methodInvalidArgs	"method_invalid_args"

class Message;
class Object;
class NEW_List;
class Table;

class MethodsProvider :		public Map22<std::string, Method*>,
							public virtual IReflection
 {
	WOSH_CLASSNAME(wosh::PropertiesProvider)
	WOSH_CLASS_ABSTRACT(false)


	public:
		/**
		 * @brief  Default constructor.
		 */
		MethodsProvider();

		MethodsProvider( const Object* owner );

		virtual ~MethodsProvider();

	public:


/** @name Method Getters (reentrant)
 * @{
 ******************************************************************************/
 	public:

		NEW_List* toList( short index ) const;
		Table* toTable( bool include_permission, bool include_arguments ) const;


//@}
/** @name Method Setters (reentrant)
 * @{
 ******************************************************************************/
 	public:
		/**
		 * @brief   Add a method to the list (map). 
		 * @param   method Method istance, it will be free by this class
		 * @note    Method::name is used as hash key.
		 */

		// add (reentrant)
		virtual WRESULT set( Method* method );

		// add (reentrant)
		virtual WRESULT unSet( const std::string& method_name );

		// add (reentrant)
		template <typename T>
		T* create( const std::string& name, const std::string& description = "", const Permission& perm = Method::defaultPermission ) {
			T* method = new T(name, description, perm);
			set(method);
			return method;
		 }

//@}
/** @name Method Getters (non-locking, no callbacks)
 * @{
 ******************************************************************************/
 	public:


/** @name Getters
 * @{
 ******************************************************************************/
 	public:

		inline const Permission& getListPermission() const	{ return this->listPermission; }
		inline Permission& getListPermission()				{ return this->listPermission; }


//@}
/** @name Setters
 * @{
 ******************************************************************************/
 	public:

		void setPermission( const Permission& value, short overwrite_mode, bool when_empty, bool nested );

//@}

	protected:
		Permission listPermission;

	private:
		MethodsProvider( const MethodsProvider& ) : Map22<std::string, Method*>() { }
		MethodsProvider& operator=( const MethodsProvider& ) { return *this; }

}; // class def

}; // namespace wosh

#endif //__WOSH_Core_MethodsProvider_H__
