/**
 * @file    Mutex.h
 * @brief   Contains definition of mutex
 * 
 *
 ****************************************************************************
 * @class   wosh::Mutex
 * @brief   Multiplatform mutex used within multi-threading, based on pthread
 *          or QT library.
 *
 ****************************************************************************
 * @class   wosh::MutexRW
 * @brief   Multiplatform mutex used within multi-threading, based QT library.
 *          Read locks are shared, write lock are exclusive.
 *
 *
 ****************************************************************************
 *
 *
 ****************************************************************************
 * @version $Id: Mutex.h 2082 2010-03-11 06:26:42Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR MutexS;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_Mutex_H__
 #define __WOSH_Core_Mutex_H__

 #include <woshDefs.h>


#define _MUTEX_QT


///////////////////////////////////////////
#if defined(_MUTEX_QT)
# include <QMutex>
# include <QReadWriteLock>
///////////////////////////////////////////
#elif defined(_MUTEX_PTHREAD)
# include <pthread.h>
///////////////////////////////////////////
#elif defined(_MUTEX_WIN32)
# include <windows.h>
#endif
///////////////////////////////////////////


namespace wosh {


class Mutex  {

	public:
		/**
		 * @brief   Empty constructor. Initialize the Mutex
		 */
		Mutex() {
#if defined(_MUTEX_QT)
			// do nothing
#elif defined(_MUTEX_PTHREAD)
			pthread_mutex_init( &this->mux, NULL );
#elif defined(_MUTEX_WIN32)
			this->mux = ::CreateMutex( NULL, FALSE, NULL );
#endif
		 }

		/**
		 * @brief   Empty destructor. Destroy the Mutex
		 */
		virtual ~Mutex() {
#if defined(_MUTEX_QT)
			// do nothing
#elif defined(_MUTEX_PTHREAD)
			pthread_mutex_destroy( &this->mux );
#elif defined(_MUTEX_WIN32)
			if ( this->mux != NULL )
				::CloseHandle( this->mux );
#endif
		 }


	public:
		/**
		 * @brief   Lock the Mutex (does not check if Mutex was already locked)
		 * @note    check function pthread_mutex_lock() for more info
		 * @note    check function QMutex::lock() for more info
		 */
		inline void lock() const {
#if defined(_MUTEX_QT)
			this->mux.lock();
#elif defined(_MUTEX_PTHREAD)
			pthread_mutex_lock( &this->mux );
#elif defined(_MUTEX_WIN32)
			DWORD d = WaitForSingleObject( this->mux, INFINITE );
			(void)d;
#endif
		 }
		/**
		 * @brief   UnLock the Mutex (does not check if Mutex was locked)
		 * @note    check function pthread_mutex_unlock() for more info
		 * @note    check function QMutex::unlock() for more info
		 */
		inline void unLock() const {
#if defined(_MUTEX_QT)
			this->mux.unlock();
#elif defined(_MUTEX_PTHREAD)
			pthread_mutex_unlock( &this->mux );
#elif defined(_MUTEX_WIN32)
			::ReleaseMutex( this->mux );
#endif
		 }
		/**
		 * @brief   Try locking the Mutex and return response
		 * @note    check function pthread_mutex_trylock() for more info
		 */
		inline bool tryLock() const {
#if defined(_MUTEX_QT)
			return this->mux.tryLock();
#elif defined(_MUTEX_PTHREAD)
			return (bool)pthread_mutex_trylock( &this->mux );
#elif defined(_MUTEX_WIN32)
			NOT_IMPLEMENTED();
			return false;
#endif
		 }

	private:
		/**
		 * @brief   Empty private copy-constructor.
		 */
		Mutex( const Mutex& ) { }
		/**
		 * @brief   Empty private assignement-operator.
		 */
		const Mutex& operator=( const Mutex& ) {
			return *this;
		 }

	protected:
#if defined(_MUTEX_QT)
		mutable QMutex mux;			/// real, hidden mutex object (from QT library)
#elif defined(_MUTEX_PTHREAD)
		mutable pthread_mutex_t mux;
#elif defined(_MUTEX_WIN32)
		HANDLE mux;
#endif

}; // class def

/*
The CReadWriteLock class is a very simple class that implements the Reader/Writer pattern, used for synchronizing multiple thread access to a shared resource. By allowing either multiple reader access or single write access, this class provides a better throughput than standard synchronization mechanisms like Critical Sections.
Background

Looking for a simple solution on the net, I faced the fact that the solutions often were advanced, and not developed in the spirit of KISS (Keep It Simple S*****). Therefore, I decided to implement a simple yet functional solution.

Please note that in order to prevent reader starvation, the CReadWriteLock should only be used when reader access is far more common than writer access. The reason for this is that the implementation does not contain any thread queue or similar to secure the access order, and writer threads will always win this race.
*/

class MutexRW  {
	public:
		/**
		 * @brief   Empty constructor.
		 */
		MutexRW() {
#if defined(_MUTEX_QT)
			// dothing
#elif defined(_MUTEX_PTHREAD)
			pthread_rwlock_init(&this->rwLock);
#elif defined(_MUTEX_WIN32)
			this->readersCount = 0;
			InitializeCriticalSection(&this->secionLockWriter);
			InitializeCriticalSection(&this->secionReaderCount);
			// writer event with manual reset and default signaled state
			//  - Signaled = Writer has currently not access
			//  - Non-signaled = Writer has currently access, block readers
			this->eventWriter = CreateEvent( NULL, TRUE, TRUE, NULL );
			// no-readers event with manual reset and default signaled state
			//  - Signaled = No readers have currently access
			//  - Non-signaled = Some readers have currently access, block writer
			this->eventNoReadersm_hNoReadersEvent = CreateEvent(NULL, TRUE, TRUE, NULL);
#endif
		 }
		/**
		 * @brief   Empty destructor.
		 */
		virtual ~MutexRW() {
#if defined(_MUTEX_QT)
			// dothing
#elif defined(_MUTEX_PTHREAD)
			pthread_rwlock_destroy(&this->rwLock);
#elif defined(_MUTEX_WIN32)
			// free critical sections
			DeleteCriticalSection(&this->secionLockWriter);
			DeleteCriticalSection(&this->secionReaderCount);
			// close handles
			CloseHandle(this->eventWriter);
			CloseHandle(this->eventNoReaders);
#endif
		 }

	public:

		inline void lockForRead() const {
#if defined(_MUTEX_QT)
			this->rwLock.lockForRead();
#elif defined(_MUTEX_PTHREAD)
			pthread_rwlock_rdlock(&this->rwLock);
#elif defined(_MUTEX_WIN32)
			while(true) {
				// wait for Writer event to be signaled
				WaitForSingleObject(this->eventWriter, INFINITE);
				// increment readers
				changeReadersCount(true);
				// If writer is become non-signaled fall back (double locking)
				if ( WaitForSingleObject(this->eventWriter, 0) != WAIT_OBJECT_0 ) {
					// decrement readers
					changeReadersCount(false);
				 }
				else // done
					break;
			}
#endif
		 }

		inline void lockForWrite() const {
#if defined(_MUTEX_QT)
			this->rwLock.lockForWrite();
#elif defined(_MUTEX_PTHREAD)
			pthread_rwlock_wrlock(&this->rwLock);
#elif defined(_MUTEX_WIN32)
			// enter critical section (prevent more than one writer)
			EnterCriticalSection(&this->secionLockWriter);
			// wait for current writer
			WaitForSingleObject(this->eventWriter, INFINITE);
			// set writer to non-signaled
			ResetEvent(this->eventWriter);
			// wait for current readers to finish
			WaitForSingleObject(this->eventNoReaders, INFINITE); 
			// leave critical section
			LeaveCriticalSection(&this->secionLockWriter);
#endif
		 }

		/**
		 * @brief   UnLock the Mutex (does not check if Mutex was locked)
		 * @note    check function QMutex::unlock() for more info
		 */
		inline void unLock() const {
#if defined(_MUTEX_QT)
			this->rwLock.unlock();
#elif defined(_MUTEX_PTHREAD)
			pthread_rwlock_unlock(&this->rwLock);
#elif defined(_MUTEX_WIN32)
			if ( this->lockedWrite ) {
				// unLock Write: signal writer event
				SetEvent(this->eventWriter);
			 }
			else {
				// unLock Read: decrement readers
				changeReadersCount(false);
			 }
#endif
		 }



		inline bool tryLockForRead() const {
#if defined(_MUTEX_QT)
			return this->rwLock.tryLockForRead();
#elif defined(_MUTEX_PTHREAD)
			return (bool)pthread_rwlock_tryrdlock( &this->rwLock );
#elif defined(_MUTEX_WIN32)
			NOT_IMPLEMENTED( "" );
#endif
			return false;
		 }

		inline bool tryLockForWrite() const {
#if defined(_MUTEX_QT)
			return this->rwLock.tryLockForWrite();
#elif defined(_MUTEX_PTHREAD)
			return (bool)pthread_rwlock_trywrlock( &this->rwLock );
#elif defined(_MUTEX_WIN32)
			NOT_IMPLEMENTED();
#endif
			return false;
		 }

	private:
		/**
		 * @brief   Empty private copy-constructor.
		 */
		MutexRW( const MutexRW& ) { }
		/**
		 * @brief   Empty private assignement-operator.
		 */
		const MutexRW& operator=( const MutexRW& ) {
			return *this;
		 }

	protected:
#if defined(_MUTEX_QT)
		mutable QReadWriteLock rwLock;	///< real, hidden mutex object (from QT library)
#elif defined(_MUTEX_PTHREAD)
		mutable pthread_rwlock_t rwLock;
#elif defined(_MUTEX_WIN32)

		HANDLE eventWriter;			///< writer access event
		HANDLE eventNoReaders;		///< no readers event
		CRITICAL_SECTION secionLockWriter;	///< critical section for protecting lock writer method.
		CRITICAL_SECTION secionReaderCount;	///< critical section for protecting reader count
		int readersCount;			///< number of readers
		bool lockedWrite;			///< currently read/write access
		/// increment/decrement number of readers
		void changeReadersCount( bool increment ) {
			EnterCriticalSection(&this->secionReaderCount);
			if ( increment ) {
				// increase
				this->readersCount++;
				 ResetEvent(this->eventNoReaders);
			 }
			else {
				// decrease
				this->readersCount--;
				if( this->readersCount <= 0 ) {
					SetEvent(this->eventNoReaders);
				 }
			 }
			LeaveCriticalSection(&this->secionReaderCount);
		 }

#endif
}; // class def



//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////// LOCKERS - PURE MULTIPLATFORM

/**
 * @class   wosh::MutexLocker
 * @brief   Multiplatform mutex locker, provided for convenience.
 *
 * @version $Id: Mutex.h 2082 2010-03-11 06:26:42Z alex $
 * @author  Alessandro Polo
 */
class MutexLocker  {
	public:
		/**
		 * @brief   Create object and locks the passed Mutex.
		 * @param lockingMutex the Mutex to be used
		 */
		inline explicit MutexLocker( const Mutex* lockingMutex ) {
			this->mutex = NULL;
			if ( lockingMutex == NULL ) return;
			this->mutex = const_cast<Mutex*>(lockingMutex);
			this->mutex->lock();
		 }
		/**
		 * @brief   Destroy current object and unlocks the Mutex.
		 */
		virtual ~MutexLocker() {
			if ( this->mutex == NULL )
				return;
			this->mutex->unLock();
		 }
	protected:
		mutable Mutex* mutex;	///< reference to Mutex object.

}; // class def
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

/**
 * @class   wosh::MutexLockerRead
 * @brief   Multiplatform mutex locker, provided for convenience.
 *
 * @version $Id: Mutex.h 2082 2010-03-11 06:26:42Z alex $
 * @author  Alessandro Polo
 */
class MutexLockerRead  {
	public:
		/**
		 * @brief   Create object and locks the passed MutexRW.
		 * @param lockingMutex the MutexRW to be used
		 */
		inline explicit MutexLockerRead( const MutexRW* lockingMutex )
		 {
			this->mutex = NULL;
			if ( lockingMutex == NULL ) return;
			this->mutex = const_cast<MutexRW*>(lockingMutex);
			this->mutex->lockForRead();
		 }
		/**
		 * @brief   Destroy current object and unlocks the MutexRW.
		 */
		virtual ~MutexLockerRead() {
			if ( this->mutex == NULL )
				return;
			this->mutex->unLock();
		 }
	protected:
		mutable MutexRW* mutex;	///< reference to MutexRW object.

}; // class def
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

/**
 * @class   wosh::MutexLockerWrite
 * @brief   Multiplatform mutex locker, provided for convenience.
 *
 * @version $Id: Mutex.h 2082 2010-03-11 06:26:42Z alex $
 * @author  Alessandro Polo
 */
class MutexLockerWrite  {
	public:
		/**
		 * @brief   Create object and locks the passed MutexRW.
		 * @param lockingMutex the MutexRW to be used
		 */
		inline explicit MutexLockerWrite( const MutexRW* lockingMutex ) {
			this->mutex = NULL;
			if ( lockingMutex == NULL ) return;
			this->mutex = const_cast<MutexRW*>(lockingMutex);
			this->mutex->lockForWrite();
		 }
		/**
		 * @brief   Destroy current object and unlocks the MutexRW.
		 */
		virtual ~MutexLockerWrite()  {
			if ( this->mutex == NULL )
				return;
			this->mutex->unLock();
		 }
	protected:
		mutable MutexRW* mutex;		///< reference to MutexRW object.

}; // class def
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////


}; // namespace wosh

#endif //__WOSH_Core_Mutex_H__
