/**
 * @class   wosh::Notification
 * @brief   A special Event message containing text or information for user(s).
 *
 *
 *
 ****************************************************************************
 * @version $Id: Notification.h 2036 2010-03-04 16:09:59Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_Notification_H__
 #define __WOSH_Core_Notification_H__

 #include <woshDefs.h>
 #include <core/MessageContent.h>
 #include <core/URI.h>



namespace wosh {
 namespace database {
  class EncoderXML_template__Notification;
 };

#define _Notification_TYPE					"wosh::Notification"


class Notification : public MessageContent
 {
	WOSH_SERIALIZABLE_ABSTRACT(wosh::Notification)

	public:
		/// priority of the notification
		enum PRIORITY {
			PRIORITY_UNKNOWN = 0,
			PRIORITY_LOW = 1,
			PRIORITY_DEFAULT = 2,
			PRIORITY_HIGH = 4,
			PRIORITY_REALTIME = 8
		};

		/// type of the notification
		enum TYPE {
			TYPE_UNKNOWN = 0,
			TYPE_TEXTUAL = 1,
			TYPE_AUDIO = 2,
			TYPE_IMAGE = 4,
			TYPE_VIDEO = 8,
			TYPE_DATA = 16
		};

		enum PROTOCOL {
			PROTOCOL_UNKNOWN = 0,
			PROTOCOL_SHELL = 1,
			PROTOCOL_HUMAN = 2
		};

	public:
		/**
		 * @brief  Empty constructor.
		 */
		Notification();
		/**
		 * @brief  Copy constructor.
		 */
		Notification( const wosh::Notification& m );
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~Notification() { }


	public:
		virtual const std::string& getPlain() const = 0;
		virtual void setPlain( const std::string& value ) = 0;

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:
		long getID() const								{ return this->uid; }
		long getFlowID() const							{ return this->flow_uid; }

		long getTimeStamp() const						{ return this->timestamp; }
		std::string getTimeStampUTF() const;

		long getSentTimeStamp() const					{ return this->sent_timestamp; }
		long getReadTimeStamp() const					{ return this->read_timestamp; }

		bool isSent() const								{ return ( this->sent_timestamp > 0 ); }
		bool isRead() const								{ return ( this->read_timestamp > 0 ); }

		const std::string& getSender_ID() const			{ return this->sender_id; }
		std::string& getSender_ID()						{ return this->sender_id; }
		const std::string& getSender_User() const		{ return this->sender_user; }
		std::string& getSender_User()					{ return this->sender_user; }

		const std::string& getRecipent_ID() const		{ return this->recipent_id; }
		std::string& getRecipent_ID()					{ return this->recipent_id; }
		const std::string& getRecipent_User() const		{ return this->recipent_user; }
		std::string& getRecipent_User()					{ return this->recipent_user; }

		const std::string& getRecipent_Group() const	{ return this->recipent_group; }
		std::string& getRecipent_Group()				{ return this->recipent_group; }

		bool isForAnyone() const						{ return ((this->recipent_user == "") && (this->recipent_group == "")); }

		const std::string& getLanguage() const			{ return this->language; }
		std::string& getLanguage()						{ return this->language; }

		Notification::PROTOCOL getProtocol() const		{ return this->protocol; }
		std::string getProtocolAsString() const			{ return getProtocolAsString(this->protocol); }

		virtual MessageContent::TYPE getType() const	{ return MessageContent::NOTIFICATION; }

		virtual Notification::TYPE getNotificationType() const = 0;

		std::string getNotificationTypeAsString() const	{ return getNotificationTypeAsString(getNotificationType()); }

		Notification::PRIORITY getPriority() const		{ return this->priority; }
		std::string getPriorityAsString() const			{ return getPriorityAsString(this->priority); }

		static std::string getPriorityAsString( Notification::PRIORITY priority );
		static std::string getProtocolAsString( Notification::PROTOCOL protocol );
		static std::string getNotificationTypeAsString( Notification::TYPE type );

		const wosh::URI& getPreferredCommunicator() const	{ return this->preferredComm; }
		wosh::URI& getPreferredCommunicator()				{ return this->preferredComm; }

//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:
		void setFlowID( long uid )							{ this->flow_uid = uid; }

		void setSender_ID( const std::string& value )		{ this->sender_id = value; }
		void setSender_User( const std::string& value )		{ this->sender_user = value; }

		void setRecipent_ID( const std::string& value )		{ this->recipent_id = value; }
		void setRecipent_User( const std::string& value )	{ this->recipent_user = value; }

		void setRecipent_Group( const std::string& value )	{ this->recipent_group = value; }

		void setLanguage( const std::string& value ) 		{ this->language = value; }

		void setProtocol( Notification::PROTOCOL proto )	{ this->protocol = proto; }

		void setTimeStamp();
		void setTimeStamp( long value )						{ this->timestamp = value; }

		void setRead( bool value, long epoch = 0 );

		void setSent( bool value, long epoch = 0 );

		void setPriority( Notification::PRIORITY prior )	{ this->priority = prior; }

		void setPreferredCommunicator( const URI& uri )		{ this->preferredComm = uri; }


//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:
		virtual wosh::Notification& operator=(const wosh::Notification& m);
		virtual bool operator==(const wosh::Notification &other) const;
		virtual bool operator!=(const wosh::Notification &other) const { return !(*this == other);  }
//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const;
		virtual short compare( const wosh::ISerializable* other ) const;
//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:
		virtual void clear();
//@}
/** @name Internal vars
 * @{
 ******************************************************************************/
	protected:
		long uid;					///<
		long flow_uid;				///<
		long timestamp;				///< timestamp (epoch) of the notification

		Notification::PRIORITY priority; ///< priority of the notification

		std::string sender_id;		///< phone_number/account SENDING message
		std::string sender_user;	///< associated WOSH user (sender_id)

		std::string recipent_id;	///< phone_number/account RECEIVING message
		std::string recipent_user;	///< associated WOSH user (recipent_id)
		std::string recipent_group;	///< associated WOSH group (not used actually)

		long read_timestamp;		///<
		long sent_timestamp;		///<

		Notification::PROTOCOL protocol;	///<
		std::string language; // it_IT

		wosh::URI preferredComm;

		friend class wosh::database::EncoderXML_template__Notification;

//@}


}; // class def

}; // namespace wosh

#endif //__WOSH_Core_Notification_H__
