/**
 * @class   wosh::ObjectAllocator
 * @brief   
 *
 *

 * Manager allocators:
 *  - Static allocator (built-in)
 *  - Dynamic allocator (as dll plugins)
 *
 *  - Remote allocators ( build a wrapper to a remote object, =location transparency)
 *
 * Rules:
 *  - Objects are destroyed by the allocator which instantiated it
 *
 * Allocation by:
 *  - classname
 *  - interface [version]
 *
 *
usa ireflection per forza

 *
 *
 *
 ****************************************************************************
 * @version 0.8.161 $Id: ObjectAllocator.h 1993 2010-02-28 23:24:14Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR ThreadS;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_ObjectAllocator_H__
 #define __WOSH_Core_ObjectAllocator_H__

 #include <woshDefs.h>
 #include <core/Object.h>
 #include <core/LogContext.h>
 #include <core/Containers.h>
 #include <core/Serialization.h>
// #include <stdio.h>

namespace wosh {


#undef WOSH_REGISTER
#define WOSH_REGISTER( CLASSNAME, PARENT_INTERFACE, VERSION, TMP ) \
	ObjectTypeAdapterT<CLASSNAME> my_register_##TMP(PARENT_INTERFACE, VERSION ); 

#undef WOSH_REGISTER_INTERFACE
#define WOSH_REGISTER_INTERFACE( CLASSNAME, PARENT_INTERFACE, VERSION, TMP ) \
	ObjectTypeAdapterAbstract my_register_##TMP(#CLASSNAME, PARENT_INTERFACE, VERSION ); 
 
/*
concepts: 

- EACH represented class has a ObjectTypeAdapter 'static' instance.
- ObjectTypeAdapter holds object's classname (+version), parent interface
- when a leaf is added nad its parent is missing, an 'emtpy' type for parent is created
- so a full tree may be built. having pure-or-declared abstract classes returning NULL on create

*/

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

class ObjectTypeInfo : public ISerializable
 {
	WOSH_SERIALIZABLE(wosh::ObjectTypeInfo)
	WOSH_COMPARE_RTTI(wosh::ObjectTypeInfo)


	public:
		ObjectTypeInfo()
			: version(0.0), allocable(false) { }

		ObjectTypeInfo( const char* className, const char* parent_interface = "", double current_version = 0.0, bool abstract_class = false )
			: name(className), version(current_version), parent(parent_interface), allocable(!abstract_class) {}

		ObjectTypeInfo( const ObjectTypeInfo& m )
			: name(m.name), version(m.version), parent(m.parent), allocable(m.allocable) { }

		virtual ~ObjectTypeInfo() { }

	public:
		ObjectTypeInfo& operator=(const ObjectTypeInfo& m);
		bool operator==(const ObjectTypeInfo &other) const;
		bool operator!=(const ObjectTypeInfo &other) const { return !(*this == other);  }

/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const;
		virtual WRESULT store( IEncoderWriter* serializer ) const;
		virtual WRESULT retrieve( IEncoderReader* deserializer );
//@}
	public:
		std::string name;
		double version;
		std::string parent;
		bool allocable;

}; // class def

/////////////////////////////////////////////////////////////////////////////////////////////////////////////

class ObjectTypeAdapter : public ObjectTypeInfo {
	public:
		ObjectTypeAdapter()
			: ObjectTypeInfo() { }

		ObjectTypeAdapter( const char* className, const char* parent_interface = "", double current_version = 0.0, bool abstract_class = false )
			: ObjectTypeInfo(className, parent_interface, current_version, abstract_class) { }

		ObjectTypeAdapter( const ObjectTypeAdapter& m )
			: ObjectTypeInfo(m) { }

		virtual ~ObjectTypeAdapter() { }

	public:
		virtual wosh::IReflection* create() = 0;
		virtual void destroy( wosh::IReflection* object ) = 0;

		virtual wosh::ObjectTypeAdapter* clone() const = 0;

	protected:
		void registerMySelf();

}; // class def

/////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <typename ObjectType>
class ObjectTypeAdapterT : public ObjectTypeAdapter
 {
	WOSH_CLONE_CONSTRUCTOR(wosh::ObjectTypeAdapterT<ObjectType>)

	public:
		ObjectTypeAdapterT( const char* parent_interface = "", double current_version = 0.0 )
			: ObjectTypeAdapter(ObjectType::className(), parent_interface, current_version, false) {
			registerMySelf();
		}
		ObjectTypeAdapterT( const ObjectTypeAdapterT& m ) // DO NOT REGISTER ITSELF!!!
			: ObjectTypeAdapter(m) { }
		virtual ~ObjectTypeAdapterT() { }

	public:
		virtual ObjectType* create()					{ return new ObjectType(); }
		virtual void destroy( wosh::IReflection* object ) {
			if ( object == NULL ) return;
			delete object;
		 }

}; // class def

/////////////////////////////////////////////////////////////////////////////////////////////////////////////

class ObjectTypeAdapterAbstract : public ObjectTypeAdapter
{
	WOSH_CLONE_CONSTRUCTOR(wosh::ObjectTypeAdapterAbstract)

	public:
		ObjectTypeAdapterAbstract() // DO NOT REGISTER ITSELF!!!
			: ObjectTypeAdapter() { }
		ObjectTypeAdapterAbstract( const char* className, const char* parent_interface = "", double current_version = 0.0 )
			: ObjectTypeAdapter(className, parent_interface, current_version, true) { 
			registerMySelf();
		}
		ObjectTypeAdapterAbstract( const ObjectTypeAdapter& m ) // DO NOT REGISTER ITSELF!!!
			: ObjectTypeAdapter(m) { }
		ObjectTypeAdapterAbstract( const ObjectTypeAdapterAbstract& m ) // DO NOT REGISTER ITSELF!!!
			: ObjectTypeAdapter(m) { }
		virtual ~ObjectTypeAdapterAbstract() { }

	public:
		virtual wosh::IReflection* create()				{ return NULL; }
		virtual void destroy( wosh::IReflection* object ) {
			if ( object == NULL ) return;
			delete object;
		 }

}; // class def

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

class ObjectTypeNode {
	public:
		ObjectTypeNode() : info(NULL) { }
		ObjectTypeNode( ObjectTypeAdapter* obj ) : info(obj) { }
		~ObjectTypeNode() {
			if ( this->info != NULL )
				delete this->info;
		 }
	public:
		inline void setChild( const std::string& name ) {
			if ( isChild(name) ) return;
			this->children.push_back(name);
		 }
		void delChild( const std::string& name ) {
			std::vector<std::string>::iterator it;
			for( it=this->children.begin(); it!=this->children.end(); ++it ) {
				if ( *it != name ) continue;
				this->children.erase(it);
				return;
			 }
		 }
		bool isChild( const std::string& name ) const {
			std::vector<std::string>::const_iterator it;
			for( it=this->children.begin(); it!=this->children.end(); ++it ) {
				if ( *it == name ) return true;
			 }
			return false;
		 }
	public:
		ObjectTypeAdapter* info;
		std::vector<std::string> children;
};

class ObjectFactory;
class ObjectAllocator;
class Tree;
typedef List22<wosh::ObjectAllocator*> tObjectAllocatorList;

class ObjectAllocator : public Object
 {
	WOSH_SERIALIZABLE(wosh::ObjectAllocator)

	typedef List22<IReflection*> tIReflectionList;
	typedef Map22<std::string, ObjectTypeNode*> tStrObjectTypeNodeMap;


	public:
		ObjectAllocator();
		virtual ~ObjectAllocator();
	private:
		ObjectAllocator( const ObjectAllocator& ) : Object() { }

	public:
		// reentrant; true if classtype exists, is NOT abstract (=allocable), match version pattern (if !=0)
		virtual bool isSupported( const std::string& classname, double version_pattern = 0.0, bool allocable_only = false );

		template <class DYNAMIC_CAST>
		DYNAMIC_CAST* createTypeOf( const std::string& className, double version_pattern = 0.0 ) {
			return dynamic_cast<DYNAMIC_CAST*>(createTypeOf(className, version_pattern));
		 }

		// reentrant, cycles all allocators untill one matches, may return NULL
		virtual IReflection* createTypeOf( const std::string& className, double version_pattern = 0.0 );

		virtual WRESULT destroy( IReflection* object );

		WRESULT gatherInfoOf( std::vector<ObjectTypeInfo>& parent_list, const std::string& className, double version_pattern = 0.0 );

		void dump();

		WRESULT mergeTree( Tree& obj_info_tree, bool children_tree = true );
		void mergeTreeNodeChild( const std::string& classname, TreeNode* parentNode );
		void mergeTreeNodeParent( const std::string& classname, TreeNode* parentNode );

	public:
		WRESULT registerType( const ObjectTypeAdapter* info );

		WRESULT unRegisterType( const char* classname );

	protected:
		void dumpChildrenOf( const std::string& classname, FILE* outfile, int indent );
		void dumpParentOf( const std::string& classname, FILE* outfile, int indent );
		int validateNode_( const ObjectTypeAdapter* currNode, double version_pattern, bool allocable_only );
		ObjectTypeNode* findChildOf_( ObjectTypeNode* currentNode, double version_pattern, bool allocable_only );

	protected:
		tIReflectionList allocatedObjects;
		tStrObjectTypeNodeMap typesInfo;

		const tIReflectionList& getAllocatedObjects() const		{ return this->allocatedObjects; }
		tIReflectionList& getAllocatedObjects()					{ return this->allocatedObjects; }

		const tStrObjectTypeNodeMap& getObjectsInfo() const		{ return this->typesInfo; }
		tStrObjectTypeNodeMap& getObjectsInfo()					{ return this->typesInfo; }

		// meyer singleton, created on first use, destroyed by compiler
		// items are NOT free on destructor
		static tObjectAllocatorList& getInstances() {
			static tObjectAllocatorList allocators_instances;
			return allocators_instances;
		 }

		mutable LogContext Log;

		friend class wosh::ObjectFactory;
}; // class def


}; // namespace wosh

#endif //__WOSH_Core_ObjectAllocator_H__
