/** @file    SecurityManager.cpp
 *  @author  Alessandro Polo
 *  @version $Id: SecurityManager.cpp 2198 2010-03-23 12:03:07Z alex $
 *  @brief
 * File containing methods for the wosh::SecurityManager class.
 * The header for this class can be found in SecurityManager.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include <core/SecurityManager.h>

 #include <core/ObjectAllocator.h>
 #include <core/BusRing.h>
 #include <core/UserManager.h>
 #include <core/BusManager.h>
 #include <core/SystemError.h>
 #include <core/WoshKernel.h>
 #include <core/MessageProcessingCommon.h>
 #include <core/NEW_List.h>
 #include <core/Table.h>
 #include <core/Fact.h>


using namespace std;
namespace wosh {

// Gamma singleton! static reference is allocated and maintained by WoshKernel
extern "C" {
	SecurityManager* SecurityManager::getInstance() { 
		if ( WoshKernel::getInstance() == NULL ) return NULL;
		return &WoshKernel::getInstance()->security();
	}
}
 // register as abstract because it is a singleton:
 WOSH_REGISTER_INTERFACE(wosh::SecurityManager, "wosh::WoshModule", 0.01, _static_SecurityManager )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

SecurityManager::SecurityManager( const WoshKernel* parent ) : WoshModule(parent)
 {
	Object::setName( _SecurityManager_NAME );
	Object::getURI().fromString( _SecurityManager_URI );
	Object::getURI().registerGlobal();

	this->eventLogMaxSize = _SecurityManager_DEFAULT_EventLogMaxSize;
	Properties.set( _SecurityManager_KEY_EventLogMaxSize,	this->eventLogMaxSize, Permission::RW_RW_R__() );
	Properties.monitor(_SecurityManager_KEY_EventLogMaxSize);

	MethodRequest* mmEventsLog = Methods.create<MethodRequest>( _SecurityManager_METHOD_eventslog, "list Security' events", Permission::R_XR__R__() );
	mmEventsLog->setMethod( this, (MethodRequestPtr)&SecurityManager::mmDoEventsLog );

	MethodWRESULT* mmClearLog = Methods.create<MethodWRESULT>( _SecurityManager_METHOD_clear_log, "clear events log", Permission::R_XR__R__() );
	mmClearLog->setMethod( this, (MethodWRESULTPtr)&SecurityManager::clearEventsLog );

 }

SecurityManager::~SecurityManager()
 {
	Log(LOG_INFO, " Destroying.." );

	Log(LOG_VERBOSE, ":~SecurityManager() : Freeing EventsLog [%d]..", EventLog.size() );
	EventLog.clear();

 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////// KERNEL EVENTS

WRESULT SecurityManager::onKernelInitializing() {
	WRESULT ret = WoshModule::onKernelInitializing();

	Log(LOG_VERBOSE, " Initializing BusRing (wosh.Bus.Security).." );
	Bus* securityBus = new BusRing(_Bus_Security);
	securityBus->setParentObject(this);
	securityBus->setSharedLocal(false);
	securityBus->setSharedNetwork(true);
	WRESULT created_bus = WoshKernel::getInstance()->busses().registerBus(securityBus);
	if ( WFAILED(created_bus) )
		Log(LOG_CRITICAL, " FAILED#%d Creating BusRing (wosh.Bus.Security)", created_bus );
	else
		Log(LOG_INFO, " Created BusRing (wosh.Bus.Security)." );

	BusSecurity.setMessageHandler(this);
	WRESULT bus_d_connected = BusSecurity.connect( _Bus_Security );
	if ( WFAILED(bus_d_connected) )
		SystemError::raise(this, Log, SystemError::CRITICAL, 0, "init", "FAILED#%d Connecting Bus (wosh.Bus.Security)..", bus_d_connected );

	return ret + bus_d_connected;
 }

/////////////////////////////////////////////////////////////////////////////////////////////// KERNEL EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENTs

bool SecurityManager::updatingProperty( bool& do_update, const Variant& value_proposed, Property& property_current, const PropertiesProvider* source )
 {
	if ( property_current.getKey() == _SecurityManager_KEY_EventLogMaxSize ) {
		this->EventLog.transactionBeginWrite();
		this->eventLogMaxSize = value_proposed.toLong(this->eventLogMaxSize);
		this->EventLog.transactionEnd();
		return true;
	 }
	// forward processing to base class
	return WoshModule::updatingProperty(do_update, value_proposed, property_current, source);
 }

///////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENTs
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// STATIC METHODS

bool SecurityManager::compareMask( short mask_pattern, short operation )
 {
	///@todo using bitset

	if ( mask_pattern == 7 )
		return true;

	if ( mask_pattern == operation )
		return true;

	if ( operation == 1 ) {
		if ( mask_pattern == 3 || mask_pattern == 5 )
			return true;
		return false;
	 }
	if ( operation == 2 ) {
		if ( mask_pattern == 3 || mask_pattern == 6 )
			return true;
		return false;
	 }
	if ( operation == 4 ) {
		if ( mask_pattern == 5 || mask_pattern == 6 )
			return true;
		return false;
	 }
	return false;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool SecurityManager::evalAuthorization( const SecurityToken* security, const Permission* permissions, Permission::PERMISSION_TYPE operation )
 {
	if ( permissions == NULL ) {
		printf("SecurityManager::evalAuthorization() BUG? permissions == NULL" );
		return false;
	 }
	if ( security == NULL ) {
		printf("SecurityManager::evalAuthorization() BUG? security == NULL" );
		return false;
	 }

	// is root accessing?
	if ( security->getUser() == User::root || UserManager::isGroupOfUser( security->getUser(), "root" ) ) {
		// any of the mask validates
//		if ( compareMask( permissions->getMask(0), operation ) || compareMask( permissions->getMask(1), operation ) || compareMask( permissions->getMask(2), operation ) )
//			return true;
//		return false;
		if ( permissions->getMask(0) == Permission::None && permissions->getMask(1) == Permission::None && permissions->getMask(2) == Permission::None )
			return false;
		if ( operation != Permission::Read && permissions->getMask(0) == Permission::Read && permissions->getMask(1) == Permission::Read && permissions->getMask(2) == Permission::Read )
			return false;

		if ( compareMask( permissions->getMask(0), operation ) || compareMask( permissions->getMask(1), operation ) || compareMask( permissions->getMask(2), operation ) )
			return true;
		
		printf("SecurityManager::evalAuthorization() WARNING, root allowed BUT probably a permission [%s] BUG!\n", permissions->toString().c_str());
		return true;
	 }

	// is the owner?
	if ( security->getUser() == permissions->getOwner() ) {
		// check owner rights
		if ( compareMask( permissions->getMask(0), operation ) )
			return true;
	 }


	// check group rights
	if ( UserManager::isGroupOfUser( security->getUser(), permissions->getGroup() ) ) {
		if ( compareMask( permissions->getMask(1), (short)operation ) )
			return true;
	 }

	// check others rights
	if ( compareMask( permissions->getMask(2), (short)operation ) )
		return true;

	return false;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

SecurityToken* SecurityManager::generateToken( const Object* object )
 {
	if ( object == NULL ) return NULL;
	SecurityToken* st = new SecurityToken(object->getPermission().getOwner(), object->getEntityID());

	return st;
 }

SecurityToken* SecurityManager::generateToken( const std::string& user, const Object* object )
 {
	if ( object == NULL ) {
		SecurityToken* st = new SecurityToken(user, "");
		
		return st;
	 }
	SecurityToken* st = new SecurityToken(user, object->getEntityID());
	
	return st;
 }

////////////////////////////////////////////////////////////////////////////////////////////// STATIC METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT SecurityManager::clearEventsLog() {
	Log(LOG_VERBOSE, "clearEventsLog()" );
	this->EventLog.clear();
	return WRET_OK;
 }

Response* SecurityManager::mmDoEventsLog( const Request* request ) {
	if ( request == NULL ) return NULL;
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, ":mmDoEventsLog(%s)", option.c_str() );

	if ( option == "s" || option == "short" ) {
		this->EventLog.transactionBeginRead();
		NEW_List* objs = new NEW_List(this->EventLog.size());
		tMessageList::ConstIterator it;
		const tMessageList::ConstIterator it_end = this->EventLog.end();
		for ( it=this->EventLog.begin(); it!=it_end; ++it ) {
			if ( *it == NULL ) continue;
			if ( (*it)->isEmpty() ) continue;
			if ( !(*it)->getContent()->isFact() ) continue;
			objs->push_back<Variant>( (*it)->getContent()->asFact()->getEvent() );
		}
		this->EventLog.transactionEnd();
		return request->replyResponse(WRET_OK, objs);
	 }
	else {
		this->EventLog.transactionBeginRead();
		Table* tbl = new Table( this->EventLog.size(), 4, true );
		tbl->setTableName("EventLog");
		tbl->setHeaderHorizontal("TimeStamp", 0);
		tbl->setHeaderHorizontal("URI", 1);
		tbl->setHeaderHorizontal("Event", 2);
		tbl->setHeaderHorizontal("Data", 3);
		unsigned int iRow = 0;
		tMessageList::ConstIterator it;
		const tMessageList::ConstIterator it_end = this->EventLog.end();
		for ( it=this->EventLog.begin(); it!=it_end; ++it, ++iRow ) {
			if ( *it == NULL ) continue;
			if ( (*it)->isEmpty() ) continue;
			const Fact* fact = (*it)->getContent()->asFact();
			tbl->set( new Variant((*it)->getTimestamp(), Variant::DATETIME), iRow, 0);
			tbl->set( (*it)->getSource().clone(), iRow, 1);
			tbl->set<Variant>( fact->getEvent(), iRow, 2);
			if ( fact->hasData() )
				tbl->set( fact->getData()->clone(), iRow, 3);
			else
				tbl->set<Variant>( "NULL", iRow, 3);

		}
		this->EventLog.transactionEnd();
		return request->replyResponse(WRET_OK, tbl);
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

}; // namespace wosh
