/**
 * @class   wosh::SecurityManager
 * @brief   Static collector of application's threads, designed for diagnostic and stats.
 *
 *
 Identity and access management (IAM) remains a challenge in distributed systems.
 
 
Security tokens are used to prove one's identity electronically (as in the case of a customer trying to access their bank account). The token is used in addition to or in place of a password to prove that the customer is who they claim to be. The token acts like an electronic key to access something.


 Use brokered authentication with a security token
 (ref http://msdn.microsoft.com/en-us/library/aa480563.aspx)
 
 
 
 * \image html diagrams/SecurityManager.png "wosh::SecurityManager UML diagram"
 *
 
 
 
 
 
 *
 * \par Initialization:
 * \msc
SecurityManager,BusManager;
SecurityManager->SecurityManager [label="WoshModule::onKernelInitializing()", URL="\ref wosh::WoshModule::onKernelInitializing()" ];
|||;
SecurityManager=>BusManager [ label="create wosh.Bus.Security", URL="\ref wosh::BusManager::createBus()" ];
SecurityManager<=SecurityManager [ label="connect wosh.Bus.Security", URL="\ref wosh::BusConnector::connect()" ];
 * \endmsc
 *
 *
 * @ingroup CoreModules
 ****************************************************************************
 * @version 0.6.030 $Id: SecurityManager.h 2200 2010-03-23 16:23:37Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/*! \page page_core
 * \htmlonly <hr/> \endhtmlonly
 * \section page_core_security Security
 *
 * wosh::SecurityManager offers common security-related operations (such as
 * wosh::Permission matching) and generates wosh::SecurityToken objects which
 * are associated with wosh::Object and wosh::User actions (messages).
 *
 *
 *
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR UserS;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_SecurityManager_H__
 #define __WOSH_Core_SecurityManager_H__

 #include <woshDefs.h>
 #include <core/WoshModule.h>

 #include <core/Permission.h>
 #include <core/SecurityToken.h>



namespace wosh {

 #define _SecurityManager_NAME						"SecurityManager"
 #define _SecurityManager_TYPE						"wosh::SecurityManager"
 #define _SecurityManager_URI						":Security"

 #define _SecurityManager_KEY_EventLogMaxSize		"EventLogMaxSize"
 #define _SecurityManager_DEFAULT_EventLogMaxSize	400

 #define _SecurityManager_METHOD_eventslog			"eventslog"
 #define _SecurityManager_METHOD_clear_log			"clear_log"

class WoshKernel;
class WoshHost;

class SecurityManager : public WoshModule
 {
	WOSH_MODULE(wosh::SecurityManager)

	typedef List22<wosh::Message*> tMessageList;


	protected:
		SecurityManager( const WoshKernel* parent = NULL );

		WRESULT onKernelInitializing();

		friend class wosh::WoshKernel;

	public:
		virtual ~SecurityManager();


///@ todo how to do AUDITING (tracing which clients accessed which resources and how)


		static bool isTrustedHost( const WoshHost* )  { return false; }


//		static bool evalMessage( const Message& message, const WoshHost* fromHost, , const WoshHost* toHost );

//@}
/** @name Public static methods (for bundles)
 * @{
 ******************************************************************************/
	public:

		static bool compareMask( short mask_pattern, short operation );

		static bool evalAuthorization( const SecurityToken* security, const Permission* permissions, Permission::PERMISSION_TYPE operation );

		static SecurityToken* generateToken( const Object* object );
		static SecurityToken* generateToken( const std::string& user, const Object* object = NULL );
//@}
/** @name IPropertiesProviderListener interface
 * @{
 ******************************************************************************/
 	protected:
		virtual bool updatingProperty( bool& do_update, const Variant& value_proposed, Property& property_current, const PropertiesProvider* source );
//@}
/** @name Internal Mehods and Events management
 * @{
 ******************************************************************************/
	private:
		static SecurityManager* getInstance();

		void logEvent( const Message& message );

		Response* mmDoEventsLog( const Request* request );
		WRESULT clearEventsLog();

//@}
/** @name Internal vars
 * @{
 ******************************************************************************/
	private:
		BusConnector BusSecurity;

		tMessageList EventLog;
		long eventLogMaxSize;

//@}


}; // class def

}; // namespace wosh

#endif //__WOSH_Core_SecurityManager_H__
