/** @file    SharedMemory.cpp
 *  @author  Alessandro Polo
 *  @version $Id: SharedMemory.cpp 2084 2010-03-11 10:49:02Z alex $
 *  @brief
 * File containing methods for the wosh::SharedMemory class.
 * The header for this class can be found in SharedMemory.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <core/SharedMemory.h>

 #include <core/ObjectAllocator.h>
 #include <core/Request.h>
 #include <core/Response.h>
 #include <core/Table.h>
 #include <core/Functional.h>


using namespace std;
namespace wosh {

// Meyer singleton! static reference is allocated on first user, destroyed by compiler
// not thread safe
extern "C" {
	List22<SharedMemory*>& SharedMemory::getInstances() {
		static List22<SharedMemory*> instances;
		return instances;
	}
}

 WOSH_REGISTER(wosh::SharedMemory, "wosh::Object", 0.05, _static_SharedMemory)

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

SharedMemory::SharedMemory( const std::string& name, Object* parent ) : Thread(parent, "wosh.DSM.thread"), Object(parent)
 {
	this->info.id = Utilities::randomLong();
	setName(name);

	this->info.seniority = Utilities::std_time();///@todo randomize decimals
	this->info.mode = 2;
	this->info.clock = 0;
	this->primary = true;
	// initialize agent list with myself
	// (basically..assume I'm the (only one) primary and resolve conflicts later on election response,
	// this should be fine since myself and my data-set are definitely out-of-date respect to another living agent,
	// moreover I easily know I was in this use-case when eventually receiving election reply)
	this->agents.set( this->info.id, &this->info);

	this->transationsHistorySize = 100;

	Log.setIndent( 4 );
	Log.setContext( Utilities::format("%s#%ld", Object::getName().c_str(), this->info.id) );
	Log.setLevel( LOG_INFO );
Log.setLevel( LOG_VERBOSE );

	SharedMemory::getInstances().transactionBeginWrite();
	SharedMemory::getInstances().push_back(this);
	SharedMemory::getInstances().transactionEnd();
 }

SharedMemory::~SharedMemory() {
	Log(LOG_VERBOSE, ":~SharedMemory() : Destroying.." );

	SharedMemory::getInstances().transactionBeginWrite();
	SharedMemory::getInstances().pop(this);
	SharedMemory::getInstances().transactionEnd();

	this->agents.erase( this->info.id, false );

	Log(LOG_INFO, ": Destroyed." );
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////// DSM CONTROL

void SharedMemory::setName( const std::string& name ) { 
	Object::getURI().setKernelLocal();
	Object::getURI().clearPathName();
	Object::getURI().getPath().push_back( "DSM" );
	if ( name.empty() ) {
		Log(LOG_WARNING, ":setName() Empty name, defaulting to "_SharedMemory_NAME );
		Object::setName( _SharedMemory_NAME, this->info.id );
		Object::getURI().getPath().push_back( _SharedMemory_NAME );
	 }
	else {
		Object::setName( name );
		Object::getURI().getPath().push_back( name );
	 }
	Object::getURI().setName( Utilities::toString(this->info.id) );
	Log(LOG_INFO, ":setName(%s) Id: %ld, URI: %s", name.c_str(), this->info.id, Object::getURI().toString().c_str() );
 }

WRESULT SharedMemory::init( unsigned long timeout ) {
	Log(LOG_INFO, ":init(%u) : Initializing..", timeout );
	if ( !BusDSM.isConnected() ) {
		BusDSM.setMessageHandler(this);
		WRESULT bus_connected = BusDSM.connect( _Bus_DSM );
		if ( WFAILED(bus_connected) ) {
			Log(LOG_CRITICAL, ":init() : FAILED#%d Connecting Bus ("_Bus_DSM")..", bus_connected );
			return WRET_ERR_INTERNAL;
		 }
	 }
	// raise an Election (intrinsically asynch)
	doElectionRequest();

	Log(LOG_VERBOSE, ":deinit() : Startng intenal thread.." );
	WRESULT ret = startThread(timeout);

	// !! list is empty, can't be synch
	return ret;
 }

WRESULT SharedMemory::deinit( unsigned long timeout ) {
	Log(LOG_INFO, ":deinit(%u) : DeInitializing..", timeout );
	// broadcast death certificate
	TransactionAgentDeath* death_certificate = new TransactionAgentDeath(this);
	death_certificate->setDiedAgentID(this->info.id);
	sendToAgents( new Request(_SharedMemory_METHOD_transaction, death_certificate) );
	Log(LOG_VERBOSE, ":deinit() : Death-Certificate sent, disconnecting bus.." );
	// no need to wait for disconnection
	WRESULT bus_disconnected = BusDSM.disconnect();
	if ( WFAILED(bus_disconnected) )
		Log(LOG_CRITICAL, ":deinit() : FAILED#%d DisConnecting Bus ("_Bus_DSM")..", bus_disconnected );
	BusDSM.setMessageHandler(NULL);

	Log(LOG_VERBOSE, ":deinit() : Stopping intenal thread.." );
	WRESULT ret = quitThread(timeout);

	Log(LOG_VERBOSE, ":deinit() : Clearing agent list.." );
	this->agents.transactionBeginWrite();
	// increase clock
	this->info.clock = 0;
	this->agents.transactionEnd();

	return ret;
 }

///////////////////////////////////////////////////////////////////////////////////////////////// DSM CONTROL
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// THREAD CONTROL

bool SharedMemory::initializingThread() {
	Log(LOG_VERBOSE, ":initializingThread() : still nothing to do.." );
	return true;
 }

void SharedMemory::runThread() {
	Log(LOG_VERBOSE, ":runThread() : Entering transaction loop.." );

	while ( this->running ) {
		Thread::sleepForMSec(50);
		setThreadAlive();
//continue;
		this->transations.transactionBeginWrite();
		if ( this->transations.size() == 0 ) {
			this->transations.transactionEnd();
			Thread::sleepForMSec(100);
			continue;
		 }
		Transaction* tr = this->transations.front();
		if ( tr == NULL ) {
			this->transations.erase(this->transations.begin());
			this->transations.transactionEnd();
			continue;
		 }

		if ( tr->state == Transaction::STATE_QUEUED ) {
			// just added? fix internal stuff calling precommit
			WRESULT precommitted = tr->precommit();
			if ( WFAILED(precommitted) ) 
				Log(LOG_CRITICAL, ":runThread() : TR#%I64d Precommit FAILED#%i ", tr->getID(), precommitted );
		 }

		if ( tr->isNACKedBySomeone() ) { // NACKs >= 1
			tr->rollback();
			removeTransaction_(tr);
		 }
		else if ( tr->getResponseOf(this->info.id) == Transaction::NO_REPLY ) {
			Log(LOG_INFO, ":runThread() : ACKing #%I64d", tr->getID() );
			// evaluate the transaction, may we ACK IT?
			tr->getAcksMap()[this->info.id] = Transaction::ACKED;
			// even my own ack is updated from message (just like (n)acks from other agents)
			sendToAgents( tr->replyACK() );
		 }
		else if ( tr->isACKedByAll() ) { // tr->getAcksCount() == agents.count()
			tr->commit();
			removeTransaction_(tr);
		 }
		else if ( tr->isTimedOut() ) {
			tr->rollback();
			// all following transactions should be aborted
			removeTransaction_(tr);
		 }
		this->transations.transactionEnd();
	 }
	Log(LOG_VERBOSE, ":runThread() : Exited transaction loop.." );
 }

void SharedMemory::exitingThread() {
	// 
	Log(LOG_INFO, ":exitingThread() : Clearing transaction queue.." );
	this->transations.transactionBeginWrite();
	this->transations.free();
	this->transations.transactionEnd();
 }

////////////////////////////////////////////////////////////////////////////////////////////// THREAD CONTROL
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// TRANSACTIONs

void SharedMemory::removeTransaction_( Transaction* transaction ) {
	if ( transaction == NULL ) return;
	this->transations.pop(transaction);
	if ( this->transationsHistorySize == 0 )
		delete transaction;
	if ( this->transationsHistorySize <= this->transationsHistory.size() )
		this->transationsHistory.erase(this->transationsHistory.begin(), true);
	this->transationsHistory.push_back(transaction);
 }

WRESULT SharedMemory::evaluateTransactionRequest( const Message& message ) {
	if ( message.isEmpty() || !message.getContent()->isRequest() ) return WRET_ERR_PARAM;
	const Request* request = message.getContent()->asRequest();
	if ( !request->hasData() || request->getData()->as<Transaction>() == NULL ) {
		Log(LOG_WARNING, ":evaluateTransactionRequest(%u) : Invalid content!", message.getID() );
		return WRET_ERR_PARAM;
	 }
	// get the transaction (copy)
	Transaction* transaction = request->getData()->clone<Transaction>();
	if ( transaction == NULL ) {
		Log(LOG_CRITICAL, ":evaluateTransactionRequest(%u) : Invalid cloned content!", message.getID() );
		return WRET_ERR_PARAM;
	 }
	// fix internal fields
	transaction->setParent(this);
	transaction->state = Transaction::STATE_QUEUED;
	this->transations.transactionBeginWrite();
	// ensure it is not already in queue
	List22<Transaction*> ::ConstIterator it_f = std::find_if(this->transations.begin(), this->transations.end(),
		Functor_Validator<Transaction, int64>(transaction->getID(), &Transaction::getID ) );
	if ( it_f != this->transations.end() && (*it_f) != NULL ) {
		this->transations.transactionEnd();
		Log(LOG_CRITICAL, ":evaluateTransactionRequest(%u) : Transaction#%I64d is already queued!", message.getID(), transaction->getID() );
		delete transaction;
		return WRET_ERR_PARAM;
	 }
	Log(LOG_INFO, ":evaluateTransactionRequest() : Enqueuing %s..", transaction->toString().c_str() );
	// enqueue it in FIFO queue!
	this->transations.push_back( transaction );
	this->transations.transactionEnd();
	return WRET_OK;
 }

WRESULT SharedMemory::evaluateTransactionResponse( const Message& message ) {
	if ( message.isEmpty() || !message.getContent()->isResponse() ) return WRET_ERR_PARAM;
	long agent_id = Utilities::fromString<long>(message.getSource().getName(), 0);
	if ( agent_id == 0 ) return WRET_ERR_PARAM;
	const Response* response = message.getContent()->asResponse();
	if ( response == NULL || !response->hasData() ) {
		Log(LOG_WARNING, ":evaluateTransactionResponse(%ld) : Invalid content!", message.getID() );
		return WRET_ERR_PARAM;
	 }
	const Variant* vID = response->getData()->as<Variant>();
	if ( vID == NULL || !vID->isNumeric() ) return WRET_ERR_PARAM;
	int64 transaction_id = vID->toInt64(0);
	if ( transaction_id == 0 ) return WRET_ERR_PARAM;
	this->transations.transactionBeginWrite();
	List22<Transaction*> ::ConstIterator it_f = std::find_if(this->transations.begin(), this->transations.end(),
		Functor_Validator<Transaction, int64>(transaction_id, &Transaction::getID ) );
	if ( it_f == this->transations.end() || (*it_f) == NULL ) {
		this->transations.transactionEnd();
		Log(LOG_WARNING, ":evaluateTransactionResponse(%I64d) : Transaction not found!", transaction_id );
		return WRET_ERR_PARAM;
	 }
	if ( WSUCCEEDED(response->getReturnValue()) ) {
		Log(LOG_VERBOSE, ":evaluateTransactionResponse(%I64d) ACKed by %ld", transaction_id, agent_id );
		(*it_f)->getAcksMap()[agent_id] = Transaction::ACKED;
	 }
	else {
		Log(LOG_INFO, ":evaluateTransactionResponse(%I64d) NACKed by %ld", transaction_id, agent_id );
		(*it_f)->getAcksMap()[agent_id] = Transaction::NACKED;
	 }
	this->transations.transactionEnd();
	return WRET_OK;
}

//////////////////////////////////////////////////////////////////////////////////////////////// TRANSACTIONs
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////// SharedObject CALLBACKS

void SharedMemory::sobj_updated( SharedObject*, ISerializable* ) {

 }

////////////////////////////////////////////////////////////////////////////////////// SharedObject CALLBACKS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////// ELECTION

void SharedMemory::doElectionRequest() {
	this->agents.transactionBeginWrite();
	// increase clock
	int64 newclock = ++this->info.clock;
	// send decorated list of known agents including myself
	NEW_List* agent_list = new NEW_List( this->agents.size() );
	agent_list->fromMapValue(this->agents.getMap(), true);
	this->agents.transactionEnd();
	Log(LOG_INFO, ":doElectionRequest() : Clock %I64d, Agents[%i]", newclock, agent_list->size() );
	sendToAgents( new Request(_SharedMemory_METHOD_election, agent_list) );
 }

void SharedMemory::election_evaluate_agents_( const NEW_List& response_agents ) {
	//add/update agent when (clock > current_clock), ignore else
	NEW_List::ConstIterator it;
	const NEW_List::ConstIterator it_end = response_agents.end();
	for( it=response_agents.begin(); it!=it_end; ++it ) {
		if ( *it == NULL ) continue;
		const SharedMemoryInfo* ainfo = (*it)->as<SharedMemoryInfo>();
		if ( ainfo == NULL ) continue;
		SharedMemoryInfo* local_ainfo = this->agents.find(ainfo->id);
		if ( local_ainfo == NULL ) {
			Log(LOG_VERBOSE, ":election_evaluate_agents() : Added agent#%ld", ainfo->id );
			this->agents.set(ainfo->id, ainfo->clone());
		 }
		else if ( ainfo->clock > local_ainfo->clock ) {
			Log(LOG_VERBOSE, ":election_evaluate_agents() : Updated agent#%ld [old_clock=%I64d,new_clock=%I64d]", ainfo->id, local_ainfo->clock, ainfo->clock );
			*local_ainfo = *ainfo;
		 }
	 }
	//update primary flag evaluating seniority
	this->primary = true;
	tLongSharedMemoryInfoMap::ConstIterator ita;
	const tLongSharedMemoryInfoMap::ConstIterator ita_end = this->agents.end();
	for( ita=this->agents.begin(); ita!=ita_end; ++ita ) {
		if ( ita->second == NULL ) continue;
		if ( ita->second == &this->info ) continue;
		if ( ita->second->seniority > this->info.seniority ) continue;
		this->primary = false;
		Log(LOG_VERBOSE, ":election_evaluate_agents() : not primary" );
		break;
	 }
	if ( this->primary )
		Log(LOG_INFO, ":election_evaluate_agents() : Primary!" );
 }

void SharedMemory::OnElectionReceivedRequest( const NEW_List& response_agents, const Message& ) {
	this->agents.transactionBeginWrite();
	// increase clock
	int64 newclock = ++this->info.clock;
	// evaluate request-data as a Received Response (see further)
	election_evaluate_agents_( response_agents );
	Log(LOG_VERBOSE, ":OnElectionReceivedRequest() : Updated Clock %I64d, Agents[%i]", newclock, this->agents.size() );
	// send decorated list of known agents including myself
	NEW_List* agent_list = new NEW_List( this->agents.size() );
	agent_list->fromMapValue(this->agents.getMap(), true);
	this->agents.transactionEnd();
	Log(LOG_INFO, ":OnElectionReceivedRequest() : Replying.. Clock %I64d, Agents[%i]", newclock, agent_list->size() );
	sendToAgents( new Response(_SharedMemory_METHOD_election, agent_list) );
 }

void SharedMemory::OnElectionReceivedResponse( const NEW_List& response_agents, const Message& ) {
	this->agents.transactionBeginWrite();
	// increase clock
	int64 newclock = ++this->info.clock;
	// evaluate request-data as a Received Response (see further)
	election_evaluate_agents_( response_agents );
	Log(LOG_INFO, ":OnElectionReceivedResponse() : Updated Clock %I64d, Agents[%i]", newclock, this->agents.size() );
	this->agents.transactionEnd();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////// ELECTION
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////// BUS EVENTS

void SharedMemory::busMessage( const Message& message, const Bus* source ) {
	if ( message.isEmpty() ) return;

	if ( message.getContent()->isRequest() ) {
		const Request* request = message.getContent()->asRequest();
		
		if ( request->getMethod() == _SharedMemory_METHOD_election ) {
			// don't eval election request from myself:
			if ( message.getSourceUUID() == this->getEntityID() ) return; 
			// evaluate request-data as a Received Response
			// reply the (updated) decorated list
			if ( !request->hasData() || !request->getData()->isClass(_NEW_List_TYPE)) return;
			const NEW_List* agent_list = request->getData()->as<NEW_List>();
			if ( agent_list == NULL ) return;
			OnElectionReceivedRequest( *agent_list, message);
			return;
		 }
		if ( request->getMethod() == _SharedMemory_METHOD_transaction ) {
			// sounds like a new transaction has started..
			evaluateTransactionRequest(message);
			return;
		 }

		// - a standard WOSH request, mostly debug stuff (not from DSMs)
		if ( request->getMethod() == _SharedMemory_METHOD_get_agents ) {
			Message* reply = message.replyTo( request->replyResponse(WRET_OK, dumpAgents()) ) ;
			BusDSM.postMessage(reply);
		 }
		return;
	 }

	if ( message.getContent()->isResponse() ) {
		const Response* response = message.getContent()->asResponse();
		
		if ( response->getMethod() == _SharedMemory_METHOD_election ) {
			// don't eval election response from myself:
			if ( message.getSourceUUID() == this->getEntityID() ) return; 
			// add/update agent when (clock > current_clock), ignore else
			// update primary flag evaluating seniority
			if ( !response->hasData() || !response->getData()->isClass(_NEW_List_TYPE)) return;
			const NEW_List* agent_list = response->getData()->as<NEW_List>();
			if ( agent_list == NULL ) return;
			OnElectionReceivedResponse( *agent_list, message );
			return;
		 }
		if ( response->getMethod() == _SharedMemory_METHOD_transaction ) {
			// a transaction was (N)ACKed by someone?
			evaluateTransactionResponse(message);
			return;
		 }
		return;
	 }

 }

////////////////////////////////////////////////////////////////////////////////////////////////// BUS EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// HELPERS

void SharedMemory::sendToAgents( MessageContent* content ) {
	Message* message = new Message(content);
	message->setSource(this);
	message->setDestinationBus(_Bus_DSM);
	message->setDestination(this->getURI());
	message->getDestination().setKernelAny();
	message->getDestination().setName(_URI_WILDCHAR);
	BusDSM.postMessage(message);
 }

// dump CURRENT agent-list to a table, data may be used to evaluate synch-status (using lamport clocks)
Table* SharedMemory::dumpAgents() {
	this->agents.transactionBeginRead();
	Table* tbl = new Table(this->agents.size()+1, 3);
	// uri, id, vector clock
	unsigned int iRow = 1;
	tbl->set( getURI().clone(), 0, 0 );
	tbl->set<Variant>( this->info.id, 0, 1 );
	tbl->set<Variant>( this->info.clock, 0, 2 );
	tLongSharedMemoryInfoMap::ConstIterator it;
	const tLongSharedMemoryInfoMap::ConstIterator it_end = this->agents.end();
	for( it=this->agents.end(); it!=it_end; ++it, ++iRow ) {
		if ( it->second == NULL ) continue;
		tbl->set<Variant>( it->first, iRow, 0 );
		tbl->set<Variant>( it->second->id, iRow, 1 );
		tbl->set<Variant>( it->second->clock, iRow, 2 );
	 }
	this->agents.transactionEnd();
	return tbl;
 }

///////////////////////////////////////////////////////////////////////////////////////////////////// HELPERS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

}; // namespace wosh
