/**
 * @class   wosh::SharedMemory
 * @brief   
 *
 ****************************************************************************
 * @version $Id: SharedMemory.h 2209 2010-03-25 17:19:12Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_SharedMemory_H__
 #define __WOSH_Core_SharedMemory_H__

 #include <woshDefs.h>
 #include <core/Bus.h>
 #include <core/BusConnector.h>
 #include <core/LogContext.h>
 #include <core/Containers.h>
 #include <core/MessageQueue.h>
 #include <core/Thread.h>
 #include <core/NEW_List.h>

 #include <core/SharedObject.h>
 #include <core/SharedTransaction.h>

namespace wosh {

 #define _SharedMemory_NAME					"wosh.SharedMemory"
 #define _SharedMemory_TYPE					"wosh::SharedMemory"

 #define _SharedMemory_METHOD_get_agents	"get_agents"

 #define _SharedMemory_METHOD_election		"election"
 
 #define _SharedMemory_METHOD_transaction	"transaction"


class Table;
class Message;
class Response;
class Request;


class SharedMemoryInfo : public virtual ISerializable
 {
	WOSH_SERIALIZABLE(wosh::SharedMemoryInfo)
	WOSH_COMPARE_RTTI(wosh::SharedMemoryInfo)

	public:
		long id;
		double seniority;
		int64 clock;
		short mode;
		long lastSeen;
		bool died;

	public:
		SharedMemoryInfo() : id(0), seniority(0), clock(0), mode(0), lastSeen(0) { }
		SharedMemoryInfo(const SharedMemoryInfo& m) {
			*this = m;
		 }
		SharedMemoryInfo& operator=(const SharedMemoryInfo& m) {
			if (this == &m) return *this; // same object?
			this->id = m.id;
			this->seniority = m.seniority;
			this->clock = m.clock;
			this->mode = m.mode;
			this->lastSeen = m.lastSeen;
			return *this;
		 }
		virtual bool operator==(const SharedMemoryInfo& other) const {
			if ( this->id != other.id ) return false;
			return true;
		 }
		virtual bool operator!=(const SharedMemoryInfo& other) const	{ return !(*this == other);  }

	public:
		std::string toString() const { return ""; }
		WRESULT store( IEncoderWriter* ) const { return WRET_ERR_PARAM; }
		WRESULT retrieve( IEncoderReader* ) { return WRET_ERR_PARAM; }

}; // class def


typedef Map22<long, SharedMemoryInfo*> tLongSharedMemoryInfoMap;
typedef List22<Transaction*> tTransactionList;
typedef Map22<std::string, SharedObject*> tStrSharedObjectMap;


class SharedMemory :	public Thread,
						public Object,
						public virtual ISharedObjectListener,
						public virtual IBusListener
 {
	WOSH_CLASSNAME(wosh::SharedMemory)
	WOSH_CLASS_ABSTRACT(false)


	public:
								enum SharedMemory_MODE {
									MODE_UNKNOWN	= 0x00,
									MODE_READ		= 0x01, ///< ignore any local writes/update
									MODE_WRITE		= 0x02, ///< ignore any remote writes/updates
									MODE_RW			= 0x03  ///< monitor local/remote writes/updates
								 };

								enum SharedMemory_PROPAGATION {
									PROPAGATION_UNKNOWN				= 0x00,

									PROPAGATION_PUSH				= 0x01, ///<
									PROPAGATION_PUSH_NOTIFICATION	= 0x03, ///<
									PROPAGATION_PUSH_DATA			= 0x05, ///<
									PROPAGATION_PUSH_OPERATION		= 0x09, ///<

									PROPAGATION_PULL_DATA			= 0x0F	///<
								 };


	public:
		SharedMemory( const std::string& name = "", Object* parent = NULL );
		virtual ~SharedMemory();

		static List22<SharedMemory*>& getInstances();


		void setName( const std::string& name );

/** @name Flow control
 * @{
 ******************************************************************************/
	public:
		// connect to bus and raise an election
		WRESULT init( unsigned long timeout = LIMIT_ULONG_MAX );
		// broadcast death certificate and disconnect bus
		WRESULT deinit( unsigned long timeout = LIMIT_ULONG_MAX );

		// force a TOTAL DSM and tuple space synchronization!
		//WRESULT invalidate();

		
		void doElectionRequest();

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:
		inline long getID() const							{ return this->info.id; }
		inline double getSeniority() const					{ return this->info.seniority; }

		inline int64 getClock() const						{ MutexLockerRead ml(this->agents.getMutex()); return this->info.clock; }
		inline short getMode() const						{ MutexLockerRead ml(this->agents.getMutex()); return this->info.mode; }
		inline bool isPrimary() const						{ MutexLockerRead ml(this->agents.getMutex()); return this->primary; }

		inline SharedMemoryInfo getInfo() const				{ MutexLockerRead ml(this->agents.getMutex()); return this->info; }

		const tTransactionList& getTransactions() const		{ return this->transations; }
		tTransactionList& getTransactions()					{ return this->transations; }

		const tTransactionList& getTransactionsHistory() const	{ return this->transationsHistory; }
		int getTransactionsHistorySize() const					{ return this->transationsHistorySize; }

		const tLongSharedMemoryInfoMap& getAgents() const	{ return this->agents; }
		tLongSharedMemoryInfoMap& getAgents()				{ return this->agents; }

//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:

//@}
/** @name Data manipulation
 * @{
 ******************************************************************************/
	public:
		const tStrSharedObjectMap& getDataSet() const	{ return this->dataset; }
		tStrSharedObjectMap& getDataSet()				{ return this->dataset; }

		template<class CONTAINER>//, class OBJECT>
		WRESULT acquireDataSet( CONTAINER& std_container ) {
			typename CONTAINER::Iterator it;
			const typename CONTAINER::ConstIterator it_end = std_container.end();
			for(it=std_container.begin(); it!=it_end; ++it) {
				this->dataset.set("asd", *it);
			 }
			return WRET_OK;
		 }
//@}
/** @name IBusListener interface
 * @{
 ******************************************************************************/
	public:
		virtual void busMessage( const Message& message, const Bus* source );
//@}
/** @name ISharedObjectListener interface
 * @{
 ******************************************************************************/
	public:
		virtual void sobj_updated( SharedObject* source, ISerializable* update_data );
//@}
/** @name Transaction managements
 * @{
 ******************************************************************************/
	protected:
		void removeTransaction_( Transaction* transaction );
		WRESULT evaluateTransactionRequest( const Message& message );
		WRESULT evaluateTransactionResponse( const Message& message );
//@}
/** @name Agent Election methods
 * @{
 ******************************************************************************/
	protected:
		void OnElectionReceivedRequest( const NEW_List& response_agents, const Message& message );
		void OnElectionReceivedResponse( const NEW_List& response_agents, const Message& message );
		void election_evaluate_agents_( const NEW_List& response_agents );
//@}
/** @name Internal helpers
 * @{
 ******************************************************************************/
	protected:
		void sendToAgents( MessageContent* content );
		Table* dumpAgents();
//@}
	protected:

		bool initializingThread();
		void runThread();
		void exitingThread();


	public:
//		void queryAgents();


	public://protected:
		BusConnector BusDSM;
		LogContext Log;

		SharedMemoryInfo info;
		bool primary;
						SharedMemory_PROPAGATION propagation;

		tLongSharedMemoryInfoMap agents;

		tStrSharedObjectMap dataset;

		tTransactionList transations;
		tTransactionList transationsHistory;
		int transationsHistorySize;

	private:
		SharedMemory(const SharedMemory&) : Thread(NULL), Object() { }	///< private copy constructor (object non-copiable)
		SharedMemory& operator=(const SharedMemory&) { return *this; }	///< private assigment operator (object non-copiable)

}; // class def

}; // namespace wosh

#endif //__WOSH_Core_SharedMemory_H__
