/** @file    SystemError.cpp
 *  @author  Alessandro Polo
 *  @version $Id: SystemError.cpp 2167 2010-03-18 06:41:05Z alex $
 *  @brief
 * File containing methods for the wosh::SystemError class.
 * The header for this class can be found in SystemError.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <core/SystemError.h>
 
 #include <core/ObjectAllocator.h>
 #include <core/SystemMonitor.h>
 #include <core/Message.h>
 #include <core/Object.h>
 #include <core/LogContext.h>
 #include <core/Table.h>

 #include <stdarg.h>

using namespace std;
namespace wosh {

 WOSH_REGISTER(wosh::SystemError, "wosh::MessageContent", 1.01, _static_SystemError)

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

SystemError::SystemError() : MessageContent()
 {
	clear();
 }

SystemError::SystemError( long errorCode ) : MessageContent()
 {
	clear();
	this->error_code = errorCode;
 }

SystemError::SystemError( long errorCode, const std::string& Description, PRIORITY Priority ) : MessageContent()
 {
	clear();
	this->error_code = errorCode;
	this->description = Description;
	this->priority = Priority;
 }

SystemError::SystemError(const SystemError& m) : MessageContent( (const MessageContent&)m )
 {
	clear();
	*this = m;
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SystemError::clear()
 {
	this->uid = Utilities::randomLong();
	this->timestamp = Utilities::std_time();
	this->priority = SystemError::UNKNOWN;

	this->error_code = 0;
	this->method = "";

	this->description = "";
 }

/////////////////////////////////////////////////////////////////////////////////////////////////// OPERATORS

SystemError& SystemError::operator=(const SystemError& m)
{
	if (this == &m) return *this; // same object?

	this->uid = m.uid;
	this->timestamp = m.timestamp;
	this->priority = m.priority;

	this->source = m.source;

	this->error_code = m.error_code;
	this->method = m.method;

	this->description = m.description;

	return *this;
}

bool SystemError::operator==(const SystemError& other) const
 {
	if ( this->uid == other.uid )
		return false;
	if ( this->error_code == other.error_code )
		return false;
	if ( this->method == other.method )
		return false;
	if ( this->priority == other.priority )
		return false;
	if ( this->description == other.description )
		return false;

	return true;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

wosh::Table* SystemError::getAsTable() const
 {
	Table* tProps = new Table( 7, 2, true );
	tProps->setTableName("Properties of SystemError#" + Utilities::toString(this->uid) );
	tProps->setHeaderHorizontal("Name", 0);
	tProps->setHeaderHorizontal("Value", 1);
	unsigned int iRow = 0;
	tProps->set<Variant>("ID", iRow, 0);
	tProps->set<Variant>(this->uid, iRow, 1);
	++iRow;
	tProps->set<Variant>("TimeStamp", iRow, 0);
	tProps->set( new Variant(this->timestamp, Variant::DATETIME), iRow, 1);
	++iRow;
	tProps->set<Variant>("Priority", iRow, 0);
	tProps->set<Variant>( getPriorityAsString(this->priority), iRow, 1);
	++iRow;
	tProps->set<Variant>("Source URI", iRow, 0);
	tProps->set( this->source.clone(), iRow, 1);
	++iRow;
	tProps->set<Variant>("Error Code", iRow, 0);
	tProps->set<Variant>( this->error_code, iRow, 1);
	++iRow;
	tProps->set<Variant>("Method", iRow, 0);
	tProps->set<Variant>( this->method, iRow, 1);
	++iRow;
	tProps->set<Variant>("Description", iRow, 0);
	tProps->set<Variant>( this->description, iRow, 1);
	++iRow;

	return tProps;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

std::string SystemError::getBrief( unsigned int max ) const
 {
	if ( this->description.empty() ) return "";
	if ( this->description.size() < max ) return this->description;
	return this->description.substr(0, max-2) + "..";
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

std::string SystemError::toString() const
 {
	return "";
 }

WRESULT SystemError::store( IEncoderWriter* serializer ) const {
	if ( serializer == NULL ) return WRET_ERR_PARAM;
	WRESULT ret = WRET_OK;
	return ret;
 }

WRESULT SystemError::retrieve( IEncoderReader* deserializer ) {
	if ( deserializer == NULL ) return WRET_ERR_PARAM;
	WRESULT ret = WRET_OK;
	return ret;
 }
 
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

long SystemError::raise( const Object* source, SystemError::PRIORITY err_priority, long err_code, const string& method_name, LogContext* log )
 {
	SystemError* sysError = new SystemError();
	sysError->method = method_name;
	sysError->priority = err_priority;
	sysError->error_code = err_code;

	if ( source != NULL )
		sysError->source = source->getURI();

	long err_id = sysError->uid;

	LogLevel type = LOG_UNKNOWN;
	switch(err_priority) {
		case SystemError::INFO:		type = LOG_INFO; break;
		case SystemError::WARNING:	type = LOG_WARNING; break;
		case SystemError::CRITICAL:	type = LOG_CRITICAL; break;
		default:					type = LOG_VERBOSE; break;
	 }
	if ( log != NULL )
		log->operator()(type, "SystemError#%ld Raised!", err_id );

	SystemMonitor::raiseError( sysError, source );

	return err_id;
 }

long SystemError::raise( const Object* source, LogContext& log, SystemError::PRIORITY err_priority, long err_code, const string& method_name, const char* stringFormat, ... )
 {
	SystemError* sysError = new SystemError();
	sysError->method = method_name;
	sysError->priority = err_priority;
	sysError->error_code = err_code;

	if ( source != NULL )
		sysError->source = source->getURI();

	long err_id = sysError->uid;

	char msgbuffer[512];
	int offset = method_name.size();
	memset(msgbuffer,'\0',512);

	if ( offset > 0 ) {
		strncpy(msgbuffer, ":", 1 );
		strncpy(msgbuffer+1, method_name.c_str(), method_name.size() );
		++offset;
		strncpy(msgbuffer+offset, "() ", 3 );
		offset += 3;
	 }

	va_list list;
	va_start(list,stringFormat);
#ifdef _OS_WINCE
	vsprintf(msgbuffer+offset,stringFormat, list);
#elif defined(WIN32_VS)
	vsprintf_s(msgbuffer+offset, 512-offset, stringFormat, list);
#else
	vsprintf(msgbuffer+offset,stringFormat, list);
#endif
	va_end(list);

	LogLevel type = LOG_UNKNOWN;
	switch(err_priority) {
		case SystemError::INFO:		type = LOG_INFO; break;
		case SystemError::WARNING:	type = LOG_WARNING; break;
		case SystemError::CRITICAL:	type = LOG_CRITICAL; break;
		default:					type = LOG_VERBOSE; break;
	 }

	log(type, msgbuffer );
	sysError->description = string(msgbuffer+offset);

	SystemMonitor::raiseError( sysError, source );
	return err_id;
 }


//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Message* SystemError::createMessage( Object* object, long error_code, PRIORITY priority, const string& description )
 {
	SystemError* sysError = new SystemError(error_code, description, priority);

	Message* msg_system = new Message();
	msg_system->setSource( object );
	msg_system->setContent( sysError );
	msg_system->setDestinationBroadcast();

	return msg_system;
 }

std::string SystemError::getPriorityAsString( SystemError::PRIORITY priority )
 {
	switch(priority) {
		case SystemError::INFO:		return "INFO";
		case SystemError::WARNING:	return "WARNING";
		case SystemError::CRITICAL:	return "CRITICAL";
		default:					return "UNKNOWN";
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

}; // namespace wosh
