/**
 * @class   wosh::SystemError
 * @brief   The class holds information about a generic system error.
 *          SystemError may be easily raised using raise() method.
 *
 ****************************************************************************
 * @version $Id: SystemError.h 2036 2010-03-04 16:09:59Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_SystemError_H__
 #define __WOSH_Core_SystemError_H__

 #include <core/MessageContent.h>
 #include <core/URI.h>

 
namespace wosh {

 #define _SystemError_TYPE					"wosh::SystemError"

 namespace database {
  class EncoderXML_template__SystemError;
 };

class Message;
class Object;
class LogContext;
class Table;

class SystemError : public MessageContent
 {
	WOSH_SERIALIZABLE(wosh::SystemError)
	WOSH_COMPARE_RTTI(wosh::SystemError)

	public:
		enum PRIORITY {
			UNKNOWN = 0,
			INFO = 1,
			WARNING = 2,
			CRITICAL = 3
		};

/*
		enum FAULT_TYPE {
			FAULT_UNKNOWN = 0,		///<
			FAULT_TRANSIENT = 1,	///< occur once and disappear
			FAULT_INTERMITTEN = 2,	///< 
			FAULT_PERMANENT = 4		///< service need to be repaired
		};
*/

	public:
		/**
		 * @brief  Empty constructor.
		 */
		SystemError();
		/**
		 * @brief  Constructor with initializer(s), provided for convenience.
		 * @param [in] errorCode the code of the error be assigned
		 */
		SystemError( long errorCode );
		/**
		 * @brief  Constructor with initializer(s), provided for convenience.
		 * @param [in] errorCode the code of the error be assigned
		 * @param [in] description the description of the error
		 * @param [in] Priority the priority of the error
		 */
		SystemError( long errorCode, const std::string& description, PRIORITY Priority = SystemError::INFO );
		/**
		 * @brief  Copy constructor.
		 */
		SystemError(const SystemError& m);
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~SystemError() { }


	public:
		virtual void clear();

	public:

		static long raise( const wosh::Object* source, wosh::SystemError::PRIORITY priority, long error_code, const std::string& method_name, wosh::LogContext* log = NULL );

		static long raise( const wosh::Object* source, wosh::LogContext& log, SystemError::PRIORITY priority, long error_code, const std::string& method_name, const char* stringFormat, ... );

	public:

		static Message* createMessage( wosh::Object* object, long error_code, wosh::SystemError::PRIORITY priority, const std::string& description );

		static std::string getPriorityAsString( SystemError::PRIORITY priority );


/** @name Getters
 * @{
 ******************************************************************************/
	public:

		long getID() const							{ return this->uid; }

		PRIORITY getPriority() const				{ return this->priority; }

		long getTimeStamp() const					{ return this->timestamp; }

		const wosh::URI& getSource() const			{ return this->source; }
		wosh::URI& getSource()						{ return this->source; }
	
		long getErrorCode() const					{ return this->error_code; }

		const std::string& getMethod() const		{ return this->method; }
		std::string& getMethod()					{ return this->method; }

		const std::string& getDescription() const	{ return this->description; }
		std::string& getDescription()				{ return this->description; }

		std::string getBrief( unsigned int max ) const;

		wosh::Table* getAsTable() const;

		/**
		 * @brief  Get MessageContent::TYPE of this implementation: a SYSTEM
		 * @return MessageContent::SYSTEM
		 */
		virtual MessageContent::TYPE getType() const { return MessageContent::SYSTEM; }

//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:

		void setPriority( SystemError::PRIORITY value )	{ this->priority = value; }

		void setErrorCode( long value )					{ this->error_code = value; }

		void setMethod( const std::string& value )		{ this->method = value; }

		void setDescription( const std::string& value )	{ this->description = value; }

//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:

		virtual SystemError& operator=(const SystemError& m);

		virtual bool operator==(const SystemError &other) const;
		virtual bool operator!=(const SystemError &other) const		{ return !(*this == other);  }

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const;
 		virtual WRESULT store( IEncoderWriter* serializer ) const;
		virtual WRESULT retrieve( IEncoderReader* deserializer );
//@}
	protected:
		long uid;
		long timestamp;

		wosh::SystemError::PRIORITY priority;

		wosh::URI source;

		long error_code;
		std::string method;

		std::string description;

		friend class wosh::database::EncoderXML_template__SystemError;

}; // class def

}; // namespace wosh

#endif //__WOSH_Core_SystemError_H__
