/** @file    SystemLogger.cpp
 *  @author  Alessandro Polo
 *  @version $Id: SystemLogger.cpp 1998 2010-03-01 06:51:42Z alex $
 *  @brief
 * File containing methods for the wosh::SystemLogger class.
 * The header for this class can be found in SystemLogger.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include <core/SystemLogger.h>

 #include <core/ObjectAllocator.h>
 #include <core/WoshKernel.h>
 #include <core/SystemError.h>
 #include <core/MethodsDef.h>
 #include <core/Table.h>
 #include <core/NEW_List.h>
 #include <core/Functional.h>
 #include <core/Utilities.h>

 #include <core/LogContext.h>
 #include <core/LogHandlers.h>


using namespace std;
namespace wosh {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Meyer singleton! static reference is allocated on first user
// not thread safe
extern "C" {
	SystemLogger& SystemLogger::getInstance() {
		static SystemLogger* ans = new SystemLogger( WoshKernel::getInstance() );
		return *ans;
	}
}
 // register as abstract because it is a singleton:
 WOSH_REGISTER_INTERFACE(wosh::SystemLogger, "wosh::WoshModule", 0.01, _static_SystemLogger )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

SystemLogger::SystemLogger( const WoshKernel* parent ) : WoshModule(parent)
 {
	Object::setName( _SystemLogger_NAME );
	Object::getURI().fromString( _SystemLogger_URI );
	Object::getURI().registerGlobal();

//	Properties.set( _SystemLogger_KEY_Count, 0, Permission( Permission::Read ) );

	//MethodRequest* mmListAllocators = Methods.create<MethodRequest>( _METHOD_List, "Get ", Permission::R_XR_XR__() );
	//mmListAllocators->setMethod( this, (MethodRequestPtr)&SystemLogger::mmDoListAllocators );

	MethodWRESULT* mmDump = Methods.create<MethodWRESULT>( "dump", "Get ", Permission::R_XR_XR__() );
	mmDump->setMethod( this, (MethodWRESULTPtr)&SystemLogger::mmDoDump);

	MethodRequest* mmGetContexts = Methods.create<MethodRequest>( _METHOD_List, "Get Contexts", Permission::R_XR_XR__() );
	mmGetContexts->setMethod( this, (MethodRequestPtr)&SystemLogger::mmDoGetContexts );

	MethodRequest* mmGetHandlers = Methods.create<MethodRequest>( _SystemLogger_METHOD_handlers, "Get Handlers", Permission::R_XR_XR__() );
	mmGetHandlers->setMethod( this, (MethodRequestPtr)&SystemLogger::mmDoGetHandlers );


 }

SystemLogger::~SystemLogger()
 {
	Log(LOG_INFO, ":~SystemLogger() Destroying.." );


	Log(LOG_VERBOSE, ":~SystemLogger() Destroyed." );
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////// KERNEL EVENTS

WRESULT SystemLogger::onKernelInitializing() {
	WRESULT ret = WoshModule::onKernelInitializing();
	return ret;
 }

/////////////////////////////////////////////////////////////////////////////////////////////// KERNEL EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT SystemLogger::mmDoDump() {

	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS

Response* SystemLogger::mmDoGetContexts( const Request* request )
 {
	if ( request == NULL ) return NULL;
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, ":mmDoGetContexts(%s)", option.c_str() );

	if ( option == "s" || option == "short" ) {
		LogContext::getInstances().transactionBeginRead();
		NEW_List* objs = new NEW_List(LogContext::getInstances().size());
		for_each( LogContext::getInstances().begin(), LogContext::getInstances().end(),
					Functor_Appender<NEW_List, LogContext, std::string>(*objs, &LogContext::getContext ) );
		LogContext::getInstances().transactionEnd();
		return request->replyResponse(WRET_OK, objs);
	 }
	else {
		LogContext::getInstances().transactionBeginRead();
		Table* tbl = new Table( LogContext::getInstances().size(), 3, true );
		tbl->setTableName("Log Contexts");
		tbl->setHeaderHorizontal("Context", 0);
		tbl->setHeaderHorizontal("Level", 1);
		tbl->setHeaderHorizontal("hasHandler", 2);
		unsigned int iRow = 0;
		List22<LogContext*>::ConstIterator it;
		const List22<LogContext*>::ConstIterator it_end = LogContext::getInstances().end();
		for ( it=LogContext::getInstances().begin(); it!=it_end; ++it, ++iRow ) {
			if ( *it == NULL ) continue;
			const LogContext* currErr = *it;
			tbl->set<Variant>( currErr->getContext(), iRow, 0);
			tbl->set<Variant>( LogContext::getLogLevelAsString(currErr->getLevel()), iRow, 1);
			tbl->set<Variant>( currErr->hasHandler(), iRow, 2);
		 }
		LogContext::getInstances().transactionEnd();
		return request->replyResponse(WRET_OK, tbl);
	 }
 }

Response* SystemLogger::mmDoGetHandlers( const Request* request )
 {
	if ( request == NULL ) return NULL;
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, ":mmDoGetHandlers(%s)", option.c_str() );

	if ( option == "s" || option == "short" ) {
		LogHandler::getInstances().transactionBeginRead();
		NEW_List* objs = new NEW_List(LogHandler::getInstances().size());
//		for_each( LogHandlers::getInstances().begin(), LogHandlers::getInstances().end(),
//					Functor_Appender<NEW_List, LogHandlers, std::string>(*objs, &LogHandlers::getContext ) );
		LogHandler::getInstances().transactionEnd();
		return request->replyResponse(WRET_OK, objs);
	 }
	else {
		LogHandler::getInstances().transactionBeginRead();
		Table* tbl = new Table( LogHandler::getInstances().size(), 4, true );
		tbl->setTableName("Log Handler");
		tbl->setHeaderHorizontal("ID", 0);
		tbl->setHeaderHorizontal("ClassType", 1);
		tbl->setHeaderHorizontal("isOpen", 2);
		tbl->setHeaderHorizontal("Protocol", 3);
		unsigned int iRow = 0;
		List22<LogHandler*>::ConstIterator it;
		const List22<LogHandler*>::ConstIterator it_end = LogHandler::getInstances().end();
		for ( it=LogHandler::getInstances().begin(); it!=it_end; ++it, ++iRow ) {
			if ( *it == NULL ) continue;
			const LogHandler* currHandler = *it;
			tbl->set<Variant>( currHandler->getID(), iRow, 0);
			tbl->set<Variant>( currHandler->getClassName(), iRow, 1);
			tbl->set<Variant>( currHandler->isOpen(), iRow, 2);
			tbl->set<Variant>( currHandler->getProtocol(), iRow, 3);
		 }
		LogHandler::getInstances().transactionEnd();
		return request->replyResponse(WRET_OK, tbl);
	 }
 }

///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// STATIC METHODS


////////////////////////////////////////////////////////////////////////////////////////////// STATIC METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

}; // namespace wosh
