/** @file    SystemMonitor.cpp
 *  @author  Alessandro Polo
 *  @version $Id: SystemMonitor.cpp 2223 2010-03-26 21:27:05Z alex $
 *  @brief
 * File containing methods for the wosh::SystemMonitor class.
 * The header for this class can be found in SystemMonitor.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include <core/SystemMonitor.h>

 #include <core/ObjectAllocator.h>
 #include <core/WoshKernel.h>
 #include <core/DataBaseManager.h>
 #include <core/SharedMemory.h>
 #include <core/MethodsDef.h>
 #include <core/NEW_List.h>
 #include <core/Table.h>
 #include <core/Functional.h>
 #include <core/Utilities.h>

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef _OS_POSIX
# include <core/posix/processes.h>
# include <core/posix/desktop.h>
#elif defined _OS_WIN32
# include <core/win32/processes.h>
# include <core/win32/desktop.h>
#elif defined _OS_WINCE
# include <core/wince/processes.h>
# include <core/wince/desktop.h>
#else
 long getCurrentProcessPID() { return 0L; }
 bool getSystemUserName( std::string& ) { return false; }
 bool retrieveProcessInfo( long , std::map<std::string, std::string>& ) { return false; }
#endif
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

using namespace std;
namespace wosh {


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////// PROFILING

/////////////////////////////////////////////////////////////////////////////////////////////////// PROFILING
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Gamma singleton! static reference is allocated and maintained by WoshKernel
extern "C" {
	SystemMonitor* SystemMonitor::getInstance() { 
		if ( WoshKernel::getInstance() == NULL ) return NULL;
		return &WoshKernel::getInstance()->monitor();
	}
};

 // register as abstract because it is a singleton:
 WOSH_REGISTER_INTERFACE(wosh::SystemMonitor, "wosh::WoshModule", 0.01, _static_SystemMonitor )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

SystemMonitor::SystemMonitor( const WoshKernel* parent ) : WoshModule(parent)
 {
	Object::setName( _SystemMonitor_NAME );
	Object::getURI().fromString( _SystemMonitor_URI );
	Object::getURI().registerGlobal();

Log.setLevel( LOG_VERBOSE );

#ifdef _ENABLE_PROFILING
	this->profiler = &SystemProfiler::getInstance();
#endif

	Properties.set(_SystemMonitor_KEY_SaveOnExit, false, Permission(Permission::RW, Permission::Read, Permission::Read) );
	Properties.set(_SystemMonitor_KEY_ProcessPID, 0, Permission(Permission::Read) );
	Properties.set(_SystemMonitor_KEY_SystemUser, "", Permission(Permission::Read) );

	MethodRequest* mmGetErrors = Methods.create<MethodRequest>( _METHOD_List, "Get Errors", Permission::R_XR_XR__() );
	mmGetErrors->setMethod( this, (MethodRequestPtr)&SystemMonitor::mmDoGetErrors );

	MethodRequest* mmGetError = Methods.create<MethodRequest>( _SystemMonitor_METHOD_retrieve, "Get Error", Permission::R_XR_XR__() );
	mmGetError->setMethod( this, (MethodRequestPtr)&SystemMonitor::mmDoGetError );

	MethodRequest* mmInspectError = Methods.create<MethodRequest>( _SystemMonitor_METHOD_inspect, "Inspect Error", Permission::R_XR_XR__() );
	mmInspectError->setMethod( this, (MethodRequestPtr)&SystemMonitor::mmDoInspectError );

	MethodRequest* mmProfilesList = Methods.create<MethodRequest>( _SystemMonitor_METHOD_profiles, "list profiling info", Permission::R_XR_XR__() );
	mmProfilesList->setMethod( this, (MethodRequestPtr)&SystemMonitor::mmDoProfilesList );

	MethodRequest* mmListDeprecated = Methods.create<MethodRequest>( _SystemMonitor_METHOD_deprecated, "list deprecated", Permission::R_XR_XR__() );
	mmListDeprecated->setMethod( this, (MethodRequestPtr)&SystemMonitor::mmDoListDeprecated );

	MethodRequest* mmListDSM = Methods.create<MethodRequest>( _SystemMonitor_METHOD_dsm, "list dsm", Permission::R_XR_XR__() );
	mmListDSM->setMethod( this, (MethodRequestPtr)&SystemMonitor::mmDoListDSM );

	MethodWRESULT* mmSaveErrors = Methods.create<MethodWRESULT>( _SystemMonitor_METHOD_save_errors, "save errors", Permission::R_XR_XR__() );
	mmSaveErrors->setMethod( this, (MethodWRESULTPtr)&SystemMonitor::mmDoSaveErrors );

	MethodWRESULT* mmCleanErrors = Methods.create<MethodWRESULT>( _SystemMonitor_METHOD_clear_errors, "clean errors", Permission::R_XR_XR__() );
	mmCleanErrors->setMethod( this, (MethodWRESULTPtr)&SystemMonitor::mmDoCleanErrors );

	MethodWRESULT* mmUpdateSysInfo = Methods.create<MethodWRESULT>( _SystemMonitor_METHOD_update_sysinfo, "update sysinfo", Permission::R_XR_XR__() );
	mmUpdateSysInfo->setMethod( this, (MethodWRESULTPtr)&SystemMonitor::updateSystemInfo );

#ifdef _ENABLE_PROFILING
	this->profiler = &SystemProfiler::getInstance();
#endif
 }

SystemMonitor::~SystemMonitor() {
	Log(LOG_VERBOSE, ":~SystemMonitor() Destroying." );
	this->errorLog.clear();
	// this->profiler is destroyed by compiler because is a static singleton (meyer)
	Log(LOG_INFO, " Destroyed." );
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////// KERNEL EVENTS

WRESULT SystemMonitor::onKernelInitializing() {
	WRESULT ret = WoshModule::onKernelInitializing();

	long pid = getCurrentProcessPID();
	Properties.update(_SystemMonitor_KEY_ProcessPID, pid );
	Log(LOG_INFO, "init() : PID updated to %ld", pid );
	ret += updateSystemInfo_(true, true);

	return ret;
 }

WRESULT SystemMonitor::onKernelStopped() {
	WRESULT ret = WRET_OK;

	Variant saveMsg = this->Properties.getValueOf(_SystemMonitor_KEY_SaveOnExit);
	if ( !saveMsg.isEmpty() && saveMsg.toBoolean(true) )
		ret += mmDoSaveErrors();

	ret = WoshModule::onKernelStopped();
	return ret;
 }

/////////////////////////////////////////////////////////////////////////////////////////////// KERNEL EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENTS

bool SystemMonitor::readingProperty( Property* property_curr, const PropertiesProvider* source ) {
	if ( property_curr != NULL && property_curr->getKey().find("SysInfo") == 0 )
		updateSystemInfo_(false, false);
	return WoshModule::readingProperty( property_curr, source );
 }

///////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT SystemMonitor::updateSystemInfo_( bool lock_props, bool force_update ) {
	 if ( !force_update ) {
		if ( Utilities::std_time() - this->systemInfoTs < _SystemMonitor_SystemInfoMinRefresh )
			 return WRET_ERR_ILLEGAL_USE;
	  }
	this->systemInfoTs = Utilities::std_time();
	Log(LOG_VERBOSE, "updateSystemInfo_(%d) Updating System Info..", lock_props );

	string username;
	bool userOk = getSystemUserName(username);
	if ( !userOk ) {
		Log(LOG_CRITICAL, "updateSystemInfo_(%d) Error retrieving system user", lock_props );
	 }

	long pid = getCurrentProcessPID();

	Properties.transactionBeginWrite();
	Properties.update(_SystemMonitor_KEY_SystemUser, username);
	Properties.transactionEnd();

	///@todo use Variant, so data is TYPED
	map<string, string> fields;
	bool memInfo = retrieveProcessInfo( pid, fields );
	if ( memInfo ) {
		Properties.transactionBeginWrite();
		std::map<string, string>::const_iterator it;
		for ( it=fields.begin(); it!=fields.end(); ++it ) {
			Properties.update(it->first, it->second);
		 }
		Properties.transactionEnd();
	 }
	else {
		Log(LOG_CRITICAL, "updateSystemInfo_() ERRORS retrieving process info." );
		return WRET_ERR_INTERNAL;
	 }
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////// COLLECTOR METHODS

WRESULT SystemMonitor::mmDoSaveErrors() {
	std::vector<ISerializable*> objects;

	this->errorLog.transactionBeginRead();
	tSystemErrorList::ConstIterator it;
	const tSystemErrorList::ConstIterator it_end = this->errorLog.end();
	for ( it=this->errorLog.begin(); it!=it_end; ++it ) {
		if ( *it == NULL ) continue;
		objects.push_back( *it );
	 }
	WRESULT ret = DataBaseManager::saveObjects( objects, "../var/database/systemerrors.xml", "xml" );

	this->errorLog.transactionEnd();
	return ret;
 }

/////////////////////////////////////////////////////////////////////////////////////////// COLLECTOR METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////// 

void SystemMonitor::busMessage( const Message& message, const Bus* source ) {
	if ( message.isEmpty() ) return;
	WoshModule::busMessage( message, source );

	if ( message.getSourceUUID() == this->getEntityID() ) return;
	if ( !message.getContent()->isSystemError() ) return;

	const SystemError* sysError = message.getContent()->asSystemError();
	Log(LOG_INFO, ":busMessage() : Adding Error#%ld", sysError->getID() );
	this->errorLog.transactionBeginWrite();
	this->errorLog.push_back( sysError->clone() );
	this->errorLog.transactionEnd();
	return;
 }

////////////////////////////////////////////////////////////////////////////////////////// THREADs OPERATIONS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS

WRESULT SystemMonitor::mmDoCleanErrors() {
	Log(LOG_INFO, ":mmCleanErrors() Cleaning.. [%d records]", this->errorLog.size() );
	this->errorLog.free();
	return WRET_OK;
 }

Response* SystemMonitor::mmDoGetErrors( const Request* request ) {
	if ( request == NULL ) return NULL;
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, ":mmDoGetErrors(%s)", option.c_str() );
	if ( option == "s" || option == "short" ) {
		this->errorLog.transactionBeginRead();
		NEW_List* objs = new NEW_List(this->errorLog.size());
		for_each( this->errorLog.begin(), this->errorLog.end(),
					Functor_Appender<NEW_List, SystemError, long>(*objs, &SystemError::getID ) );
		this->errorLog.transactionEnd();
		return request->replyResponse(WRET_OK, objs);
	 }
	else {
		this->errorLog.transactionBeginRead();
		Table* tbl = new Table( this->errorLog.size(), 6, true );
		tbl->setTableName("Errors");
		tbl->setHeaderHorizontal("ID", 0);
		tbl->setHeaderHorizontal("TimeStamp", 1);
		tbl->setHeaderHorizontal("Priority", 2);
		tbl->setHeaderHorizontal("Source", 3);
		tbl->setHeaderHorizontal("Method", 4);
		tbl->setHeaderHorizontal("Brief", 5);
		unsigned int iRow = 0;
		tSystemErrorList::ConstIterator it;
		const tSystemErrorList::ConstIterator it_end = this->errorLog.end();
		for ( it=this->errorLog.begin(); it!=it_end; ++it, ++iRow ) {
			if ( *it == NULL ) continue;
			const SystemError* currErr = *it;
			tbl->set<Variant>( currErr->getID(), iRow, 0);
			tbl->set( new Variant(currErr->getTimeStamp(), Variant::DATETIME), iRow, 1);
			tbl->set<Variant>( SystemError::getPriorityAsString(currErr->getPriority()), iRow, 2);
			tbl->set( currErr->getSource().clone(), iRow, 3);
			tbl->set<Variant>( currErr->getMethod(), iRow, 4);
			tbl->set<Variant>( currErr->getBrief(30), iRow, 5);
		 }
		this->errorLog.transactionEnd();
		return request->replyResponse(WRET_OK, tbl);
	 }
 }

Response* SystemMonitor::mmDoGetError( const Request* request ) {
	if ( request == NULL ) return NULL;
	long errorId = request->getArgument().toLong(0);
	if ( errorId == 0 ) {
		Log(LOG_WARNING, ":mmDoGetError(%s) Invalid ID", request->getArgument().toString().c_str() );
		return request->replyResponse(WRET_ERR_PARAM, "Invalid ID");
	 }
	SystemError* errorPtr = NULL;
	this->errorLog.transactionBeginRead();
	tSystemErrorList::ConstIterator it;
	const tSystemErrorList::ConstIterator it_end = this->errorLog.end();
	for ( it=this->errorLog.begin(); it!=it_end; ++it ) {
		if ( *it == NULL ) continue;
		if ( (*it)->getID() != errorId ) continue;
		errorPtr = *it;
	 }
	if ( errorPtr == NULL ) {
		this->errorLog.transactionEnd();
		Log(LOG_WARNING, ":mmDoGetError(%ld) ID not found", errorId );
		return request->replyResponse(WRET_ERR_PARAM, "ID not found!");
	 }
	Log(LOG_VERBOSE, ":mmDoGetError(%ld)", errorId );

	errorPtr = errorPtr->clone();
	this->errorLog.transactionEnd();
	return request->replyResponse(WRET_OK, errorPtr);
 }

Response* SystemMonitor::mmDoInspectError( const Request* request ) {
	if ( request == NULL ) return NULL;
	long errorId = request->getArgument().toLong(0);
	if ( errorId == 0 ) {
		Log(LOG_WARNING, ":mmDoInspectError(%s) Invalid ID", request->getArgument().toString().c_str() );
		return request->replyResponse(WRET_ERR_PARAM, "Invalid ID");
	 }
	SystemError* errorPtr = NULL;
	this->errorLog.transactionBeginRead();
	tSystemErrorList::ConstIterator it;
	const tSystemErrorList::ConstIterator it_end = this->errorLog.end();
	for ( it=this->errorLog.begin(); it!=it_end; ++it ) {
		if ( *it == NULL ) continue;
		if ( (*it)->getID() != errorId ) continue;
		errorPtr = *it;
	 }
	if ( errorPtr == NULL ) {
		this->errorLog.transactionEnd();
		Log(LOG_WARNING, ":mmDoInspectError(%ld) ID not found", errorId );
		return request->replyResponse(WRET_ERR_PARAM, "ID not found!");
	 }
	Log(LOG_VERBOSE, ":mmDoInspectError(%ld)", errorId );
	wosh::Table* tProps = errorPtr->getAsTable();
	this->errorLog.transactionEnd();
	return request->replyResponse(WRET_OK, tProps);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* SystemMonitor::mmDoProfilesList( const Request* request ) {
	if ( request == NULL ) return NULL;
	Variant target;
	Variant option1;
	request->extractArguments(target, option1);
	if ( target.isStringEmpty() ) {
		return request->replyResponse(WRET_ERR_PARAM, "specify target {properties, methods, connectors}");
	 }
	Log(LOG_VERBOSE, ":mmDoProfilesList(%s)", target.asString().c_str() );
#ifdef _ENABLE_PROFILING
	if ( this->profiler == NULL )
		return request->replyResponse(WRET_ERR_INTERNAL, "Internal error: Profiler is NULL");

	if ( target.asString() == "properties" )
		return request->replyResponse(WRET_OK, this->profiler->getProfilesPropertiesTable(option1.toInt64(1)) );
	else if ( target.asString() == "methods" )
		return request->replyResponse(WRET_OK, this->profiler->getProfilesMethodsTable(option1.toInt64(0)) );
	else if ( target.asString() == "connectors" )
		return request->replyResponse(WRET_OK, this->profiler->getProfilesConnectorsTable(option1.toInt64(0)) );
#else
	return request->replyResponse(WRET_ERR_ILLEGAL_USE, "Profiling is not enabled in this built.");
#endif
	return request->replyResponse(WRET_ERR_PARAM, "specify valid target {properties, methods, connectors}");
 }

Response* SystemMonitor::mmDoListDeprecated( const Request* request ) {
	if ( request == NULL ) return NULL;
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, ":mmDoListDeprecated(%s)", option.c_str() );

												WOSH_DEPRECATE( "SystemMonitor::mmDoListDeprecated", "SAMPLE" )

#ifdef _ENABLE_PROFILING
	if ( this->profiler == NULL )
		return request->replyResponse(WRET_ERR_INTERNAL, "Internal error: Profiler is NULL");
	return request->replyResponse(WRET_OK, this->profiler->getDeprecatedTable() );
#else
	return request->replyResponse(WRET_ERR_ILLEGAL_USE, "Profiling is not enabled in this built.");
#endif
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* SystemMonitor::mmDoListDSM( const Request* request ) {
	if ( request == NULL ) return NULL;
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, ":mmDoListDSM(%s)", option.c_str() );
	if ( option == "s" || option == "short" ) {
		SharedMemory::getInstances().transactionBeginRead();
		NEW_List* objs = new NEW_List(SharedMemory::getInstances().size());
		for_each( SharedMemory::getInstances().begin(), SharedMemory::getInstances().end(),
					Functor_Appender<NEW_List, SharedMemory, const string&>(*objs, &SharedMemory::getName ) );
		SharedMemory::getInstances().transactionEnd();
		return request->replyResponse(WRET_OK, objs);
	 }
	else {
		SharedMemory::getInstances().transactionBeginRead();
		Table* tbl = new Table( SharedMemory::getInstances().size(), 4, true );
		tbl->setTableName("DSM");
		tbl->setHeaderHorizontal("Name", 0);
		tbl->setHeaderHorizontal("ID", 1);
		tbl->setHeaderHorizontal("Clock", 2);
		tbl->setHeaderHorizontal("Seniority", 3);
		unsigned int iRow = 0;
		List22<SharedMemory*>::ConstIterator it;
		const List22<SharedMemory*>::ConstIterator it_end = SharedMemory::getInstances().end();
		for ( it=SharedMemory::getInstances().begin(); it!=it_end; ++it, ++iRow ) {
			if ( *it == NULL ) continue;
			const SharedMemory* currDSM = *it;
			tbl->set<Variant>( currDSM->getName(), iRow, 0);
			tbl->set<Variant>( currDSM->getID(), iRow, 1);
			tbl->set<Variant>( currDSM->getClock(), iRow, 2);
			tbl->set<Variant>( currDSM->getSeniority(), iRow, 3);
		 }
		SharedMemory::getInstances().transactionEnd();
		return request->replyResponse(WRET_OK, tbl);
	 }
 }

///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// STATIC METHODS

WRESULT SystemMonitor::raiseError( wosh::SystemError* error, const Object* source ) {
	if (SystemMonitor::getInstance() == NULL) return WRET_ERR_INTERNAL;
	Message* msg_system = new Message(error);
	msg_system->setSource( source );
	msg_system->setDestinationBroadcast(false);
	msg_system->setDestinationBus(_Bus_Core);
	SystemMonitor::getInstance()->BusCore.postMessage(msg_system);
	return WRET_OK;
 }

////////////////////////////////////////////////////////////////////////////////////////////// STATIC METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

}; // namespace wosh
