/**
 * @class   wosh::SystemProfiler
 * @brief   
 ****************************************************************************
 * @version 0.8.161 $Id: SystemProfiler.h 1937 2010-02-24 20:04:23Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/*! \page page_debug Debugging
 * \htmlonly <hr/> \endhtmlonly
 * \section page_debug_profiling WOSH Profiling
 *
 * \note Profiling has been officially introduced in WOSH 0.8.151 [icarus]
 * \warning Profiling is still experimental.
 *
 * Profiling allows you to learn where your program spent its time and which
 * functions called which other functions while it was executing.
 *
 * @todo [..]
 *
 * \warning Profiling data is *very* huge and it's generally too big for current TCP buffers.
 *          In other words, such listing (table) often can't be transmitted to the WOSH network.
 *          The I/O within an host (working on local-host) obviously works fine.
 *
 * \par Linux, GCC:
 *  
 * \see http://www.cs.utah.edu/dept/old/texinfo/as/gprof.html
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_debug_deprecated Deprecated-List functionalities
 *
 * \note Deprecated-List has been officially introduced in WOSH 0.8.151 [icarus]
 * \warning Deprecated-List is still experimental.
 *
 * \note Deprecated-List is built in the Profiling system (then it will be eventually disabled as well).
 *
 * @todo [..]
 *
 * \code
void myClass::myDeprecatedMethod( int arg1 ) {
	WOSH_DEPRECATE( "myClass::myDeprecatedMethod", "myClass::newOptimizedMethod" )
 }
 * \endcode
 *
 *
 *
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR MutexS;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 // IDEA:  specialization for { Property, Method, BusConnector }
 // more specific profiling should be done with standard/specific c++ tools


#ifndef __WOSH_Core_SystemProfiler_H__
 #define __WOSH_Core_SystemProfiler_H__

 #include <woshDefs.h>
 #include <core/Containers.h>
 #include <core/Mutex.h>

namespace wosh {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef _ENABLE_PROFILING
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 #define PROFILING_INIT( OBJECT, PARENT )			wosh::SystemProfiler::getInstance().initializeProfileOf( OBJECT, PARENT );
 #define PROFILING_ENTER( OBJECT, ARGUMENT )	wosh::SystemProfiler::getInstance().enterProfileOf( OBJECT, ARGUMENT );
 #define PROFILING_EXIT( OBJECT, ARGUMENT )	wosh::SystemProfiler::getInstance().exitProfileOf( OBJECT, ARGUMENT );

// fptr : it is platform specific and only works on the x86 family of CPUs
 #ifdef _MSC_VER
 # define WOSH_DEPRECATE( OLD_FUNC_NAME, NEW_FUNC_NAME ) {	\
		void* fptr;											\
		__asm { mov fptr, ebp }								\
		wosh::SystemProfiler::getInstance().deprecated(OLD_FUNC_NAME, NEW_FUNC_NAME, fptr, __FILE__, __LINE__); }
 #else
 # define WOSH_DEPRECATE( OLD_FUNC_NAME, NEW_FUNC_NAME ) {	\
		wosh::SystemProfiler::getInstance().deprecated(OLD_FUNC_NAME, NEW_FUNC_NAME, 0, __FILE__, __LINE__); }
 #endif


 // profiling classes:
 class Method;
 class MethodRunner;
 class Property;
 class IPropertiesProviderListener;
 class BusConnector;
 class IBusListener;

 // outputs:
 class NEW_List;
 class Table;


class DeprecatedFunctionInfo {
	public:
		DeprecatedFunctionInfo( const char * oldFunctionName, const char * newFunctionName,
								void* calledFrom, const char * filename, unsigned int file_line )
			: OldFunctionName(oldFunctionName), NewFunctionName(newFunctionName),
				CalledFrom(calledFrom), FileName(filename), FileLine(file_line) { }
		DeprecatedFunctionInfo( const DeprecatedFunctionInfo& m ) { *this = m; }
		~DeprecatedFunctionInfo() { }
		//TODO OPERATOR =, ==

	public:
		DeprecatedFunctionInfo& operator=( const wosh::DeprecatedFunctionInfo& m ) {
			if (this == &m) return *this; // same object?
			this->OldFunctionName = m.OldFunctionName;
			this->NewFunctionName = m.NewFunctionName;
			this->CalledFrom = m.CalledFrom;

			this->FileName = m.FileName;
			this->FileLine = m.FileLine;
			return *this;
		 }
		bool operator==(const wosh::DeprecatedFunctionInfo &other) const {
			if ( this->OldFunctionName != other.OldFunctionName )
				return false;
			if ( this->NewFunctionName != other.NewFunctionName )
				return false;
			if ( this->CalledFrom != other.CalledFrom )
				return false;

			if ( this->FileName != other.FileName )
				return false;
			if ( this->FileLine != other.FileLine )
				return false;

			return true;
		 }
		bool operator!=(const wosh::DeprecatedFunctionInfo &other) const { return !(*this == other);  }

	public:
		std::string OldFunctionName;
		std::string NewFunctionName;
		void* CalledFrom;

		std::string FileName;
		unsigned int FileLine;
};


class ProfileInfo {
	public:
		ProfileInfo() : accessCount(0), concurrentCount(0), parentPtr(NULL), parentClass("VOID") { }
		virtual ~ProfileInfo() { }

	public:
		virtual void enter( short opt ) = 0;
		virtual void exit( short ) {
			this->mux.lockForWrite();
			--this->concurrentCount;
			this->mux.unLock();
		 }
		void setParentInfo( const IReflection* parent );

	public:
		std::string name;
		int64 accessCount;
		int concurrentCount;
		const void* parentPtr;
		std::string parentClass;
		std::string parentName;
		MutexRW mux;
};


class ProfileInfoMethod : public ProfileInfo {
	public:
		ProfileInfoMethod()
			: ProfileInfo(), methodClass("VOID"), linkedFunction(false), linkedParent(false) { }

	public:
		void enter( short opt );

		void setMethod( const Method* method, const MethodRunner* parent );
	public:
		std::string methodClass;
		bool linkedFunction;
		bool linkedParent;

};

class ProfileInfoProperty : public ProfileInfo {
	public:
		ProfileInfoProperty()
			: ProfileInfo(), readCount(0), writeCount(0) { }

	public:
		void enter( short read_or_write );

		void setProperty( const Property* prop, const IPropertiesProviderListener* parent );
	public:
		int64 readCount;
		int64 writeCount;
};

class ProfileInfoConnector : public ProfileInfo {
	public:
		ProfileInfoConnector()
			: ProfileInfo(), inputCount(0), outputCount(0) { }

	public:
		void enter( short input_or_output );

		void setConnector( const BusConnector* prop, const IBusListener* parent );
	public:
		int64 inputCount;
		int64 outputCount;
};



class SystemProfiler
 {
	typedef Map22<const void*, ProfileInfoMethod*> tVoidProfileInfoMethodMap;
	typedef Map22<const void*, ProfileInfoProperty*> tVoidProfileInfoPropertyMap;
	typedef Map22<const void*, ProfileInfoConnector*> tVoidProfileInfoConnectorMap;

	typedef List22<DeprecatedFunctionInfo> tDeprecatedFunctionInfoList;

	//----------------------------------- used directly by SystemMonitor
	public:
		Table* getDeprecatedTable() const;
		Table* getProfilesMethodsTable( int64 access_threshold ) const;
		Table* getProfilesPropertiesTable( int64 access_threshold ) const;
		Table* getProfilesConnectorsTable( int64 access_threshold ) const;

	//----------------------------------- used directly by macros
	public:
		static SystemProfiler& getInstance();

	public:
		void deprecated( const char * OldFunctionName, const char * NewFunctionName,
						void* CalledFrom, const char * filename, unsigned int file_line );

	public:
		template<class OBJECT, class PARENT>
		void initializeProfileOf( const OBJECT* object, const PARENT* owner ) {
			getCreateProfile(object, owner);
		 }
		template<class OBJECT>
		void enterProfileOf( const OBJECT* object, short argument ) {
			ProfileInfo* info = static_cast<ProfileInfo*>( getCreateProfile(object, NULL) );
			if ( info == NULL ) return;
			info->enter(argument);
		 }
		template<class OBJECT>
		void exitProfileOf( const OBJECT* object, short argument ) {
			ProfileInfo* info = static_cast<ProfileInfo*>( getCreateProfile(object, NULL) );
			if ( info == NULL ) return;
			info->exit(argument);
		}

	protected:
	//	template<class OBJECT, class PARENT>
	//	ProfileInfo* getCreateProfile( OBJECT* , PARENT*  ) {
	//		return NULL;
	//	 }

		ProfileInfoProperty* getCreateProfile( const Property* object, const IPropertiesProviderListener* owner );
		ProfileInfoConnector* getCreateProfile( const BusConnector* object, const IBusListener* owner );
		ProfileInfoMethod* getCreateProfile( const Method* object, const MethodRunner* owner );

	public:
		virtual ~SystemProfiler();

	protected:
		SystemProfiler() { }

		tVoidProfileInfoMethodMap profilesMethods;
		tVoidProfileInfoPropertyMap profilesProperties;
		tVoidProfileInfoConnectorMap profilesConnectors;

		tDeprecatedFunctionInfoList deprecatedList;


}; // class def
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
//////////////////////////////////////////////////////////////////////////////////////////////////////////////


}; // namespace wosh

#endif //__WOSH_Core_SystemProfiler_H__
