/** @file    Table.cpp
 *  @author  Alessandro Polo
 *  @version $Id: Table.cpp 2172 2010-03-19 00:38:53Z alex $
 *  @brief
 * File containing methods for the wosh::Table class.
 * The header for this class can be found in Table.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include <core/Table.h>

 #include <core/ObjectAllocator.h>
 #include <core/EncoderReader.h>
 #include <core/EncoderWriter.h>
 #include <core/Utilities.h>


using namespace std;
namespace wosh {

 WOSH_REGISTER(wosh::Table, "", 1.01, _static_Table)

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

Table::Table()
 {
	this->rows = 0;
	this->cols = 0;
	this->items = NULL;
	this->headerVertical = NULL;
	this->headerHorizontal = NULL;

	this->table_name = "Table";
 }

Table::Table( unsigned int row_count, unsigned int col_count, bool horizontalHeader, bool verticalHeader )
 {
	this->rows = 0;
	this->cols = 0;
	this->items = NULL;
	this->headerVertical = NULL;
	this->headerHorizontal = NULL;
	this->table_name = "Table";

	alloc(row_count, col_count, horizontalHeader, verticalHeader);
 }

Table::Table( const wosh::Table& m )
 {
	this->rows = 0;
	this->cols = 0;
	this->items = NULL;
	this->headerVertical = NULL;
	this->headerHorizontal = NULL;
	*this = m;
 }

Table::~Table()
 {
	clear(true);
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT Table::alloc( unsigned int row_count, unsigned int col_count, bool horizontalHeader, bool verticalHeader )
 {
	if ( this->items != NULL ) {
		return WRET_ERR_INTERNAL;
	 }

	this->rows = row_count;
	this->cols = col_count;
	unsigned int count = this->rows * this->cols;
	if ( count > 0 ) {
		this->items = new wosh::ISerializable*[count];
		for(unsigned int i=0; i<count; i++) {
			this->items[i] = NULL;
		 }
	 }

	allocHeaderHorizontal(horizontalHeader);
	allocHeaderVertical(verticalHeader);
	return WRET_OK;
 }

void Table::allocHeaderHorizontal( bool value )
 {
	if ( this->headerHorizontal != NULL ) {
		for(unsigned int i=0; i<this->cols; i++) {
			if ( this->headerHorizontal[i] == NULL ) continue;
			delete this->headerHorizontal[i]; this->headerHorizontal[i] = NULL;
		 }
		 delete [] this->headerHorizontal; this->headerHorizontal = NULL;
	 }
	if (!value)
		return;
	this->headerHorizontal = new wosh::ISerializable*[this->cols];
	for(unsigned int i=0; i<this->cols; i++) {
		this->headerHorizontal[i] = NULL;
	 }
 }

void Table::allocHeaderVertical( bool value )
 {
	if ( this->headerVertical != NULL ) {
		for(unsigned int i=0; i<this->rows; i++) {
			if ( this->headerVertical[i] == NULL ) continue;
			delete this->headerVertical[i]; this->headerVertical[i] = NULL;
		 }
		 delete [] this->headerVertical; this->headerVertical = NULL;
	 }
	if (!value)
		return;
	this->headerVertical = new wosh::ISerializable*[this->rows];
	for(unsigned int i=0; i<this->rows; i++) {
		this->headerVertical[i] = NULL;
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool Table::isEmpty() const
 {
	if ( this->items == NULL )
		return true;
	unsigned int count = this->rows * this->cols;
	if ( count == 0 )
		return true;
	for(unsigned int i=0; i<count; i++) {
		if ( this->items[i] != NULL )
			return false;
	 }
	return true;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Table::clear( bool deleteContents )
 {
	allocHeaderVertical(false);
	allocHeaderHorizontal(false);
	clearContents(deleteContents);
	delete [] this->items; this->items = NULL;
	this->rows = 0;
	this->cols = 0;
 }

void Table::clearContents( bool deleteContents )
 {
	if ( this->items == NULL ) return;
	unsigned int count = this->rows * this->cols;
	for(unsigned int i=0; i<count; i++) {
		if ( this->items[i] == NULL ) continue;
		if ( deleteContents )
			delete this->items[i];
		this->items[i] = NULL;
	 }

	//delete [] this->items; this->items = NULL;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// SETTERS

WRESULT Table::set( wosh::ISerializable* object, unsigned int row, unsigned int col, bool deletePrevious )
 {
	if ( this->items == NULL ) return WRET_ERR_PARAM;
	if ( row >= this->rows ) return WRET_ERR_PARAM;
	if ( col >= this->cols ) return WRET_ERR_PARAM;

	unsigned int raster_index = row * this->cols + col;
	return setAt(object, raster_index, deletePrevious );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT Table::setAt( wosh::ISerializable* object, unsigned int raster_index, bool deletePrevious )
 {
	if ( this->items == NULL ) return WRET_ERR_PARAM;
	unsigned int count = this->rows * this->cols;
	if ( raster_index >= count ) return WRET_ERR_PARAM;

	if ( deletePrevious && this->items[raster_index] != NULL )
		delete this->items[raster_index];

	this->items[raster_index] = object;
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT Table::setHeaderHorizontal( wosh::ISerializable* object, unsigned int col, bool deletePrevious )
 {
	if ( this->headerHorizontal == NULL ) return WRET_ERR_PARAM;
	if ( col >= this->cols ) return WRET_ERR_PARAM;

	if ( deletePrevious && this->headerHorizontal[col] != NULL )
		delete this->headerHorizontal[col];

	this->headerHorizontal[col] = object;
	return WRET_OK;
 }

WRESULT Table::setHeaderVertical( wosh::ISerializable* object, unsigned int row, bool deletePrevious )
 {
	if ( this->headerVertical == NULL ) return WRET_ERR_PARAM;
	if ( row >= this->rows ) return WRET_ERR_PARAM;

	if ( deletePrevious && this->headerVertical[row] != NULL )
		delete this->headerVertical[row];

	this->headerVertical[row] = object;
	return WRET_OK;
 }

///////////////////////////////////////////////////////////////////////////////////////////////////// SETTERS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// GETTERS

const wosh::ISerializable* Table::get( unsigned int row, unsigned int col ) const {
	if ( this->items == NULL ) return NULL;
	if ( row >= this->rows ) return NULL;
	if ( col >= this->cols ) return NULL;

	unsigned int raster_index = row * this->cols + col;
	return getAt( raster_index );
 }
wosh::ISerializable* Table::get( unsigned int row, unsigned int col ) {
	if ( this->items == NULL ) return NULL;
	if ( row >= this->rows ) return NULL;
	if ( col >= this->cols ) return NULL;

	unsigned int raster_index = row * this->cols + col;
	return getAt( raster_index );
 }

wosh::ISerializable* Table::getAt( unsigned int raster_index ) {
	if ( this->items == NULL ) return NULL;
	unsigned int count = this->rows * this->cols;
	if ( raster_index >= count ) return NULL;

	return this->items[raster_index];
 }
const wosh::ISerializable* Table::getAt( unsigned int raster_index ) const {
	if ( this->items == NULL ) return NULL;
	unsigned int count = this->rows * this->cols;
	if ( raster_index >= count ) return NULL;

	return this->items[raster_index];
 }

wosh::ISerializable* Table::getHeaderHorizontal( unsigned int col ) {
	if ( this->headerHorizontal == NULL ) return NULL;
	if ( col >= this->cols ) return NULL;
	return this->headerHorizontal[col];
 }
const wosh::ISerializable* Table::getHeaderHorizontal( unsigned int col ) const {
	if ( this->headerHorizontal == NULL ) return NULL;
	if ( col >= this->cols ) return NULL;
	return this->headerHorizontal[col];
 }

wosh::ISerializable* Table::getHeaderVertical( unsigned int row ) {
	if ( this->headerVertical == NULL ) return NULL;
	if ( row >= this->rows ) return NULL;
	return this->headerVertical[row];
 }
const wosh::ISerializable* Table::getHeaderVertical( unsigned int row ) const {
	if ( this->headerVertical == NULL ) return NULL;
	if ( row >= this->rows ) return NULL;
	return this->headerVertical[row];
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

std::string Table::getClassNameOf( unsigned int row, unsigned int col ) const
 {
	const ISerializable* obj = get( row, col );
	if ( obj == NULL )
		return "";
	return obj->getClassName();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////// OPERATORS

wosh::Table& Table::operator=(const wosh::Table& m)
 {
	if (this == &m) return *this; // same object?

	this->table_name = m.table_name;
	clear(true);
	alloc( m.rows, m.cols );
	unsigned int count = this->rows * this->cols;
	for(unsigned int i=0; i<count; i++) {
		if ( m.items[i] == NULL ) continue;
		this->items[i] = m.items[i]->clone();
	 }
	if ( m.hasHeaderHorizontal() ) {
		allocHeaderHorizontal(true);
		for(unsigned int i=0; i<this->cols; i++) {
			if ( m.headerHorizontal[i] == NULL ) continue;
			this->headerHorizontal[i] = m.headerHorizontal[i]->clone();
		 }
	 }
	if ( m.hasHeaderVertical() ) {
		allocHeaderVertical(true);
		for(unsigned int i=0; i<this->rows; i++) {
			if ( m.headerVertical[i] == NULL ) continue;
			this->headerVertical[i] = m.headerVertical[i]->clone();
		 }
	 }

	return *this;
 }

bool Table::operator==(const wosh::Table &other) const
 {
	if ( this->rows != other.rows )
		return false;
	if ( this->cols != other.cols )
		return false;
	if ( this->table_name != other.table_name )
		return false;

	return true;
 }

/////////////////////////////////////////////////////////////////////////////////////////////////// OPERATORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

const ISerializable* Table::findMappedColumnItem( unsigned int col_primary, unsigned int return_column, const ISerializable* other, short compare_mode ) const
 {
	if ( col_primary >= this->cols || return_column >= this->cols  ) return NULL;
	unsigned int row_start = 0;
	unsigned int row_end = this->rows;
	for (unsigned int row=row_start; row<row_end; row++) {
		const ISerializable* iobj = this->items[row * this->cols + col_primary];
		if ( iobj == other ) return other;
		if ( iobj == NULL ) continue;
		if ( iobj->compare(other) != compare_mode ) continue;
		iobj = this->items[row * this->cols + return_column];
		return iobj;
	 }
	return NULL;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT Table::swapRow( unsigned int source_index, unsigned int dest_index ) {
	if ( source_index >= this->rows ) return WRET_ERR_PARAM;
	if ( dest_index >= this->rows ) return WRET_ERR_PARAM;
	if ( source_index == dest_index ) return WRET_OK;
	// backup old row
	wosh::ISerializable** tempRow = new wosh::ISerializable*[this->cols];
	wosh::ISerializable* tempVHeader = NULL;
	for(unsigned int i=0; i<this->cols; i++) {
		tempRow[i] = this->items[dest_index * this->cols + i];
	 }
	if ( this->headerVertical != NULL )
		tempVHeader = this->headerVertical[dest_index];
	// overwrite pointers of old row
	for(unsigned int i=0; i<this->cols; i++) {
		this->items[dest_index * this->cols + i] = this->items[source_index * this->cols + i];
	 }
	if ( this->headerVertical != NULL )
		this->headerVertical[dest_index] = this->headerVertical[source_index];
	// restore the cached row
	for(unsigned int i=0; i<this->cols; i++) {
		this->items[source_index * this->cols + i] =tempRow[i];
	 }
	if ( this->headerVertical != NULL )
		this->headerVertical[source_index] = tempVHeader;
	delete [] tempRow;
	return WRET_OK;
 }

WRESULT Table::sortByColumn( int column, short order ) {
	if ( column == -1 && this->headerVertical == NULL ) return WRET_ERR_PARAM;
	if ( column < 0 || column >= (int)this->cols ) return WRET_ERR_PARAM;
	// bubble sort
	int row = this->rows+1;
	while( --row > 0 ) {
		for( int i=0; i<row-1; ++i ) {
			wosh::ISerializable* obj1 = this->items[i * this->cols + column];
			wosh::ISerializable* obj2 = this->items[(i+1) * this->cols + column];
			if ( obj1 == NULL && obj2 == NULL )
				continue;
			if ( obj1 == NULL ) {
				if ( order == 1 )
					swapRow(i+1, i); // move forward
				continue;
			 }
			if ( obj2 == NULL ) {
				if ( order == 0 )
					swapRow(i+1, i); // move forward
				continue;
			 }
			short comparison = obj1->compare(obj2);
			if ( comparison == 0 )
				continue;
			else if ( (comparison < 0) ^ (order == 1) )
				swapRow(i+1, i); // move forward
		 }
	 }

	return WRET_OK;
 }
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// HELPERS

std::string Table::getClassNameOfCol( unsigned int col ) const
 {
	std::string classname = "";
	for(unsigned int row=0; row<this->rows; row++) {
		const ISerializable* obj = get( row, col );
		if ( obj == NULL )
			return "";
		if ( classname.size() == 0 ) {
			classname = obj->getClassName();
			continue;
		 }
		if ( obj->getClassName() != classname )
			return "";
	 }
	return classname;
}

std::string Table::getClassNameOfItems() const
 {
	std::string classname = "";
	unsigned int count = this->rows * this->cols;
	for(unsigned int i=0; i<count; i++) {
		if ( this->items[i] == NULL )
			continue;
		if ( classname.size() == 0 ) {
			classname = this->items[i]->getClassName();
			continue;
		 }
		if ( this->items[i]->getClassName() != classname )
			return "";
	 }
	return classname;
}

///////////////////////////////////////////////////////////////////////////////////////////////////// HELPERS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

std::string Table::toString() const {
	return Utilities::format("%s[%d,%d]",this->table_name.c_str(), this->rows, this->cols );
 }

WRESULT Table::store( IEncoderWriter* serializer ) const {
	if ( serializer == NULL ) return WRET_ERR_PARAM;
	WRESULT ret = WRET_OK;

	serializer->push( this->table_name, "name", SERIALIZE_FIELD );
	serializer->push( this->rows, "rows", SERIALIZE_FIELD );
	serializer->push( this->cols, "cols", SERIALIZE_FIELD );

	if ( hasHeaderVertical() ) {
		for (unsigned int row=0; row<rows; row++) {
			if ( this->headerVertical[row] == NULL ) continue;
			//serializer->push( this->headerVertical[row], "header", SERIALIZE_NODE );
		}
	 }


	return ret;
 }
WRESULT Table::retrieve( IEncoderReader* deserializer ) {
	if ( deserializer == NULL ) return WRET_ERR_PARAM;
	WRESULT ret = WRET_OK;

	return ret;
 }

///////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////
/*
class TableWrapper {
public:
	explicit TableWrapper(Table& output_table, bool vertical_iter = true, int rows_start = 0, int cols_start = 0 ) 
		: targetTarget(output_table), iterVertical(vertical_iter), row(cols_start), col(rows_start) { }

	template <typename OBJECT_TYPE>
	void push_back( const OBJECT_TYPE& item ) {
		if ( iterVertical )
			this->targetTarget.set( new Variant(item), row++, col);
		else
			this->targetTarget.set( new Variant(item), row, col++);
	 }
	void push_back( ISerializable* item ) {
		if ( iterVertical )
			this->targetTarget.set(item, row++, col);
		else
			this->targetTarget.set(item, row, col++);
	 }

private:
	Table& targetTarget;
	bool iterVertical;
	int row;
	int col;
};

// always iterate on items
//  - one wrapper for each  wor/col

// table memory access is the same (since stored as raster-array)
// simmetric for rows vs. cols

/*
FOR-EACH( COLUMN )
{
	FOR-EACH( item )
	 {
		add( item::METHOD_1, row, COL )
		add( item::METHOD_2, row, COL )
	 }
	++row;
}




template <class OBJECT_TYPE, typename VALUE_TYPE>
class TableColumnWrapper {
public:
	typedef VALUE_TYPE (OBJECT_TYPE::*MethodExtractor)(void) const;

	explicit TableColumnWrapper(Table& output_table, int column, MethodExtractor fptr, ISerializable* header = NULL ) 
		: targetTarget(output_table), col(column), fExtractorPtr(fptr), tblHeader(header), row(0) { }

	void push_back( const OBJECT_TYPE* item ) {
		this->targetTarget.set( new Data(item), row++, col);
	 }

private:
	MethodExtractor fExtractorPtr;
	Table& targetTarget;
	ISerializable* tblHeader;
	bool iterVertical;
	const int col;
	int row;
};

template <class OBJECT_TYPE>
class ITableSectionWrapper {
public:
	virtual ISerializable* extract( OBJECT_TYPE* object ) const = 0;

 };


// idea TableColumnWrapper<OBJECT_TYPE, VALUE_TYPE>::MethodExtractor
template <class OBJECT_TYPE, typename VALUE_TYPE, class PROXY_TYPE>
class TableSectionWrapper : public ITableSectionWrapper<OBJECT_TYPE> {
public:
	typedef VALUE_TYPE (OBJECT_TYPE::*MethodExtractor)(void) const;

	explicit TableSectionWrapper(MethodExtractor fptr ) 
		: fExtractorPtr(fptr) { }


	ISerializable* extract( OBJECT_TYPE* object ) const {
		return new PROXY_TYPE( (object->*fExtractorPtr)() );
	 }

private:
	MethodExtractor fExtractorPtr;
};

template<class OBJECT_TYPE>
class TableWapperAllocator {
public:
	typedef ITableSectionWrapper<OBJECT_TYPE> tableSection;

	explicit TableWapperAllocator(Table& output_table) 
		: targetTable(output_table), iRow(0) {
		this->cols = output_table.getCols();
		this->columns = new tableSection*[cols];
		for( unsigned int col=0; col<cols; ++col ) {
			this->columns[col] = NULL;
		 }
	}
	~TableWapperAllocator() {
		//delete this->columns;
		// acquired from table!? for( int col=0; col<cols; ++col ) { delete this->header[col]; }
		//delete this->header;
	 }

//	template <typename VALUE_TYPE, class PROXY_TYPE>
	void set( int col, tableSection* fptr, ISerializable* cHeader = NULL ) {
		this->columns[col] = fptr;
		targetTable.setHeaderHorizontal( cHeader, col );
	 }

	void operator() ( OBJECT_TYPE* object ) const {
		if ( object == NULL ) {
			++this->iRow;
			return;
		 }
		// fill colums
		for( unsigned int col=0; col<cols; ++col ) {
			if ( this->columns[col] != NULL ) {
				ISerializable* obj = this->columns[col]->extract(object);
				targetTable.set( obj, this->iRow, col );
			}
		 }
		++this->iRow;
	 }

private:
	mutable Table& targetTable;
	mutable unsigned int iRow;
	unsigned int cols;
	tableSection** columns;
};

void asd( Table* oTbl, std::vector<Object*>& objects2 ) {
	TableWapperAllocator<Object> tw(*oTbl);
	TableSectionWrapper<Object, const std::string&, Variant> tw0(&Object::getName);
	TableSectionWrapper<Object, const std::string&, Variant> tw1(&Object::getEntityID);
	tw.set(0, &tw0, new Variant("Name") );
	tw.set(1, &tw1, new Variant("EntityID") );

	for_each( objects2.begin(), objects2.end(), tw );

	
	for ( unsigned int col=0; col<oTbl->getCols(); ++col ) {
		ISerializable* issss = oTbl->getHeaderHorizontal(col);
		Variant* dsss = dynamic_cast<Variant*>(issss);
		if ( dsss == NULL )
			printf("NULL\t" );
		else
			printf("%s\t", dsss->toString().c_str() );
	}
	printf("\n------------------------------------\n");
	for ( unsigned int row=0; row<oTbl->getRows(); ++row ) {
		for ( unsigned int col=0; col<oTbl->getCols(); ++col ) {
			ISerializable* issss = oTbl->get(row, col);
			if ( issss == NULL )
				printf("NULL\t" );
			else
				printf("%s\t", issss->toString().c_str() );
		 }
		printf("\n");
	 }
int f = 0;

//NO	tw.set(0, &Bundle::getName, "Name" );
// tw.set(1, &Bundle::getPermission, "Permission" );
}
*/
///////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////

}; // namespace wosh

