/** @file    Thread.cpp
 *  @author  Alessandro Polo
 *  @version $Id: Thread.cpp 2193 2010-03-23 06:46:47Z alex $
 *  @brief
 * File containing methods for the wosh::Thread class.
 * The header for this class can be found in Thread.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include <core/Thread.h>

 #include <core/ObjectAllocator.h>
 #include <core/ThreadManager.h>
 #include <core/Utilities.h>


using namespace std;
namespace wosh {

 WOSH_REGISTER_INTERFACE(wosh::Thread, "", 0.8, _static_Thread )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

Thread::Thread( const Object* owner, const std::string& threadName ) : QThread()
 {
	this->id = 0;
	this->running = false;
	this->threadname = threadName;

	this->aliveTs = -1;
	this->aliveVariance = 10000;

	this->ownerObj = owner;
	this->threadListener = NULL;

	// changes the thread affinity for this object and its children
	moveToThread(this);

	setThreadState( Thread::STATE_CREATED );
 }

Thread::~Thread() {
	setThreadState( Thread::STATE_DESTROYED );
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////// START/STOP

WRESULT Thread::startThread( unsigned long synch_timeout ) {
	QThread::start();
	if ( synch_timeout == 1 ) {
		printf("\nWRESULT Thread::startThread( 1 ) [%s]\n", this->getThreadName().c_str() );
	}

	bool started = Thread::waitState( this, Thread::STATE_RUNNING, synch_timeout );
	if ( started )
		return WRET_OK;
	// probably the initializeThread() returned false! or waiting timed out..
	// let' the thread exit RUN call
	// else we may edit/destroy thread while still running
	// this is because initializeThread() is inside RUN()!
	if ( this->threadState == STATE_FAILURE )
		QThread::wait();
	if ( synch_timeout == 0 ) // was asynch
		return WRET_OK;
	return WRET_ERR_INTERNAL;
 }

WRESULT Thread::quitThread( unsigned long synch_timeout )
 {
	this->running = false; // stop implemented loop if any
	QThread::quit();
	if ( synch_timeout == 1 ) {
		printf("\nWRESULT Thread::quitThread( 1 ) [%s]\n", this->getThreadName().c_str() );
	}
	bool stopped = Thread::waitState( this, Thread::STATE_STOPPED, synch_timeout );
	if ( stopped || synch_timeout == 0 )
		return WRET_OK;
	return WRET_ERR_INTERNAL;
 }

bool Thread::waitThread( unsigned long time ) {
	return QThread::wait(time);
}

void Thread::terminateThread() {
	QThread::terminate();
 }

////////////////////////////////////////////////////////////////////////////////////////////////// START/STOP
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// RUN FUNCTION

void Thread::run() {
	this->id = (long)QThread::currentThreadId();

	bool initialized = initializingThread();
	if ( !initialized ) {
		this->running = false;
		setThreadState( Thread::STATE_FAILURE );
		return;
	 }

	setThreadState( Thread::STATE_RUNNING );

	this->running = true;
	runThread();
	this->running = false;

	exitingThread();

	setThreadState( Thread::STATE_STOPPED );
 }

//////////////////////////////////////////////////////////////////////////////////////////////// RUN FUNCTION
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool Thread::waitForMSec( long millisec ) {
	while( this->running && millisec > 0 ) {
		millisec -= 50;
		Thread::sleepForMSec(50);
	 }
	setThreadAlive();
	return this->running;
 }

bool Thread::isThreadRunning() const {
	if ( this->threadState == Thread::STATE_RUNNING )
		return true;
	return false;
 }

void Thread::setThreadAlive()
 {
	this->aliveTs = Utilities::std_time();
	if ( this->threadState == Thread::STATE_CRASHED ) {
		// must be a wrong notification (due to timeout)
		// restore the state
		log(LOG_WARNING, "Thread::setThreadAlive()", "Forcing state-swtich from CRASHED state [%l, %s]", this->id, this->threadname.c_str()  );
		if ( QThread::isRunning() )
			setThreadState( Thread::STATE_RUNNING, true );
		else
			setThreadState( Thread::STATE_FAILURE, true );
	 }
 }

bool Thread::isThreadAlive( long sec_span ) const {
	if ( this->aliveTs == 0 ) return false;
	if ( sec_span < 0 )
		sec_span = this->aliveVariance/10;
	if ( Utilities::std_time() - (long)this->aliveTs < sec_span )
		return true;
	return false;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Thread::setThreadState( Thread::THREAD_STATE state, bool raiseEvent ) {
	this->threadState = state;

	if ( this->threadListener != NULL )
		this->threadListener->thread_event( state, this );

	if ( raiseEvent )
		ThreadManager::push_event( state, this );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// STATIC METHODS

bool Thread::waitState( const Thread* thread, Thread::THREAD_STATE state, unsigned long maxtime_msec ) {
	if ( thread == NULL ) return false;
	if ( maxtime_msec > 0 )
		maxtime_msec = (unsigned long)(maxtime_msec / 10) + 1;
	unsigned long i = 0;
	while( ++i < maxtime_msec ) {
		if ( thread->getThreadState() == state )
			return true;
		if ( state != Thread::STATE_FAILURE && thread->getThreadState() == Thread::STATE_FAILURE )
			return false;
		Thread::sleepForMSec(10);
	 }
	return ( thread->getThreadState() == state );
 }


const char* ThreadStateStr[] = {
	"UNKNOWN",		///< (THREAD_STATE)0
	"CREATED",		///< (THREAD_STATE)1
	"STARTING",		///< (THREAD_STATE)2
	"RUNNING",		///< (THREAD_STATE)3
	"STOPPING",		///< (THREAD_STATE)4
	"STOPPED",		///< (THREAD_STATE)5
	"DESTROYED",	///< (THREAD_STATE)6
	"CRASHED",		///< (THREAD_STATE)7
	"FAILURE"		///< (THREAD_STATE)8
};

Enum2StrFromArray(Thread::getThreadStateAsString, Thread::THREAD_STATE, STATE_CREATED, STATE_FAILURE, 0, ThreadStateStr)

Str2EnumByArray(Thread::getThreadStateFromString, Thread::THREAD_STATE, 0, STATE_FAILURE, STATE_UNKNOWN, ThreadStateStr)

////////////////////////////////////////////////////////////////////////////////////////////// STATIC METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

}; // namespace wosh
