/**
 * @class   wosh::User
 * @brief   Represents an user of WOSH system
 *
 ****************************************************************************
 * @version $Id: User.h 2191 2010-03-22 23:07:12Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR UserS;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_User_H__
 #define __WOSH_Core_User_H__

 #include <woshDefs.h>
 #include <core/Object.h>
 #include <core/Variant.h>
 #include <core/MethodsProvider.h>
 #include <core/PropertiesProvider.h>
 #include <core/UserInfoCommmunicators.h>
 #include <core/UserInfoLocation.h>


namespace wosh {

 #define _User_TYPE					"wosh::User"

 #define _User_KEY_IsAtHome			"IsAtHome"

 #define _User_EVENT_UserLogged		"UserLogged"


class Request;
class Response;
class UserManager;
class SecurityManager;
 namespace database {
  class EncoderXML_template__User;
 }

class User :	public Object,
				public IPropertiesProviderListener,
				public MethodRunner
 {
	WOSH_SERIALIZABLE(wosh::User)
	WOSH_COMPARE_RTTI(wosh::User)


	private:
		/**
		 * @brief  Empty constructor.
		 */
		User();

		User( long id, const std::string& name, const std::string& password );

		User( long id, const std::string& name, const std::string& password, const std::string& groupsAsString  );

		friend class wosh::UserManager;
		friend class EncoderXML_template__User;

	public:
		/**
		 * @brief  Copy constructor.
		 */
		User( const User& m );

		/**
		 * @brief  Empty destructor.
		 */
		virtual ~User();

	public:

		void updateLocation( const std::string& new_location, long timestamp, double probability = 0.0 );

		void updateCommmunicator( const std::string& new_communicator, long timestamp, double ranking_offset = 0.0 );
		
		void setAtHome( bool value );


/** @name Getters
 * @{
 ******************************************************************************/
	public:

		inline long getID() const							{ return this->id; }

		bool isMemberOf( const std::string& group) const;

		bool isAtHome() const;

		const std::vector<std::string>& getGroups() const	{ return this->groups; }
		std::vector<std::string>& getGroups()				{ return this->groups; }

		const PropertiesProvider& getProperties() const		{ return this->Properties; }
		PropertiesProvider& getProperties() 				{ return this->Properties; }

		const MethodsProvider& getMethods() const			{ return this->Methods; }
		MethodsProvider& getMethods()						{ return this->Methods; }

		Variant getValueOf( const std::string& key ) const
			{ return Properties.getValueOf(key); }


		UserInfoLocation& getLocation()						{ return this->Location; }
		const UserInfoLocation& getLocation() const 		{ return this->Location; }

		UserInfoCommmunicators& getCommunicators()			{ return this->Communicators; }
		const UserInfoCommmunicators& getCommunicators( )const { return this->Communicators; }

//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:

		virtual void setName( const std::string& name );

		virtual void setPermission( const Permission& value );

		virtual void setPermission( const Permission& value, short overwrite_mode, bool when_empty = false );

		WRESULT setProperty( const std::string& key, const Variant& value )
															{ return Properties.set(key, value); }


//@}
/** @name IPropertiesProviderListener interface
 * @{
 ******************************************************************************/
 	public:
		virtual bool readingProperty( Property* , const PropertiesProvider*  ) { return true; }
		virtual bool updatingProperty( bool& do_update, const Variant& value_proposed, Property& property_current, const PropertiesProvider* source );

//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:
		virtual User& operator=(const User& m);
		virtual bool operator==(const User &other) const;
		virtual bool operator!=(const User &other) const	{ return !(*this == other);  }

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const				{ return Utilities::format("%ld:%s", getID(), getName().c_str()); }
		virtual WRESULT store( IEncoderWriter* serializer ) const;
		virtual WRESULT retrieve( IEncoderReader* deserializer );
//@}
	protected:

		void initUser();
		virtual void clear();

		const std::string& getPassword() const				{ return this->pwd; }

	protected:
		Response* mmDoConnect( const Request* request );
		Response* mmDoDisconnect( const Request* request );

	protected:

		long id;						///< Unique ID of the user
		std::string pwd;				///< login password 
		std::string pwd_proto;			///< protocol of the password {plain,md5, ..}

		std::vector<std::string> groups;

		mutable MethodsProvider Methods;
		mutable PropertiesProvider Properties;	///< properties collection


		UserInfoLocation Location;
		UserInfoCommmunicators Communicators;

	protected:
		friend class wosh::SecurityManager;

		friend class wosh::database::EncoderXML_template__User;


	public:

		static std::string root;

}; // class def

}; // namespace wosh

#endif //__WOSH_Core_User_H__
