/** @file    UserManager.cpp
 *  @author  Alessandro Polo
 *  @version $Id: UserManager.cpp 2215 2010-03-26 16:43:35Z alex $
 *  @brief
 * File containing methods for the wosh::UserManager class.
 * The header for this class can be found in UserManager.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include <core/UserManager.h>

 #include <core/ObjectAllocator.h>
 #include <core/WoshKernel.h>
 #include <core/FileSystem.h>
 #include <core/DataBaseManager.h>
 #include <core/Settings.h>
 #include <core/Thread.h>
 #include <core/SystemError.h>

 #include <core/MethodsDef.h>
 #include <core/MessageProcessingCommon.h>
 #include <core/MessageFilter.h>
 #include <core/Fact.h>
 #include <core/NEW_List.h>
 #include <core/Table.h>
 #include <core/Functional.h>
 #include <core/Utilities.h>

 #include <interfaces/services/CommunicatorService.h>


using namespace std;
namespace wosh {

// Gamma singleton! static reference is allocated and maintained by WoshKernel
extern "C" {
	UserManager* UserManager::getInstance() { 
		if ( WoshKernel::getInstance() == NULL ) return NULL;
		return &WoshKernel::getInstance()->users();
	}
}
 // register as abstract because it is a singleton:
 WOSH_REGISTER_INTERFACE(wosh::UserManager, "wosh::WoshModule", 0.01, _static_UserManager )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

UserManager::UserManager( const WoshKernel* parent ) : WoshModule(parent)
 {
	Object::setName( _UserManager_NAME );
	Log.setContext( Object::getName() );
	Object::getURI().fromString( _UserManager_URI );
	Object::getURI().registerGlobal();

	Properties.set( _UserManager_KEY_Users, 0,	Permission(Permission::Read) );
	Properties.set( _UserManager_KEY_UsersDB, "$DATABASE/users/users.xml",	Permission(Permission::RW, Permission::Read, Permission::Read) );

	Properties.set( _UserManager_KEY_Groups, 0,	Permission(Permission::Read) );
	Properties.set( _UserManager_KEY_GroupsDB, "$DATABASE/users/groups.xml",	Permission(Permission::RW, Permission::Read, Permission::Read) );


	Log(LOG_VERBOSE, " Registering Methods.." );
	MethodRequest* mmGetUsers = Methods.create<MethodRequest>( _METHOD_List, "Get Users", Permission::R_XR_XR__() );
	mmGetUsers->setMethod( this, (MethodRequestPtr)&UserManager::mmDoGetUsers );

	MethodRequest* mmGetUserGroups = Methods.create<MethodRequest>( _UserManager_METHOD_listgroups, "Get Groups", Permission::R_XR_XR__() );
	mmGetUserGroups->setMethod( this, (MethodRequestPtr)&UserManager::mmDoGetUserGroups );

	MethodRequest* mmGetUsersLocations = Methods.create<MethodRequest>( _UserManager_METHOD_listloc, "Get Users' Locations", Permission::R_XR_XR__() );
	mmGetUsersLocations->setMethod( this, (MethodRequestPtr)&UserManager::mmDoGetUsersLocations );

	MethodRequest* mmGetUsersCommunicators = Methods.create<MethodRequest>( _UserManager_METHOD_listcomm, "Get Users' Communicators", Permission::R_XR_XR__() );
	mmGetUsersCommunicators->setMethod( this, (MethodRequestPtr)&UserManager::mmDoGetUsersCommunicators );

	MethodMessageResponse* mmForceUserIn = Methods.create<MethodMessageResponse>( _UserManager_METHOD_forcelocation, "force user in a location", Permission::R_XR__R__() );
	mmForceUserIn->setMethod( this, (MethodMessageResponsePtr)&UserManager::mmDoForceUserIn );

	MethodMessageResponse* mmSearchUser = Methods.create<MethodMessageResponse>( _UserManager_METHOD_search, "search user matching pattners", Permission(Permission::RX) );
	mmSearchUser->setMethod( this, (MethodMessageResponsePtr)&UserManager::mmDoSearchUser );

	registerGroup_( new UserGroup(1,	"root") );
	registerGroup_( new UserGroup(10,	"virtual") );
	registerGroup_( new UserGroup(20,	"nobody") );
	registerGroup_( new UserGroup(110,	"inhabitants") );
	registerGroup_( new UserGroup(3200,	"friends") );

	registerUser_( new User(1,	"root",		"wosh",	"root;virtual") );
	registerUser_( new User(10,	"wosh",		"",		"root;virtual") );
	registerUser_( new User(50,	"nobody",	"",		"nobody;virtual") );
	registerUser_( new User(52,	"anonymous","",		"nobody;virtual") );
 }

UserManager::~UserManager() {
	Log(LOG_INFO, " Destroying.." );

	if ( this->users.count() != 0 )
		Log(LOG_VERBOSE, ":~UserManager() : Freeing Users [%d]..", this->users.count() );
	this->users.clear();

	if ( this->groups.count() != 0 )
		Log(LOG_VERBOSE, ":~UserManager() : Freeing Groups [%d]..", this->groups.count() );
	this->groups.clear();

	Log(LOG_VERBOSE, ":~UserManager() Destroyed." );
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////// KERNEL EVENTS

WRESULT UserManager::onKernelInitializing() {
	WRESULT ret = WoshModule::onKernelInitializing();

	BusSecurity.setMessageHandler(this);
	WRESULT bus_d_connected = BusSecurity.connect( _Bus_Security );
	if ( WFAILED(bus_d_connected) )
		SystemError::raise(this, Log, SystemError::CRITICAL, 0, "init", "FAILED#%d Connecting Bus (wosh.Bus.Security)..", bus_d_connected );
	else
		Log(LOG_INFO, " Created BusRing (wosh.Bus.Security)." );

	ret += UserManager::loadUsers();
	//ret += UserManager::loadGroups();
	return ret;
 }

/////////////////////////////////////////////////////////////////////////////////////////////// KERNEL EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////// PROCESS BUS MESSAGES

void UserManager::busMessage( const Message& message, const Bus* source )
 {
	if ( message.isEmpty() ) return;
	// avoid message replicas
	if ( !MessageFilter::filterReplica( message, &BusCore ) )
		return;

	if ( message.getContent()->isFact() ) {
		const Fact* fact = message.getContent()->asFact();
		if ( fact->getEvent() != _CommunicatorService_EVENT_UserOnline &&
				fact->getEvent() != _CommunicatorService_EVENT_UserOffline &&
				fact->getEvent() != _CommunicatorService_EVENT_UserInteraction )
			return;
		if ( !fact->hasData() || !fact->getData()->isClass(_Variant_TYPE) )
			return;
		const Variant* dataUser = dynamic_cast<const Variant*>( fact->getData() );
		if ( dataUser->isStringEmpty() )
			return;
		this->users.transactionBeginWrite();
		User* cUser = this->users.find( dataUser->asString() );
		if ( cUser == NULL ) {
			this->users.transactionEnd();
			return;
		 }

		long timestamp = message.getTimestamp();
		if ( fact->getEvent() == _CommunicatorService_EVENT_UserOnline )
			cUser->updateCommmunicator(message.getSource().toString(), timestamp);
		else if ( fact->getEvent() == _CommunicatorService_EVENT_UserOffline )
			cUser->updateCommmunicator(message.getSource().toString(), -timestamp);
		else if ( fact->getEvent() == _CommunicatorService_EVENT_UserInteraction )
			cUser->updateCommmunicator(message.getSource().toString(), timestamp );

		this->users.transactionEnd();
		return;
	 }

	if ( !message.getContent()->isRequest() ) return;

	if ( !message.getDestination().isChildOf(this->getURI(), true) ) {
		WoshModule::busMessage(message, source);
		return;
	 }

	Log(LOG_VERBOSE, ":busMessage() = Message-Request for Child '%s'", message.getDestination().getName().c_str() );
	this->users.transactionBeginRead(); // following operation won't ever really add/delete a channel item
	MPC_mmDo_InContainer<User>(message, this->users, Log, BusCore);
	this->users.transactionEnd();
 }

//////////////////////////////////////////////////////////////////////////////////////// PROCESS BUS MESSAGES
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////// COLLECTOR METHODS

WRESULT UserManager::registerUser( User* user ) {
	MutexLockerWrite mL(this->users.getMutex());
	return registerUser_( user );
 }

WRESULT UserManager::registerUser_( User* user ) {
	if ( user == NULL ) return WRET_ERR_PARAM;
	WRESULT ret = WRET_OK;

	user->getURI().clear();
	user->getURI().setParent( this->getURI() );
	user->getURI().setName( user->getName() );
	user->getURI().registerGlobal();

	this->groups.transactionBeginWrite();
	tStrUserGroupMap::ConstIterator it_g;
	const tStrUserGroupMap::ConstIterator it_g_end = this->groups.end();
	std::vector<std::string>::iterator it;
	bool editing = true;
	while (editing) {
		editing = false;
		for ( it=user->getGroups().begin(); it!=user->getGroups().end(); ++it ) {
			bool found = false;
			for ( it_g=this->groups.begin(); it_g!=it_g_end; ++it_g ) {
				if ( it_g->second == NULL ) continue;
				if ( it_g->second->getName() != *it ) continue;
				it_g->second->setUserCount( it_g->second->getUserCount() + 1 );
				found = true;
				break;
			}
			if ( !found ) {
				Log(LOG_WARNING, ":registerUser_(%s) Group %s NOT FOUND!", user->getName().c_str(), (*it).c_str() );
				user->getGroups().erase(it);
				editing = true;
				break;
			 }
		 }
	 }
	this->users.set(user->getName(), user);
	Properties.update( _UserManager_KEY_Users, this->users.size() );

	if ( WSUCCEEDED(ret) ) {
		Log(LOG_INFO, ":registerUser_(%s) id#%d", user->getName().c_str(), user->getID() );
	 }
	else
		Log(LOG_CRITICAL, ":registerUser_() : FAILED Registering %s", user->getName().c_str() );

	this->groups.transactionEnd();
	return ret;
 }

WRESULT UserManager::registerGroup_( UserGroup* usergroup ) {
	if ( usergroup == NULL ) return WRET_ERR_PARAM;
	this->groups.transactionBeginWrite();
	if ( this->groups.exists(usergroup->getName()) ) {
		Log(LOG_CRITICAL, ":registerGroup_() : FAILED Registering %s (already registered)", usergroup->getName().c_str() );
		this->groups.transactionEnd();
		return WRET_ERR_PARAM;
	 }
	this->groups.set(usergroup->getName(), usergroup);
	Properties.update( _UserManager_KEY_Groups, this->groups.size() );
	Log(LOG_INFO, ":registerGroup_(%s) id#%ld", usergroup->getName().c_str(), usergroup->getID() );
	this->groups.transactionEnd();
	return WRET_OK;
 }


//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT UserManager::unRegisterUser( User* user ) {
	if ( user == NULL ) return WRET_ERR_PARAM;
	WRESULT ret = WRET_OK;

	user->getURI().unRegisterGlobal();
	this->users.remove( user->getName() );

	if ( WSUCCEEDED(ret) ) {
		Log(LOG_INFO, ":unRegisterUser() : %s", user->getName().c_str() );
	 }
	else
		Log(LOG_CRITICAL, ":unRegisterUser() : FAILED UnRegistering %s", user->getName().c_str() );

	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT UserManager::loadGroups() {
	// may also be a folder of .xml files
	string xmlfile = Properties.getValueOf(_UserManager_KEY_GroupsDB).toString();
	Log(LOG_VERBOSE, ":loadGroups() File %s ..", xmlfile.c_str() );
	std::vector<ISerializable*> objects_load;
	WRESULT ret = DataBaseManager::loadObjects( objects_load, "wosh::UserGroup", xmlfile, "xml" );
	if ( WFAILED(ret) ) {
		Log(LOG_CRITICAL, ":loadGroups() FAILED#%ld loading objects!", ret );
		return ret;
	 }

	this->groups.transactionBeginWrite();
	std::vector<ISerializable*>::iterator it_load;
	for ( it_load=objects_load.begin(); it_load!=objects_load.end(); ++it_load ) {
		UserGroup* currObj = dynamic_cast<UserGroup*>(*it_load);
		if ( currObj == NULL ) continue;
		UserGroup* oldObj = this->groups.find(currObj->getName());
		if ( oldObj != NULL) {
			Log(LOG_WARNING, ":loadGroups() Group %s exists, ignored", currObj->getName().c_str() );
			// group already registered, 
			///@todo should merge info!
			delete currObj;
		 }
		else
			registerGroup_( currObj );
	 }
	this->groups.transactionEnd();

	Log(LOG_VERBOSE, ":loadGroups() Loaded %d groups", this->groups.count() );

	return ret;
 }

WRESULT UserManager::loadUsers() {
	// may also be a folder of .xml files
	string xmlfile = Properties.getValueOf(_UserManager_KEY_UsersDB).toString();
	Log(LOG_VERBOSE, ":loadUsers() File %s ..", xmlfile.c_str() );
	std::vector<ISerializable*> objects_load;
	WRESULT ret = DataBaseManager::loadObjects( objects_load, "wosh::User", xmlfile, "xml" );
	if ( WFAILED(ret) ) {
		Log(LOG_CRITICAL, ":loadUsers() FAILED#%d loading objects!", ret );
		return ret;
	 }

	this->users.transactionBeginWrite();
	std::vector<ISerializable*>::iterator it_load;
	for ( it_load=objects_load.begin(); it_load!=objects_load.end(); ++it_load ) {
		User* currObj = dynamic_cast<User*>(*it_load);
		if ( currObj == NULL ) continue;
		User* oldObj = this->users.find(currObj->getName());
		if ( oldObj != NULL) {
			Log(LOG_WARNING, ":loadUsers() User %s exists, intersecting properties", currObj->getName().c_str() );
			// user already registered, 
			///@todo should merge info!
			oldObj->Properties.intersect(currObj->Properties, true, false);
			delete currObj;
		 }
		else
			registerUser_( currObj );
	 }
	this->users.transactionEnd();

	Log(LOG_VERBOSE, ":loadUsers() Loaded %d users", this->users.size() );

	return ret;
 }

/////////////////////////////////////////////////////////////////////////////////////////// COLLECTOR METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS

Response* UserManager::mmDoSearchUser( const Message* requestMessage ) {
	if ( requestMessage == NULL || requestMessage->isEmpty() ) return NULL;
	if ( !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();
	if ( !request->hasData() ) {
		return request->replyResponse(WRET_ERR_PARAM, "Invalid data");
	 }
	string key;
	Variant propValue;
	if ( request->getData()->isClass(_Property_TYPE) ) {
		const Property* prop = request->getData()->as<Property>();
		if ( prop == NULL )
			return request->replyResponse(WRET_ERR_PARAM, "Invalid data");
		key = prop->getKey();
		propValue = prop->getValue();
	 }
	std::vector<std::string> fusers = findUserByProperty( key, propValue, requestMessage->getSecurityToken() );
	Table* tbl = new Table(fusers.size(), 1, true);
	tbl->setTableName("Users matching -");
	tbl->setHeaderHorizontal("Name", 0);
	std::vector<std::string>::iterator it;
	unsigned int iRow = 0;
	for ( it=fusers.begin(); it!=fusers.end(); ++it, ++iRow ) {
		tbl->set<Variant>(*it, iRow, 0);
	 }
	return request->replyResponse(WRET_OK, tbl);
/*
	NEW_List* list = new NEW_List(fusers.size());
	std::vector<std::string>::iterator it;
	for ( it=fusers.begin(); it!=fusers.end(); ++it ) {
		list->push_back<Variant>(*it);
	 }
	return request->replyResponse(WRET_OK, list);
*/
 }

Response* UserManager::mmDoGetUsers( const Request* request ) {
	if ( request == NULL ) return NULL;
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, ":mmDoGetUsers(%s)", option.c_str() );
	if ( option == "s" || option == "short" ) {
		this->users.transactionBeginRead();
		NEW_List* objs = new NEW_List( this->users.size() );
//		for_each( this->users.begin(), this->users.end(),
//					Functor_Appender_TY<User, const std::string&, Variant>(objs, &User::getName ) );
		this->users.transactionEnd();
		return request->replyResponse(WRET_OK, objs);
	 }
	else {
		this->users.transactionBeginRead();
		Table* tbl = new Table( this->users.size(), 6, true );
		tbl->setTableName("Users");
		tbl->setHeaderHorizontal("ID", 0);
		tbl->setHeaderHorizontal("Name", 1);
		tbl->setHeaderHorizontal("Type", 2);
		tbl->setHeaderHorizontal("URI", 3);
		tbl->setHeaderHorizontal("Groups", 4);
		tbl->setHeaderHorizontal("Location", 5);
		unsigned int iRow = 0;
		tStrUserMap::ConstIterator it;
		const tStrUserMap::ConstIterator it_end = this->users.end();
		for ( it=this->users.begin(); it!=it_end; ++it, ++iRow ) {
			if ( it->second == NULL ) continue;
			tbl->set( new Variant(it->second->getID()), iRow, 0);
			tbl->set( new Variant(it->second->getName()), iRow, 1);
			tbl->set( new Variant(it->second->getClassName()), iRow, 2);
			tbl->set( new Variant(it->second->getURI().toString()), iRow, 3);
			tbl->set( new Variant(Utilities::joinVector(it->second->getGroups(),";")), iRow, 4);
			tbl->set( new Variant(it->second->getLocation().getLastName()), iRow, 5);
		}
		this->users.transactionEnd();
		return request->replyResponse(WRET_OK, tbl);
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* UserManager::mmDoGetUserGroups( const Request* request ) {
	if ( request == NULL ) return NULL;
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, ":mmDoGetUserGroups(%s)", option.c_str() );
	if ( option == "s" || option == "short" ) {
		this->groups.transactionBeginRead();
		NEW_List* objs = new NEW_List( this->groups.size() );
//		for_each( this->groups.begin(), this->groups.end(),
//					Functor_Appender_TY<UserGroup, const std::string&, Variant>(objs, &UserGroup::getName ) );
		this->groups.transactionEnd();
		return request->replyResponse(WRET_OK, objs);
	 }
	else {
		this->groups.transactionBeginRead();
		Table* tbl = new Table( this->groups.size(), 5, true );
		tbl->setTableName("Groups");
		tbl->setHeaderHorizontal("ID", 0);
		tbl->setHeaderHorizontal("Name", 1);
		tbl->setHeaderHorizontal("Type", 2);
		tbl->setHeaderHorizontal("URI", 3);
		tbl->setHeaderHorizontal("Users", 4);
		unsigned int iRow = 0;
		tStrUserGroupMap::ConstIterator it;
		const tStrUserGroupMap::ConstIterator it_end = this->groups.end();
		for ( it=this->groups.begin(); it!=it_end; ++it, ++iRow ) {
			if ( it->second == NULL ) continue;
			tbl->set( new Variant(it->second->getID()), iRow, 0);
			tbl->set( new Variant(it->second->getName()), iRow, 1);
			tbl->set( new Variant(it->second->getClassName()), iRow, 2);
			tbl->set( new Variant(it->second->getURI().toString()), iRow, 3);
			tbl->set( new Variant(it->second->getUserCount()), iRow, 4);
		}
		this->groups.transactionEnd();
		return request->replyResponse(WRET_OK, tbl);
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* UserManager::mmDoGetUsersCommunicators( const Request* request ) {
	if ( request == NULL ) return NULL;
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, ":mmDoGetUsersCommunicators(%s)", option.c_str() );
	if ( option == "s" || option == "short" ) {
		this->users.transactionBeginRead();
		NEW_List* objs = new NEW_List(this->users.size());
		tStrUserMap::ConstIterator it;
		const tStrUserMap::ConstIterator it_end = this->users.end();
		for ( it=this->users.begin(); it!=it_end; ++it ) {
			if ( it->second == NULL ) continue;
			objs->push_back<Variant>( it->second->getCommunicators().getLastName() );
		}
		this->users.transactionEnd();
		return request->replyResponse(WRET_OK, objs);
	 }
	else {
		this->users.transactionBeginRead();
		Table* tbl = new Table( this->users.size(), 4, true );
		tbl->setTableName("Users' Communicators");
		tbl->setHeaderHorizontal("Username", 0);
		tbl->setHeaderHorizontal("LastCommunicator", 1);
		tbl->setHeaderHorizontal("TimeStamp", 2);
		tbl->setHeaderHorizontal("Ranking", 3);
		unsigned int iRow = 0;
		tStrUserMap::ConstIterator it;
		const tStrUserMap::ConstIterator it_end = this->users.end();
		for ( it=this->users.begin(); it!=it_end; ++it, ++iRow ) {
			if ( it->second == NULL ) continue;
			tbl->set( new Variant(it->second->getName()), iRow, 0);
			string commURI;
			UserInfoCommmunicatorData data;
			it->second->getCommunicators().getCommmunicatorLast(commURI, data);
			tbl->set( new Variant(commURI), iRow, 1);
			tbl->set( new Variant(data.timestamp), iRow, 2);
			tbl->set( new Variant(data.ranking), iRow, 3);
		}
		this->users.transactionEnd();
		return request->replyResponse(WRET_OK, tbl);
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* UserManager::mmDoGetUsersLocations( const Request* request ) {
	if ( request == NULL ) return NULL;
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, ":mmDoGetUsersLocations(%s)", option.c_str() );
	if ( option == "s" || option == "short" ) {
		this->users.transactionBeginRead();
		NEW_List* objs = new NEW_List(this->users.size());
		tStrUserMap::ConstIterator it;
		const tStrUserMap::ConstIterator it_end = this->users.end();
		for ( it=this->users.begin(); it!=it_end; ++it ) {
			if ( it->second == NULL ) continue;
			objs->push_back<Variant>( it->second->getLocation().getLastName() );
		}
		this->users.transactionEnd();
		return request->replyResponse(WRET_OK, objs);
	 }
	else {
		this->users.transactionBeginRead();
		Table* tbl = new Table( this->users.size(), 5, true );
		tbl->setTableName("Users' Locations");
		tbl->setHeaderHorizontal( "Name", 0);
		tbl->setHeaderHorizontal( "AtHome", 1);
		tbl->setHeaderHorizontal( "LastCommunicator", 2);
		tbl->setHeaderHorizontal( "LastCommTs", 3);
		tbl->setHeaderHorizontal( "LastCommProb", 4);
		unsigned int iRow = 0;
		tStrUserMap::ConstIterator it;
		const tStrUserMap::ConstIterator it_end = this->users.end();
		for ( it=this->users.begin(); it!=it_end; ++it, ++iRow ) {
			if ( it->second == NULL ) continue;
			tbl->set<Variant>( it->second->getName(), iRow, 0);
			tbl->set<Variant>( it->second->isAtHome(), iRow, 1);
			string locURI;
			UserInfoLocationData data;
			it->second->getLocation().getLocationLast(locURI, data);
			tbl->set<Variant>( locURI, iRow, 2);
			tbl->set( new Variant(data.timestamp, Variant::DATETIME), iRow, 3);
			tbl->set<Variant>( data.probability, iRow, 4);
		}
		this->users.transactionEnd();
		return request->replyResponse(WRET_OK, tbl);
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* UserManager::mmDoForceUserIn( const Message* requestMessage ) {
	if ( requestMessage == NULL || requestMessage->isEmpty() ) return NULL;
	if ( !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();
	if ( !request->hasData() ) {
		return request->replyResponse(WRET_ERR_PARAM, "Invalid data");
	 }
	Variant dUsername;
	Variant dLocation;
	request->extractArguments(dUsername, dLocation);
	if ( dUsername.isStringEmpty() || dLocation.isStringEmpty() ) {
		return request->replyResponse(WRET_ERR_PARAM, "Invalid arguments [username, location]");
	 }
	std::string sender = "";
	if ( requestMessage->getSecurityToken() != NULL )
		sender = requestMessage->getSecurityToken()->getUser();
	Log(LOG_INFO, "mmDoForceUserIn() : Force %s in %s", dUsername.asString().c_str(), dLocation.asString().c_str() );
	this->users.transactionBeginWrite();
	User* oldObj = this->users.find(dUsername.asString());
	if ( oldObj == NULL ) {
		this->users.transactionEnd();
		return request->replyResponse(WRET_ERR_PARAM, "Invalid username");
	 }
	oldObj->updateLocation(dLocation.asString(), Utilities::std_time(), 1.0);
	this->users.transactionEnd();
	return request->replyResponse(WRET_OK);
 }

///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// STATIC METHODS

bool UserManager::isUser( const std::string& user ) {
	UserManager* usrman = UserManager::getInstance();
	if ( usrman == NULL ) return false;

	MutexLockerRead mL(usrman->users.getMutex());
	User* oldObj = usrman->users.find(user);
	return ( oldObj != NULL );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool UserManager::isAuthorized( const std::string& user, const std::string& password ) {
	UserManager* usrman = UserManager::getInstance();
	if ( usrman == NULL ) return false;
	//Thread::sleepForMSec( Utilities::randomInt() % 100 );

	MutexLockerRead mL(usrman->users.getMutex());
	User* oldObj = usrman->users.find(user);
	if ( oldObj == NULL )
		return false;
	return ( oldObj->getPassword() == password );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool UserManager::isGroupOfUser( const std::string& user, const std::string& group ) {
	UserManager* usrman = UserManager::getInstance();
	if ( usrman == NULL ) return false;

	MutexLockerRead mL(usrman->users.getMutex());
	User* oldObj = usrman->users.find(user);
	if ( oldObj == NULL )
		return false;
	return oldObj->isMemberOf(group);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Variant UserManager::getUserProperty( const std::string& user, const std::string& key, const SecurityToken* )
 {
	UserManager* usrman = UserManager::getInstance();
	if ( usrman == NULL ) return Variant();

	MutexLockerRead mL(usrman->users.getMutex());
	User* oldObj = usrman->users.find(user);
	if ( oldObj == NULL )
		return Variant();
	return oldObj->getValueOf(key);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

std::string UserManager::getUserLocation( const std::string& user, const SecurityToken* queryOwner )
 {
	return UserManager::getUserProperty(user, "LocationLast", queryOwner).toString();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

UserInfoCommmunicators* UserManager::getUserCommunicators( const std::string& user, const SecurityToken* )
 {
	UserManager* usrman = UserManager::getInstance();
	if ( usrman == NULL ) return NULL;
	MutexLockerRead mL(usrman->users.getMutex());
	User* cUser = usrman->users.find(user);
	if ( cUser == NULL )
		return NULL;
	return cUser->getCommunicators().clone();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

int UserManager::getUserCountAtHome()
 {
	UserManager* usrman = UserManager::getInstance();
	if ( usrman == NULL ) return -1;

	int count = 0;
	MutexLockerRead mL(usrman->users.getMutex());
	tStrUserMap::ConstIterator it;
	const tStrUserMap::ConstIterator it_end = usrman->users.end();
	for ( it=usrman->users.begin(); it!=it_end; ++it ) {
		if ( it->second == NULL ) continue;
		if ( it->second->isAtHome() )
			++count;
	 }
	return count;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT UserManager::setUserProperty( const std::string& user, const std::string& key, const Variant& data, const SecurityToken* )
 {
	UserManager* usrman = UserManager::getInstance();
	if ( usrman == NULL ) return WRET_ERR_ILLEGAL_USE;

	MutexLockerRead mL(usrman->users.getMutex());
	User* oldObj = usrman->users.find(user);
	if ( oldObj == NULL )
		return WRET_ERR_PARAM;
	return oldObj->setProperty(key, data);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT UserManager::setUserLocation( const std::string& user, const string& new_location, double probability, short at_home )
 {
	UserManager* usrman = UserManager::getInstance();
	if ( usrman == NULL ) return WRET_ERR_ILLEGAL_USE;

	usrman->users.transactionBeginRead();
	User* oldObj = usrman->users.find(user);
	if ( oldObj == NULL ) {
		usrman->users.transactionEnd();
		return WRET_ERR_PARAM;
	 }
	oldObj->updateLocation(new_location, Utilities::std_time(), probability);
	if ( at_home != 2 )
		oldObj->setAtHome(at_home);

	int usersAtHome = 0;
	tStrUserMap::ConstIterator it;
	const tStrUserMap::ConstIterator it_end = usrman->users.end();
	for ( it=usrman->users.begin(); it!=it_end; ++it ) {
		if ( it->second == NULL ) continue;
		if ( it->second->isAtHome() )
			++usersAtHome;
	 }

	usrman->users.transactionEnd();

	NEW_List* dL = new NEW_List();
	dL->push_back<Variant>( user );
	dL->push_back<Variant>( new_location );
	dL->push_back<Variant>( probability );
	dL->push_back<Variant>( at_home );
	Message* msg_event = new Message( new Fact( _UserManager_EVENT_located, dL) );
	msg_event->setSource( usrman );
	msg_event->setDestinationBroadcast();
	msg_event->setDestinationBus(_Bus_Security);
	usrman->BusSecurity.postMessage(msg_event);

	if ( usersAtHome == 0 ) {
		Message* msg_event = new Message( new Fact( _UserManager_EVENT_UsersAtHome, new Variant(usersAtHome)) );
		msg_event->setSource( usrman );
		msg_event->setDestinationBroadcast();
		msg_event->setDestinationBus(_Bus_Security);
		usrman->BusSecurity.postMessage(msg_event);
	 }
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT UserManager::setUserCommunicator( const std::string& user, const string& communicator, long timestamp, double ranking_offset )
 {
	UserManager* usrman = UserManager::getInstance();
	if ( usrman == NULL ) return WRET_ERR_ILLEGAL_USE;

	MutexLockerRead mL(usrman->users.getMutex());
	User* oldObj = usrman->users.find(user);
	if ( oldObj == NULL )
		return WRET_ERR_PARAM;
	oldObj->updateCommmunicator(communicator, timestamp, ranking_offset);
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

std::vector<std::string> UserManager::findUserByProperty( const string& key, const Variant& value, const SecurityToken* )
 {
	std::vector<std::string> res;
	UserManager* usrman = UserManager::getInstance();
	if ( usrman == NULL ) return res;

	MutexLockerRead mL(usrman->users.getMutex());
	tStrUserMap::ConstIterator it;
	const tStrUserMap::ConstIterator it_end = usrman->users.end();
	for ( it=usrman->users.begin(); it!=it_end; ++it ) {
		if ( it->second == NULL ) continue;
		if ( !it->second->getProperties().exists(key) ) continue;
		if ( !value.isEmpty() && it->second->getValueOf(key) != value ) continue;
		res.push_back( it->second->getName() );
	 }
	return res;
 }

////////////////////////////////////////////////////////////////////////////////////////////// STATIC METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

}; // namespace wosh
