/**
 * @class   wosh::UserManager
 * @brief   Static collector of application's threads, designed for diagnostic and stats.
 * 
 * \image html diagrams/UserManager.png "wosh::UserManager UML diagram"
 *
 *
 *
 * @ingroup CoreModules
 ****************************************************************************
 * @version 0.7.000 $Id: UserManager.h 2203 2010-03-23 18:33:39Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/*! \page page_core
 * \htmlonly <hr/> \endhtmlonly
 * \section page_core_users Users
 *
 * wosh::UserManager collects users' object instances (wosh::User) and offers
 * methods for accessing and updating information (properties) of Users.
 *
 * Users are loaded from a WOSH compatible database (see \ref page_core_database).
 * 
 *
 *
 *
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR UserS;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_UserManager_H__
 #define __WOSH_Core_UserManager_H__

 #include <woshDefs.h>
 #include <core/WoshModule.h>

 #include <core/User.h>
 #include <core/UserGroup.h>


namespace wosh {

 #define _UserManager_NAME					"UserManager"
 #define _UserManager_TYPE					"wosh::UserManager"
 #define _UserManager_URI					":Users"

 #define _UserManager_KEY_Users				"Users"
 #define _UserManager_KEY_UsersDB			"UsersDB"
 #define _UserManager_KEY_Groups			"Groups"
 #define _UserManager_KEY_GroupsDB			"GroupsDB"

 #define _UserManager_METHOD_listgroups		"listgroups"

 #define _UserManager_METHOD_listloc		"listloc"
 #define _UserManager_METHOD_forcelocation	"forcelocation"

 #define _UserManager_METHOD_listcomm		"listcomm"
 #define _UserManager_METHOD_rankcomm		"rankcomm"

 #define _UserManager_METHOD_search			"search"

 #define _UserManager_EVENT_located			"UserLocated"

 #define _UserManager_EVENT_UsersAtHome		"UsersAtHome"


class WoshKernel;

class UserManager : public WoshModule
 {
	WOSH_MODULE(wosh::UserManager)

	typedef Map22<std::string, wosh::User*> tStrUserMap;
	typedef Map22<std::string, wosh::UserGroup*> tStrUserGroupMap;


	protected:
		UserManager( const WoshKernel* parent = NULL );

		WRESULT onKernelInitializing();

		friend class wosh::WoshKernel;

	public:
		virtual ~UserManager();


		virtual void busMessage( const Message& message, const Bus* source );

	public:
		// deprecated
		static bool isAuthorized( const std::string& user, const std::string& password );

/*
		static bool signInUser( const string& user,
									const string& authorization_protocol,
									const char* auth_data,
									long auth_data_size
									)  { return false; }
*/

//@}
/** @name Public methods (not for bundles, usually hidden to most classes)
 * @{
 ******************************************************************************/
	public:

		virtual WRESULT registerUser( User* user );

		virtual WRESULT unRegisterUser( User* user );

		virtual WRESULT loadUsers();

		virtual WRESULT loadGroups();

//@}
/** @name Public static methods (for bundles)
 * @{
 ******************************************************************************/
	public:

		static bool isUser( const std::string& user );

		static bool isGroupOfUser( const std::string& user, const std::string& group );

		static int getUserCountAtHome();

		static std::vector<std::string> findUserByProperty( const std::string& key, const Variant& value, const SecurityToken* queryOwner = NULL );

		static Variant getUserProperty( const std::string& user, const std::string& key, const SecurityToken* queryOwner = NULL );
		static WRESULT setUserProperty( const std::string& user, const std::string& key, const Variant& data, const SecurityToken* queryOwner );

		static std::string getUserLocation( const std::string& user, const SecurityToken* queryOwner = NULL );
		static UserInfoCommmunicators* getUserCommunicators( const std::string& user, const SecurityToken* queryOwner = NULL );


		///@todo ADD MessageSecurity to all calls:

		static WRESULT setUserLocation( const std::string& user, const std::string& new_location, double probability = 0.0, short at_home = 2 );
		static WRESULT setUserCommunicator( const std::string& user, const std::string& communicator, long timestamp, double ranking_offset = 0.0 );

//@}
/** @name Public methods (mapped from Kernel)
 * @{
 ******************************************************************************/

//@}

	protected:
		WRESULT registerUser_( User* user );
		WRESULT registerGroup_( UserGroup* usergroup );


		friend class wosh::User;

//@}
/** @name Internal Mehods and Events management
 * @{
 ******************************************************************************/
	private:

		static UserManager* getInstance();

//@}
	private:
		Response* mmDoGetUsers( const Request* request );
		Response* mmDoGetUsersCommunicators( const Request* request );
		Response* mmDoGetUsersLocations ( const Request* request );
		Response* mmDoGetUserGroups( const Request* request );
		Response* mmDoForceUserIn( const Message* requestMessage );
		Response* mmDoSearchUser( const Message* requestMessage );

	private:
		tStrUserMap users;			///< map of users [name, pointer]
		tStrUserGroupMap groups;	///< map of groups [name, pointer]
		BusConnector BusSecurity;	///< connector to wosh.Bus.Security


}; // class def

}; // namespace wosh

#endif //__WOSH_Core_UserManager_H__
