/**
 * @class   wosh::Variant
 * @brief   
 *
 ****************************************************************************
 * @version $Id: Variant.h 1937 2010-02-24 20:04:23Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_Variant_H__
 #define __WOSH_Core_Variant_H__

 #include <woshDefs.h>
 #include <core/Serialization.h>


namespace wosh {

#define _Variant_TYPE		"wosh::Variant"

class Variant : public ISerializable
 {
	WOSH_SERIALIZABLE(wosh::Variant)
	WOSH_COMPAREALL_RTTI(wosh::Variant)

	public:
		/// types of the data
		enum DATA_TYPE {
			EMPTY		= 0x00, // memset(0), unknown = empty
			BOOLEAN		= 0x01, // as bool
			CHAR		= 0x02, // as char
			INTEGER		= 0x03, // as long
			LONG		= 0x04, // as long
			INT64		= 0x05, // as long long
			DATETIME	= 0x06, // as long long
			TIMESPAN	= 0x07, // as long long
			DOUBLE		= 0x08, // as double
			FLOAT		= 0x09, // as float

			CHARS		= 0x10, // 16
			STRING		= 0x11
		 };
		// MISSING: enum type (with name info), bitset


		// pure C struct,  about 8 bytes?
		 union tVariantUnion {
			bool v_bool;
			char v_char;
			long v_long;
			int64 v_int64;
			double v_double;
			float v_float;

			char* v_pchar;
			std::string* v_pstr;
		 };


	public:
		Variant( )
			: vType(Variant::EMPTY) {
			memset(&this->vHolder, 0, sizeof(vHolder));
		 }

		// 'explicit' constructos for implicit conversion
		// a template would makes things harder to debug,
		// for example you could acquire pointers as booleans..

		Variant( const bool& value )
			: vType(Variant::BOOLEAN) {
			this->vHolder.v_bool = value;
		 }
		Variant( const int& value )
			: vType(Variant::INTEGER) {
			this->vHolder.v_long = value;
		 }
		Variant( const unsigned int& value )
			: vType(Variant::INTEGER) {
			this->vHolder.v_long = value; // upcasted
		 }
		Variant( const long& value )
			: vType(Variant::LONG) {
			this->vHolder.v_long = value;
		 }
		Variant( const unsigned long& value )
			: vType(Variant::INT64) {
			this->vHolder.v_int64 = value; // upcasted
		 }
		Variant( const int64& value )
			: vType(Variant::INT64) {
			this->vHolder.v_int64 = value;
		 }
		Variant( const double& value )
			: vType(Variant::DOUBLE) {
			this->vHolder.v_double = value;
		 }
		Variant( const float& value )
			: vType(Variant::FLOAT) {
			this->vHolder.v_float = value;
		 }
		Variant( const std::string& value )
			: vType(Variant::STRING) {
			this->vHolder.v_pstr = new std::string(value);
		 }
		Variant( const char* value )
			: vType(Variant::STRING) {
			this->vHolder.v_pstr = new std::string(value);
		 }

		template <typename T>
		explicit Variant( const T& value, Variant::DATA_TYPE forceType )
			: vType(Variant::EMPTY) {
			set( (const T&)value, forceType);
		 }

		Variant( const wosh::Variant& m )
			: vType(Variant::EMPTY) {
			*this = m;
		 }

		explicit Variant( const ISerializable* ) : vType(Variant::EMPTY) { }
		explicit Variant( ISerializable* ) : vType(Variant::EMPTY) { }

	public:

		virtual ~Variant() {
			free();
		 }


	public:
		inline void free() {
			if ( this->vType == Variant::STRING && this->vHolder.v_pstr != NULL ) {
				delete this->vHolder.v_pstr; this->vHolder.v_pstr = NULL;
			 }
			else if ( this->vType == Variant::CHARS && this->vHolder.v_pchar != NULL ) {
				delete [] this->vHolder.v_pchar; this->vHolder.v_pchar = NULL;
			 }
		 }

		void clear() {
			free();
			memset(&this->vHolder, 0, sizeof(vHolder));
			this->vType = Variant::EMPTY;
		 }

/** @name Setters
 * @{
 ******************************************************************************/
	public:

		inline void set( const bool& value ) {
			free();
			this->vType = Variant::BOOLEAN;
			this->vHolder.v_bool = value;
		 }
		inline void set( const char& value ) {
			free();
			this->vType = Variant::CHAR;
			this->vHolder.v_char = value;
		 }
		inline void set( const unsigned char& value ) {
			free();
			this->vType = Variant::CHAR;
			this->vHolder.v_char = value;
		 }
		inline void set( const int& value ) {
			free();
			this->vType = Variant::INTEGER;
			this->vHolder.v_long = value;
		 }
		inline void set( const unsigned int& value ) {
			free();
			this->vType = Variant::INTEGER;
			this->vHolder.v_long = value; // upcasted
		 }
		// because often we have date in long-type (epoch)
		inline void set( const long& value, DATA_TYPE dataType = Variant::LONG ) {
			free();
			if ( dataType < Variant::INTEGER || dataType > Variant::TIMESPAN ) {
				// force default, set aligned
				this->vType = Variant::LONG;
				this->vHolder.v_long = value;
			 }
			else {  // casting
				this->vType = dataType;
				this->vHolder.v_int64 = (int64)value;
			 }
		 }
		// because often we have date in long-type (epoch)
		inline void set( const unsigned long& value, DATA_TYPE dataType = Variant::INT64 ) {
			free();
			if ( dataType == Variant::LONG ) {
				this->vType = Variant::LONG;
				this->vHolder.v_long = (long)value;
			 }
			else if ( dataType < Variant::INT64 || dataType > Variant::TIMESPAN ) {
				// force default, set aligned
				this->vType = Variant::INT64;
				this->vHolder.v_int64 = (int64)value; // upcasted
			 }
			else {  // casting
				this->vType = dataType;
				this->vHolder.v_int64 = (int64)value; // upcasted
			 }
		 }
		// because also time-types share same memory type:
		inline void set( const int64& value, DATA_TYPE dataType = Variant::INT64 ) {
			free();
			this->vHolder.v_int64 = value;
			if ( dataType < Variant::INT64 || dataType > Variant::TIMESPAN )
				this->vType = Variant::INT64;
			else
				this->vType = dataType;
		 }

		inline void set( const double& value ) {
			free();
			this->vType = Variant::DOUBLE;
			this->vHolder.v_double = value;
		 }
		inline void set( const float& value ) {
			free();
			this->vType = Variant::FLOAT;
			this->vHolder.v_float = value;
		 }

		// create NEW or string assignement-operator
		inline void set( const std::string& value ) {
			if ( this->vType == Variant::STRING && this->vHolder.v_pstr != NULL )
				*this->vHolder.v_pstr = value;
			else {
				free();
				this->vType = Variant::STRING;
				this->vHolder.v_pstr = new std::string(value);
			 }
		 }

		// must be NULL terminated, considered as Variant::STRING!!!!
		inline void set( const char* value ) {
			if ( this->vType == Variant::STRING && this->vHolder.v_pstr != NULL )
				this->vHolder.v_pstr->assign(value);
			else {
				free();
				this->vType = Variant::STRING;
				this->vHolder.v_pstr = new std::string(value);
			 }
		 }

	private:
		 inline void set( const ISerializable* ) {}
	public:

//		inline void set( char* value ) {
			//bug
//			;
//		 }

		Variant::DATA_TYPE fromString( const std::string& data, Variant::DATA_TYPE type_tip = Variant::EMPTY );

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:
		Variant::DATA_TYPE getType() const				{ return this->vType; }

		inline bool isEmpty() const						{ return (this->vType == Variant::EMPTY); }
		inline bool isBoolean() const					{ return (this->vType == Variant::BOOLEAN); }
		inline bool isChar() const						{ return (this->vType == Variant::CHAR); }
		inline bool isInteger() const					{ return (this->vType == Variant::INTEGER); }
		inline bool isLong() const						{ return (this->vType == Variant::LONG); }
		inline bool isInt64() const						{ return (this->vType == Variant::INT64); }
		inline bool isDateTime() const					{ return (this->vType == Variant::DATETIME); }
		inline bool isTimeSpan() const					{ return (this->vType == Variant::TIMESPAN); }
		inline bool isDouble() const					{ return (this->vType == Variant::DOUBLE); }
		inline bool isFloat() const						{ return (this->vType == Variant::FLOAT); }
		inline bool isCharBuffer() const				{ return (this->vType == Variant::CHARS); }

		inline bool isString( bool check_valid = true ) const {
			if (this->vType != Variant::STRING ) return false;
			if ( check_valid && this->vHolder.v_pstr == NULL ) return false;
			return true;
		 }
		// return false only is is a non-empty string
		inline bool isStringEmpty() const {
			if ( !isString(true) ) return true;
			return this->vHolder.v_pstr->empty();
		 }

		inline bool isNumeric() const					{ return (this->vType >= Variant::INTEGER && this->vType <= Variant::FLOAT ); }

		// NOT type-check SAFE 
	inline bool asBoolean() const				{ return this->vHolder.v_bool; }
	inline bool& asBoolean()						{ return this->vHolder.v_bool; }

		inline char asChar() const						{ return this->vHolder.v_char; }
		inline int asInteger() const					{ return this->vHolder.v_long; }
		inline long asLong() const						{ return this->vHolder.v_long; }
		inline int64 asInt64() const					{ return this->vHolder.v_int64; }
		inline int64 asDateTime() const					{ return this->vHolder.v_int64; }
		inline int64 asTimeSpan() const					{ return this->vHolder.v_int64; }
		inline double asDouble() const					{ return this->vHolder.v_double; }
		inline float asFloat() const					{ return this->vHolder.v_float; }

		// should be NULL terminated
		inline char* asCharPtr() const					{ return this->vHolder.v_pchar; }

		inline const std::string* asStringPtr() const	{ return this->vHolder.v_pstr; }
		// not safe, should previously check calling isString(true)
		inline const std::string& asString() const		{ return *this->vHolder.v_pstr; }
		inline std::string& asString()					{ return *this->vHolder.v_pstr; }

		// SAFE + CONVERSION (when required)
		bool toBoolean( bool default_value ) const;
		int toInteger( int default_value ) const;
		long toLong( long default_value ) const;
		int64 toInt64( int64 default_value ) const;
		double toDouble( double default_value ) const;
		float toFloat( float default_value ) const;
		// toString() follows ISerializable interface !!

		inline const char* getTypeAsString( bool short_name = false ) const {
			return getTypeAsString(this->vType, short_name);
		 }

		static Variant::DATA_TYPE getTypeFromString( const char* type, short short_name = 0 );

		static const char* getTypeAsString( Variant::DATA_TYPE type, bool short_name = false );

//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:
		template <typename T>
		inline Variant& operator=( const T& m ) {
			if (this == &m) return *this; // same object?
		
			this->set(m);
			return *this;
		 }

		Variant& operator=( const wosh::Variant& m );
		inline Variant& operator=( const bool& m )				{ set(m); return *this; }
		inline Variant& operator=( const char& m )				{ set(m); return *this; }
		inline Variant& operator=( const int& m )				{ set(m); return *this; }
		inline Variant& operator=( const long& m )				{ set(m); return *this; }
		inline Variant& operator=( const int64& m )				{ set(m); return *this; }
		inline Variant& operator=( const double& m )			{ set(m); return *this; }
		inline Variant& operator=( const float& m )				{ set(m); return *this; }
		inline Variant& operator=( const std::string& m )		{ set(m); return *this; }


		const Variant operator+( const Variant& other ) const;

		bool operator==( const Variant& other ) const;
		inline bool operator==( const bool& other ) const			{ if (!isBoolean()) return false; return (asBoolean()==other); }
		inline bool operator==( const char& other ) const			{ if (!isChar()) return false; return (asChar()==other); }
		inline bool operator==( const int& other ) const			{ if (!isInteger()) return false; return (asInteger()==other); }
		inline bool operator==( const long& other ) const			{ if (!isLong()) return false; return (asLong()==other); }
		inline bool operator==( const int64& other ) const			{ if (!isInt64()) return false; return (asInt64()==other); }
		inline bool operator==( const double& other ) const			{ if (!isDouble()) return false; return (asDouble()==other); }
		inline bool operator==( const float& other ) const			{ if (!isFloat()) return false; return (asFloat()==other); }
		inline bool operator==( const std::string& other ) const {
			if (!isString()) return false;
			if ( this->vHolder.v_pstr == NULL ) return false;
			return ( *this->vHolder.v_pstr == other );
		 }
		bool operator==( const char* other ) const; // considered as string

		inline bool operator!=( const Variant& other ) const		{ return !(*this == other);  }
		inline bool operator!=( const bool& other ) const			{ return !(*this == other);  }
		inline bool operator!=( const char& other ) const			{ return !(*this == other);  }
		inline bool operator!=( const int& other ) const			{ return !(*this == other);  }
		inline bool operator!=( const long& other ) const			{ return !(*this == other);  }
		inline bool operator!=( const int64& other ) const			{ return !(*this == other);  }
		inline bool operator!=( const double& other ) const			{ return !(*this == other);  }
		inline bool operator!=( const float& other ) const			{ return !(*this == other);  }
		inline bool operator!=( const std::string& other ) const	{ return !(*this == other);  }
		inline bool operator!=( const char* other ) const			{ return !(*this == other);  }


		bool operator>( const Variant& other ) const;
		bool operator<( const Variant& other ) const;

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const;
		virtual WRESULT store( IEncoderWriter* serializer ) const;
		virtual WRESULT retrieve( IEncoderReader* deserializer );
//@}

	protected:
		DATA_TYPE vType;
		tVariantUnion vHolder;

	public:
		static const Variant EmptyVariant;

}; // class def

}; // namespace wosh

#endif //__WOSH_Core_Variant_H__
