/** @file    WoshHost.cpp
 *  @author  Alessandro Polo
 *  @version $Id: WoshHost.cpp 2171 2010-03-19 00:08:59Z alex $
 *  @brief
 * File containing methods for the wosh::WoshHost class.
 * The header for this class can be found in WoshHost.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include <core/WoshHost.h>

 #include <core/Table.h>
 #include <core/Property.h>
 #include <core/Utilities.h>


using namespace std;
namespace wosh {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

WoshHost::WoshHost()
	: wosh::Object(),
	hopsCount(-1),
	kernel_state(0),
	discoveryProtocolVersion(0),
	notifyFrequency(0),
	lastSeen(0),
	lastSeenRemote(0)
 { }

WoshHost::WoshHost( const std::string& WoshKernelName )
	: wosh::Object(NULL, WoshKernelName),
	hopsCount(-1),
	kernel_state(0),
	discoveryProtocolVersion(0),
	notifyFrequency(0),
	lastSeen(0),
	lastSeenRemote(0)
 { }

WoshHost::WoshHost( const wosh::WoshHost& m )
	: wosh::Object(m),
	discoverSource(m.discoverSource),
	discoverBindAddress(m.discoverBindAddress),
	hopsCount(m.hopsCount),
	kernel_state(m.kernel_state),
	discoveryProtocolVersion(m.discoveryProtocolVersion),
	notifyFrequency(m.notifyFrequency),
	lastSeen(m.lastSeen),
	lastSeenRemote(m.lastSeenRemote),
	protocols(m.protocols),
	connections(m.connections),
	neighbours(m.neighbours)
 { }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// SETTERS

void WoshHost::clear()
 {
	this->discoverSource = "";
	this->discoverBindAddress = "";

	this->discoveryProtocolVersion = 0;

	this->hopsCount = -1;

	this->kernel_state = 0;

	this->notifyFrequency = 0;
	this->lastSeen = 0;
	this->lastSeenRemote = 0;

	this->protocols.clear();
	this->connections.clear();
 }

void WoshHost::setSeen( long time ) {
	if ( time == -1 )
		this->lastSeen = Utilities::std_time();
	else
		this->lastSeen = time;
 }

///////////////////////////////////////////////////////////////////////////////////////////////////// SETTERS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// GETTERS

int WoshHost::isConnected() const {
	int count = 0;
	std::map<std::string,long>::const_iterator it;
	for ( it=this->connections.begin(); it!=this->connections.end(); ++it ) {
		if ( it->second == 0 ) continue;
		count++;
	 }
	return count;
 }

bool WoshHost::isTimedOut( long span_seconds ) const {
	if ( this->notifyFrequency <= 0  )
		return false;
	long expir = this->lastSeen + (long)this->notifyFrequency + span_seconds;
	if ( expir >= Utilities::std_time() )
		return false;
	return true;
 }

///////////////////////////////////////////////////////////////////////////////////////////////////// GETTERS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////// OPERATORS

wosh::WoshHost& WoshHost::operator=( const wosh::WoshHost& m ) {
	if (this == &m) return *this; // same object?
	(Object&)*this = (const Object&)m;

	this->discoverSource = m.discoverSource;
	this->discoverBindAddress = m.discoverBindAddress;

	this->hopsCount = m.hopsCount;

	this->kernel_state = m.kernel_state;

	this->discoveryProtocolVersion = m.discoveryProtocolVersion;

	this->notifyFrequency = m.notifyFrequency;
	this->lastSeen = m.lastSeen;
	this->lastSeenRemote = m.lastSeenRemote;

	this->connections = m.connections;
	this->protocols = m.protocols;
	this->neighbours = m.neighbours;

	return *this;
 }

bool WoshHost::operator==( const wosh::WoshHost &other ) const
 {
	if ( getName() != other.getName() )
		return false;

	if ( this->discoverSource != other.discoverSource )
		return false;
	if ( this->discoverBindAddress != other.discoverBindAddress )
		return false;

	if ( this->kernel_state != other.kernel_state )
		return false;

	return true;
 }

/////////////////////////////////////////////////////////////////////////////////////////////////// OPERATORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////// PROTOCOLS & CONNECTIONS

void WoshHost::setProtocolSupported( const std::string& value )
 {
	std::vector<std::string>::const_iterator itp;
	for ( itp=this->protocols.begin(); itp!=this->protocols.end(); ++itp ) {
		if ( *itp == value )
			return;
	 }
	this->protocols.push_back(value);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

std::vector<std::string> WoshHost::getNeighboursName() const
 {
	std::vector<std::string> nnames;
	std::map<std::string, long>::const_iterator itp;
	for ( itp=this->neighbours.begin(); itp!=this->neighbours.end(); ++itp ) {
		nnames.push_back(itp->first);
	 }
	return nnames;
 }

///////////////////////////////////////////////////////////////////////////////////// PROTOCOLS & CONNECTIONS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// HELPERS

wosh::Property* WoshHost::getProperty( const std::string& key ) const
 {
	Variant data;
	if ( key == _KEY_name )									data.set( this->getName() );
	else if ( key == _KEY_type )							data.set( this->getClassName() );
	else if ( key == _WoshHost_KEY_Seen )					{ data.set( this->lastSeen, Variant::DATETIME); }
	else if ( key == _WoshHost_KEY_SeenRemote )				{ data.set( this->lastSeenRemote, Variant::DATETIME); }

	else if ( key == _WoshHost_KEY_DiscoverSource )			data.set( this->discoverSource );
	else if ( key == _WoshHost_KEY_DiscoveryBindAddress )	data.set( this->discoverBindAddress );
	else if ( key == _WoshHost_KEY_DiscoveryProtocolVersion ) data.set( (int)this->discoveryProtocolVersion );
	else if ( key == _WoshHost_KEY_NotifyFrequency )		data.set( (int)this->notifyFrequency );

	else if ( key == _WoshHost_KEY_KernelState )			data.set( (int)this->kernel_state );

	else if ( key == _WoshHost_KEY_HopsCount )				data.set( this->hopsCount );

	else if ( key == _WoshHost_KEY_ProtocolCount )			data.set( (int)this->protocols.size() );
	else if ( key == _WoshHost_KEY_ConnectionCount )		data.set( this->isConnected() );

	else if ( key == _WoshHost_KEY_Protocols ) {
		std::string result = "";
		std::map<std::string,long>::const_iterator it;
		for ( it=this->connections.begin(); it!=this->connections.end(); ) {
			result += it->first;
			++it;
			if ( it!=this->connections.end() )
				result += "\n";
		 }
		data.set( result );
	 }
	else if ( key == _WoshHost_KEY_Connections ) {
		std::string result = "";
		std::map<std::string,long>::const_iterator it;
		for ( it=this->connections.begin(); it!=this->connections.end(); ) {
			if ( it->second == 0 ) { // not connected 
				++it;
				continue;
			 }
			result += Utilities::format("%s{%ld};", it->first.c_str(), it->second);
			++it;
			if ( it!=this->connections.end() )
				result += "\n";
		 }
		data.set( result );
	 }
	else if ( key == _WoshHost_KEY_Neighbours ) {
		std::string result = "";
		std::map<std::string,long>::const_iterator it;
		for ( it=this->neighbours.begin(); it!=this->neighbours.end(); ) {
			if ( it->second == 0 ) { // not connected 
				++it;
				continue;
			 }
			result += Utilities::format("%s{%d};", it->first.c_str(), it->second);
			++it;
			if ( it!=this->neighbours.end() )
				result += "\n";
		 }
		data.set( result );
	 }

	else
		return NULL;
	return new Property(key, data, this->getPermission() );
 }

wosh::Table* WoshHost::getProperties() const
 {
	Table* table = new Table( 15, 2, true );
	table->setTableName("Properties");
	table->setHeaderHorizontal("Name", 0);
	table->setHeaderHorizontal("Value", 1);
	unsigned int iRow = 0;
	table->set<Variant>( _KEY_name, iRow, 0);
	table->set<Variant>( this->getName(), iRow, 1);
	++iRow;
	table->set<Variant>( _KEY_type, iRow, 0);
	table->set<Variant>( this->getClassName(), iRow, 1);
	++iRow;
	table->set<Variant>( _WoshHost_KEY_Seen, iRow, 0);
	table->set( new Variant(this->lastSeen, Variant::DATETIME), iRow, 1);
	++iRow;
	table->set<Variant>( _WoshHost_KEY_SeenRemote, iRow, 0);
	table->set( new Variant(this->lastSeenRemote, Variant::DATETIME), iRow, 1);
	++iRow;
	table->set<Variant>( _WoshHost_KEY_NotifyFrequency, iRow, 0);
	table->set<Variant>( (int)this->notifyFrequency, iRow, 1);
	++iRow;
	table->set<Variant>( _WoshHost_KEY_DiscoverSource, iRow, 0);
	table->set<Variant>( this->discoverSource, iRow, 1);
	++iRow;
	table->set<Variant>( _WoshHost_KEY_DiscoveryBindAddress, iRow, 0);
	table->set<Variant>( this->discoverBindAddress, iRow, 1);
	++iRow;
	table->set<Variant>( _WoshHost_KEY_DiscoveryProtocolVersion, iRow, 0);
	table->set( new Variant((int)this->discoveryProtocolVersion), iRow, 1);
	++iRow;
	table->set<Variant>( _WoshHost_KEY_KernelState, iRow, 0);
	table->set( new Variant((int)this->kernel_state), iRow, 1);
	++iRow;
	table->set<Variant>(_WoshHost_KEY_ProtocolCount, iRow, 0);
	table->set( new Variant((int)this->protocols.size()), iRow, 1);
	++iRow;
	table->set<Variant>( _WoshHost_KEY_ConnectionCount, iRow, 0);
	table->set( new Variant((int)this->connections.size()), iRow, 1);

	++iRow;
	table->set<Variant>( _WoshHost_KEY_Protocols, iRow, 0);
	Variant* data_proto = new Variant();
	std::string result_p = "";
	std::vector<std::string>::const_iterator it_p;
	for ( it_p=this->protocols.begin(); it_p!=this->protocols.end(); it_p++) {
		result_p += *it_p + ";";
	 }
	data_proto->set( result_p );
	table->set( data_proto, iRow, 1);

	++iRow;
	table->set<Variant>( _WoshHost_KEY_Connections, iRow, 0);
	Variant* data_conn = new Variant();
	std::string result_c = "";
	std::map<std::string,long>::const_iterator it_c;
	for ( it_c=this->connections.begin(); it_c!=this->connections.end(); it_c++) {
		result_c += Utilities::format("%s{%d};", it_c->first.c_str(), it_c->second);
	 }
	data_conn->set( result_c );
	table->set( data_conn, iRow, 1);

	++iRow;
	table->set<Variant>( _WoshHost_KEY_Neighbours, iRow, 0);
	Variant* data_neigh = new Variant();
	std::string result_n = "";
	std::map<std::string,long>::const_iterator it_n;
	for ( it_n=this->neighbours.begin(); it_n!=this->neighbours.end(); it_n++) {
		result_n += Utilities::format("%s{%d};", it_n->first.c_str(), it_n->second);
	 }
	data_neigh->set( result_n );
	table->set( data_neigh, iRow, 1);

	return table;
 }

///////////////////////////////////////////////////////////////////////////////////////////////////// HELPERS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

std::string WoshHost::toString() const
 {
	return "";
 }

WRESULT WoshHost::store( IEncoderWriter* serializer ) const {
	if ( serializer == NULL ) return WRET_ERR_PARAM;
	WRESULT ret = WRET_OK;

	ret += Object::store(serializer);

	ret += serializer->push( this->discoverSource, "discoverSource", SERIALIZE_FIELD );
	ret += serializer->push( this->discoverBindAddress, "discoverBindAddress", SERIALIZE_FIELD );
	ret += serializer->push( this->hopsCount, "hopsCount", SERIALIZE_FIELD );
	ret += serializer->push( this->discoveryProtocolVersion, "discoveryProtocolVersion", SERIALIZE_FIELD );
	ret += serializer->push( this->notifyFrequency, "notifyFrequency", SERIALIZE_FIELD );
	ret += serializer->push( this->lastSeen, "lastSeen", SERIALIZE_FIELD );
	ret += serializer->push( this->lastSeenRemote, "lastSeenRemote", SERIALIZE_FIELD );

//	ret += serializer->push( this->protocols, "lastSeenRemote" );

//	std::vector<std::string> protocols;
//	std::map<std::string, long> connections;
//	std::map<std::string, long> neighbours;

	return ret;
 }
WRESULT WoshHost::retrieve( IEncoderReader* deserializer ) {
	if ( deserializer == NULL ) return WRET_ERR_PARAM;
	WRESULT ret = WRET_OK;

	ret += Object::retrieve(deserializer);

	ret += deserializer->pop( this->discoverSource, "discoverSource", SERIALIZE_FIELD );
	ret += deserializer->pop( this->discoverBindAddress, "discoverBindAddress", SERIALIZE_FIELD );
	ret += deserializer->pop( this->hopsCount, "hopsCount", SERIALIZE_FIELD );
	ret += deserializer->pop( this->discoveryProtocolVersion, "discoveryProtocolVersion", SERIALIZE_FIELD );
	ret += deserializer->pop( this->notifyFrequency, "notifyFrequency", SERIALIZE_FIELD );
//	ret += deserializer->pop( this->lastSeen, "lastSeen", SERIALIZE_FIELD );
//	ret += deserializer->pop( this->lastSeenRemote, "lastSeenRemote", SERIALIZE_FIELD );

	return ret;
 }



//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////// WoshHost_node SOURCE
/*
WoshHost_node* WoshHost_node::addChild( const std::string& name )
 {
	std::vector<WoshHost_node*>::iterator it;
	for(it=this->children.begin();it!=this->children.end(); ++it) {
		if ( *it == NULL ) continue;
		if ( (*it)->name != name ) continue;
		(*it)->timestamp = Utilities::std_time();
		(*it)->hops = this->hops + 1;
		return *it;
	 }
	WoshHost_node* child = new WoshHost_node();
	child->name = name;
	child->setParent(this, false);
	child->timestamp = Utilities::std_time();
	child->hops = this->hops + 1;
	this->children.push_back(child);
	return child;
 }

WoshHost_node* WoshHost_node::getChild( int index )
 {
	if ( index < 0 ) return NULL;
	int i = 0;
	std::vector<WoshHost_node*>::iterator it;
	for(it=this->children.begin();it!=this->children.end(); ++it, i++) {
		if ( i != index ) continue;
		return *it;
	 }
	return NULL;
 }

int WoshHost_node::getChildIndex( WoshHost_node* item )
 {
	int i = 0;
	std::vector<WoshHost_node*>::iterator it;
	for(it=this->children.begin();it!=this->children.end(); ++it, i++) {
		if ( *it != item ) continue;
		return i;
	 }
	return -1;
 }

std::vector<WoshHost_node*> WoshHost_node::getChildRecursive()
 {
	std::vector<WoshHost_node*> allChild;
	std::vector<WoshHost_node*>::iterator it;
	for(it=this->children.begin();it!=this->children.end(); ++it) {
		if ( *it == NULL ) continue;
		allChild.push_back(*it);
		std::vector<WoshHost_node*> childChildren = (*it)->getChildRecursive();
		if ( childChildren.empty() ) continue;
		allChild.insert(it, childChildren.begin(), childChildren.end() );
	 }
	return allChild;
 }
*/
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

}; // namespace wosh
