/** @file    WoshKernel.cpp
 *  @author  Alessandro Polo
 *  @version $Id: WoshKernel.cpp 2216 2010-03-26 17:10:19Z alex $
 *  @brief
 * File containing methods for the wosh::WoshKernel class.
 * The header for this class can be found in WoshKernel.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <core/WoshKernel.h>

 // modules
 #include <core/ObjectAllocator.h>
 #include <core/ObjectFactory.h>
 #include <core/BundleManager.h>
 #include <core/ThreadManager.h>
 #include <core/UserManager.h>
 #include <core/BusManager.h>
 #include <core/DeviceManager.h>
 #include <core/SecurityManager.h>
 #include <core/DataBaseManager.h>
 #include <core/FileSystem.h>
 #include <core/NetworkManager.h>
 #include <core/SystemMonitor.h>
 #include <core/SystemLogger.h>

 #include <core/MethodsDef.h>
 #include <core/MessageProcessingCommon.h>
 #include <core/MessageFilter.h>
 #include <core/Notification.h>
 #include <core/Table.h>
 #include <core/Fact.h>
 #include <core/NEW_List.h>
 #include <stdio.h>


using namespace std;
namespace wosh {

// Gamma singleton!
extern "C" {
	WoshKernel* ms_WoshKernel_instance = NULL; ///< private reference of static instance
	WoshKernel* WoshKernel::getInstance() { return ms_WoshKernel_instance; }
}

WRESULT WoshKernel::importInstance( WoshKernel* global_instance ) {
	if ( global_instance == NULL ) return WRET_ERR_PARAM;
	if ( ms_WoshKernel_instance != NULL ) return WRET_ERR_ILLEGAL_USE;
	ms_WoshKernel_instance = global_instance;
	return WRET_OK;
 }

 // register as abstract because it is a singleton:
 WOSH_REGISTER_INTERFACE(wosh::WoshKernel, "wosh::Object", 0.01, _static_WoshKernel )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

WoshKernel::WoshKernel( const string& WoshKernel_name, Settings* settings_default ) : Object(), IBusListener(), MethodRunner()
 {
	this->state = WoshKernel::STATE_UNKNOWN;
	if ( settings_default != NULL )		this->SettingsDefault = settings_default;
	else								this->SettingsDefault = new Settings();

	// configure object's info
	if ( WoshKernel_name.size() == 0 )	Object::setName( Utilities::format("wosh%ld", Utilities::randomLong()) );
	else								Object::setName( WoshKernel_name );
	Object::getURI().setKernelName(WoshKernel_name);
	Object::getURI().clearPathName();
	Object::getURI().registerGlobal();
	Object::setPermission( Permission( "root", "root", Permission::RWX, Permission::RX, Permission::Read) );

	// configure my own logger
	Log.setIndent( 1 );
	Log.setContext( "WoshKernel" );
	Log.setLevel( LOG_INFO );
Log.setLevel( LOG_VERBOSE );

	Log(LOG_VERBOSE, " Initializing local vars.." );
	this->state = WoshKernel::STATE_INITIALIZING;
	this->loopAcquired = false;
	this->stopRequest = false;
	this->listener = NULL;
	ms_WoshKernel_instance = this;

	this->modules = new WoshModule*[_WoshKernel_Modules_SIZE];
	for(int i=0; i<_WoshKernel_Modules_SIZE; i++) {
		modules[i] = NULL;
	 }
	this->Factory = NULL;
	this->Busses = NULL;
	this->Threads = NULL;
	this->Security = NULL;
	this->Bundles = NULL;
	this->Devices = NULL;
	this->Users = NULL;
	this->Db = NULL;
	this->Network = NULL;
	this->FS = NULL;
	this->Monitor = NULL;
	this->SysLog = NULL;


	Log(LOG_VERBOSE, " Setting default properties and permissions.." );
	Properties.set( _KEY_Version,	_WoshKernel_VERSION, Permission(Permission::Read) );
	Properties.set( _WoshModule_KEY_State,	0, Permission( Permission::Read ) );
	Properties.set( _KEY_DebugLevel,	string(LogContext::getLogLevelAsString( Log.getLevel() )), Permission(Permission::RW,Permission::RW,Permission::Read) );
	Properties.set( _KEY_interfaces,	_WoshKernel_TYPE,					Permission(Permission::Read) );
	Properties.set( _WoshKernel_KEY_BuiltRev,		"$Rev $",				Permission(Permission::Read) );
	Properties.set( _WoshKernel_KEY_BuiltDate,		__DATE__" : "__TIME__,	Permission(Permission::Read) );
	Properties.set( _WoshKernel_KEY_BuiltPlatform,	_WOSH_BUILT_PLATFORM,	Permission(Permission::Read) );
	Properties.set( _WoshKernel_KEY_WoshVersionMayor,	_WOSH_VER_MAJOR,	Permission(Permission::Read) );
	Properties.set( _WoshKernel_KEY_WoshVersionMinor,	_WOSH_VER_MINOR,	Permission(Permission::Read) );
	Properties.set( _WoshKernel_KEY_WoshCodeName,	_WOSH_VER_CODENAME,	Permission(Permission::Read) );
 	Properties.monitor( _KEY_DebugLevel );
	Properties.updateObjectKeys( this );
	Properties.setPermission( Object::getPermission(), Permission::Overwrite_All, true, true );
	Properties.setPropertiesProviderListener(this);

	Log(LOG_VERBOSE, " Allocating ObjectFactory.." );
	// can't access to almost anything!
	this->SysLog = &SystemLogger::getInstance();

	Log(LOG_VERBOSE, " Allocating ObjectFactory.." );
	// can't access to almost anything!
	this->Factory = &ObjectFactory::getInstance();
	this->Factory->setParentObject(this);

	Log(LOG_VERBOSE, " Allocating ThreadManager.." );
	// can't access to any wosh module!
	this->Threads = &ThreadManager::getInstance();
	this->Threads->setParentObject(this);

	Log(LOG_VERBOSE, " Allocating SystemMonitor.." );
	// can't access to the bus!
	this->Monitor = new SystemMonitor(this);

	// since even core-services may access BUS, initialization must be ordered
	Log(LOG_VERBOSE, " Allocating BusManager.." );
	this->Busses = new BusManager(this);

	Log(LOG_VERBOSE, " Allocating FileSystem.." );
	this->FS = new FileSystem(this);

	Log(LOG_VERBOSE, " Allocating SecurityManager.." );
	this->Security = new SecurityManager(this);

	Log(LOG_VERBOSE, " Allocating BundleManager.." );
	this->Bundles = new BundleManager(this);

	Log(LOG_VERBOSE, " Allocating DeviceManager.." );
	this->Devices = new DeviceManager(this);

	Log(LOG_VERBOSE, " Allocating UserManager.." );
	this->Users = new UserManager(this);

	Log(LOG_VERBOSE, " Allocating DataBaseManager.." );
	this->Db = new DataBaseManager(this);

	Log(LOG_VERBOSE, " Allocating NetworkManager.." );
	this->Network = new NetworkManager(this);

	int i=0;
	modules[i++] = this->SysLog;
	modules[i++] = this->Threads;
	modules[i++] = this->Factory;
	modules[i++] = this->FS;
	modules[i++] = this->Security;
	modules[i++] = this->Busses;
	modules[i++] = this->Monitor;
	modules[i++] = this->Network;
	modules[i++] = this->Db;
	modules[i++] = this->Users;
	modules[i++] = this->Devices;
	modules[i++] = this->Bundles;

	Log(LOG_VERBOSE, " Registering Kernel's Methods.." );
	Methods.setPermission( Object::getPermission(), Permission::Overwrite_All, true, true );

	MethodMessageResponse* mmPropertyGet = Methods.create<MethodMessageResponse>( _METHOD_GetProperty, "Get Property" );
	mmPropertyGet->setMethod( this, (MethodMessageResponsePtr)&WoshKernel::mmDoPropertyGet );
	mmPropertyGet->getPermission().set( Object::getPermission(), Permission::RX );

	MethodMessageResponse* mmPropertySet = Methods.create<MethodMessageResponse>( _METHOD_SetProperty, "Set Property" );
	mmPropertySet->setMethod( this, (MethodMessageResponsePtr)&WoshKernel::mmDoPropertySet );
	mmPropertySet->getPermission().set( Object::getPermission(), Permission::RX, Permission::RX, Permission::Read );

	MethodMessageResponse* mmPropertyList = Methods.create<MethodMessageResponse>( _METHOD_ListProperties, "List Properties" );
	mmPropertyList->setMethod( this, (MethodMessageResponsePtr)&WoshKernel::mmDoPropertyList );
	mmPropertyList->getPermission().set( Object::getPermission(), Permission::RX );

	MethodMessageResponse* mmMethodList = Methods.create<MethodMessageResponse>( _METHOD_ListMethods, "List Methods" );
	mmMethodList->setMethod( this, (MethodMessageResponsePtr)&WoshKernel::mmDoMethodList );
	mmMethodList->getPermission().set( Object::getPermission(), Permission::RX );

	MethodWRESULT* mmShutdown = Methods.create<MethodWRESULT>( _WoshKernel_METHOD_shutdown );
	mmShutdown->setMethod( this, (MethodWRESULTPtr)&WoshKernel::shutdown );
	mmShutdown->getPermission().set( Object::getPermission(), Permission::RX, Permission::RX, Permission::Read );

	MethodRequest* mmListMod = Methods.create<MethodRequest>( _METHOD_List, "List WoshKernel modules" );
	mmListMod->setMethod( this, (MethodRequestPtr)&WoshKernel::mmDoListMod );
	mmListMod->getPermission().set( Object::getPermission(), Permission::RX );

	MethodRequest* mmListURI = Methods.create<MethodRequest>( _WoshKernel_METHOD_lsuri, "List URIs" );
	mmListURI->setMethod( this->Busses, (MethodRequestPtr)&WoshKernel::mmDoListURI );
	mmListURI->getPermission().set( Object::getPermission(), Permission::RX );

	Variant busType;
	if ( this->SettingsDefault->getSettingsOf("WoshKernel") != NULL )
		busType = this->SettingsDefault->getSettingsOf("WoshKernel")->getValueOf("BusType");
	if ( busType.isStringEmpty() )
		busType.set("wosh::BusRing");

	Log(LOG_INFO, " Initializing Bus "_Bus_Core" (%s)..", busType.asString().c_str() );
	WRESULT created_bus = this->Busses->createBus( _Bus_Core, busType.asString() );
	if ( WFAILED(created_bus) )
		Log(LOG_CRITICAL, ": FAILED#%d Registering Bus (wosh.Bus.Core) [%s]..", created_bus, busType.asString().c_str() );
	else {
		BusCore.setMessageHandler(this);
		WRESULT bus_b_connected = BusCore.connect( _Bus_Core );
		if ( WFAILED(bus_b_connected) )
			Log(LOG_CRITICAL, ": FAILED#%d Connecting Bus (wosh.Bus.Core)..", bus_b_connected );
	 }

	Log(LOG_INFO, "  WOSH %g.%d [%s] - WoshKernel v.%g $Rev $ " __DATE__ " : " __TIME__"\n",
				_WOSH_VER_MAJOR, _WOSH_VER_MINOR, _WOSH_VER_CODENAME, _WoshKernel_VERSION );
 }

WoshKernel::~WoshKernel()
 {
	Log(LOG_INFO, " Destroying WoshKernel.." );

	WRESULT destroying = setState(WoshKernel::STATE_DESTROYING, true);
	if ( WFAILED(destroying) )
		Log(LOG_CRITICAL, ":~WoshKernel() : FAILED#%d (pre)Destroyig WOSH, see previous logs. But Resuming!..", destroying );
	else
		Log(LOG_INFO, " WOSH (pre)Initializing completed. [INITIALIZING]" );

	Log(LOG_VERBOSE, ":~WoshKernel() : Freeing BundleManager.." );
	if ( this->Bundles != NULL )
		delete this->Bundles; this->Bundles = NULL;

	Log(LOG_VERBOSE, ":~WoshKernel() : Freeing DeviceManager.." );
	if ( this->Devices != NULL )
		delete this->Devices; this->Devices = NULL;

	Log(LOG_VERBOSE, ":~WoshKernel() : Freeing UserManager.." );
	if ( this->Users != NULL )
		delete this->Users; this->Users = NULL;

	Log(LOG_VERBOSE, ":~WoshKernel() : Freeing SecurityManager.." );
	if ( this->Security != NULL )
		delete this->Security; this->Security = NULL;

	Log(LOG_VERBOSE, ":~WoshKernel() : Freeing DataBase.." );
	if ( this->Db != NULL )
		delete this->Db; this->Db = NULL;

	Log(LOG_VERBOSE, ":~WoshKernel() : Freeing NetworkManager.." );
	if ( this->Network != NULL )
		delete this->Network; this->Network = NULL;

	Log(LOG_VERBOSE, ":~WoshKernel() : Freeing FileSytem.." );
	if ( this->FS != NULL )
		delete this->FS; this->FS = NULL;

	Log(LOG_VERBOSE, ":~WoshKernel() : Freeing BusManager.." );
	if ( this->Busses != NULL )
		delete this->Busses; this->Busses = NULL;

	Log(LOG_VERBOSE, ":~WoshKernel() : Freeing SystemMonitor.." );
	if ( this->Monitor != NULL )
		delete this->Monitor; this->Monitor = NULL;

	Log(LOG_VERBOSE, ":~WoshKernel() : Freeing ThreadManager.." );
	if ( this->Threads != NULL )
		delete this->Threads; this->Threads = NULL;

	Log(LOG_VERBOSE, ":~WoshKernel() : Freeing ObjectFactory.." );
	if ( this->Factory != NULL )
		delete this->Factory; this->Factory = NULL;

	Log(LOG_VERBOSE, ":~WoshKernel() : Freeing SystemLogger.." );
	if ( this->SysLog != NULL )
		delete this->SysLog; this->SysLog = NULL;

	delete [] this->modules;

	if ( this->SettingsDefault != NULL ) {
		Log(LOG_VERBOSE, ":~WoshKernel() : Freeing Settings.." );
		delete this->SettingsDefault; this->SettingsDefault = NULL;
	 }

	Log(LOG_VERBOSE, ":~WoshKernel() : DESTROYED." );
	ms_WoshKernel_instance = NULL;
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////// STATE CONTROL

WRESULT WoshKernel::init()
 {
	// force the current state again, execute
	// SYNCH call to WoshModule::onKernelInitializing();
	// also broadcast kernel-state event
	WRESULT initializing = setState(WoshKernel::STATE_INITIALIZING, true);
	if ( WFAILED(initializing) )
		Log(LOG_CRITICAL, ":init() : FAILED#%d (pre)Initializing WOSH, see previous logs. But Resuming!..", initializing );
	else
		Log(LOG_INFO, " WOSH (pre)Initializing completed. [INITIALIZING]" );

	// notify that.. all modules (hopefully) are initialized.
	// it's a sort of post initialization to eventually resolve
	// init conflicts between modules
	WRESULT initialized = setState(WoshKernel::STATE_INITIALIZED, true);
	if ( WFAILED(initialized) )
		Log(LOG_CRITICAL, ":init() : FAILED#%d (post)Initializing WOSH, see previous logs.", initialized );
	else
		Log(LOG_INFO, " WOSH (post)Initializing completed. [INITIALIZED]" );

	return initializing+initialized;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT WoshKernel::start() {

	WRESULT started = setState(WoshKernel::STATE_RUNNING, true);

	if ( WFAILED(started) )
		Log(LOG_CRITICAL, ":start() : FAILED#%d (pre)Initializing WOSH, see previous logs.", started );
	else
		Log(LOG_INFO, " WOSH is now running." );

	return started;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT WoshKernel::shutdown() {
	if ( this->state == WoshKernel::STATE_STOPPED ) {
		Log(LOG_CRITICAL, ":shutdown() : Already in STOPPED state!" );
		return WRET_ERR_ILLEGAL_USE;
	 }
	if ( this->state == WoshKernel::STATE_STOPPING ) {
		// then it's another thread asking to stop..
		Log(LOG_WARNING, ":shutdown() : Already stopping.." );
		this->stopRequest = true;
		return WRET_ERR_ILLEGAL_USE;
	 }
	if ( this->loopAcquired ) {
		// then it's another thread asking to stop.. then break the loop
		this->stopRequest = true;
		// so stopping is asych
		return WRET_OK;
	 }

	// synch stop
	// SYNCH call to WoshModule::onKernelStopping();
	// also broadcast kernel-state event
	WRESULT stopping = setState(WoshKernel::STATE_STOPPING, true);
	if ( WFAILED(stopping) )
		Log(LOG_CRITICAL, ":init() : FAILED#%d (pre)DeInitializing WOSH, see previous logs. But Resuming!..", stopping );
	else
		Log(LOG_INFO, " WOSH (pre)DeInitializing completed. [STOPPING]" );

	// notify that.. all modules (hopefully) are deinitialized.
	// it's a sort of post deinitialization to eventually resolve
	// init conflicts between modules
	WRESULT stopped = setState(WoshKernel::STATE_STOPPED, true);
	if ( WFAILED(stopped) )
		Log(LOG_CRITICAL, ":init() : FAILED#%d (post)DeInitializing WOSH, see previous logs.", stopped );
	else
		Log(LOG_INFO, " WOSH (post)DeInitializing completed. [STOPPED]" );

	return stopping+stopped;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT WoshKernel::enterLoop() {
	if ( this->state != WoshKernel::STATE_RUNNING ) {
		Log(LOG_CRITICAL, ":enterLoop() : FAILED: Not in RUNNING state!" );
		return WRET_ERR_ILLEGAL_USE;
	 }
	Log(LOG_INFO, ":enterLoop() : Aquired Thread!" );
	this->loopAcquired = true;
	while( !this->stopRequest ) {
		Thread::sleepForMSec(500);
	}
	this->loopAcquired = false;
	Log(LOG_INFO, ":enterLoop() : Stopping and Leaving Thread!" );
	return shutdown();

 }

/////////////////////////////////////////////////////////////////////////////////////////////// STATE CONTROL
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////// PROCESS BUS MESSAGES

void WoshKernel::busMessage( const Message& message, const Bus* ) {
	if ( message.isEmpty() || !message.getContent()->isRequest() ) return;
	if ( !MessageFilter::isMessageTarget(message, this) ) return;
	// similar to BundleGeneric, common implementation in MessageProcessingCommon.cpp
	Message* reply = MPC_busMessageRequest( message, Methods, Log );
	if ( reply == NULL ) return;
	reply->setSource(this);
	BusCore.postMessage(reply);
 }

//////////////////////////////////////////////////////////////////////////////////////// PROCESS BUS MESSAGES
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENTS

bool WoshKernel::updatingProperty( bool& , const Variant& value_proposed, Property& property_current, const PropertiesProvider* )
 {
	if ( property_current.getKey() == _KEY_DebugLevel )
		return MPC_PropertyParse_LogLevel( value_proposed, this->Log );

	// not handled, sound strange but probably its ok since we are the lower (base) implementation
	return true;
 }

///////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* WoshKernel::mmDoListMod( const Request* request ) {
	if ( request == NULL ) return NULL;
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, ":mmDoListMod(%s)", option.c_str() );

	if ( option == "s" || option == "short" ) {
		NEW_List* mods =  new NEW_List(_WoshKernel_Modules_SIZE);
		for(int i=0; i<_WoshKernel_Modules_SIZE; i++) {
			if ( modules[i] == NULL ) continue;
			mods->push_back<Variant>( modules[i]->getName() );
		 }
		return request->replyResponse(WRET_OK, mods);
	 }
	else {
		Table* tbl = new Table( _WoshKernel_Modules_SIZE, 3, true );
		tbl->setTableName("Modules");
		tbl->setHeaderHorizontal("Name", 0);
		tbl->setHeaderHorizontal("Type", 1);
		tbl->setHeaderHorizontal("URI", 2);
		//tbl->set("Permission", 0, 3);
		unsigned int iRow = 0;
		for( int i=0; i<_WoshKernel_Modules_SIZE; i++, ++iRow ) {
			if ( modules[i] == NULL ) continue;
			tbl->set<Variant>( modules[i]->getName(), iRow, 0);
			tbl->set<Variant>( modules[i]->getClassName(), iRow, 1);
			tbl->set( modules[i]->getURI().clone(), iRow, 2);
			//tbl->set( modules[i]->getPermission().clone(), iRow, 3);
		 }
		return request->replyResponse(WRET_OK, tbl);
	 }
 }

Response* WoshKernel::mmDoListURI( const Request* request )
 {
	if ( request == NULL ) return NULL;
	
	NEW_List* obj_list = new NEW_List();
	URI temp("*:**");
	URICollector::getInstance().list( &temp, true, obj_list->getVector() );

	///@todo implement here
	return request->replyResponse(WRET_OK, obj_list);
 }

Response* WoshKernel::mmDoPropertySet( const Message* requestMessage ) {
	return MPC_mmDoPropertySet( requestMessage, this->Properties, this->Log );
 }
Response* WoshKernel::mmDoPropertyGet( const Message* requestMessage ) {
	return MPC_mmDoPropertyGet( requestMessage, this->Properties, this->Log );
 }
Response* WoshKernel::mmDoPropertyList( const Message* requestMessage ) {
	return MPC_mmDoPropertyList( requestMessage, this->Properties, this->Log );
 }
Response* WoshKernel::mmDoMethodList( const Message* requestMessage ) {
	return MPC_mmDoMethodList( requestMessage, this->Methods, this->Log );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT WoshKernel::setState( WoshKernel::KERNEL_STATE new_state, bool raise_event ) {
	WoshKernel::KERNEL_STATE old_state = this->state;
	if ( _GET_Abs(static_cast<int>(new_state)-static_cast<int>(old_state)) > 1 ) {
		Log(LOG_CRITICAL, ":setState(%s) : KERNEL-FAILURE [can't switch from %s]", getStateAsString(new_state), getStateAsString(old_state) );
		return WRET_ERR_ILLEGAL_USE;
	 }
	WRESULT ret = WRET_OK;
	Log(LOG_VERBOSE, ":setState() Switching state %s => %s..", getStateAsString(old_state), getStateAsString(new_state) );
	this->state = new_state;

	for(int i=0; i<_WoshKernel_Modules_SIZE; i++) {
		if ( modules[i] == NULL ) continue;
		ret = modules[i]->kernel_StateChanged( static_cast<int>(new_state), static_cast<int>(old_state) );
	 }

	if ( this->listener != NULL )
		this->listener->kernel_StateChanged( old_state, new_state, this );

	Log(LOG_INFO, " State %s", getStateAsString(new_state) );
	this->state = new_state;
	Properties.update( _WoshKernel_KEY_State, getStateAsString(new_state) );

	if ( !raise_event )
		return ret;

	Fact* fact = new Fact(_WoshKernel_EVENT_StateChanged, new Variant(static_cast<int>(new_state)) );
	Message* message = new Message( fact );
	message->setSource(this);
	BusCore.postMessage(message);
	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////// STATIC

const char* WoshKernel::getKernelName() {
	if ( WoshKernel::getInstance() == NULL )
		return "";
	return WoshKernel::getInstance()->getName().c_str();
 }

WoshKernel::KERNEL_STATE WoshKernel::getKernelState() {
	if ( WoshKernel::getInstance() == NULL )
		return WoshKernel::STATE_FAILURE;
	return WoshKernel::getInstance()->getState();
 }

const char* WoshKernelStateStr[] = {
	"UNKNOWN",		///< (KERNEL_STATE)0
	"INITIALIZING",	///< (KERNEL_STATE)1
	"INITIALIZED",	///< (KERNEL_STATE)2
	"RUNNING",		///< (KERNEL_STATE)3
	"STOPPING",		///< (KERNEL_STATE)4
	"STOPPED",		///< (KERNEL_STATE)5
	"DESTROYING",	///< (KERNEL_STATE)6
	"",				///< (KERNEL_STATE)7
	"FAILURE"		///< (KERNEL_STATE)8
};

Enum2StrFromArray(WoshKernel::getStateAsString, WoshKernel::KERNEL_STATE, STATE_INITIALIZING, STATE_FAILURE, 0, WoshKernelStateStr)

Str2EnumByArray(WoshKernel::getStateFromString, WoshKernel::KERNEL_STATE, 0, STATE_FAILURE, STATE_UNKNOWN, WoshKernelStateStr)

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

}; // namespace wosh
