/**
 * @class   wosh::WoshModule
 * @brief   Interface of a generic CoreModule. Such modules are embedded into
 *          the kernel itself.
 *
 ****************************************************************************
 * \defgroup CoreModules Core-Modules
 *
 * Core modules are low level services embedded into wosh::WoshKernel
 *
 *
 ****************************************************************************
 * @version 0.8.151 $Id: WoshModule.h 2208 2010-03-25 15:45:43Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_WoshModule_H__
 #define __WOSH_Core_WoshModule_H__

 #include <woshDefs.h>
 #include <core/Object.h>
 #include <core/Bus.h>
 #include <core/LogContext.h>
 #include <core/BusConnector.h>
 #include <core/MethodsProvider.h>
 #include <core/PropertiesProvider.h>
 #include <core/InterfacesProvider.h>
 #include <core/Settings.h>


namespace wosh {

 #define _WoshModule_NAME					"wosh.WoshModule"
 #define _WoshModule_TYPE					"wosh::WoshModule"

 #define _WoshModule_KEY_State				"State"

#define WOSH_MODULE(CLASSNAME)					\
		WOSH_CLASSNAME(CLASSNAME)				\
		WOSH_CLASS_ABSTRACT(false)

//		WOSH_COPY_FORBIDDEN(CLASSNAME)


class WoshKernel;

class WoshModule :	public Object,
					public virtual IBusListener,
					protected virtual IPropertiesProviderListener,
					public MethodRunner
 {
	WOSH_CLASSNAME(wosh::WoshModule)
	WOSH_CLASS_ABSTRACT(true)


	protected:
		WoshModule( const WoshKernel* parent = NULL );
		friend class WoshKernel;

	public:
		virtual ~WoshModule();


/** @name Public methods (bundles)
 * @{
 ******************************************************************************/
	public:
		/**
		 * @brief  Initialize the module
		 */
		virtual WRESULT diagnostic();

		virtual bool isInitialized() const			{ return (this->initState == 1); }
		virtual bool isDeinitialized() const		{ return (this->initState == 2); }

//@}
/** @name Kernels Events 
 * @{
 ******************************************************************************/
	public:
		/**
		 * @brief  Event raised synchonously by the Kernel when switching to WoshKernel::STATE_INITIALIZING state
		 * @return WRET_OK on success, custom error code on errors
		 */
		virtual WRESULT onKernelInitializing();
		/**
		 * @brief  Event raised synchonously by the Kernel when switching to WoshKernel::STATE_INITIALIZED state
		 * @return WRET_OK on success, custom error code on errors
		 */
		virtual WRESULT onKernelInitialized();
		/**
		 * @brief  Event raised synchonously by the Kernel when switching to WoshKernel::STATE_RUNNING state
		 * @return WRET_OK on success, custom error code on errors
		 */
		virtual WRESULT onKernelRunning();
		/**
		 * @brief  Event raised synchonously by the Kernel when switching to WoshKernel::STATE_STOPPING state
		 * @return WRET_OK on success, custom error code on errors
		 */
		virtual WRESULT onKernelStopping();
		/**
		 * @brief  Event raised synchonously by the Kernel when switching to WoshKernel::STATE_STOPPED state
		 * @return WRET_OK on success, custom error code on errors
		 */
		virtual WRESULT onKernelStopped();
//@}
/** @name Message Events (incoming message)
 * @{
 ******************************************************************************/
	public:
		virtual void busMessage( const Message& message, const Bus* source );
//@}
/** @name IPropertiesProviderListener interface
 * @{
 ******************************************************************************/
 	protected:
		virtual inline bool readingProperty( Property* , const PropertiesProvider*  ) { return true; }
		virtual bool updatingProperty( bool& do_update, const Variant& value_proposed, Property& property_current, const PropertiesProvider* source );
		friend class wosh::PropertiesProvider;
//@}
/** @name Internal Mehods
 * @{
 ******************************************************************************/
	protected:
		Response* mmDoMethodList( const Message* requestMessage );
		Response* mmDoPropertyGet( const Message* requestMessage );
		Response* mmDoPropertySet( const Message* requestMessage );
		Response* mmDoPropertyList( const Message* requestMessage );
		Response* mmDoPropertiesGet( const Message* requestMessage );
//@}
/** @name Internal vars
 * @{
 ******************************************************************************/
	protected:
		BusConnector BusCore;					///< connector to Core
		mutable LogContext Log;					///< my log
		mutable MethodsProvider Methods;		///< hosted methods
		mutable PropertiesProvider Properties;	///< my properties
		mutable InterfacesProvider Interfaces;	///< my interfaces
		short initState;						///< 0=created; 1=initialized 2=DE-initialized
//@}
	protected:
		virtual WRESULT kernel_StateChanged( int new_state, int old_state );

	private:
		WoshModule(const WoshModule&) : Object() { }	///< private copy constructor (object non-copiable)
		WoshModule& operator=(const WoshModule&) { return *this; }	///< private assigment operator (object non-copiable)


};


};

#endif //__WOSH_Core_WoshModule_H__
