/** @file    Automation.cpp
 *  @author  Alessandro Polo
 *  @version $Id: Automation.cpp 2057 2010-03-08 21:32:15Z alex $
 *  @brief
 * File containing methods for the wosh::automations::Automation class.
 * The header for this class can be found in Automation.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include <framework/automations/Automation.h>

 #include <core/Message.h>
 #include <core/SecurityManager.h>
 #include <core/Utilities.h>
 #include <core/MessageProcessingCommon.h>


using namespace std;
namespace wosh {
 namespace automations {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

Automation::Automation( ) : Object()
 {
	this->uid = Utilities::randomLong();
	Object::setName("Automation", this->uid);
//	this->parent = NULL;

	Log.setIndent( 4 );
	Log.setContext( "Automation" );
	Log.setLevel( LOG_INFO );

	Properties.set( _Automation_KEY_enabled, true, Permission::RW_RW_R__() );
	Properties.monitor(_KEY_name);
	Properties.monitor(_KEY_DebugLevel);
	Properties.monitor(_Automation_KEY_enabled);

	clear();
 }

Automation::Automation( const Automation& m  ) : Object(m)
 {
//	this->parent = NULL;

	clear();
	*this = m;
 }

Automation::~Automation() { }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// SETTERS

void Automation::clear() {
	setEnabled(true);

	setLastExecutedTime(0);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Automation::setName( const std::string& value, long id )
 {
	Object::setName(value, id);
	Log.setContext(Object::getName());
	Properties.update( _KEY_name, value );
 }

void Automation::setEnabled( bool value ) {
	this->enabled = value;
	Properties.update( _Automation_KEY_enabled, value );
 }

void Automation::setLastExecutedTime( int64 time ) {
	if ( time == -1 )
		this->lastExecutedTime = Utilities::std_time();
	else
		this->lastExecutedTime = time;
	Properties.update( _Automation_KEY_LastExecutedTime, Variant(this->lastExecutedTime, Variant::DATETIME) );
 }

void Automation::setPriority( int value ) {
	this->priority = value;
	Properties.update( _Automation_KEY_priority, value );
 }

void Automation::setDescription( const std::string& value ) {
	Properties.update( _Automation_KEY_description, value );
 }

///////////////////////////////////////////////////////////////////////////////////////////////////// SETTERS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////// PROCESS BUS MESSAGES

WRESULT Automation::postMessage( Message * message )
 {
	if ( message == NULL ) return WRET_ERR_PARAM;

//	if ( this->parent == NULL ) {
		delete message;
		return WRET_ERR_INTERNAL;
//	 }
	message->setSource(this);
	message->setSecurityToken(SecurityManager::generateToken(this));

//	bool sent = this->parent->automation_post_message( message, this );
//	if ( !sent )
//		return WRET_ERR_INTERNAL;
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////// PROCESS BUS MESSAGES
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENTS

bool Automation::updatingProperty( bool& , const Variant& value_proposed, Property& property_current, const PropertiesProvider* )
 {
	if ( property_current.getKey() == _KEY_name ) { // if we are here, we allow to edit name
		Log(LOG_VERBOSE, " Changing Bundle Name, Log-Context will change to '%s'", value_proposed.toString().c_str() );
		Object::setName(value_proposed.toString());
		Log.setContext(value_proposed.toString());
		return true;
	 }
	else if ( property_current.getKey() == _KEY_DebugLevel )
		return MPC_PropertyParse_LogLevel( value_proposed, this->Log );

	else if ( property_current.getKey() == _Automation_KEY_enabled ) {
		this->enabled = value_proposed.toBoolean(true);
		return true;
	 }
	// not handled, sound strange but probably its ok since we are the lower (base) implementation
	return true;
 }

///////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////// OPERATORS

Automation& Automation::operator=(const Automation& m)
{
	if (this == &m) return *this; // same object?

	(Object)*this = (const Object)m;

	this->uid = m.uid;
	this->enabled = m.enabled;
	this->lastExecutedTime = m.lastExecutedTime;

	this->Properties = m.Properties;

	return *this;
 }
 
 
bool Automation::operator==(const Automation &other) const
 {
	if ( this->enabled != other.enabled )
		return false;
	if ( this->Properties != other.Properties )
		return false;

	return true;
 }

/////////////////////////////////////////////////////////////////////////////////////////////////// OPERATORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////// HELPERS/CONVERTERS

std::string Automation::toString() const
 {
	return "";
 }

////////////////////////////////////////////////////////////////////////////////////////// HELPERS/CONVERTERS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace automations
}; // namespace wosh
