/**
 * @class   wosh::automations::Automation
 * @brief   Interface for home automations, holds properties and basic methods.
 *          Messages are evaluated by isTrigger() function, when the call
 *          returns true, the manager will run execute().
 *
 * @see     wosh::interfaces::services::TaskManager
 ****************************************************************************
 * @version $Id: Automation.h 2125 2010-03-13 18:58:46Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Automations_Automation_H__
 #define __WOSH_Automations_Automation_H__

 #include <woshDefs.h>
 #include <core/Object.h>
 #include <core/PropertiesProvider.h>
 #include <core/MethodsProvider.h>
 #include <core/LogContext.h>


namespace wosh {
 namespace automations {

 #define _Automation_TYPE					"wosh::automations::Automation"

 #define _Automation_KEY_id					"id"
 #define _Automation_KEY_enabled			"enabled"
 #define _Automation_KEY_description		"description"
 #define _Automation_KEY_LastExecutedTime	"LastExecutedTime"
 
 #define _Automation_KEY_priority			"priority"


class Automation : public Object
 {
	WOSH_SERIALIZABLE_ABSTRACT(wosh::automations::Automation)

	public:
		/**
		 * @brief  Empty constructor.
		 */
		Automation();
		/**
		 * @brief  Copy constructor.
		 */
		Automation( const Automation& m );
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~Automation();


	public:


		virtual WRESULT execute( const Message* message = NULL, int64 time = 0 ) = 0;

		virtual bool isTrigger( const Message* message = NULL, int64 time = 0 ) const = 0;


		virtual void clear();


/** @name Getters
 * @{
 ******************************************************************************/
	public:
		virtual long getID() const							{ return this->uid; }

		bool isEnabled() const								{ return this->enabled; }

		int getPriority() const								{ return this->priority; }

		long getLastExecutedTime() const					{ return this->lastExecutedTime; }

		std::string getDescription() const					{ return this->Properties.getValueOf(_Automation_KEY_description).toString(); }

		/**
		 * @brief  Get the Properties of the object
		 * @return (constant) Interface list reference
		 */
		const PropertiesProvider& getProperties() const			{ return this->Properties; }
		PropertiesProvider& getProperties() 					{ return this->Properties; }

//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:
		void setEnabled( bool value );

		virtual void setName( const std::string& value, long id );

		void setDescription( const std::string& value );

		void setPriority( int priority );

//@}
/** @name IPropertiesProviderListener interface
 * @{
 ******************************************************************************/
 	public:
		virtual bool readingProperty( Property*, const PropertiesProvider* ) { return true; }
		virtual bool updatingProperty( bool& doUpdate, const Variant& value_proposed, Property& property_current, const PropertiesProvider* source );
//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:
		virtual Automation& operator=(const Automation& m);

		virtual bool operator==(const Automation &other) const;

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const;

//@}
	protected:
		virtual WRESULT postMessage( Message * message );

		void setLastExecutedTime( int64 time = -1 );

	protected:
		long uid;					///< unique ID of 
		bool enabled;				///< flag indicating if Automation is enabled

		int priority;

		long lastExecutedTime;		///< timestamp (epoch) of last execution

	//	std::string kernelHost;		///< kernel name where the automation code is
	//	std::string bundleOwner;	///< bundle name, owner of the object

	//	std::string busTrigger;
	//	std::string busEffect;

		wosh::LogContext Log;			///< my log
		PropertiesProvider Properties;	///< hosted properties


}; // class def

 }; // namespace automations
}; // namespace wosh

#endif //__WOSH_Automations_Automation_H__
