/** @file    CronJob.cpp
 *  @author  Alessandro Polo
 *  @version $Id: CronJob.cpp 1980 2010-02-27 17:12:51Z alex $
 *  @brief
 * File containing methods for the wosh::CronJob class.
 * The header for this class can be found in CronJob.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include "CronJob.h"

 #include <core/Message.h>
 #include <core/Utilities.h>
 #include <string.h>

 #include <stdlib.h>
 #include <time.h>


using namespace std;
namespace wosh {
 namespace automations {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

CronJob::CronJob( ) : Automation() {
	clear();
	this->enabled = false;
 }

CronJob::CronJob( const CronJob& m  ) : Automation(m) {
	clear();
	*this = m;
 }

CronJob::~CronJob() {
	clear();
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT CronJob::execute( const Message*, int64 time )
 {
	setLastExecutedTime(time);

	tMessageList::ConstIterator it;
	const tMessageList::ConstIterator it_end = this->effects.end();
	for ( it=this->effects.begin(); it!=it_end; ++it ) {
		if ( *it == NULL ) continue;
		postMessage( (*it)->clone() );
	 }

	// eval next interupt
	updatedTime();
	return WRET_OK;
 }

bool CronJob::isTrigger( const Message*, int64 time ) const {
	if ( this->nextInterrupt == 0 )
		return false;
	if ( time >= this->nextInterrupt )
		return true;
	return false;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CronJob::clear()
 {
	Automation::clear();

	this->year = 0;
	this->month = 0;
	this->day = 0;
	this->hour = 0;
	this->min = 0;
	this->sec = 0;
	updatedTime();

	this->lastExecutedTime = 0;

	this->effects.clear();
 }

void CronJob::setDate( int year_v, int month_v, int day_v ) {
	this->year = year_v;
	this->month = month_v;
	this->day = day_v;
	updatedTime();
 }

void CronJob::setTime( int hour_v, int min_v, int sec_v ) {
	this->hour = hour_v;
	this->min = min_v;
	this->sec = sec_v;
	updatedTime();
 }

WRESULT CronJob::addEffect( Message* effect )
 {
	this->effects.push_back(effect);
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CronJob::updatedTime() {
	this->nextInterrupt = 0;

	time_t rawtime;
	struct tm * timeinfo;

	time ( &rawtime );
	timeinfo = localtime ( &rawtime );

	if ( !isYearly() )
		timeinfo->tm_year = this->year - 1900;
	if ( !isMonthly() )
		timeinfo->tm_mon = this->month - 1;
	if ( !isDaily() )
		timeinfo->tm_mday = this->day;
	if ( !isHourly() )
		timeinfo->tm_hour = this->hour;
	if ( !isMinutely() )
		timeinfo->tm_min = this->min;

	rawtime = mktime( timeinfo );
	this->nextInterrupt = rawtime;

	Properties.update( _CronJob_KEY_NextInterrupt, Variant(this->nextInterrupt, Variant::DATETIME) );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CronJob::updatingProperty( bool& do_update, const Variant& value_proposed, Property& property_current, const PropertiesProvider* source )
 {

//	else // maybe is a (common) Automation property, let base implementation process it
	return Automation::updatingProperty(do_update, value_proposed, property_current, source);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

CronJob& CronJob::operator=(const CronJob& m){
	if (this == &m) return *this; // same object?



	return *this;
 }
 
 
bool CronJob::operator==(const CronJob &other) const
 {
	if ( this->year != other.year )
		return false;


	return true;
 }

//////////////////////////////////////////////////////////////////////////////////////////////// DEBUG & INFO
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////// SERIALIZATION

std::string CronJob::getDate() const
 {
	char buffer [63];
	memset(buffer, '\0', 63);

/// #if defined(_MSC_VER) && !defined(_OS_WINCE)
/// 	sprintf_s(buffer, "%i-%i-%i", this->year, this->month, this->day);
/// #else
	sprintf(buffer, "%i-%i-%i", this->year, this->month, this->day);
/// #endif

	return string(buffer);
 }

std::string CronJob::getTime() const
 {
	char buffer [63];
	memset(buffer, '\0', 63);
/// #if defined(_MSC_VER) && !defined(_OS_WINCE)
/// 	sprintf_s(buffer, "%i:%i", this->hour, this->min);
/// #else
	sprintf(buffer, "%i:%i", this->hour, this->min);
/// #endif

	return string(buffer);
 }

std::string CronJob::toString() const
 {
	return "";
 }

WRESULT CronJob::store( IEncoderWriter* serializer ) const {
	if ( serializer == NULL ) return WRET_ERR_PARAM;
	WRESULT ret = WRET_OK;

	return ret;
 }
WRESULT CronJob::retrieve( IEncoderReader* deserializer ) {
	if ( deserializer == NULL ) return WRET_ERR_PARAM;
	WRESULT ret = WRET_OK;

	return ret;
 }

/////////////////////////////////////////////////////////////////////////////////////////////// SERIALIZATION
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace automations
}; // namespace wosh
