/** @file    Appliance.cpp
 *  @author  Alessandro Polo
 *  @version $Id: Appliance.cpp 2167 2010-03-18 06:41:05Z alex $
 *  @brief
 * File containing methods for the wosh::building::Appliance class.
 * The header for this class can be found in Appliance.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <framework/building/Appliance.h>

 #include <core/ObjectAllocator.h>
 #include <core/NEW_List.h>
 #include <core/MethodsDef.h>


namespace wosh {
 namespace building {

 WOSH_REGISTER(wosh::building::Appliance, "wosh::building::BuildingObject", _Appliance_VERSION, _static_Appliance )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

Appliance::Appliance() : BuildingObject()
 {
	BuildingObject::setName( Utilities::format("wosh.building.Appliance#%ld", Utilities::randomLong()) );
 }

void Appliance::init()
 {
	Log(LOG_VERBOSE, " Setting default properties and permissions.." );
	Properties.set( _Appliance_KEY_ExternalAppliance, false, Permission(Permission::RW, Permission::RW, Permission::Read) );
	Properties.set( _Appliance_KEY_PowerWatt, false, Permission(Permission::RW, Permission::RW, Permission::Read) );

	this->state = Appliance::APPLIANCE_UNKNOWN; this->level = -1;
	setState(Appliance::APPLIANCE_UNKNOWN, -1, false);

	Log(LOG_VERBOSE, " Registering methods.." );
	MethodWRESULTwSecurityToken* mmTurnOn = Methods.create<MethodWRESULTwSecurityToken>( _Appliance_METHOD_on, "turn appliance on" );
	mmTurnOn->setMethod( this, (MethodWRESULTwSecurityTokenPtr)&Appliance::mmDoTurnOn );
	mmTurnOn->getPermission().setMask( Permission::RX, Permission::RX, Permission::RX );

	MethodWRESULTwSecurityToken* mmTurnOff = Methods.create<MethodWRESULTwSecurityToken>( _Appliance_METHOD_off, "turn appliance off" );
	mmTurnOff->setMethod( this, (MethodWRESULTwSecurityTokenPtr)&Appliance::mmDoTurnOff );
	mmTurnOff->getPermission().setMask( Permission::RX, Permission::RX, Permission::RX );

	MethodMessageResponse* mmFade = Methods.create<MethodMessageResponse>( _Appliance_METHOD_fade, "fade appliance" );
	mmFade->setMethod( this, (MethodMessageResponsePtr)&Appliance::mmDoFade );
	mmFade->getPermission().setMask( Permission::RX, Permission::RX, Permission::RX );

	MethodMessageResponse* mmDim = Methods.create<MethodMessageResponse>( _Appliance_METHOD_dim, "dim appliance" );
	mmDim->setMethod( this, (MethodMessageResponsePtr)&Appliance::mmDoDim );
	mmDim->getPermission().setMask( Permission::RX, Permission::RX, Permission::RX );

	MethodMessageResponse* mmBright = Methods.create<MethodMessageResponse>( _Appliance_METHOD_bright, "bright appliance" );
	mmBright->setMethod( this, (MethodMessageResponsePtr)&Appliance::mmDoBright );
	mmBright->getPermission().setMask( Permission::RX, Permission::RX, Permission::RX );

	BuildingObject::init();
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////// PROCESS CHILD MESSAGES

void Appliance::busMessageChild( const Message& childMessage )
 {
	if ( childMessage.isEmpty() ) return;
	const Variant* data = NULL;
	if ( childMessage.getContent()->isResponse() ) {
		const Response* response = childMessage.getContent()->asResponse();
		if ( response == NULL ) return;
		if ( !response->hasData() ) return;
		if ( !response->getData()->isClass(_Property_TYPE) ) return;
		const Property* prop = dynamic_cast<const Property*>(response->getData());
		if ( prop == NULL ) return;
		if ( prop->getKey() != _Device_KEY_DeviceStatus ) return;
		data = &prop->getValue();
	 }
	else if ( childMessage.getContent()->isFact() ) {
		const Fact* fact = childMessage.getContent()->asFact();
		if ( fact == NULL ) return;
		if ( fact->getEvent() != _Device_EVENT_StateChanged ) {
			Log(LOG_WARNING, ":busMessageChild(%s) UNKNOWN Event '%s'", childMessage.getSource().getName().c_str(), fact->getEvent().c_str() );
			return;
		 }
		if ( !fact->hasData() ) {
			Log(LOG_WARNING, ":busMessageChild(%s) Event '"_Device_EVENT_StateChanged"' INVALID, no Content!", childMessage.getSource().getName().c_str() );
			return;
		 }
		if ( fact->getData()->isClass(_Variant_TYPE) )
			data = dynamic_cast<const Variant*>( fact->getData() );
			
		if ( data == NULL ) {
			Log(LOG_WARNING, ":busMessageChild(%s) EVENT '"_Device_EVENT_StateChanged"' Invalid, Not Data-Content! [%s]", childMessage.getSource().getName().c_str(), fact->getData()->getClassName() );
			return;
		 }
	}
	else { // ignoring requests, ..
		return;
	 }
	if ( data == NULL )
		return;
	if ( !data->isNumeric() ) {
		Log(LOG_WARNING, ":busMessageChild(%s) NOT-INT [%s] Switch-State '%s'", childMessage.getSource().getName().c_str(), data->getTypeAsString(), data->toString().c_str() );
		return;
	 }
	int devValue = data->toInteger(-1);
	if ( devValue < 0 || devValue > 100 ) {
		Log(LOG_WARNING, ":busMessageChild(%s) OutOfRange Switch-State [%i]", childMessage.getSource().getName().c_str(), devValue );
	 }
	Log(LOG_VERBOSE, ":busMessageChild(%s) Switch-State => %d", childMessage.getSource().getName().c_str(), devValue );

	if ( devValue == 0 )
		setState(Appliance::APPLIANCE_OFF, devValue, true);
	else if ( devValue == 100 )
		setState(Appliance::APPLIANCE_ON, devValue, true);
	else
		setState(Appliance::APPLIANCE_FADED, devValue, true);
 }

////////////////////////////////////////////////////////////////////////////////////// PROCESS CHILD MESSAGES
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////// GENERATED EVENTS

void Appliance::setState( Appliance::APPLIANCE_STATE new_state, int new_level, bool raise_event )
 {
	if ( this->state != new_state || this->level != new_level ) {
		Properties.update( _BuildingObject_KEY_State, getStateAsString() );
	 }
	Properties.update( _BuildingObject_KEY_StateTs, Variant(Utilities::std_time(), Variant::DATETIME) );

	if ( this->raiseAlwaysEvents || raise_event ) {
		Log(LOG_VERBOSE, ":setState(%s,%i) Raising Event.. ", getStateAsString(), new_level );
		if ( new_state == Appliance::APPLIANCE_FADED ) {
			NEW_List* dList = new NEW_List(2);
			dList->push_back<Variant>( getStateAsString() );
			dList->push_back<Variant>( new_level );
			BuildingObject::generateMessage( new Fact( _BuildingObject_EVENT_StateChanged, dList ) );
		 }
		else
			BuildingObject::generateMessage( new Fact( _BuildingObject_EVENT_StateChanged, new Variant(getStateAsString()) ) );
	 }
	else {
		Log(LOG_INFO, ":setState(%s,%i) Not Raising Event.", getStateAsString(), new_level );
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////// GENERATED EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT Appliance::turn( int value, const SecurityToken* security )
 {
	Log(LOG_INFO, ":turn(%i)", value );
	// state update is asynch. raised by Child-Device (as Fact)
	Request* request = new Request(_Dimmer_METHOD_set, new Variant(value) );
	request->setOmitResponse(true);
	BuildingObject::controlChild( getMainSwitchName(), request, security );
	return WRET_OK;
 }

WRESULT Appliance::dim( int value, const SecurityToken* security )
 {
	Log(LOG_INFO, ":dim(%i)", value );
	if ( value <= 0 )
		value = 4;
	// state update is asynch. raised by Child-Device (as Fact)
	Request* request = new Request(_Dimmer_METHOD_dim, new Variant(value) );
	request->setOmitResponse(true);
	BuildingObject::controlChild( getMainSwitchName(), request, security );
	return WRET_OK;
 }

WRESULT Appliance::bright( int value, const SecurityToken* security )
 {
	Log(LOG_INFO, ":bright(%i)", value );
	if ( value <= 0 )
		value = 4;
	// state update is asynch. raised by Child-Device (as Fact)
	Request* request = new Request(_Dimmer_METHOD_bright, new Variant(value) );
	request->setOmitResponse(true);
	BuildingObject::controlChild( getMainSwitchName(), request, security );
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// DEVICE METHODS

WRESULT Appliance::mmDoTurnOn( const SecurityToken* security )
 {
	Log(LOG_INFO, ":mmDoTurnOn()" );
	// state update is asynch. raised by Child-Device (as Fact)
	Request* request = new Request(_Switch_METHOD_on);
	request->setOmitResponse(true);
	BuildingObject::controlChild( getMainSwitchName(), request, security );
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT Appliance::mmDoTurnOff( const SecurityToken* security )
 {
	Log(LOG_INFO, ":mmDoTurnOff()" );
	// state update is asynch. raised by Child-Device (as Fact)
	Request* request = new Request(_Switch_METHOD_off);
	request->setOmitResponse(true);
	BuildingObject::controlChild( getMainSwitchName(), request, security );
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* Appliance::mmDoFade( const Message* requestMessage )
 {
	if ( requestMessage == NULL || requestMessage->isEmpty() ) return NULL;
	if ( !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();
	int value = request->getArgument().toInteger(-10000);
	Log(LOG_INFO, ":mmDoFade(%i)", value );

	WRESULT ret = turn( value, requestMessage->getSecurityToken() );
	return request->replyResponse(ret);
 }

Response* Appliance::mmDoDim( const Message* requestMessage )
 {
	if ( requestMessage == NULL || requestMessage->isEmpty() ) return NULL;
	if ( !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();
	int value = request->getArgument().toInteger(-10000);
	Log(LOG_INFO, ":mmDoDim(%i)", value );

	WRESULT ret = dim( value, requestMessage->getSecurityToken() );
	return request->replyResponse(ret);
 }

Response* Appliance::mmDoBright( const Message* requestMessage )
 {
	if ( requestMessage == NULL || requestMessage->isEmpty() ) return NULL;
	if ( !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();
	int value = request->getArgument().toInteger(-10000);
	Log(LOG_INFO, ":mmDoBright(%i)", value );

	WRESULT ret = bright( value, requestMessage->getSecurityToken() );
	return request->replyResponse(ret);
 }

////////////////////////////////////////////////////////////////////////////////////////////// DEVICE METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

const char* ApplianceStateStr[] = {
	"OFF",		///< (APPLIANCE_STATE)0
	"ON",		///< (APPLIANCE_STATE)100
	"FADED",	///< (APPLIANCE_STATE)101
	"UNKNOWN"	///< (APPLIANCE_STATE)102
 };

Appliance::APPLIANCE_STATE Appliance::getStateFromString( const char* state ) {
	if ( state == NULL ) return Appliance::APPLIANCE_UNKNOWN;
	if ( strcmp( state, ApplianceStateStr[0]) == 0 )
		return Appliance::APPLIANCE_OFF;
	if ( strcmp( state, ApplianceStateStr[1]) == 0 )
		return Appliance::APPLIANCE_ON;
	if ( strcmp( state, ApplianceStateStr[2]) == 0 )
		return Appliance::APPLIANCE_FADED;
	return Appliance::APPLIANCE_UNKNOWN;
 }

const char* Appliance::getStateAsString( Appliance::APPLIANCE_STATE state ) {
	switch(state) {
		case Appliance::APPLIANCE_OFF:		return ApplianceStateStr[0];
		case Appliance::APPLIANCE_ON:		return ApplianceStateStr[1];
		case Appliance::APPLIANCE_FADED:	return ApplianceStateStr[2];
		case Appliance::APPLIANCE_UNKNOWN:
		default:							return ApplianceStateStr[3];
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace building
}; // namespace wosh
