/**
 * @class   wosh::building::Appliance
 * @brief   BuildingObject representing a generic (real) appliance.
 *          This class provides standard Methods and properties to 
 *          control the mapped wosh::interfaces::devices::Switch or 
 *          wosh::interfaces::devices::Dimmer
 *          wosh::building::Appliance is usually a child of wosh::building::Room
 *
 ****************************************************************************
 * @version $Id: Appliance.h 2208 2010-03-25 15:45:43Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Building_Appliance_H__
 #define __WOSH_Building_Appliance_H__

 #include <framework/building/BuildingObject.h>

 #include <interfaces/devices/Switch.h>
 #include <interfaces/devices/Dimmer.h>



namespace wosh {
 namespace building {

 #define _Appliance_TYPE					"wosh::building::Appliance"
 #define _Appliance_VERSION					0.900

 #define _Appliance_KEY_PowerWatt			"PowerWatt"
 #define _Appliance_KEY_ExternalAppliance	"ExternalAppliance"
 #define _Appliance_KEY_Level				"Level"

 #define _Appliance_METHOD_on				"turn_on"
 #define _Appliance_METHOD_off				"turn_off"
 #define _Appliance_METHOD_dim				"dim"
 #define _Appliance_METHOD_bright			"bright"
 #define _Appliance_METHOD_fade				"fade"

 #define _Appliance_METHOD_updatedchildren	"updatedchildren"


class Appliance : public BuildingObject
 {
	WOSH_BUILDING(wosh::building::Appliance)

	public:
		/**
		 * @brief   appliance states
		 */
		enum APPLIANCE_STATE {
			APPLIANCE_OFF		= 0,	///< appliace is off
			APPLIANCE_ON		= 100,	///< appliace is on
			APPLIANCE_FADED		= 101,	///< appliace is on, but not at maximum level
			APPLIANCE_UNKNOWN	= 102	///< appliace state is unknown
		};

	public:
		/**
		 * @brief  Empty constructor.
		 */
		Appliance();
		/**
		 * @brief  Copy constructor.
		 */
		Appliance( const Appliance& m ) : BuildingObject(m) { }
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~Appliance() { }


	public:
		virtual void busMessageChild( const Message& message );

//@}
/** @name Appliance Interface - 
 * @{
 ******************************************************************************/
	public:
		virtual WRESULT turnOn()						{ return mmDoTurnOn(NULL);}
		virtual WRESULT turnOff()						{ return mmDoTurnOff(NULL);}
		virtual WRESULT turn( int dimvalue, const SecurityToken* security = NULL );
		virtual WRESULT dim( int value, const SecurityToken* security = NULL );
		virtual WRESULT bright( int value, const SecurityToken* security = NULL );

	public:
		virtual std::string getMainSwitchName() const	{ return getChildByInterface( _Switch_INTERFACE ); }

		/**
		 * @brief   Get last known appliance state (opened, closed, unknown)
		 * @return  current appliance state (enum)
		 */
		Appliance::APPLIANCE_STATE getState() const		{ return this->state; }
		/**
		 * @brief   Get appliance state as a literal string
		 * @return  current appliance state as literal string
		 */
		inline const char* getStateAsString() const		{ return getStateAsString(this->state); }
		/**
		 * @brief   Get appliance state as a literal string
		 * @param state [in] appliance state to be retrieved as literal string
		 * @return  passed appliance state as literal string
		 */
		static const char* getStateAsString( Appliance::APPLIANCE_STATE state );
		/**
		 * @brief   Get appliance state from a literal string
		 * @param state [in] appliance state as literal string
		 * @return  appliance state (as enum) associated with the argument 
		 */
		static Appliance::APPLIANCE_STATE getStateFromString( const char* state );

//@}
	private:
		WRESULT mmDoTurnOn( const SecurityToken* security );
		WRESULT mmDoTurnOff( const SecurityToken* security );

		Response* mmDoFade( const Message* requestMessage );
		Response* mmDoDim( const Message* requestMessage );
		Response* mmDoBright( const Message* requestMessage );

	protected:
		virtual void init();
		void setState( Appliance::APPLIANCE_STATE state, int level = -1, bool raiseEvent = true );

	protected:
		/// level as enum, fade level is saved in Properties[_Appliance_KEY_Level] and level field
		Appliance::APPLIANCE_STATE state;
		int level; ///< Properties[_Appliance_KEY_Level] is shadow var


}; // class def

 }; // namespace building
}; // namespace wosh

#endif //__WOSH_Building_Appliance_H__
