/** @file    BuildingObject.cpp
 *  @author  Alessandro Polo
 *  @version $Id: BuildingObject.cpp 2167 2010-03-18 06:41:05Z alex $
 *  @brief
 * File containing methods for the wosh::building::BuildingObject class.
 * The header for this class can be found in BuildingObject.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <framework/building/BuildingObject.h>

 #include <core/ObjectAllocator.h>
 #include <core/MethodsDef.h>
 #include <core/MessageFilter.h>
 #include <core/SecurityManager.h>
 #include <core/MessageProcessingCommon.h>
 #include <core/SystemError.h>
 #include <core/Table.h>
 #include <core/NEW_List.h>


using namespace std;
namespace wosh {
 namespace building {

 WOSH_REGISTER(wosh::building::BuildingObject, "wosh::Object", 1.0, _static_BuildingObject )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

BuildingObject::BuildingObject( ) : Object(), MethodRunner(), IPropertiesProviderListener(), IBusListener()
 {
	Object::setName( Utilities::format("wosh.BuildingObject#%ld", Utilities::randomLong()) );
	Object::getURI().setName( Object::getName() );
	Properties.set( _KEY_name, Object::getName() );

	Properties.set( _KEY_type, _BuildingObject_TYPE, Permission(Permission::Read) );

	initBO();
 }

BuildingObject::BuildingObject( const BuildingObject& m ) : Object(m), MethodRunner(), IPropertiesProviderListener(), IBusListener()
 {
	this->parent = m.parent;
	this->raiseAlwaysEvents = m.raiseAlwaysEvents;

	Interfaces = m.Interfaces;
	Properties = m.Properties;
	Geometry = m.Geometry;

	initBO();
 }

BuildingObject::~BuildingObject()
 {
	this->children.free();

	Log(LOG_VERBOSE, ":~BuildingObject() Destroyed." );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void BuildingObject::initBO()
 {
	this->parent = "";
	this->raiseAlwaysEvents = false;

	Log.setIndent( 4 );
	Log.setContext( Object::getName() );
	Log.setLevel( LOG_INFO );

//Log.setLevel(VERBOSE);

	// initialize permissions of default properties
	Properties.set( _KEY_DebugLevel, LogContext::getLogLevelAsString(LOG_INFO), Permission(Object::getPermission(), Permission::RW, Permission::Read, Permission::Read) );
	Properties.set( _BuildingObject_KEY_AlwaysRaiseEvents, this->raiseAlwaysEvents, Permission(Object::getPermission(), Permission::RW, Permission::Read, Permission::Read) );
	Properties.set( _BuildingObject_KEY_Location, "", Permission(Object::getPermission(), Permission::RW, Permission::Read, Permission::Read) );
	Properties.monitor(_KEY_name);
	Properties.monitor(_KEY_DebugLevel);
	Properties.monitor(_BuildingObject_KEY_AlwaysRaiseEvents);

	// register default methods
	MethodMessageResponse* mmPropertyGet = Methods.create<MethodMessageResponse>( _METHOD_GetProperty, "Get Property", Permission(Permission::RX) );
	mmPropertyGet->setMethod( this, (MethodMessageResponsePtr)&BuildingObject::mmDoPropertyGet );

	MethodMessageResponse* mmPropertySet = Methods.create<MethodMessageResponse>( _METHOD_SetProperty, "Set Property", Permission::R_XR_XR__() );
	mmPropertySet->setMethod( this, (MethodMessageResponsePtr)&BuildingObject::mmDoPropertySet );

	MethodMessageResponse* mmPropertiesGet = Methods.create<MethodMessageResponse>( _METHOD_GetProperties, "Get Properties", Permission(Permission::RX) );
	mmPropertiesGet->setMethod( this, (MethodMessageResponsePtr)&BuildingObject::mmDoPropertiesGet );

	MethodMessageResponse* mmPropertyList = Methods.create<MethodMessageResponse>( _METHOD_ListProperties, "List Properties", Permission(Permission::RX) );
	mmPropertyList->setMethod( this, (MethodMessageResponsePtr)&BuildingObject::mmDoPropertyList );

	MethodMessageResponse* mmMethodList = Methods.create<MethodMessageResponse>( _METHOD_ListMethods, "List Methods", Permission(Permission::RX) );
	mmMethodList->setMethod( this, (MethodMessageResponsePtr)&BuildingObject::mmDoMethodList );

	MethodMessageResponse* mmListChildren = Methods.create<MethodMessageResponse>( _METHOD_List, "List Children", Permission(Permission::RX) );
	mmListChildren->setMethod( this, (MethodMessageResponsePtr)&BuildingObject::mmDoListChildren );

	MethodRequest* mmInterfaceList = Methods.create<MethodRequest>( _METHOD_ListInterfaces, "List Interfaces", Permission(Permission::RX) );
	mmInterfaceList->setMethod( this, (MethodRequestPtr)&BuildingObject::mmDoInterfaceList );

	MethodWRESULTwSecurityToken* mmUpdatedChildren = Methods.create<MethodWRESULTwSecurityToken>( _BuildingObject_METHOD_force_update, "force update children" );
	mmUpdatedChildren->setMethod( this, (MethodWRESULTwSecurityTokenPtr)&BuildingObject::mmDoUpdatedChildren );
	mmUpdatedChildren->getPermission().setMask( Permission::RX, Permission::RX, Permission::Read );

 }

void BuildingObject::init()
 {
	Properties.updateObjectKeys( this );
	Properties.setPermission( Object::getPermission(), Permission::Overwrite_All, true, true );
	Properties.setPropertiesProviderListener(this);

	Interfaces.createFor(this);

	// register to BUS(ses)
	BusBuilding.setMessageHandler(this);
	WRESULT bus_b_connected = BusBuilding.connect( _Bus_Building );
	if ( WFAILED(bus_b_connected) )
		SystemError::raise(this, Log, SystemError::CRITICAL, 0, "initBO", "FAILED#%i Connecting Bus (wosh.Bus.Building)..", bus_b_connected );

	BusDevices.setMessageHandler(this);
	WRESULT bus_d_connected = BusDevices.connect( _Bus_Devices );
	if ( WFAILED(bus_d_connected) )
		SystemError::raise(this, Log, SystemError::CRITICAL, 0, "initBO", "FAILED#%i Connecting Bus (wosh.Bus.Devices)..", bus_b_connected );

 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////// PROCESS BUS MESSAGES

void BuildingObject::busMessage( const Message& message, const Bus*  )
 {
	if ( message.isEmpty() ) return;

	// avoid double-response, since there is route devices<->building,
	// if source bus is DEVICE and current is BUILDING, then it's a double message and we drop
	if ( message.getSourceBus() == BusBuilding.getBusName() && message.getCurrentBus() == BusDevices.getBusName() )
		return;
	if ( message.getSourceBus() == _Bus_Core && message.getCurrentBus() == BusDevices.getBusName() )
		return;

	if ( MessageFilter::isMessageTarget(message, this->getURI(), &this->Properties) ) {
		if ( message.getSourceBus() == BusBuilding.getBusName() || message.getCurrentBus() == BusDevices.getBusName() ) {
			if ( isChild(message.getSource()) ) {
				Log(LOG_VERBOSE, ":busMessage() : Message from Child '%s' [%s]", message.getSource().getName().c_str(), message.getSourceBus().c_str() );
				busMessageChild(message);
				return;
			 }
		 }

		if ( !message.getContent()->isRequest() )
			return;

		Message* reply = MPC_busMessageRequest( message, Methods, Log );
		if ( reply == NULL )
			return;
		reply->setSource( this );

		if ( message.getCurrentBus() == BusDevices.getBusName() )
			BusDevices.postMessage(reply);
		else// if ( message.getCurrentBus() == BusBuilding.getBusName() )
			BusBuilding.postMessage(reply);

		return;
	 }


 }

//////////////////////////////////////////////////////////////////////////////////////// PROCESS BUS MESSAGES
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////// GENERATED EVENTS

void BuildingObject::generateMessage( MessageContent* content )
 {
	Log(LOG_VERBOSE, ":generateMessage() : Generating Message.." );
	Message* message = new Message();
	message->setSource( this );
	message->setContent( content );
	message->setDestinationBroadcast(true);
	message->setDestinationBus(_Bus_Building);
	BusBuilding.postMessage( message );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void BuildingObject::controlChild( const std::string& child_uri, MessageContent* content, const SecurityToken* security )
 {
	if ( child_uri.empty() || content == NULL ) return;
	Log(LOG_VERBOSE, ":controlChild(%s) : Generating Message..", child_uri.c_str() );
	Message* message = new Message();
	message->setContent( content, true );
	if ( security == NULL ) {
		message->setSecurityToken( SecurityManager::generateToken(this->getPermission().getOwner()) );
		Log(LOG_WARNING, ":controlChild(%s) : NULL Security, Inherited", child_uri.c_str() );
	 }
	else
		message->setSecurityToken( security->clone() );

	controlChild(child_uri, message);
 }

void BuildingObject::controlChild( const std::string& child_uri, Message* message )
 {
	if ( message == NULL || child_uri.empty() ) return;
	Log(LOG_VERBOSE, ":controlChild(%s) : Injecting Message..", child_uri.c_str() );
	message->setSource( this );
	message->getDestination().fromString(child_uri);
	///@bug BuildingObject::controlChild! using always Bus.Devices!!
	BusDevices.postMessage( message );
 }

int BuildingObject::controlChildren( const std::string& child_type, const Message& message )
 {
	int found = 0;
	MutexLockerRead mL(this->children.getMutex());
	std::vector<std::string> devices;
	BuildingObject::getChildrenByInterface_( child_type, devices );
	Log(LOG_VERBOSE, ":controlChildren(%s) : Generating %i Message(s)..", child_type.c_str(), devices.size() );
	std::vector<std::string>::iterator it;
	for ( it=devices.begin(); it!=devices.end(); ++it ) {
		controlChild( *it, message.clone() );
		++found;
	 }
	return found;
 }

int BuildingObject::controlChildren( const std::string& child_type, const MessageContent& content, const SecurityToken* security )
 {
	MutexLockerRead mL(this->children.getMutex());
	return controlChildren_( child_type, content, security );
 }

int BuildingObject::controlChildren_( const std::string& child_type, const MessageContent& content, const SecurityToken* security )
 {
	int found = 0;
	std::vector<std::string> devices;

	BuildingObject::getChildrenByInterface_( child_type, devices );
	Log(LOG_VERBOSE, ":controlChildren(%s) : Generating %i Message(s)..", child_type.c_str(), devices.size() );
	std::vector<std::string>::iterator it;
	for ( it=devices.begin(); it!=devices.end(); ++it ) {
		controlChild( *it, content.clone(), security );
		++found;
	 }
	return found;
 }

//////////////////////////////////////////////////////////////////////////////////////////// GENERATED EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENT

bool BuildingObject::updatingProperty( bool& , const Variant& value_proposed, Property& property_current, const PropertiesProvider* )
 {
	if ( property_current.getKey() == _KEY_name ) { // if we are here, we allow to edit name
		Log(LOG_VERBOSE, " Changing Bundle Name, Log-Context will change to '%s'", value_proposed.toString().c_str() );
		Object::setName( value_proposed.toString() );
		Log.setContext( value_proposed.toString() );
		return true;
	 }
	else if ( property_current.getKey() == _KEY_DebugLevel )
		return MPC_PropertyParse_LogLevel( value_proposed, this->Log );

	else if ( property_current.getKey() == _BuildingObject_KEY_AlwaysRaiseEvents ) {
		this->raiseAlwaysEvents = value_proposed.toBoolean(this->raiseAlwaysEvents);
		Log(LOG_INFO, " Changing "_BuildingObject_KEY_AlwaysRaiseEvents" to '%i'", this->raiseAlwaysEvents );
		return true;
	 }
	return true;
 }

////////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENT
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////// CHILD(s) OPERATIONS

WRESULT BuildingObject::setChild( const std::string& object_uri, InterfacesProvider* interfaces )
 {
	if ( object_uri.size() < 2 ) return WRET_ERR_PARAM;
	MutexLockerWrite mL(this->children.getMutex());

	if ( this->children.exists(object_uri) ) {
		InterfacesProvider* interfaces_old = this->children.find(object_uri);
		if ( interfaces_old != NULL )
			delete interfaces_old;
	 }

	this->children.set(object_uri, interfaces );
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT BuildingObject::removeChild( const URI& object_uri )
 {
	MutexLockerWrite mL(this->children.getMutex());
	if ( this->children.exists(object_uri) ) {
		this->children.remove(object_uri, true);
	 }
	else
		return WRET_ERR_PARAM;

	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT BuildingObject::updatedChildren_( const URI& object_uri, const SecurityToken* security )
 {
	Log(LOG_INFO, ":updatedChildren_(%s)", object_uri.toString().c_str() );

	InterfacesProvider* ip = this->children.find(object_uri);
	if ( ip == NULL )
		return WRET_ERR_PARAM;

	Request* requestApps = NULL;
	if ( ip->exists(_Device_TYPE) )
		requestApps = new Request( _METHOD_GetProperty, new Variant(_Device_KEY_DeviceStatus) );
	else if ( ip->exists(_BuildingObject_TYPE) )
		requestApps = new Request( _METHOD_GetProperty, new Variant(_BuildingObject_EVENT_StateChanged) );
	else {
	 }
	Message* message = new Message(requestApps);
	message->setSource( this );
	message->setDestination(object_uri);

	if ( security == NULL )
		message->setSecurityToken(SecurityManager::generateToken(this->getPermission().getOwner()));
	else
		message->setSecurityToken(security->clone());

	if ( ip->exists(_Device_TYPE) ) {
		message->setDestinationBus(_Bus_Devices);
		BusDevices.postMessage( message );
	 }
	else if ( ip->exists(_BuildingObject_TYPE) ) {
		message->setDestinationBus(_Bus_Building);
		BusBuilding.postMessage( message );
	 }
	else
		BusBuilding.postMessage( message );

	return WRET_OK;
 }

WRESULT BuildingObject::updatedChildren_( const SecurityToken* security )
 {
	if ( security == NULL ) {
		Log(LOG_CRITICAL, ":updatedChildren_(NULL) FAILED" );
		return WRET_ERR_PARAM;
	 }
	Log(LOG_INFO, ":updatedChildren_()" );

	tURIInterfacesProviderMap::ConstIterator it;
	const tURIInterfacesProviderMap::ConstIterator it_end = this->children.end();
	for ( it=this->children.begin(); it!=it_end; ++it ) {
		if ( it->second == NULL ) continue;
		if ( it->second->exists(_Device_TYPE) ) {
			Request* requestApps = new Request( _METHOD_GetProperty, new Variant(_Device_KEY_DeviceStatus) );
			Message* message = new Message(requestApps);
			message->setSource( this );
			message->setDestination(it->first);
			message->setSecurityToken(security->clone());
			BusDevices.postMessage( message );
			continue;
		 }
		else if ( it->second->exists(_BuildingObject_TYPE) ) {
			Request* requestApps = new Request( _METHOD_GetProperty, new Variant(_BuildingObject_EVENT_StateChanged) );
			Message* message = new Message(requestApps);
			message->setSource( this );
			message->setDestination(it->first);
			message->setSecurityToken(security->clone());
			BusBuilding.postMessage( message );
			continue;
		 }
		else {
			Log(LOG_WARNING, ":updatedChildren_(%s) Unknown Child-Type! [%s]", it->first.toString().c_str(), it->second->toString().c_str() );
		 }
	}

	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool BuildingObject::isChild( const URI& object_uri ) const
 {
	if ( this->children.exists(object_uri) )
		return true;
	return false;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool BuildingObject::isChildA( const URI& object_uri, const std::string& interface ) const
 {
	MutexLockerRead mL(this->children.getMutex());
	if ( !this->children.exists(object_uri) )
		return false;
	if ( this->children.find(object_uri) == NULL )
		return false;
	return this->children.find(object_uri)->exists(interface);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

std::string BuildingObject::getChildByInterface( const std::string& type ) const
 {
	MutexLockerRead mL(this->children.getMutex());
	return getChildByInterface_( type );
 }

std::string BuildingObject::getChildByInterface_( const std::string& type ) const
 {
	tURIInterfacesProviderMap::ConstIterator it;
	const tURIInterfacesProviderMap::ConstIterator it_end = this->children.end();
	for ( it=this->children.begin(); it!=it_end; ++it ) {
		if ( it->second == NULL ) continue;
		if ( !it->second->exists(type) ) continue;
		return it->first.toString();
	 }
	return "";
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

int BuildingObject::getChildrenByInterface( const std::string& type, std::vector<std::string>& results ) const
 {
	MutexLockerRead mL(this->children.getMutex());
	return getChildrenByInterface_( type, results );
 }

int BuildingObject::getChildrenByInterface_( const std::string& type, std::vector<std::string>& results ) const
 {
	results.clear();
	tURIInterfacesProviderMap::ConstIterator it;
	const tURIInterfacesProviderMap::ConstIterator it_end = this->children.end();
	for ( it=this->children.begin(); it!=it_end; ++it ) {
		if ( it->second == NULL ) continue;
		if ( !it->second->exists(type) ) continue;
		results.push_back( it->first.toString() );
	 }
	return results.size();
 }

int BuildingObject::getChildren( std::vector<std::string>& results ) const
 {
	MutexLockerRead mL(this->children.getMutex());
	tURIInterfacesProviderMap::ConstIterator it;
	const tURIInterfacesProviderMap::ConstIterator it_end = this->children.end();
	for ( it=this->children.begin(); it!=it_end; ++it ) {
		results.push_back( it->first .toString());
	 }
	return results.size();
 }

///////////////////////////////////////////////////////////////////////////////////////// CHILD(s) OPERATIONS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS

WRESULT BuildingObject::mmDoUpdatedChildren( const SecurityToken* security )
 {
	Log(LOG_INFO, ":mmDoUpdatedChildren()" );
	// state update is asynch. raised by Child-Device (as Fact)
	MutexLockerRead mL(this->children.getMutex());
	return updatedChildren_(security);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* BuildingObject::mmDoListChildren( const Message* requestMessage )
 {
	if ( requestMessage == NULL || requestMessage->isEmpty() ) return NULL;
	if ( !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, ":mmDoListChildren(%s)", option.c_str() );

	if ( option == "s" || option == "short" ) {
		this->children.transactionBeginRead();
		NEW_List* objs = new NEW_List(this->children.size());
		tURIInterfacesProviderMap::ConstIterator it;
		const tURIInterfacesProviderMap::ConstIterator it_end = this->children.end();
		for ( it=this->children.begin(); it!=it_end; ++it ) {
			objs->push_back<Variant>( it->first.getName() );
		}
		this->children.transactionEnd();
		return request->replyResponse(WRET_OK, objs);
	 }
	else {
		this->children.transactionBeginRead();
		Table* tbl = new Table( this->children.size(), 2, true );
		tbl->setTableName("Children");
		tbl->setHeaderHorizontal("URI", 0);
		tbl->setHeaderHorizontal("Interfaces", 1);
		unsigned int iRow = 0;
		tURIInterfacesProviderMap::ConstIterator it;
		const tURIInterfacesProviderMap::ConstIterator it_end = this->children.end();
		for ( it=this->children.begin(); it!=it_end; ++it, ++iRow ) {
			tbl->set( it->first.clone(), iRow, 0);
			if ( it->second != NULL )
				tbl->set<Variant>( it->second->toString(), iRow, 1);
		}
		this->children.transactionEnd();
		return request->replyResponse(WRET_OK, tbl);
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* BuildingObject::mmDoPropertySet( const Message* requestMessage ) {
	return MPC_mmDoPropertySet( requestMessage, Properties, Log );
 }

Response* BuildingObject::mmDoPropertyGet( const Message* requestMessage ) {
	return MPC_mmDoPropertyGet( requestMessage, this->Properties, this->Log );
 }

Response* BuildingObject::mmDoPropertiesGet( const Message* requestMessage ) {
	return MPC_mmDoPropertiesGet( requestMessage, this->Properties, this->Log );
 }

Response* BuildingObject::mmDoPropertyList( const Message* requestMessage ) {
	return MPC_mmDoPropertyList( requestMessage, this->Properties, this->Log );
 }

Response* BuildingObject::mmDoMethodList( const Message* requestMessage ) {
	return MPC_mmDoMethodList( requestMessage, this->Methods, this->Log );
 }

Response* BuildingObject::mmDoInterfaceList( const Request* request ) {
	if ( request == NULL ) return NULL;
	string option = request->getArgument().toString();
	Log(LOG_VERBOSE, ":mmDoInterfaceList(%s)", option.c_str() );
	if ( option == "s" || option == "short" )
		return request->replyResponse(WRET_OK, this->Interfaces.toList() );
	return request->replyResponse(WRET_OK, this->Interfaces.toTable() );
 }

///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// COMMON/HELPERS

void BuildingObject::setName( const std::string& value )
 {
	Log(LOG_VERBOSE, ":setName('%s')", value.c_str() );
	Object::setName(value);
	Object::getURI().setName( value );
	Log.setContext(value);
	Properties.updateObjectKeys(this);
 }

void BuildingObject::setType( const std::string& value )
 {
	Log(LOG_VERBOSE, ":setType('%s')", value.c_str() );
//	Object::setType(value);
	Properties.set( _KEY_type, value, Permission( Permission::Read ) );
 }

void BuildingObject::setParentBObj( const URI& value )
 {
	Log(LOG_VERBOSE, ":setParentBObj('%s')", value.toString().c_str() );
	this->parent = value.getName();
	Properties.updateObjectKeys(this);
	
	///@bug should update previous parent, and new parent!
 }

void BuildingObject::setPermission( const Permission& value )
 {
	Log(LOG_VERBOSE, ":setPermission('%s')", value.toUnixStyle().c_str() );
	Object::setPermission( value );
	Properties.updateObjectKeys(this);

	Properties.setPermission( value, Permission::Overwrite_All, true, true );
	Methods.setPermission( value, Permission::Overwrite_All, true, true );
 }

////////////////////////////////////////////////////////////////////////////////////////////// COMMON/HELPERS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

std::string BuildingObject::toString() const
 {
	return "[BuildingObject]";
 }

WRESULT BuildingObject::store( IEncoderWriter* serializer ) const {
	if ( serializer == NULL ) return WRET_ERR_PARAM;

	return WRET_OK;
 }
WRESULT BuildingObject::retrieve( IEncoderReader* deserializer ) {
	if ( deserializer == NULL ) return WRET_ERR_PARAM;

	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace building
}; // namespace wosh
