/**
 * @class   wosh::building::Location
 * @brief   
 *
 *
 *

Generic Location. There is hierarchy and cross relationships.

A Room is a Location! Rooms may have sub-locations (such as one defined by a motion sensor)







 *
 *
 *
 *
 *
 *
 *
 ****************************************************************************
 * @version $Id: Location.h 2080 2010-03-10 05:38:54Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Building_Location_H__
 #define __WOSH_Building_Location_H__

 #include <core/Object.h>
 #include <core/UserInfoLocation.h>

 #include <vector>
 #include <map>


namespace wosh {
 namespace building {

 #define _Location_TYPE				"wosh::building::Location"
 #define _Location_VERSION			0.02


class Location : public Object
 {
	WOSH_SERIALIZABLE(wosh::building::Location)

	public:
		/// types of the data
		enum LOCATION_TYPE {
			TYPE_UNKNOWN = 0,
			TYPE_BUILDING = 1,
			TYPE_DEVICE = 2,
			TYPE_VIRTUAL = 8,
			TYPE_GENERIC = 16
		};

	public:
		/**
		 * @brief  Empty constructor.
		 */
		Location();
		/**
		 * @brief  Copy constructor.
		 */
		Location( const Location& m );
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~Location() { }


	public:
		bool isUserInLocation( const std::string& username ) const;


/** @name Getters
 * @{
 ******************************************************************************/
	public:

		const std::string& getParentName() const			{ return this->parentName; }
		std::string& getParentName()						{ return this->parentName; }

		const std::vector<std::string>& getChildren() const	{ return this->children; }
		std::vector<std::string>& getChildren()				{ return this->children; }

		const std::map<std::string, double>& getAccessTo() const	{ return this->accessTo; }
		std::map<std::string, double>& getAccessTo()				{ return this->accessTo; }
		bool hasAccessToLocations() const					{ return !this->accessTo.empty(); }

		const std::map<URI, std::string>& getDevices() const		{ return this->devices; }
		std::map<URI, std::string>& getDevices()					{ return this->devices; }

		std::vector<std::string> getDevicesURI() const;

		const std::map<std::string, UserInfoLocationData>& getUsers() const	{ return this->users; }
		std::map<std::string, UserInfoLocationData>& getUsers()				{ return this->users; }

		std::string getUser( unsigned int index ) const;

		std::vector<std::string> getUsersName() const;

		Location::LOCATION_TYPE getLocationType() const	{ return this->locationType; }

//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:

		void setParentName2( const std::string& value )		{ this->parentName = value; }

		void setParent( Location* location, bool update_parent );
		void addChild( Location* location, bool update_parent );


		void addAccessTo( Location* location, bool update_other, double probability = -1.0 );


		void setLocationType( Location::LOCATION_TYPE value ) { this->locationType = value; }

//@}
/** @name internal fields
 * @{
 ******************************************************************************/
	protected:

		Location::LOCATION_TYPE locationType;

	//	std::string location;				///< location uri (such as home/ground_floor/cucina)
		std::string parentName;				///< parent location
		std::vector<std::string> children;			///< sub-locations
		std::map<std::string, double> accessTo;	///< locations may be accessed => probability

//@}
/** @name Devices available in the location
 * @{
 ******************************************************************************/
	protected:
		std::map<URI, std::string> devices;		///< URI of Device => interfaces

//@}
/** @name Users in the location
 * @{
 ******************************************************************************/
	protected:
		mutable std::map<std::string, UserInfoLocationData> users;	///< synch with UserManager?!

//@}
/** @name internal optional fields
 * @{
 ******************************************************************************/
	protected:
		double latitude;				///< latitude of the location (optional) [North is positive]
		double longitude;				///< longitude of the location (optional) [East is positive]
		double latlong_variance;		///< variance of the lat/long position (optional)

//@}


}; // class def

 }; // namespace building
}; // namespace wosh

#endif //__WOSH_Building_Location_H__
