/** @file    Room.cpp
 *  @author  Alessandro Polo
 *  @version $Id: Room.cpp 2167 2010-03-18 06:41:05Z alex $
 *  @brief
 * File containing methods for the wosh::building::Room class.
 * The header for this class can be found in Room.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include <framework/building/Room.h>
 
 #include <core/ObjectAllocator.h>
 #include <core/NEW_List.h>
 #include <core/MethodsDef.h>
 
 #include <framework/building/Appliance.h>
 #include <framework/building/AudioBox.h>
 #include <framework/building/Light.h>

 #include <interfaces/Device.h>
 #include <interfaces/devices/SensorLight.h>
 #include <interfaces/devices/SensorMotion.h>


namespace wosh {
 namespace building {

 WOSH_REGISTER(wosh::building::Room, "wosh::building::BuildingObject", _Room_VERSION, _static_Room )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

Room::Room() : BuildingObject()
 {
	BuildingObject::setName( Utilities::format("wosh.building.Room#%ld", Utilities::randomLong()) );
 }

void Room::init()
 {
	Log(LOG_VERBOSE, " Setting default properties and permissions.." );
	Properties.set( _Room_KEY_AccessTo, "", Permission(Permission::RW, Permission::RW, Permission::Read) );

	Properties.set( _Room_KEY_PresenceState, "", Permission(Permission::RW, Permission::Read, Permission::Read) );
	Properties.set( _Room_KEY_PresenceStateTs, Variant(0L, Variant::DATETIME), Permission(Permission::Read) );
	this->statePresence = Room::PRESENCE_UNKNOWN;
	setPresenceState(Room::PRESENCE_UNKNOWN, false);

	Properties.set( _Room_KEY_LightState, "", Permission(Permission::RW, Permission::Read, Permission::Read) );
	Properties.set( _Room_KEY_LightStateTs, Variant(0L, Variant::DATETIME), Permission(Permission::Read) );
	this->stateLight = Room::LIGHT_UNKNOWN;
	setLightState(Room::LIGHT_UNKNOWN, false);

	Properties.set( _Room_KEY_NoiseState, "", Permission(Permission::RW, Permission::Read, Permission::Read) );
	Properties.set( _Room_KEY_NoiseStateTs, Variant(0L, Variant::DATETIME), Permission(Permission::Read) );
	this->stateNoise = Room::NOISE_UNKNOWN;
	setNoiseState(Room::NOISE_UNKNOWN, false);

	Log(LOG_VERBOSE, " Registering methods.." );
	MethodWRESULTwSecurityToken* mmLightsTurnOn = Methods.create<MethodWRESULTwSecurityToken>( _Room_METHOD_lights_on, "turn lights on" );
	mmLightsTurnOn->setMethod( this, (MethodWRESULTwSecurityTokenPtr)&Room::mmDoLightsTurnOn );
	mmLightsTurnOn->getPermission().setMask( Permission::RX, Permission::RX, Permission::Read );

	MethodWRESULTwSecurityToken* mmLightsTurnOff = Methods.create<MethodWRESULTwSecurityToken>( _Room_METHOD_lights_off, "turn lights off" );
	mmLightsTurnOff->setMethod( this, (MethodWRESULTwSecurityTokenPtr)&Room::mmDoLightsTurnOff );
	mmLightsTurnOff->getPermission().setMask( Permission::RX, Permission::RX, Permission::Read );

//Log.setLevel(VERBOSE);

	BuildingObject::init();
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////// PROCESS CHILD MESSAGES

void Room::busMessageChild( const Message& childMessage )
 {
	if ( childMessage.isEmpty() ) return;
	const Variant* data = NULL;
	if ( childMessage.getContent()->isResponse() ) {
		const Response* response = childMessage.getContent()->asResponse();
		if ( response == NULL ) return;
		if ( !response->hasData() ) return;
		if ( !response->getData()->isClass(_Property_TYPE) ) return;
		const Property* prop = dynamic_cast<const Property*>(response->getData());
		if ( prop == NULL ) return;
		if ( prop->getKey() != _Device_KEY_DeviceStatus && prop->getKey() != _BuildingObject_KEY_State ) return;
		data = &prop->getValue();
	 }
	else if ( childMessage.getContent()->isFact() ) {
		const Fact* fact = childMessage.getContent()->asFact();
		if ( fact == NULL ) return;
		if ( fact->getEvent() != _BuildingObject_EVENT_StateChanged && fact->getEvent() != _Device_EVENT_StateChanged ) {
			Log(LOG_VERBOSE, ":busMessageChild(%s) UNKNOWN Event '%s'", childMessage.getSource().getName().c_str(), fact->getEvent().c_str() );
			return;
		 }
		if ( !fact->hasData() ) {
			Log(LOG_WARNING, ":busMessageChild(%s) Event '"_Device_EVENT_StateChanged"' INVALID, no Content!", childMessage.getSource().getName().c_str() );
			return;
		 }
		if ( fact->getData()->isClass(_Variant_TYPE) )
			data = dynamic_cast<const Variant*>( fact->getData() );
		else if ( fact->getData()->isClass(_NEW_List_TYPE) ) {
			// BUGGY BUG DEPRECATED
		 }
			
		if ( data == NULL ) {
			Log(LOG_WARNING, ":busMessageChild(%s) EVENT '"_Device_EVENT_StateChanged"' Invalid, Not Data-Content! [%s]", childMessage.getSource().getName().c_str(), fact->getData()->getClassName() );
			return;
		 }
	}
	else { // ignoring requests, ..
		return;
	 }
	if ( data == NULL || data->isStringEmpty() ) {
		return;
	 }

	// sub devices events:
	if ( isChildA( childMessage.getSource(), _SensorMotion_INTERFACE ) ) {
		Log(LOG_VERBOSE, ":busMessageChild() MOTIONs => %s from %s [sMotion]", data->asString().c_str(), childMessage.getSource().getName().c_str() );
		if ( data->asString() == "0" )
			setPresenceState( Room::PRESENCE_EMPTY, true);
		else
			setPresenceState( Room::PRESENCE_DETECTED, true);
	 }
	else if ( isChildA( childMessage.getSource(), _SensorLight_INTERFACE ) ) {
		Log(LOG_VERBOSE, ":busMessageChild() LIGHTs => %s from %s [sLight]", data->asString().c_str(), childMessage.getSource().getName().c_str() );
		if ( data->asString() == "100" )
			setLightState( Room::LIGHT_DARK, true);
		else if ( data->asString() == "0" )
			setLightState( Room::LIGHT_BRIGHT, true);
		else {
			setLightState( Room::LIGHT_NATURAL, true);
		 }
	 }
	// sub appliances events:
	else if ( isChildA( childMessage.getSource(), _Light_TYPE ) ) {
		// note that LIGHTS are APPLIANCES, so type must be checked before _Appliance_TYPE (else it would validate Appliance TYPE!)
		Log(LOG_VERBOSE, ":busMessageChild() LIGHT => %s from %s [Light]", data->asString().c_str(), childMessage.getSource().getName().c_str() );
		Light::APPLIANCE_STATE newState = Light::getStateFromString( data->asString().c_str() );
		if ( newState == Light::APPLIANCE_ON || newState == Light::APPLIANCE_FADED )
			setLightState( Room::LIGHT_BRIGHT, true);
		// off state should be updated by light sensor
	 }
	else if ( isChildA( childMessage.getSource(), _Appliance_TYPE ) ) {
		Log(LOG_VERBOSE, ":busMessageChild() APPLIANCE => %s from %s [Appliance]", data->asString().c_str(), childMessage.getSource().getName().c_str() );
	 }
	else if ( isChildA( childMessage.getSource(), _AudioBox_TYPE ) ) {
		Log(LOG_VERBOSE, ":busMessageChild() AUDIO => %s from %s [AudioBox]", data->asString().c_str(), childMessage.getSource().getName().c_str() );
		AudioBox::AUDIOBOX_STATE newState = AudioBox::getStateFromString( data->asString().c_str() );
		if ( newState == AudioBox::AUDIOBOX_PLAYING )
			setNoiseState( Room::NOISE_DETECTED, true);
		else if ( newState == AudioBox::AUDIOBOX_IDLE )
			setNoiseState( Room::NOISE_SILENT, true);
		else
			setNoiseState( Room::NOISE_UNKNOWN, true);
	 }
	else
		Log(LOG_VERBOSE, ":busMessageChild() EVENT '"_Device_EVENT_StateChanged"' => %s from %s [???]", data->asString().c_str(), childMessage.getSource().getName().c_str() );

 }

////////////////////////////////////////////////////////////////////////////////////// PROCESS CHILD MESSAGES
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////// GENERATED EVENTS

void Room::setPresenceState( Room::PRESENCE_STATE new_state, bool raise_event )
 {
	if ( this->statePresence != new_state ) {
		Properties.update( _Room_KEY_PresenceState, getPresenceStateAsString() );
	 }
	Properties.update( _Room_KEY_PresenceStateTs, Variant(Utilities::std_time(), Variant::DATETIME) );

	if ( this->raiseAlwaysEvents || raise_event ) {
		Log(LOG_VERBOSE, ":setPresenceState(%s) Raising Event.. ", getPresenceStateAsString() );
		BuildingObject::generateMessage( new Fact( _Room_EVENT_PresenceChanged, new Variant(getPresenceStateAsString()) ) );
	 }
	else {
		Log(LOG_INFO, ":setPresenceState(%s) Not Raising Event.", getPresenceStateAsString() );
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Room::setLightState( Room::LIGHT_STATE new_state, bool raise_event )
 {
	if ( this->stateLight != new_state ) {
		Properties.update( _Room_KEY_LightState, getLightStateAsString() );
	 }
	Properties.update( _Room_KEY_LightStateTs, Variant(Utilities::std_time(), Variant::DATETIME) );

	if ( this->raiseAlwaysEvents || raise_event ) {
		Log(LOG_VERBOSE, ":setLightState(%s) Raising Event.. ", getLightStateAsString() );
		BuildingObject::generateMessage( new Fact( _Room_EVENT_LightChanged, new Variant(getLightStateAsString()) ) );
	 }
	else {
		Log(LOG_INFO, ":setLightState(%s) Not Raising Event.", getLightStateAsString() );
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Room::setNoiseState( Room::NOISE_STATE new_state, bool raise_event )
 {
	if ( this->stateNoise != new_state ) {
		Properties.update( _Room_KEY_NoiseState, getNoiseStateAsString() );
	 }
	Properties.update( _Room_KEY_NoiseStateTs, Variant(Utilities::std_time(), Variant::DATETIME) );

	if ( this->raiseAlwaysEvents || raise_event ) {
		Log(LOG_VERBOSE, ":setNoiseState(%s) Raising Event.. ", getNoiseStateAsString() );
		BuildingObject::generateMessage( new Fact( _Room_EVENT_NoiseChanged, new Variant(getNoiseStateAsString()) ) );
	 }
	else {
		Log(LOG_INFO, ":setNoiseState(%s) Not Raising Event.", getNoiseStateAsString() );
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////// GENERATED EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// DEVICE METHODS

WRESULT Room::mmDoLightsTurnOn( const SecurityToken* security )
 {
	Log(LOG_INFO, ":mmDoLightsTurnOn()" );
	// state update is asynch. raised by Child-Device (as Fact)
	Request request( _Appliance_METHOD_on );
	request.setOmitResponse(true);
	BuildingObject::controlChildren( _Appliance_TYPE, request, security );
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT Room::mmDoLightsTurnOff( const SecurityToken* security )
 {
	Log(LOG_INFO, ":mmDoLightsTurnOff()" );
	// state update is asynch. raised by Child-Device (as Fact)
	Request request( _Appliance_METHOD_off );
	request.setOmitResponse(true);
	BuildingObject::controlChildren( _Appliance_TYPE, request, security );
	return WRET_OK;
 }

////////////////////////////////////////////////////////////////////////////////////////////// DEVICE METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

const char* RoomPresenceStateStr[] = {
	"UNKNOWN",	///< (PRESENCE_STATE)0
	"EMPTY",	///< (PRESENCE_STATE)1
	"DETECTED"	///< (PRESENCE_STATE)2
 };

Enum2StrFromArray(Room::getPresenceStateAsString, Room::PRESENCE_STATE, PRESENCE_EMPTY, PRESENCE_DETECTED, 0, RoomPresenceStateStr)

Str2EnumByArray(Room::getPresenceStateFromString, Room::PRESENCE_STATE, 0, PRESENCE_DETECTED, PRESENCE_UNKNOWN, RoomPresenceStateStr)

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

const char* RoomLightStateStr[] = {
	"UNKNOWN",	///< (LIGHT_STATE)0
	"DARK",		///< (LIGHT_STATE)1
	"BRIGHT",	///< (LIGHT_STATE)2
	"NATURAL"	///< (LIGHT_STATE)3
 };

Enum2StrFromArray(Room::getLightStateAsString, Room::LIGHT_STATE, LIGHT_DARK, LIGHT_NATURAL, 0, RoomLightStateStr)

Str2EnumByArray(Room::getLightStateFromString, Room::LIGHT_STATE, 0, LIGHT_NATURAL, LIGHT_UNKNOWN, RoomLightStateStr)

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

const char* RoomNoiseStateStr[] = {
	"UNKNOWN",	///< (NOISE_STATE)0
	"SILENT",	///< (NOISE_STATE)1
	"DETECTED"	///< (NOISE_STATE)2
 };

Enum2StrFromArray(Room::getNoiseStateAsString, Room::NOISE_STATE, NOISE_SILENT, NOISE_DETECTED, 0, RoomNoiseStateStr)

Str2EnumByArray(Room::getNoiseStateFromString, Room::NOISE_STATE, 0, NOISE_DETECTED, NOISE_UNKNOWN, RoomNoiseStateStr)


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace building
}; // namespace wosh
