/**
 * @class   wosh::communication::AudioMessage
 * @brief   wosh::Notification representing an audio message (based on audio-file).
 *          The file containing the audio message is stored in a file
 *          (for example, it may be an mp3).
 *          When message is sent over the network, file is not copied or,
 *          its URI must be valid on all system as the phisical resource.
 *          (see $SHARED_TEMP alias)
 *
 * @see     wosh::interfaces::devices::PlayerAudio
 ****************************************************************************
 * @version $Id: AudioMessage.h 2036 2010-03-04 16:09:59Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Communication_AudioMessage_H__
 #define __WOSH_Communication_AudioMessage_H__

 #include <woshDefs.h>
 #include <core/Notification.h>



namespace wosh {
 namespace communication {

 #define _AudioMessage_TYPE					"wosh::communication::AudioMessage"


class AudioMessage : public Notification
 {
	WOSH_SERIALIZABLE(wosh::communication::AudioMessage)
	WOSH_COMPARE_RTTI(wosh::communication::AudioMessage)

	public:

		/// type of audio message
		enum AUDIO_TYPE {
			AUDIO_GENERIC = 0,
			AUDIO_TEXT = 1,
			AUDIO_DMTF = 2,
			AUDIO_MUSIC = 4
		};

	public:

		AudioMessage();

		/**
		 * @brief  Constructor with initializer(s), provided for convenience.
		 */
		AudioMessage( const std::string& filename );

		AudioMessage( const AudioMessage& m );

		virtual ~AudioMessage() { }

	public:
		virtual bool fromNotification( const Notification* notif );


	public:

		virtual const std::string& getPlain() const				{ return this->optPlain; }

		virtual std::string getSummary( int maxsize = 1023 ) const { (void)maxsize; return this->optPlain;; }

		virtual void setPlain( const std::string& text )		{ this->optPlain = text; }


//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:

//		void setFileUri( const URI& uri )				{ this->file_uri = uri; }
		void setFile( const std::string& path )				{ this->file_path = path; }

		void setAudioType( AUDIO_TYPE type )			{ this->audioType = type; }

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:

		const std::string& getFile() const					{ return this->file_path; }
		std::string& getFile()								{ return this->file_path; }

//		const URI& getFileUri() const					{ return this->file_uri; }
//		URI& getFileUri()								{ return this->file_uri; }

		AUDIO_TYPE getAudioType() const					{ return this->audioType; }

		virtual Notification::TYPE getNotificationType() const { return Notification::TYPE_AUDIO; }

//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:
		virtual AudioMessage& operator=(const AudioMessage& m);

		virtual bool operator==(const AudioMessage &other) const;
		virtual bool operator!=(const AudioMessage &other) const { return !(*this == other);  }

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const;
		virtual WRESULT store( IEncoderWriter* serializer ) const;
		virtual WRESULT retrieve( IEncoderReader* deserializer );
//@}
	protected:
		virtual void clear();

/** @name Internal vars
 * @{
 ******************************************************************************/
	protected:

		AUDIO_TYPE audioType;

		//URI file_uri;
		std::string file_path;

		std::string optPlain;

//@}


}; // class def

 }; // namespace communication
}; // namespace wosh

#endif //__WOSH_Communication_AudioMessage_H__
