/**
 * @class   wosh::communication::ContactInfo
 * @brief   
 *
 *
 *
 ****************************************************************************
 * @version $Id: ContactInfo.h 2051 2010-03-07 08:27:48Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/


#ifndef __WOSH_ContactInfo_H__
 #define __WOSH_ContactInfo_H__

 #include <woshDefs.h>

 #include <core/Notification.h>
 #include <core/Serialization.h>

 #include <core/Utilities.h>



namespace wosh {
 namespace communication {

 #define _ContactInfo_TYPE			"wosh::communication::ContactInfo"


class ContactInfo : public virtual ISerializable
 {
	WOSH_SERIALIZABLE(wosh::communication::ContactInfo)
	WOSH_COMPARE_RTTI(wosh::communication::ContactInfo)

	public:

		ContactInfo() {
			this->userID = "";
			this->userWosh = "";
			this->lastSeen = 0;
			this->protocol = Notification::PROTOCOL_UNKNOWN;
			this->language = "";
			this->lastTargetWoshUser = "";
		 }

		ContactInfo( const ContactInfo& m ) {
			*this = m;
		 }

		virtual ~ContactInfo() { }


//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:

		void setUserID( const std::string& value )			{ this->userID = value; }

		void setUser( const std::string& value )				{ this->userWosh = value; }

		void setLastSeen( long value )					{ this->lastSeen = value; }

		void setProtocol( Notification::PROTOCOL pr )	{ this->protocol = pr; }

		void setLanguage( const std::string& value )			{ this->language = value; }

		void setLastTargetUser( const std::string& value )	{ this->lastTargetWoshUser = value; }

		void set( const std::string& custom_user, const std::string& wosh_user, long last_seen ) {
			this->userID = custom_user;
			this->userWosh = wosh_user;
			this->lastSeen = last_seen;
		 }

		void setLastSeen()								{ this->lastSeen = Utilities::std_time(); }

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:

		const std::string& getUserID() const					{ return this->userID; }
		std::string& getUserID()								{ return this->userID; }

		const std::string& getUser() const					{ return this->userWosh; }
		std::string& getUser()								{ return this->userWosh; }

		long getLastSeen() const						{ return this->lastSeen; }

		Notification::PROTOCOL getProtocol()			{ return this->protocol; }

		const std::string& getLanguage() const			{ return this->language; }
		std::string& getLanguage()						{ return this->language; }

		const std::string& getLastTargetUser() const	{ return this->lastTargetWoshUser; }
		std::string& getLastTargetUser()				{ return this->lastTargetWoshUser; }

		bool isUserSetAndSeen() const					{ return ( !this->userWosh.empty() && this->lastSeen > 0 ); }

		bool isTimedOut( long seconds ) const			{ return ( this->lastSeen < 0 || (long)Utilities::std_time() - this->lastSeen > seconds ); }

		bool isOnline() const							{ return ( this->lastSeen > 0 ); }
		bool isOffline() const							{ return ( this->lastSeen <= 0 ); }

//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:
		virtual ContactInfo& operator=(const ContactInfo& m) {
			this->userID = m.userID;
			this->userWosh = m.userWosh;
			this->lastSeen = m.lastSeen;
			this->protocol = m.protocol;
			this->language = m.language;
			this->lastTargetWoshUser = m.lastTargetWoshUser;
			return *this;
		 }

		virtual bool operator==(const ContactInfo &other) const {
			if ( this->userID != other.userID )
				return false;
			if ( this->userWosh != other.userWosh )
				return false;
			if ( this->lastSeen != other.lastSeen )
				return false;
			return true;
		 }

		virtual bool operator!=(const ContactInfo &other) const	{ return !(*this == other);  }

		virtual bool operator>(const ContactInfo& other) const { return ( this->userWosh > other.userWosh ); }

		virtual bool operator<(const ContactInfo& other) const { return ( this->userWosh < other.userWosh ); }

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const			{ return ""; }

		virtual WRESULT store( IEncoderWriter* serializer ) const {
			if ( serializer == NULL ) return WRET_ERR_PARAM;

			return WRET_OK;
		 }
		virtual WRESULT retrieve( IEncoderReader* deserializer ) {
			if ( deserializer == NULL ) return WRET_ERR_PARAM;

			return WRET_OK;
		 }
//@}
	protected:
		std::string userID;					///< custom account name
		std::string userWosh;				///< wosh account name
		long lastSeen;						///< negative long value means is actuall offline, use abs() to get epoch

		Notification::PROTOCOL protocol;	///< current protocol
		std::string language;				///< current language

		std::string lastTargetWoshUser;		///< wosh account name


}; // class def


 }; // namespace communication
}; // namespace wosh

#endif //__WOSH_ContactInfo_H__
