/** @file    Session.cpp
 *  @author  Alessandro Polo
 *  @version $Id: Session.cpp 2167 2010-03-18 06:41:05Z alex $
 *  @brief
 * File containing methods for the wosh::communication::Session class.
 * The header for this class can be found in Session.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <framework/communication/Session.h>

 #include <core/ObjectAllocator.h>
 #include <framework/communication/SessionProtocol.h>
 #include <core/Utilities.h>


using namespace std;
namespace wosh {
 namespace communication {

 WOSH_REGISTER_INTERFACE(wosh::communication::Session, "wosh::Object", 1.0, _static_Session )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

Session::Session() : Object()
 {
	this->id = Utilities::randomInt();
	Object::setName("Session", this->id);

	this->parentSServer = NULL;
	this->protocol = NULL;

	this->state = Session::IDLE;

	this->username = "";
	this->usernameLogged = "";

	this->msgWelcome = "";
	this->skipWelcome = false;
	this->currentBus = _Bus_ANY;

	this->inputTimeout = 0;
	this->initDelay = 0;
	this->lastInputTS = 0;

	this->historyMaxSize = 20;
	this->currentLocation.setKernelLocal();
 }

Session::~Session()
 {
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Session::parseMessage( const string& message )
 {
	setLastInputNow();
	if ( this->protocol == NULL )
		return;

	this->protocol->parse_command( *this, message );
	saveHistory(message);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool Session::su( const string& user )
 {
	if ( this->usernameLogged == "root" ) {
		setUsername(user);
		return true;
	 }
	return false;
 }

void Session::changeToUserHome()
 {
	URI home;
	home.setKernelLocal();
	home.setPath("Users");
	home.setName(this->username);
	setCurrentLocation(home);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Session::setUsername( const string& value )
 {
	if ( value.empty() ) return;
	this->username = value;
	if ( this->usernameLogged.empty() )
		this->usernameLogged = this->username;
 }

void Session::setUsernameLogged( const string& value )
 {
	if ( value.empty() ) return;
	this->usernameLogged = value;
	if ( this->username.empty() )
		this->username = this->usernameLogged;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Session::setLastInputNow() const
 {
	this->lastInputTS = Utilities::std_time();
 }

void Session::saveHistory( const std::string& value )
 {

 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT Session::setSkipWelcome( bool value )
 {
	if ( this->state == Session::OPENED ) return WRET_ERR_ILLEGAL_USE;
	this->skipWelcome = value;
	return WRET_OK;
 }

WRESULT Session::setInitDelay( int value )
 {
	if ( this->state == Session::OPENED ) return WRET_ERR_ILLEGAL_USE;
	this->initDelay = value;
	return WRET_OK;
 }

WRESULT Session::setInputTimeOut( long value )
 {
	if ( this->state == Session::OPENED ) return WRET_ERR_ILLEGAL_USE;
	this->initDelay = value;
	return WRET_OK;
 }

WRESULT Session::setWelcomeMessage( const string& value )
 {
	if ( this->state == Session::OPENED ) return WRET_ERR_ILLEGAL_USE;
	this->msgWelcome = value;
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

string Session::getStateAsString( Session::STATE state )
 {
	switch(state) {
		case Session::IDLE:			return "IDLE";
		case Session::CONNECTED:	return "CONNECTED";
		case Session::LOGGING:		return "LOGGING";
		case Session::OPENED:		return "OPENED";
		case Session::CLOSED:		return "CLOSED";
		default:					return "UNKNOWN";
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace communication
}; // namespace wosh
