/**
 * @class   wosh::communication::Session
 * @brief   Abstract class (interface) which holds properties and settings of
 *          a session. A Session is a channel between user and the WOSH system,
 *          similar to an unix shell (or Windows' prompt).
 *
 * @see     wosh::communication::SessionProtocol
 * @see     wosh::services::SessionServerBundle
 ****************************************************************************
 * @version $Id: Session.h 2082 2010-03-11 06:26:42Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Communication_Session_H__
 #define __WOSH_Communication_Session_H__

 #include <core/Object.h>
 #include <core/Containers.h>
 #include <core/MessageQueue.h>
 
 #include <framework/communication/ShellCommand.h>
 #include <interfaces/services/SessionServer.h>

 #include <iostream>



namespace wosh {
 namespace communication {

 #define _Session_TYPE			"wosh::communication::Session"

#define WOSH_SESSION(CLASSNAME)			\
		WOSH_CLASSNAME(CLASSNAME)		\
		WOSH_CLASS_ABSTRACT(false)


class SessionProtocol;

class Session : public Object
 {
	WOSH_CLASSNAME(wosh::communication::Session)
	WOSH_CLASS_ABSTRACT(false)

	public:
		/**
		 * @brief   states of a session
		 */
		enum STATE {
			UNKNOWN,
			IDLE,			///< "allocated"
			CONNECTED,		///< just connected, going to login, or a failed login (before disconnecting?)
			LOGGING,		///< logging (user:pass)
			OPENED,			///< session is active
			CLOSED			///< session is closed, may be deleted
		};

	public:

		Session();

		virtual ~Session();



/** @name Session Interface
 * @{
 ******************************************************************************/
	public:

		virtual WRESULT sessionStart( unsigned long timeout = LIMIT_ULONG_MAX ) = 0;
		virtual WRESULT sessionStop( unsigned long timeout = LIMIT_ULONG_MAX ) = 0;

		virtual void printMessage( const std::string& msg ) = 0;

		virtual void parseMessage( const std::string& message );

		virtual void appendMessage( Message* message )		{ this->incomingMsg.push(message); }

//@}
/** @name Getters
 * @{
 ******************************************************************************/
 	public:

		inline long getID()	const							{ return this->id; }
		inline const std::string& getUsername() const		{ return this->username; }
		inline const std::string& getUsernameLogged() const	{ return this->usernameLogged; }

		inline long getLastInputTS() const					{ return this->lastInputTS; }

		inline const URI& getCurrentLocation() const 		{ return this->currentLocation; }

		inline const std::string& getCurrentBus() const		{ return this->currentBus; }

		inline int getInitDelay() const						{ return this->initDelay; }
		inline long getInputTimeOut() const					{ return this->inputTimeout; }
		inline bool getSkipWelcome() const					{ return this->skipWelcome; }

		inline const std::string& getWelcomeMessage() const	{ return this->msgWelcome; }
		inline std::string& getWelcomeMessage()				{ return this->msgWelcome; }

		inline Session::STATE getState() const				{ return this->state; }
		inline bool isIdle() const							{ return (this->state == Session::IDLE); }
		inline bool isConnected() const						{ return (this->state == Session::CONNECTED); }
		inline bool isOpened() const						{ return (this->state == Session::OPENED); }
		inline bool isClosed() const						{ return (this->state == Session::CLOSED); }

		std::string getStateAsString() const				{ return getStateAsString(this->state); }

		static std::string getStateAsString( Session::STATE state );

//@}
/** name Setters
 * @{
 ******************************************************************************/
 	public:
		using Object::setParentObject;

		virtual bool su( const std::string& username );

		virtual void setUsername( const std::string& value );
		virtual void setUsernameLogged( const std::string& value );

		virtual void setCurrentLocation( const URI& uri )		{ this->currentLocation = uri; }
		virtual void setCurrentBus( const std::string& value )	{ this->currentBus = value; }

		void changeToUserHome();

		WRESULT setSkipWelcome( bool value );
		WRESULT setInitDelay( int seconds );
		WRESULT setInputTimeOut( long seconds );

		WRESULT setWelcomeMessage( const std::string& value );

		void setSessionServer( wosh::interfaces::services::SessionServer* parent )
															{ this->parentSServer = parent; }

//@}
/** @name Internal methods, provided for implementations
 * @{
 ******************************************************************************/
	protected:

		virtual void setSessionState( Session::STATE value ) { this->state = value; }

		virtual void saveHistory( const std::string& value );

		void setLastInputNow() const;


//@}
	protected:

		long id;							///< session id
		std::string username;				///< current user
		std::string usernameLogged;			///< original logged user

		Session::STATE state;				///< session state

		mutable URI currentLocation;		///< target URI, it is like a system path (exec 'pwd' to prompt)
		std::string currentBus;				///< target Bus, see Message::destinationBus, default should be wosh.Bus.Core

		int initDelay;						///< force delay before login, in seconds (may be 0)
		long inputTimeout;					///< input timeout (will make session expire), in seconds (may be 0)
		mutable long lastInputTS;			///< timestamp of last user input

		std::string msgWelcome;				///< welcome message (after login)
		bool skipWelcome;					///< skip welcome message

		mutable List22<std::string> history; ///< last executed commands/messages, see saveHistory()
		unsigned int historyMaxSize;		///< max items of history, see saveHistory()

		MessageQueue incomingMsg;			///< local message queue (contain copy of messages)
		SessionProtocol* protocol;			///< the i/o processor (convert user's input and responses)

		wosh::interfaces::services::SessionServer* parentSServer;	///< dad Shell Server

	public://reserved
	
		void setSessionProtocol( SessionProtocol* protocol )		{ this->protocol = protocol; }


}; // class def

 }; // namespace communication
}; // namespace wosh

#endif //__WOSH_Communication_Session_H__
