/** @file    SessionAsynch.cpp
 *  @author  Alessandro Polo
 *  @version $Id: SessionAsynch.cpp 2167 2010-03-18 06:41:05Z alex $
 *  @brief
 * File containing methods for the wosh::communication::SessionAsynch class.
 * The header for this class can be found in SessionAsynch.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <framework/communication/SessionAsynch.h>

 #include <core/ObjectAllocator.h>
 #include <core/Utilities.h>
 #include <core/WoshKernel.h>
 #include <core/Request.h>


 #define _ENDLINE	"\r\n"

 
using namespace std;
namespace wosh {
 namespace communication {

 WOSH_REGISTER(wosh::communication::SessionAsynch, "wosh::communication::Session", 1.0, _static_SessionAsynch )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

SessionAsynch::SessionAsynch() : Session(), Thread(NULL, "wosh.SessionAsynch.Thread")
 {
	this->listener = NULL;

	// override some base class fields
	this->skipWelcome = true;
 }

SessionAsynch::~SessionAsynch() {
	if ( isThreadRunning() )
		sessionStop();
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool SessionAsynch::su( const string& username )
 {
	bool ret = Session::su(username);
	if ( !ret )
		return ret;
	if ( this->listener != NULL )
		this->listener->session_UsernameChanged( username, this );
	return ret;
 }

void SessionAsynch::setSessionState( Session::STATE state )
 {
	Session::setSessionState(state);
	if ( this->listener != NULL )
		this->listener->session_StateChanged( state, this );
 }

void SessionAsynch::setCurrentBus( const std::string& value )
 {
	Session::setCurrentBus(value);
	if ( this->listener != NULL )
		this->listener->session_UriChanged( this->currentLocation, this );
 }

void SessionAsynch::setCurrentLocation( const URI& uri )
 {
	Session::setCurrentLocation(uri);
	if ( this->listener != NULL )
		this->listener->session_UriChanged( this->currentLocation, this );
 }

void SessionAsynch::appendMessage( Message* message )
 {
	if ( this->listener != NULL )
		this->listener->session_MessageQueued( message, this );
	Session::appendMessage( message );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT SessionAsynch::sessionStart( unsigned long timeout )
 {
	if ( this->parentSServer == NULL )
		return WRET_ERR_ILLEGAL_USE;
	if ( this->protocol == NULL )
		return WRET_ERR_ILLEGAL_USE;
	if ( this->listener == NULL)
		return WRET_ERR_ILLEGAL_USE;
	return startThread(timeout);
 }

WRESULT SessionAsynch::sessionStop(unsigned long timeout ) {
	return quitThread(timeout);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool SessionAsynch::initializingThread()
 {
	setSessionState(Session::CONNECTED);

	if ( !this->skipWelcome ) {
		if ( this->listener != NULL )
			this->listener->session_PostWelcome( this->msgWelcome, this );
	 }

	setSessionState(Session::OPENED);
	return true;
 }

void SessionAsynch::runThread()
 {
	this->running = true;
	while( this->running )
	 {
		Thread::sleepForMSec(100);
		setThreadAlive();

		if ( this->incomingMsg.isEmpty() ) {
			Thread::sleepForMSec(100);
			continue;
		 }

		Message* message = this->incomingMsg.pop();
		if ( this->listener != NULL )
			this->listener->session_MessageParsing( message, this );

		this->protocol->evalMessage( *this, message );

		delete message;
	 }
 }

void SessionAsynch::exitingThread()
 {
	setSessionState(Session::CLOSED);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SessionAsynch::printMessage( const string& message )
 {
	if ( this->listener != NULL )
		this->listener->session_PrintMessage( message, this );

 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace communication
}; // namespace wosh
