/**
 * @class   wosh::communication::SessionAsynch
 * @brief   The class represent and manage life of an asynchronous
 *          session, used by GUI.
 *          Callback is asynchronous, input messages are appended to a local
 *          queue and processed by internal thread (loop).
 *
 * @see     wosh::gui::FormConsole
 * @see     wosh::services::SessionServerBundle
 ****************************************************************************
 * @version $Id: SessionAsynch.h 2082 2010-03-11 06:26:42Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Communication_SessionAsynch_H__
 #define __WOSH_Communication_SessionAsynch_H__

 #include <framework/communication/Session.h>
 #include <framework/communication/ShellCommand.h>
 #include <framework/communication/SessionProtocol.h>
 #include <core/Thread.h>

 #include <map>


namespace wosh {
 namespace communication {

 #define _SessionAsynch_TYPE			"wosh::communication::SessionAsynch"


class SessionAsynch;

class ISessionAsynchListener  {

	public:
		virtual void session_StateChanged( Session::STATE state, SessionAsynch* source ) = 0;
		virtual void session_UriChanged( const URI& location, SessionAsynch* source ) = 0;
		virtual void session_UsernameChanged( const std::string& username, SessionAsynch* source ) = 0;

		virtual void session_MessageQueued( Message* message, SessionAsynch* source ) = 0;
		virtual void session_MessageParsing( Message* message, SessionAsynch* source ) = 0;

		virtual void session_PostWelcome( const std::string& message, SessionAsynch* source ) = 0;
		virtual void session_PrintMessage( const std::string& message, SessionAsynch* source ) = 0;

//		virtual void session_( Message* message, SessionAsynch* source ) = 0;

//virtual void printData( const string& message ) = 0;

	public:
		virtual ~ISessionAsynchListener() { }
};


class SessionAsynch :	public Session,
						protected Thread
 {
	WOSH_SESSION(wosh::communication::SessionAsynch)

	public:

		SessionAsynch();

		virtual ~SessionAsynch();


	public:

		void appendMessage( Message* message );


/** @name Session Interface
 * @{
 ******************************************************************************/
	public:

		virtual WRESULT sessionStart( unsigned long timeout = LIMIT_ULONG_MAX );

		virtual WRESULT sessionStop( unsigned long timeout = LIMIT_ULONG_MAX );


		virtual void printMessage( const std::string& msg );


//@}
/** @name Getters
 * @{
 ******************************************************************************/
 	public:

		ISessionAsynchListener* getListener()				{ return this->listener; }

//@}
/** name Setters
 * @{
 ******************************************************************************/
 	public:
		virtual bool su( const std::string& username );

		virtual void setCurrentLocation( const URI& uri );
		virtual void setCurrentBus( const std::string& value );

		void setListener( ISessionAsynchListener* listPtr )	{ this->listener = listPtr; }


//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:

		virtual void setSessionState( Session::STATE value );


		virtual bool initializingThread();
		virtual void runThread();
		virtual void exitingThread();

//@}
	protected:

		ISessionAsynchListener* listener;


}; // class def

 }; // namespace communication
}; // namespace wosh

#endif //__WOSH_Communication_SessionAsynch_H__
