/** @file    SessionProtocolShell.cpp
 *  @author  Alessandro Polo
 *  @version $Id: SessionProtocolShell.cpp 2216 2010-03-26 17:10:19Z alex $
 *  @brief
 * File containing methods for the wosh::communication::SessionProtocolShell class.
 * The header for this class can be found in SessionProtocolShell.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <framework/communication/SessionProtocolShell.h>

 //#include <bundles/services/SessionServer/SessionServer.h>

 #include <core/Bundle.h>
 #include <core/Containers.h>
 #include <core/NEW_List.h>
 #include <core/Table.h>
 #include <core/Request.h>
 #include <core/Response.h>
 #include <core/Property.h>
 #include <framework/communication/TextMessage.h>

 #include <core/BundleManager.h>
 #include <core/Utilities.h>
 #include <core/MethodsDef.h>
 #include <core/WoshKernel.h>
 #include <core/NetworkManager.h>
  
 #define _ENDLINE	"\r\n"
 
 
using namespace std;
namespace wosh {
 using namespace communication;
 namespace communication {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

SessionProtocolShell::SessionProtocolShell() : SessionProtocol()
 {
	// setup default asliases
	this->aliases["methods"] = new ShellCommand( _METHOD_ListMethods );
	this->aliases["props"] = new ShellCommand( _METHOD_ListProperties );
	this->aliases["interfaces"] = new ShellCommand( _METHOD_ListInterfaces );

	this->aliases["set"] = new ShellCommand( _METHOD_SetProperty );
	this->aliases["get"] = new ShellCommand( _METHOD_GetProperty );

 }

SessionProtocolShell::~SessionProtocolShell() {
	map<string,ShellCommand*>::iterator it;
	for ( it=this->aliases.begin(); it != this->aliases.end(); ++it ) {
		if ( it->second == NULL ) continue;
		delete it->second; it->second = NULL;
	 }
	this->aliases.clear();
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SessionProtocolShell::parse_command( Session& session, const string& dataline )
 {
	ShellCommand curr_cmd;
	if ( dataline == "." ) {
		curr_cmd = this->lastCommand;
	 }
	else {
		if ( this->aliases.find(dataline) != this->aliases.end() ) {
			ShellCommand* refCmd = this->aliases[dataline];
			curr_cmd = *refCmd;
		 }
		else if ( !curr_cmd.fromString(dataline ) ) {
			session.printMessage( "  Error: can't parse string" _ENDLINE );
			return;
		 }
	 }
	if ( this->aliases.find(curr_cmd.getMainCommand()) != this->aliases.end() ) {
		ShellCommand* refCmd = this->aliases[curr_cmd.getMainCommand()];
		curr_cmd.setMainCommand( refCmd->getMainCommand() );
	 }
	this->lastCommand = curr_cmd;
	bool found = internal_command( session, &curr_cmd );
	if ( found )
		return;

	if ( this->parent == NULL ) {
		session.printMessage( "  Error: can't post messages (NULL parent)" _ENDLINE );
		return;
	 }
	Request* request = curr_cmd.toRequest();
	if ( request == NULL ) {
		session.printMessage( "  Error: can't create Message -Request" _ENDLINE );
		return;
	 }
	Message* message = new Message(request);
	message->setSource( &session );
	message->setDestination( session.getCurrentLocation() );
	message->setDestinationBus( session.getCurrentBus() );

	bool sent = this->parent->session_post_message( &session, message );
	if ( !sent )
		session.printMessage( "  Error: can't post messages (UNKNOWN)" _ENDLINE );
 }


bool SessionProtocolShell::internal_command( Session& session, ShellCommand* command )
 {
	if ( command == NULL )
		return false;
		
//qDebug() << ">|" << command->getMainCommand().c_str() << "|";

	if ( command->getMainCommand() == "exit" || command->getMainCommand() == "quit" ) {
		session.sessionStop(); // ByUser
		return true;
	 }
	else if ( command->getMainCommand() == "help" ) {
		session.printMessage(	_ENDLINE "WOSH Shell Protocol - Commands:" _ENDLINE \
								"\t help : \t print this screen" _ENDLINE \
								"\t man : \t show manual" _ENDLINE \
								"\t session : \t print session info" _ENDLINE \
								"\t whoami : \t print username" _ENDLINE \
								"\t su : \t switch user" _ENDLINE \
								"\t echo : \t reply input" _ENDLINE \
								"\t proto : \t change session protocol" _ENDLINE \
								"\t notify : \t notify a message to user(s)" _ENDLINE \
								"\t pwd : \t print current path" _ENDLINE \
								"\t ck : \t change kernel" _ENDLINE \
								"\t cd : \t change destination (location)" _ENDLINE \
								"\t cb : \t change bus (target)" _ENDLINE \
								"\t home : \t change to home URI" _ENDLINE \
								"\t scd : \t (smart) change destination (name)" _ENDLINE \
"\t which : \t find method" _ENDLINE \
								"\t . : \t repeat last command-line" _ENDLINE \
								"\t exit : \t close session" _ENDLINE _ENDLINE
							);
		return true;
	 }
	//---------------------------------------------------------------------------------------------------------
	else if ( command->getMainCommand() == "whoami" ) {
		if ( session.getUsername() == session.getUsernameLogged() )
			session.printMessage( "  " + session.getUsername() + _ENDLINE );
		else
			session.printMessage( "  " + session.getUsername() + " [" + session.getUsernameLogged() + "]" + _ENDLINE );
		return true;
	 }
	//---------------------------------------------------------------------------------------------------------
	else if ( command->getMainCommand() == "hello" ) {
		session.printMessage( "  Hi " + session.getUsername() + _ENDLINE );
		return true;
	 }
	//---------------------------------------------------------------------------------------------------------
	else if ( command->getMainCommand() == "su" ) {
		bool done = false;
		if ( command->getArgument(0).isStringEmpty() )
			done = session.su("root");
		else
			done = session.su( command->getArgument(0).asString() );
		if ( done ) {
		 }
		return true;
	 }
	//---------------------------------------------------------------------------------------------------------
	else if ( command->getMainCommand() == "home" ) {
		session.changeToUserHome();
		return true;
	 }
	//---------------------------------------------------------------------------------------------------------
	else if ( command->getMainCommand() == "echo" ) {
		session.printMessage( command->toString(false) + _ENDLINE );
		return true;
	 }
	//---------------------------------------------------------------------------------------------------------
	else if ( command->getMainCommand() == "proto" ) {
		if ( command->getArgument(0).isStringEmpty() )
			session.printMessage( Utilities::format("  SessionProtocol :\t%s %f"_ENDLINE, this->getProtocolName().c_str(), this->getProtocolVersion() ) );
		else if ( this->parent == NULL )
			session.printMessage( "  SessionProtocol :\tINTERNAL ERROR" _ENDLINE );
		else {
			WRESULT ret = this->parent->changeSessionProtocol( &session, command->getArgument(0).asString() );
			if ( WSUCCEEDED(ret) )
				session.printMessage( Utilities::format("  SessionProtocol :\t%s"_ENDLINE, command->getArgument(0).asString().c_str()) );
			else
				session.printMessage( Utilities::format("  ERROR#%i %s:"_ENDLINE, ret, this->getProtocolName().c_str()) );
		 }
		return true;
	 }
	//---------------------------------------------------------------------------------------------------------
	else if ( command->getMainCommand() == "session" ) {
		session.printMessage(	_ENDLINE "Session Information:" _ENDLINE \
								"\tID :\t\t" + Utilities::toString(session.getID()) + _ENDLINE \
								"\tUser :\t\t" + session.getUsername() + _ENDLINE \
								"\tPath :\t\t" + session.getCurrentLocation().toString() + _ENDLINE _ENDLINE
							);
		return true;
	 }
	//---------------------------------------------------------------------------------------------------------
	else if ( command->getMainCommand() == "pwd" ) {
		if ( session.getCurrentLocation().isEmpty() )
			session.printMessage( "  /" _ENDLINE );
		else
			session.printMessage( "  " + session.getCurrentLocation().toString() + _ENDLINE );
		return true;
	 }
	//---------------------------------------------------------------------------------------------------------
	else if ( command->getMainCommand() == "ck" ) {
		string target = "";
		if ( command->getArgument(0).isStringEmpty() )
			session.printMessage( "  Kernel: " + session.getCurrentLocation().getKernelName() + _ENDLINE );
		else if ( command->getArgument(0).asString() == "." )
			target = WoshKernel::getKernelName();
		else {
			target = command->getArgument(0).asString();
			if ( !WoshKernel::getInstance()->network().isHost(target) )
				session.printMessage( "  Warning: Host seems invalid!" _ENDLINE );
		 }
		URI newUri = session.getCurrentLocation();
		newUri.setKernelName( target );
		session.setCurrentLocation(newUri);
		return true;
	 }
	//---------------------------------------------------------------------------------------------------------
	else if ( command->getMainCommand() == "cd" ) {
		if ( command->getArgument(0).isEmpty() ) {
			session.printMessage( "  " + session.getCurrentLocation().toString() + _ENDLINE );
			return true;
		 }

		string target = command->getArgument(0).toString();
		if ( target == "@" ) { // index selection mode, enter the i-entry of of current location's children
			long index = command->getArgument(1).toLong(-1);
			NEW_List uri_list;
			URI childWild(session.getCurrentLocation(), "*");
			long count = URICollector::getInstance().list( &childWild, false, uri_list.getVector() );
			if ( count == 0 ) {
				session.printMessage( "  Error: No children (or unable to list)." _ENDLINE );
				return true;
			 }			
			if ( index < 0 || index >= uri_list.size() ) {
				session.printMessage( "  Error: specify valid numeric index." _ENDLINE );
				renderList(session, &uri_list, "Known Children:" );
				return true;
			 }
			URI* sel_child = dynamic_cast<URI*>(uri_list.itemAt(index));
			if ( sel_child == NULL ) {
				session.printMessage( "  Error: Intenral error retrieving selected uri." _ENDLINE );
				return true;
			 }	
			session.setCurrentLocation(*sel_child);
			uri_list.clear();
			return true;
		 }

		URI newUri = session.getCurrentLocation();
		if ( target.find("/") == 0 ) {
			newUri.clearPathName();
			target = target.substr(1);
		 }
		if ( target == ".." ) {
			if ( newUri.getPath().empty() )
				newUri.setName("");
			else {
				newUri.setName(newUri.getPath().back());
				newUri.getPath().pop_back();
			 }
		 }
		else {
			if ( !newUri.getName().empty() )
				newUri.getPath().push_back(newUri.getName());
			newUri.setName(target);
			if ( !newUri.isValid() )
				session.printMessage( "  Warning: URI seems invalid!" _ENDLINE );
			else {
				if ( newUri.isKernelLocal() && !newUri.exists() )
					session.printMessage( "  Warning: URI seems invalid or not found" _ENDLINE );
			 }
		 }
		session.setCurrentLocation(newUri);
		return true;
	 }
	//---------------------------------------------------------------------------------------------------------
	else if ( command->getMainCommand() == "scd" ) {
		if ( command->getArgument(0).isStringEmpty() ) {
			session.printMessage( "  " + session.getCurrentLocation().toString() + _ENDLINE );
			return true;
		 }
		URI newUri = session.getCurrentLocation();

		string target = "*:**/~" + command->getArgument(0).asString();	// PATTERN SAMPLE *:**/~PlayerCuccina
		std::vector<std::string> results;
		URI target_pattern;
		target_pattern.fromString(target);
///@todo warning about uri...
/*
		int count = URICollector::pregMatch( results, &target_pattern );
		if (  count == 0 ) {
			session.printMessage( "  Warning: URI seems invalid or not found" _ENDLINE );
		 }
		else if (  count == 1 ) {
			newUri.fromString( results[0] );
		 }
		else {
			newUri.fromString( results[0] );
			session.printMessage( Utilities::format("  Guessing URI.. (%i found)"_ENDLINE, count).c_str() );
		 }
*/
		session.setCurrentLocation(newUri);
		return true;
	 }
	//---------------------------------------------------------------------------------------------------------
	else if ( command->getMainCommand() == "cb" ) {
		if ( command->getArgument(0).isStringEmpty() ) {
			session.setCurrentBus(_Bus_ANY);
			session.printMessage( "  Current Bus: " _Bus_ANY " (default)" _ENDLINE );
			return true;
		 }
		session.setCurrentBus( command->getArgument(0).asString() );
		session.printMessage( "  Current Bus: " + session.getCurrentBus() + _ENDLINE );
		return true;
	 }
	//---------------------------------------------------------------------------------------------------------
	else if ( command->getMainCommand() == "which" ) {
		if ( command->getArgCount() < 1 ) {
			return true;
		 }
	 }
	//---------------------------------------------------------------------------------------------------------
	else if ( command->getMainCommand() == "man" ) {
		ISerializable* arg = NULL;
		if ( command->getArguments().empty() )
			arg = session.getCurrentLocation().clone();
		else {
			if ( command->hasArgumentFields() )
				arg = command->toArgumentTable();
			else
				arg = command->toArgumentList();
		 }
		Request* request = new Request(_METHOD_Man, arg );
		Message* message = new Message(request);
		message->setSource( &session );
		message->getDestination().fromString("*:Bundles/Man");
		message->setDestinationBus( _Bus_Core );
		bool sent = this->parent->session_post_message( &session, message );
		if ( !sent )
			session.printMessage( "  Error: can't post messages (unknown)" _ENDLINE );
		return true;
	 }
	//---------------------------------------------------------------------------------------------------------
	else if ( command->getMainCommand() == "notify" ) {
		if ( command->getArguments().size() < 2 || command->getArgument(0).isStringEmpty() ) {
			return true;
		 }
		string target_user = command->getArgument(0).asString();
		string not_message = command->getArgument(1).toString();;
		TextMessage* notif = new TextMessage();
		notif->setSender_User( session.getUsername() );
		if ( target_user != "all" && target_user != "anyone" )
			notif->setRecipent_User( target_user );
		notif->setPlain( not_message );

		Message* message = new Message(notif);
		message->setSource( &session );
		message->getDestination().fromString("*:Communication");
		message->setDestinationBus( _Bus_Core );
		bool sent = this->parent->session_post_message( &session, message );
		if ( !sent )
			session.printMessage( "  Error: can't post messages (unknown)" _ENDLINE );
		return true;
	 }
	//---------------------------------------------------------------------------------------------------------

	return false;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////// RENDERING

void renderSerializable( stringstream& sbuffer, const wosh::ISerializable* object, int maxsize )
 {
	if ( object == NULL ) return;
	if ( object->isClass(_Variant_TYPE) ) {
		const wosh::Variant* obj_data = dynamic_cast<const wosh::Variant*>( object );
		sbuffer << obj_data->toString();
	 }
	else if ( object->isClass(_Permission_TYPE) ) {
		const wosh::Permission* obj_perm = dynamic_cast<const wosh::Permission*>( object );
		sbuffer << obj_perm->toUnixStyle();
	 }
	else {
		string temp = object->toString();
		if ( temp.empty() )
			sbuffer << "-[" << object->getClassName() << "]";
		else
			sbuffer << temp;
	 }
 }

int evalStringSize( const wosh::ISerializable* object, int maxsize  )
 {
	if ( object == NULL ) return 0;
	if ( object->isClass(_Variant_TYPE) ) {
		const wosh::Variant* obj_data = dynamic_cast<const wosh::Variant*>( object );
		if ( obj_data == NULL ) return 0;
		if ( !obj_data->isStringEmpty() )
			return obj_data->asString().size();
		return obj_data->toString().size();
	 }
	else if ( object->isClass(_Permission_TYPE) ) {
		const wosh::Permission* obj_perm = dynamic_cast<const wosh::Permission*>( object );
		if ( obj_perm == NULL ) return 0;
		return obj_perm->toUnixStyle_size();
	 }
	else { // DEPRECATED
		string temp = object->toString();
		return temp.size();
	}
	return 0;
 }

void SessionProtocolShell::renderList( Session& session, const wosh::NEW_List* list, const std::string& title )
 {
	if ( list == NULL ) return;
	stringstream sbuffer;
	sbuffer << title << _ENDLINE;
	NEW_List::ConstIterator it;
	const NEW_List::ConstIterator it_end = list->end();
	for( it=list->begin(); it!=list->end(); ++it ) {
		if ( *it == NULL ) continue;
		renderSerializable(sbuffer, *it, 0);
//		sbuffer << (*it)->toString();
		sbuffer << "\r\n";
	 } 
	sbuffer << std::flush;
	session.printMessage( sbuffer.str() );
 }

void SessionProtocolShell::renderTable( Session& session, const wosh::Table* table, const std::string& title )
 {
	if ( table == NULL ) return;
	int* lenghts = new int[table->getCols()];
	int spacing = 1;
	int tot_lenght = 0;
	for(unsigned int col=0; col<table->getCols(); col++) {
		lenghts[col] = 0;
		for(unsigned int row=0; row<table->getRows(); row++) {
			const wosh::ISerializable* object = table->get(row, col);
			int csize = evalStringSize(object, 40);
			if ( csize > lenghts[col] )
				lenghts[col] = csize;
		}
		lenghts[col] += spacing;
		tot_lenght += lenghts[col];
	 }
	if ( table->hasHeaderHorizontal() ) {
		for(unsigned int col=0; col<table->getCols(); col++) {
			const wosh::ISerializable* object = table->getHeaderHorizontal(col);
			int csize = evalStringSize(object, 40);
			if ( csize > lenghts[col] )
				lenghts[col] = csize + spacing;
		 }
	 }

	stringstream sbuffer;
	sbuffer << title;
	if ( table->hasHeaderHorizontal() ) {
		for(unsigned int col=0; col<table->getCols(); col++) {
			sbuffer.fill(' ');
			sbuffer.width(lenghts[col]);
			const wosh::ISerializable* object = table->getHeaderHorizontal(col);
			renderSerializable(sbuffer, object, 40);
		 }
		sbuffer << "\r\n";
		sbuffer.fill('-');
		sbuffer.width(tot_lenght);
		sbuffer << "-";
		sbuffer << "\r\n";
	 }
	for(unsigned int row=0; row<table->getRows(); row++) {
		for(unsigned int col=0; col<table->getCols(); col++) {
			sbuffer.fill(' ');
			sbuffer.width(lenghts[col]);
			const wosh::ISerializable* object = table->get(row, col);
			renderSerializable(sbuffer, object, 0);
		 }
		sbuffer << "\r\n";
	 }
	sbuffer << "\r\n";
	session.printMessage( sbuffer.str() );

	delete [] lenghts;
	return;
 }

void SessionProtocolShell::renderMan( Session& session, const Response* response, const Message* message ) {
	if ( message == NULL || response == NULL || response->getData() == NULL ) return;
	if ( response->getData()->isClass(_Variant_TYPE) ) {
		const Variant* data = dynamic_cast<const Variant*>(response->getData());
		session.printMessage( data->toString() + _ENDLINE );
	 }
 }

/////////////////////////////////////////////////////////////////////////////////////////////////// RENDERING
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SessionProtocolShell::evalMessage( Session& session, const Message* message )
 {
	if ( message == NULL || message->isEmpty() ) return;
//	if ( this->state != SessionProtocolShell::OPENED ) return;

	string source = "";
	if ( message->getSource() != session.getCurrentLocation() )
		source = message->getSource().toString() + " :: ";

	if ( message->getContent()->isNotification() ) {
		const Notification* notification = message->getContent()->asNotification();
		session.printMessage( Utilities::format("  %sNotification [for %s]:"_ENDLINE"\t%s"_ENDLINE,
												source.c_str(), notification->getRecipent_User().c_str(), notification->getPlain().c_str() ));
	 }

	else if ( message->getContent()->isResponse() ) {
		const Response* response = message->getContent()->asResponse();

		if ( WFAILED(response->getReturnValue()) ) {
			if ( response->hasData() )
				session.printMessage( Utilities::format("  %sMethod: %s returned ERROR#%i"_ENDLINE"  Data: %s"_ENDLINE,
										source.c_str(), response->getMethod().c_str(), response->getReturnValue(),
										response->getData()->toString().c_str() ));
			else 
				session.printMessage( Utilities::format("  %sMethod: %s returned ERROR#%i [Empty]"_ENDLINE,
										source.c_str(), response->getMethod().c_str(), response->getReturnValue() ));
			return;
		 }

		if ( !response->hasData() ) {
			session.printMessage( Utilities::format("  %sMethod: %s returned OK [No data]"_ENDLINE,
									source.c_str(), response->getMethod().c_str(), response->getReturnValue() ));
			return;
		 }

		if ( response->getMethod() == _METHOD_GetProperty ||
				response->getMethod() == _METHOD_SetProperty     ) {
			if ( response->getData()->isClass(_Property_TYPE) ) {
				const Property* property = dynamic_cast<const Property*>( response->getData() );
				session.printMessage( Utilities::format("  %sProperty:"_ENDLINE"\t%s"_ENDLINE,
										source.c_str(), property->toStringUnixStyle().c_str() ));
				return;
			}
		 }
		else if ( response->getMethod() == _METHOD_Man ) {
			renderMan( session, response, message );
			return;
		 } 

		if ( response->getData()->isClass(_Variant_TYPE) ) {
			const wosh::Variant* data = dynamic_cast<const wosh::Variant*>( response->getData() );
			session.printMessage( Utilities::format("  %sResponse of '%s' : OK, {Data::%s}"_ENDLINE" %s"_ENDLINE, 
									source.c_str(), response->getMethod().c_str(),
									data->getTypeAsString(true), data->toString().c_str() ));
			return;
		 }


		else if ( response->getData()->isClass(_NEW_List_TYPE) ) {
			const NEW_List* objects = dynamic_cast<const NEW_List*>( response->getData() );
			string title = "";
			renderList( session, objects, "  " + source + title );
			return;
		 }

		else if ( response->getData()->isClass(_Table_TYPE) ) {
			const wosh::Table* table = dynamic_cast<const wosh::Table*>( response->getData() );
			renderTable(session, table, "  " + source + table->getTableName() + ":""\r\n" );
			return;
		 }

		// unknown method (response)
		session.printMessage( Utilities::format("  %sResponse of '%s' : %i, %s"_ENDLINE,
								source.c_str(), response->getMethod().c_str(),
								response->getReturnValue(), response->getData()->getClassName() ));
	 }




 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SessionProtocolShell::printPrompt( Session& session ) const
 {
	session.printMessage( Utilities::format("%s@%s# ", session.getUsername().c_str(), session.getCurrentLocation().toString().c_str()) );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace communication
}; // namespace wosh
