/** @file    SessionShell.cpp
 *  @author  Alessandro Polo
 *  @version $Id: SessionShell.cpp 2167 2010-03-18 06:41:05Z alex $
 *  @brief
 * File containing methods for the wosh::communication::SessionShell class.
 * The header for this class can be found in SessionShell.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <framework/communication/SessionShell.h>

 #include <core/ObjectAllocator.h>
 #include <core/Utilities.h>
 #include <core/WoshKernel.h>

 //#include <termios.h>


 #define _ENDLINE	"\r\n"

 
using namespace std;
namespace wosh {
 namespace communication {

 WOSH_REGISTER(wosh::communication::SessionShell, "wosh::communication::Session", 1.0, _static_SessionShell )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

SessionShell::SessionShell() : Session(), Thread(NULL, "wosh.SessionShell.Thread")
 {
	this->streamInput = NULL;
	this->streamOutput = NULL;
	this->streamOwn = NULL;

 }

SessionShell::~SessionShell() {
	if ( isThreadRunning() )
		sessionStop();

	if ( this->streamOwn != NULL )
		delete this->streamOwn; this->streamOwn = NULL;
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT SessionShell::sessionStart( unsigned long timeout )
 {
	if ( this->streamInput == NULL || this->streamOutput == NULL )
		return WRET_ERR_ILLEGAL_USE;
	if ( this->parentSServer == NULL )
		return WRET_ERR_ILLEGAL_USE;
	if ( this->protocol == NULL )
		return WRET_ERR_ILLEGAL_USE;
	return startThread(timeout);
 }

WRESULT SessionShell::sessionStop( unsigned long timeout ) {
	return quitThread(timeout);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool SessionShell::initializingThread()
 {
	setSessionState(Session::CONNECTED);


	setSessionState(Session::OPENED);
	return true;
 }

void SessionShell::runThread()
 {
	// little trick for output order
	Thread::sleepForMSec(500);
	if ( !this->skipWelcome ) {
		bool sentok = ( *streamOutput << this->msgWelcome << std::endl );
		if ( !sentok ) {
		 }
	 }

	this->username = "root";

	char buf[256];
	std::string deprecated;
	this->running = true;
	while( this->running && streamInput->good() )
	 {
		this->protocol->printPrompt( *this );

		streamInput->getline(buf, 256);
		deprecated.assign(buf);
		if ( !deprecated.empty() ) {
			setLastInputNow();
			this->protocol->parse_command( *this, deprecated );
			saveHistory(deprecated);
		 }

		while ( !this->incomingMsg.isEmpty() ) {
			Message* message = this->incomingMsg.pop();
			this->protocol->evalMessage( *this, message );
			delete message;
		 }
	 }
 }

void SessionShell::exitingThread()
 {
	setSessionState(Session::CLOSED);

	*streamOutput << "Logged Out." << std::endl;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT SessionShell::setStreamInput( std::istream* input_stream )
 {
	if ( isThreadRunning() ) return WRET_ERR_ILLEGAL_USE;
	this->streamInput = input_stream;
	return WRET_OK;
 }
WRESULT SessionShell::setStreamOutput( std::ostream* output_stream )
 {
	if ( isThreadRunning() ) return WRET_ERR_ILLEGAL_USE;
	this->streamOutput = output_stream;
	return WRET_OK;
 }

WRESULT SessionShell::setStreamStdIO()
 {
	if ( isThreadRunning() ) return WRET_ERR_ILLEGAL_USE;
	setStreamInput( &std::cin );
	setStreamOutput( &std::cout );
	return WRET_OK;
 }

WRESULT SessionShell::setStreamOwn( std::iostream* ios )
 {
	if ( isThreadRunning() ) return WRET_ERR_ILLEGAL_USE;

	if ( this->streamOwn != NULL )
		delete this->streamOwn;

	if ( ios != NULL )
		this->streamOwn = ios;
	else {
		std::stringbuf* strbuf = new std::stringbuf();
		this->streamOwn = new std::iostream( strbuf );
	 }

	setStreamInput( this->streamOwn );
	setStreamOutput( this->streamOwn );
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*

struct termios tbuf, tsave;
int    res;
tbuf.c_lflag    &= ~ICANON;
//tbuf.c_lflag &= ~ECHO;//echo off 
tbuf.c_cc[VMIN]  = sizeof(buf);
tbuf.c_cc[VTIME] = 2;
res = tcsetattr (fileno(stdin), TCSANOW, &tbuf);
res = tcgetattr (fileno(stdin), &tbuf);
assert(res >= 0);
tsave = tbuf;
tbuf.c_lflag    &= ~ICANON;
tbuf.c_cc[VMIN]  = sizeof(buf);
tbuf.c_cc[VTIME] = 2;
res = tcsetattr (fileno(stdin), TCSANOW, &tbuf);
assert(res >= 0);

 for (;;) {
   res = read (fileno (stdin), buf, sizeof buf -1);
   assert(res > 0 && res < sizeof(buf));
   buf[res] = 0;
   printf("got: %d `%s'\n", res, buf);
 }

int cci = 0;
while( this->running ) {
cci = fgetc(stdin);

if ( cci == 't' ) {
	int i=0;
	i++;
}
}

	char ch = 0;
	char buf[1024];
	unsigned int pos = 0;

char input[2+1] ;
while( this->running ) {
	if ( fgets(input, 2, stdin) != NULL ) {

		if (input[0] == 12) {
			int i=0;
			i++;
		}

	}
}

while( false && this->running ) {

	fgets(&ch, 1, stdin );

	if ( ch == 9 ) { // 9=TAB
		// do..
	 }
	else if( ch == 10 ) { // 10=NEWLINE
		buf[pos] = 0;

		setLastInputNow();
		if ( pos > 0 ) {
			this->protocol->parse_command( *this, string(buf) );
//			saveHistory(buffer);
			pos = 0;
		 }
		while ( !this->incomingMsg.isEmpty() ) {
			Message* message = this->incomingMsg.pop();
			this->protocol->evalMessage( *this, message );
			delete message;
		 }

		this->protocol->printPrompt( *this );
	 }
	else if ( ch > 31 && ch < 127 ) { // 32=SPACE, 127=DEL
		buf[pos++] = ch;
	 }
}

	// accpect white spaces, tabs, and special chars
	*streamInput >> std::noskipws;


	char ch;
	char buf[1024];
	unsigned int pos = 0;


streamInput->clear();
*streamInput >> ch;
while( this->running ) {

//	if ( streamInput->eof() ) {
		Thread::sleepForMSec(5);
//		continue;
//	 }
streamInput->get(ch);
//	*streamInput >> ch;

if ( streamInput->fail() ) {
	streamInput->clear();
		continue;
}
if ( streamInput->bad() ) {
	break;
}

	if ( ch == 9 ) { // 9=TAB
		// do..
	 }
	else if( ch == 10 ) { // 10=NEWLINE
		buf[pos] = 0;

		setLastInputNow();
		if ( pos > 0 ) {
			this->protocol->parse_command( *this, string(buf) );
//			saveHistory(buffer);
			pos = 0;
		 }
		while ( !this->incomingMsg.isEmpty() ) {
			Message* message = this->incomingMsg.pop();
			this->protocol->evalMessage( *this, message );
			delete message;
		 }

		this->protocol->printPrompt( *this );
	 }
	else if ( ch > 31 && ch < 127 ) { // 32=SPACE, 127=DEL
		buf[pos++] = ch;
	 }

}

-----------------------------------------------

int main()
{
   fd_set fdset;
   struct timeval timeout;
   int  rc;
   int  val;

   timeout.tv_sec = 6;   // wait for 6 seconds for data
   timeout.tv_usec = 0;


   FD_ZERO(&fdset);

   FD_SET(0, &fdset);

   rc = select(1, &fdset, NULL, NULL, &timeout);
   if (rc == -1)  // select failed
   {
       printf("ERROR path\n");
       val='E';
   }
   else if (rc == 0)  // select timed out
   {
       printf("DEFAULT path\n");
       val='D';
   }
   else 
   {
      if (FD_ISSET(0, &fdset)) 
      {
         val = getchar();
      }
   }
   printf("VAL is %c\n", val);
}
*/
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace communication
}; // namespace wosh
