/**
 * @file    framework/gui.h
 * @brief   Include all components of the gui framework
 * @version 0.8161 $Id: gui.h 1690 2010-01-25 05:19:57Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/*! @namespace wosh::gui
 * @brief     Contains classes regarding GUI. There are many built-in graphical interface
 *            for core-modules, most objects and services.
 *
 * \warning WOSH GUI Framework is under revision from 0.8.013.
 *
 *
 * Implementation is based on QT library, most component inherits QWidget class.
 *
 * Source tree is organized as follows:
 *  - bundles
 *    - devices
 *    - services
 *  - kiosk
 *  - modules
 *  - {..} common classes
 *
 * WOSH GUI architecture follows strongly the QT software models, some components are
 * designed as shared adapter to the WOSH underlying system.
 *
 *
 * WOSH GUI framework is not built-in by default, makefile must include directive:
 * \code
CONFIG += WOSH_GUI
 * \endcode
 * It will force inclusion of <a href="../../src/framework/gui/gui.pro">framework/gui/gui.pro</a>
 * which defines following \c qmake directive (to enable QT.Gui headers and linking references):
 * \code
QT += gui
 * \endcode
 * 
 *
 ****************************************************************************
 * \defgroup Gui GUI
 *
 * Graphical interfaces to WOSH components, modules and bundles.
 *
 * See \ref wosh::gui for details.
 *
 *
 ****************************************************************************/
/*! \page page_framework
 * \htmlonly <hr/> \endhtmlonly
 * \section page_framework_gui GUI 
 *
 * WOSH gui namespace is a rich set of classes based on QT4.
 * \ref page_applications_woshshop "WOSH WoshShop" and
 * \ref page_applications_woshremote "WOSH Remote"
 *
 * GUI Framework is not included by default, you have to define a flag in the application's
 * project file. woshshop.pro (WOSH WorkShop) includes this directive:
 * \verbatim
CONFIG += WOSH_GUI
\endverbatim
 *
 * [..]
 *
 * wosh::gui::IWidget is  [..]
 *
 *
 * \par Bus and Messages
 *  You should have read page_core_bus before going on, moreover you should
 *  have some basic knowledge of QT framework and its signals/slots mechanism.
 *  Because of QT threading and classical inter-thread calls, a custom
 *  wosh::Bus implementation: wosh::gui::BusQt is provided to maintain the standard.
 *  In short, you can't update a QWidget within IBusListener::busMessage() of
 *  a generic wosh::Bus implementation, but you may do processing and emit signals
 *  to local slots. Since code would often look repetitive, wosh::gui::BusConnectorQt 
 *  has been implemented to masquerade the singal/slot code, and it is fine to interact
 *  with graphical objects (usually a QWidget) within its scope.
 *
 * The good news is that you shouldn't usually care about this, because the base
 * GUI interface wosh::gui::IWidget masquerades the lower-layer implementation,
 * a simple (but common) sample of bus I/O is:
 * \code

void FormNetworkManager::refreshHosts() {
	Request* request_hosts = new Request( _NetworkManager_METHOD_Hosts_list );
	postMessageForTarget( new Message(request_hosts) );
	this->m_ui->wUpBarHosts->sayRetrieving();
 }
void FormNetworkManager::busMessage( const Message& message, Bus* source ) {
	if ( !message.getContent()->isResponse() ) return;
	if ( !isMessageForMeOrFromTarget(message) ) return;

	const wosh::Response* response = message.getContent()->asResponse();
	if ( response->getMethod() == _NetworkManager_METHOD_Hosts_list ) {
		if ( !response->hasData() )
			return;
		if ( response->getData()->isClass(_Table_TYPE) ) {
			const wosh::Table* hostsTable = dynamic_cast<const wosh::Table*>(response->getData());
			this->wTableModelHosts->loadTable(hostsTable);
			this->m_ui->tableViewHosts->clearSelection();
			this->m_ui->tableViewHosts->resizeColumnsToContents();
			this->m_ui->tableViewHosts->horizontalHeader()->setResizeMode(0,QHeaderView::Stretch);
			this->m_ui->wUpBarHosts->sayReady();
		 }
	 }
	// [..]
 }

\endcode
 *
 *
 * WOSH Applications will often share some features and functionalities,
 * most common have been included in the framework for RAD.
 *
 *  - wosh::gui::WidgetQMenu
 *    A menu listing child-objects of a specific group, such as bundles listing,
 *    devices, core-modules and so on. Gathering of information is done through 
 *    messages (Request->Response).
 *    See wosh::application::WoshShopWindow::createMenus() as sample use.
 *
 *  - wosh::gui::WidgetUriSelector
 *    
 *
 *  - wosh::gui::WoshTableModel
 *    
 *
 *
 *  - wosh::gui::WidgetUpdateBar
 *    
 *
 *
 *  - wosh::gui::WidgetProperties
 *    
 *
 *  - wosh::gui
 *    
 *
 *
 *  - wosh::gui::WoshObject
 *    
 *
 *
 * Some graphical interfaces of the \ref page_framework_communication framework are
 * wosh::gui::WidgetNotification, wosh::gui::WidgetTextMessage, wosh::gui::WidgetAudioMessage.
 *
 * GUI of core-modules are: (a screenshot is provided in each page)
 *  - wosh::gui::FormBundleManager
 *  - wosh::gui::FormBusManager
 *  - wosh::gui::FormDeviceManager
 *  - wosh::gui::FormDataBaseManager
 *  - wosh::gui::FormFileSystem
 *  - wosh::gui::FormNetworkManager
 *  - wosh::gui::FormSecurityManager
 *  - wosh::gui::FormThreadManager
 *  - wosh::gui::FormUserManager
 *  - wosh::gui::FormSystemMonitor
 *
 *
 * wosh::application::GuiAllocatorStatic is an helper class
 * which maps type-name to its interface and allocates it.
 *
 *
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_FrameWork_Gui_H__
 #define __WOSH_FrameWork_Gui_H__

 #include <woshDefs.h>

namespace wosh {
 namespace gui { };
};




#endif //__WOSH_FrameWork_Gui_H__
