/** @file    FormConsole.cpp
 *  @author  Alessandro Polo
 *  @version $Id: FormConsole.cpp 2222 2010-03-26 20:49:35Z alex $
 *  @brief
 * File containing methods for the wosh::gui::FormConsole class.
 * The header for this class can be found in FormConsole.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include "FormConsole.h"
 #include "ui_FormConsole.h"

 #include <bundles/services/SessionServer/SessionServerBundle.h>

 #include <QFontDialog>
 #include <QColorDialog>
 #include <QMessageBox>

using namespace std;
namespace wosh {
 namespace gui {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

FormConsole::FormConsole(QWidget *parent) : IWidgetGeneric(parent)
 {
	this->m_ui = new Ui::FormConsole();
	this->m_ui->setupUi(this);

	this->scrollAuto = true;
	this->m_ui->chkScrollAuto->setChecked(this->scrollAuto);

	this->sessionDebug = false;
	this->m_ui->cmbSessionDebug->setCurrentIndex(this->sessionDebug);

	this->m_ui->cmbWordWrap->setCurrentIndex(0);

	this->session = new SessionAsynch();
	this->session->setSkipWelcome(false);
	this->session->setUsername("root");
	this->session->setListener(this);

	this->setWindowTitle( tr("Terminal") + " - " + this->session->getURI().getName().c_str() );
	this->getWidgetURI().setName("FormConsole#" + this->session->getURI().getName() );
	this->setTargetURI(this->session->getURI());

	this->m_ui->txtLoggedUser->setText( this->session->getUsernameLogged().c_str() );
	this->m_ui->txtCurrentUser->setText( this->session->getUsername().c_str() );

	this->m_ui->txtCurrentKernel->setText( this->session->getCurrentLocation().getKernelName().c_str() );
	this->m_ui->txtCurrentBus->setText( this->session->getCurrentBus().c_str() );

	WRESULT registered = wosh::services::SessionServerBundle::registerSession(this->session, true);

	if ( WFAILED(registered) ) {
		QMessageBox::critical(this, this->windowTitle(), QString("FAILED Registering Session!") );
	 }

	connect(this, SIGNAL(s_sentMessage(QString)), this, SLOT(e_sentMessage(QString)), Qt::QueuedConnection);
	connect(this, SIGNAL(s_sessionInfoChanged()), this, SLOT(e_sessionInfoChanged()), Qt::QueuedConnection);

	connect(this->m_ui->txtInput, SIGNAL(returnPressed()), this, SLOT(shellSend()));
	
	connect(this->m_ui->txtCurrentUser, SIGNAL(editingFinished()), this, SLOT(txtCurrentUser_editingFinished()));

	connect(this->m_ui->txtCurrentBus, SIGNAL(editingFinished()), this, SLOT(txtCurrentBus_editingFinished()));
	connect(this->m_ui->txtCurrentKernel, SIGNAL(editingFinished()), this, SLOT(txtCurrentKernel_editingFinished()));

	connect(this->m_ui->cmbSessionDebug, SIGNAL(activated(int)), this, SLOT(cmbSessionDebug_activated(int)));
	
	connect(this->m_ui->cmbWordWrap, SIGNAL(activated(int)), this, SLOT(cmbWordWrap_activated(int)));
	connect(this->m_ui->chkScrollAuto, SIGNAL(clicked()), this, SLOT(chkScrollAuto_clicked()));

	connect(this->m_ui->btnFontColorChange, SIGNAL(clicked()), this, SLOT(btnFontColorChange_clicked()));
	connect(this->m_ui->btnFontChange, SIGNAL(clicked()), this, SLOT(btnFontChange_clicked()));
	connect(this->m_ui->btnBackgroundChange, SIGNAL(clicked()), this, SLOT(btnBackgroundChange_clicked()));

	this->m_ui->txtInput->setFocus();
 }

FormConsole::~FormConsole()
 {
	this->session->sessionStop();
	this->session->setListener(NULL);

	WRESULT unregistered = wosh::services::SessionServerBundle::unRegisterSession( this->session );
	if ( WFAILED(unregistered) ) {
		QMessageBox::critical(this, this->windowTitle(), QString("FAILED UnRegistering Session!") );
	 }
	delete this->m_ui;
 }

void FormConsole::changeEvent(QEvent *e) {
	QWidget::changeEvent(e);
	if ( e->type() == QEvent::LanguageChange )
		this->m_ui->retranslateUi(this);
 }

void FormConsole::closeEvent(QCloseEvent*) {
	this->deleteLater();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// SEND COMMAND

void FormConsole::shellSend() {
	if ( this->session == NULL ) return;
	if ( this->m_ui->txtInput->text().isEmpty() )
		return;

	this->m_ui->txtConsole->append( QString("%1@%2# %3")
									.arg(this->session->getUsername().c_str())
									.arg(this->session->getCurrentLocation().toString().c_str())
									.arg(this->m_ui->txtInput->text()) );
	QTextCursor cursor(this->m_ui->txtConsole->textCursor());
	cursor.movePosition(QTextCursor::End, QTextCursor::MoveAnchor);

	this->session->parseMessage( this->m_ui->txtInput->text().toLatin1().data() );

	this->m_ui->txtInput->clear();
 }

//////////////////////////////////////////////////////////////////////////////////////////////// SEND COMMAND
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// SIGNAL->EVENTS

void FormConsole::e_sentMessage( QString message ) {
	this->m_ui->txtConsole->append( message );
 	QTextCursor cursor(this->m_ui->txtConsole->textCursor());
	cursor.movePosition(QTextCursor::End, QTextCursor::MoveAnchor);
 }

void FormConsole::e_sessionInfoChanged()
 {
	this->m_ui->lblCurrentURI->setText( QString("%1@%2#")
									.arg(this->session->getUsername().c_str())
									.arg(this->session->getCurrentLocation().toString().c_str()) );

	this->m_ui->txtCurrentBus->setText( this->session->getCurrentBus().c_str() );
	this->m_ui->txtCurrentKernel->setText( this->session->getCurrentLocation().getKernelName().c_str() );

	this->m_ui->txtLoggedUser->setText( this->session->getUsernameLogged().c_str() );
	this->m_ui->txtCurrentUser->setText( this->session->getUsername().c_str() );
 }

////////////////////////////////////////////////////////////////////////////////////////////// SIGNAL->EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// SESSION EVENTS

void FormConsole::session_PrintMessage( const string& message, SessionAsynch* ) {
	emit s_sentMessage( QString(message.c_str()) );
 }

void FormConsole::session_UriChanged( const URI& , SessionAsynch* ) {
	emit s_sessionInfoChanged();
 }

void FormConsole::session_UsernameChanged( const string& , SessionAsynch* ) {
	emit s_sessionInfoChanged();
 }

void FormConsole::session_StateChanged( Session::STATE state, SessionAsynch* )
 {
	string stateStr = Session::getStateAsString(state);
	if ( state == Session::OPENED ) {
		this->m_ui->lblCurrentURI->setText( QString("%1@%2#")
											.arg(this->session->getUsername().c_str())
											.arg(this->session->getCurrentLocation().toString().c_str()) );
		if ( !this->sessionDebug ) return;
		emit s_sentMessage( QString("<font color='green'>SESSION-STATE : %1</font>").arg(stateStr.c_str()) );
	 }
	else {
		if ( !this->sessionDebug ) return;
		emit s_sentMessage( QString("<font color='red'>SESSION-STATE : %1</font>").arg(stateStr.c_str()) );
	 }
 }

void FormConsole::session_MessageQueued( Message* message, SessionAsynch* ) {
	if ( message == NULL ) return;
	if ( !this->sessionDebug ) return;
	emit s_sentMessage( QString("<font color='gray'>SESSION-MESSAGE/QUEUED : %1</font>").arg(message->getID()) );
 }

void FormConsole::session_MessageParsing( Message* message, SessionAsynch* ) {
	if ( message == NULL ) return;
	if ( !this->sessionDebug ) return;
	emit s_sentMessage( QString("SESSION-MESSAGE/PARSING : %1").arg(message->getID()) );
 }

void FormConsole::session_PostWelcome( const string& message, SessionAsynch* ) {
	if ( !this->sessionDebug ) return;
	emit s_sentMessage( QString("SESSION-WELCOME : %1").arg(message.c_str()) );
 }

////////////////////////////////////////////////////////////////////////////////////////////// SESSION EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormConsole::busMessage( const Message& message, const Bus* ) {
	(void)message;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////// SETTINGS

void FormConsole::txtCurrentUser_editingFinished() {
	if ( this->session == NULL ) return;
	bool done = this->session->su( this->m_ui->txtCurrentUser->text().toLatin1().data() );
	if ( !done ) {
		QMessageBox::critical(this, this->windowTitle(), QString("FAILED Switching User!") );
	 }
	// will cause callback update
 }

void FormConsole::txtCurrentBus_editingFinished() {
	if ( this->session == NULL ) return;
	this->session->setCurrentBus( this->m_ui->txtCurrentBus->text().toLatin1().data() );
	// will cause callback update
 }

void FormConsole::txtCurrentKernel_editingFinished() {
	if ( this->session == NULL ) return;
	URI newUri = this->session->getCurrentLocation();
	newUri.setKernelName( this->m_ui->txtCurrentKernel->text().toLatin1().data() );
	this->session->setCurrentLocation(newUri);
	// will cause callback update
 }

void FormConsole::chkScrollAuto_clicked() {
	this->scrollAuto = this->m_ui->chkScrollAuto->isChecked();
 }

void FormConsole::cmbSessionDebug_activated(int index) {
	if ( index == 0 )
		this->sessionDebug = false;
	else
		this->sessionDebug = true;
 }

void FormConsole::cmbWordWrap_activated(int index) {
	if ( index == 0 )
		this->m_ui->txtConsole->setLineWrapMode(QTextEdit::NoWrap);
	else
		this->m_ui->txtConsole->setLineWrapMode(QTextEdit::WidgetWidth);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormConsole::btnFontChange_clicked() {
	bool ok;
	QFont font = QFontDialog::getFont( &ok, this->m_ui->txtConsole->font(), this);
	if ( !ok )
		return;
	this->m_ui->txtConsole->setFont(font);
 }

void FormConsole::btnFontColorChange_clicked() {
	QColor color = QColorDialog::getColor(Qt::green, this);
	if ( !color.isValid() )
		return;
	QPalette pal = this->m_ui->txtInput->palette();
	pal.setColor(QPalette::Text, color);
	this->m_ui->txtConsole->setPalette(pal);
	this->m_ui->lblCurrentURI->setPalette(pal);
	this->m_ui->txtInput->setPalette(pal);
 }

void FormConsole::btnBackgroundChange_clicked() {
	QColor color = QColorDialog::getColor(Qt::green, this);
	if ( !color.isValid() )
		return;
	QPalette pal = this->m_ui->txtInput->palette();
	pal.setColor(QPalette::Background, color);
	pal.setColor(QPalette::Window, color);
	this->m_ui->txtConsole->setPalette(pal);
	this->m_ui->lblCurrentURI->setPalette(pal);
	this->m_ui->txtInput->setPalette(pal);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////// SETTINGS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace gui
}; // namespace wosh
