/** @file    IWidget.cpp
 *  @author  Alessandro Polo
 *  @version $Id: IWidget.cpp 1708 2010-01-26 22:28:01Z alex $
 *  @brief
 * File containing methods for the wosh::gui::IWidget class.
 * The header for this class can be found in IWidget.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include "IWidget.h"

 #include <core/ObjectAllocator.h>
 #include <core/Bus.h>
 #include <core/Response.h>
 #include <core/SecurityManager.h>
 #include <framework/database/EncoderXML.h>
 #include <framework/gui/GuiCache.h>
 #include <QSignalMapper>
 #include <QtGui/QMessageBox>


using namespace std;
namespace wosh {
 namespace gui {

 // IWidget
 WOSH_REGISTER_INTERFACE(wosh::gui::IWidget, "", 1.00, _static_IWidget )
 // IWidgetGeneric
 WOSH_REGISTER_INTERFACE(wosh::gui::IWidgetGeneric, "wosh::gui::IWidget", 1.00, _static_IWidgetGeneric )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT IWidget::init() {
	if ( this->myURI.getName().empty() )
		this->myURI.setName( Utilities::format("%s#%ld", getClassName(), Utilities::randomLong()) );
	BusGui.setMessageHandler(this);
	WRESULT bus_connected = BusGui.connect( _Bus_Gui );
	return bus_connected;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void IWidget::postMessageForTarget( Message* message ) {
	message->setSource( this->myURI );

	if ( message->getDestination().isEmpty() )
		message->setDestination( this->targetURI );

	if ( message->getDestinationBus().empty() )
		message->setDestinationBus( _Bus_Core );

	if ( !message->hasSecurityToken() ) {
		SecurityToken* msec = SecurityManager::generateToken( GuiCache::CurrentUser );
		message->setSecurityToken(msec);
	 }
	BusGui.postMessage(message);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void IWidget::setupFor( const std::string& uri_str ) {
	this->myURI.setName( Utilities::format("%s#%10lu", getClassName(), Utilities::randomLong()) );
	this->myURI.setKernelLocal();
	this->targetURI.fromString( uri_str );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

IWidgetGeneric::IWidgetGeneric(QWidget *parent ) : QWidget(parent), IWidget() {
	this->signalMapperBtn = new QSignalMapper(this);
	connect(this->signalMapperBtn, SIGNAL(mapped(const QString &)), this, SLOT(executeRemoteMethod(const QString &)));
 }

IWidgetGeneric::~IWidgetGeneric() {
	delete this->signalMapperBtn; this->signalMapperBtn = NULL;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void IWidgetGeneric::mapMethodButton( const string& method_name, QObject* sender_button ) {
	connect(sender_button, SIGNAL(clicked()), this->signalMapperBtn, SLOT(map()));
	this->signalMapperBtn->setMapping(sender_button, method_name.c_str() );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool IWidgetGeneric::promptMethodResponse( const wosh::Message& message, bool filter_mapped, bool filter_failed, bool filter_target ) {
	if ( message.isEmpty() || !message.getContent()->isResponse() ) return false;
	if ( filter_target && !URI::pregMatch(message.getDestination(), this->getWidgetURI()) ) return false;
	const Response* response = message.getContent()->asResponse();
	if ( filter_mapped ) {
		QObject* objMapped = this->signalMapperBtn->mapping( response->getMethod().c_str() );
		if ( objMapped == NULL )
			return false;
	 }
	else if ( filter_failed && WSUCCEEDED(response->getReturnValue()) ) return false;
	promptMethodResponse(response, this->windowTitle() );
	return true;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void IWidgetGeneric::promptMethodResponse( const Response* response, const QString& parent_title ) {
	QString msgstr = "";
	if ( WSUCCEEDED(response->getReturnValue()) ) {
		msgstr = tr("SUCCEEDED\nExecuting method '%1'").arg(response->getMethod().c_str()).arg(response->getReturnValue());
	 }
	else if ( WFAILED(response->getReturnValue()) ) {
		msgstr = tr("FAILED\n (Error# %1)Executing method '%2'.\n").arg(response->getReturnValue()).arg(response->getMethod().c_str());
		msgstr += tr("Generic error description is:\n");
		switch( response->getReturnValue() ) {
			case WRET_ERR_PARAM:		msgstr += tr("missing/invalid/illegal parameter(s)"); break;
			case WRET_ERR_INTERNAL:		msgstr += tr("internal (routine) error"); break;
			case WRET_ERR_ILLEGAL_USE:	msgstr += tr("illegal use of function (forbidden)"); break;
			case WRET_ERR_DEPRECATED:	msgstr += tr("method is deprecated"); break;
			case WRET_ERR_WRONG_STATE:	msgstr += tr("call is not valid in current state"); break;
			case WRET_ERR_UNKNOWN:		msgstr += tr("unknown (not handled) error"); break;
			case WRET_ERR_SECURITY:		msgstr += tr("security/permission error"); break;
			case WRET_NOT_IMPLEMENTED:	msgstr += tr("not implemented"); break;
			default:					msgstr += tr("description not available"); break;
		 }
		if ( response->hasData() ) {
			msgstr += tr("\nSpecific Error description:\n%1\n").arg(response->getData()->toString().c_str());
		 }
	 }
	char* buffer = NULL;
	QMessageBox msgBox;
	msgBox.setWindowTitle(tr("%1 Error").arg(parent_title));
	msgBox.setText(msgstr);
	if ( WSUCCEEDED(response->getReturnValue()) )
		msgBox.setIcon(QMessageBox::Information);
	else {
		msgBox.setIcon(QMessageBox::Warning);
		char* buffer = new char[12000];
		wosh::database::EncoderXML::serializeTo( response, buffer, 12000 );
		msgBox.setDetailedText( buffer );
	}
	msgBox.resize( QSize(400, 270) );
	msgBox.exec();
	if ( buffer != NULL )
		delete [] buffer;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace gui
}; // namespace wosh
