/** @file    WidgetProperties.cpp
 *  @author  Alessandro Polo
 *  @version $Id: WidgetProperties.cpp 2189 2010-03-22 21:22:06Z alex $
 *  @brief
 * File containing methods for the wosh::gui::WidgetProperties class.
 * The header for this class can be found in WidgetProperties.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "WidgetProperties.h"
 #include "ui_WidgetProperties.h"

 #include <core/Request.h>
 #include <core/Response.h>
 #include <core/MethodsDef.h>
 #include <core/SecurityManager.h>

 #include <QtGui/QMenu>
 #include <QtGui/QContextMenuEvent>
 #include <QtGui/QInputDialog>
 #include <QtGui/QMessageBox>
 #include <QHeaderView>


using namespace std;
namespace wosh {
 namespace gui {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WidgetProperties::WidgetProperties(QWidget *parent) : QWidget(parent), IWidget()
 {
	this->m_ui = new Ui::WidgetProperties();
	this->m_ui->setupUi(this);

	this->menuKey = NULL;
	this->menuValue = NULL;
	this->menuPerm = NULL;

	GenericItemDelegate* tblDelegate = new GenericItemDelegate(this);
	this->m_ui->tableView->setItemDelegate(tblDelegate);

	this->m_ui->wUpBarProps->setEnabled(false);

	connect(this->m_ui->tableView, SIGNAL(tableCell_changed(unsigned int,unsigned int, ISerializable*)), this, SLOT(cell_valueChanged(unsigned int,unsigned int, ISerializable*)));

	//connect(this->m_ui->tableWidget, SIGNAL(itemChanged(QTableWidgetItem*)), this, SLOT(cell_valueChanged(QTableWidgetItem*)));
	//connect(this->m_ui->tableWidget, SIGNAL(itemDoubleClicked(QTableWidgetItem*)), this, SLOT(cell_DblClicked(QTableWidgetItem*)));

	connect(this->m_ui->wUpBarProps, SIGNAL(updateClicked()), this, SLOT(refreshSmart()));
}

WidgetProperties::~WidgetProperties()
{
	delete this->m_ui;
	if ( this->menuKey != NULL )
		delete this->menuKey;
	if ( this->menuValue != NULL )
		delete this->menuValue;
	if ( this->menuPerm != NULL )
		delete this->menuPerm;
}

void WidgetProperties::buildMenus()
 {
	this->menuKey = new QMenu(this);
	this->actionKeyUpdate = this->menuKey->addAction(tr("Update Property"));
	this->menuKey->addSeparator();
	this->actionKeyCopyTo= this->menuKey->addAction(tr("Copy To.."));
	this->actionKeyNew = this->menuKey->addAction(tr("New Property.."));

	this->actionKeyMan = this->menuKey->addAction(tr("Man.."));

	this->menuValue = new QMenu(this);
	this->actionValueUpdate = this->menuValue->addAction(tr("Update Property"));

	this->menuPerm = new QMenu(this);
	this->actionPermChange = this->menuPerm->addAction(tr("Change.."));
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void WidgetProperties::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void WidgetProperties::contextMenuEvent( QContextMenuEvent* menu_event )
 {
	QModelIndex index = this->m_ui->tableView->indexAt( menu_event->pos() );
	if ( !index.isValid() )
		return;
	if ( this->menuKey == NULL )
		buildMenus();

	Variant propKey = this->m_ui->tableView->getItemAsVariant( index.row()-1, index.column() );

	QAction* selected = NULL;
	if ( index.column() == 0 )
		selected = this->menuKey->exec( menu_event->globalPos() );
	else if ( index.column() == 1 )
		selected = this->menuValue->exec( menu_event->globalPos() );
	else if ( index.column() == 2 )
		selected = this->menuPerm->exec( menu_event->globalPos() );

	if ( selected == this->actionKeyUpdate || selected == this->actionValueUpdate ) {
		Message* message = new Message( new Request(_METHOD_GetProperty, new Variant(propKey) ) );
		message->setDestinationBus( _Bus_ANY );
		postMessageForTarget(message);
	 }
	else if ( selected == this->actionKeyNew ) {
		// open new widget
	 }
	else if ( selected == this->actionKeyCopyTo ) {
		QString text = QInputDialog::getText(this, tr("Copy Property"), tr("Copy property '%1' to:").arg(propKey.toString().c_str()) );
		if ( text.isEmpty() ) return;
	 }
	else if ( selected == this->actionKeyMan ) {
		Message* message = new Message( new Request(_METHOD_Man, new Variant(propKey) ) );
		message->setDestinationBus( _Bus_ANY );
		postMessageForTarget(message);
	 }
	

 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void WidgetProperties::setTargetURI( const URI& uri )
{
	IWidget::setTargetURI( uri );
	this->m_ui->wUpBarProps->setEnabled(this->getTargetURI().isValid());
}

WRESULT WidgetProperties::reset()
 {
	IWidget::setTargetURI( URI() );
	this->m_ui->wUpBarProps->setEnabled( false );

	this->m_ui->tableView->loadObject(NULL);
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void WidgetProperties::refreshData( bool force_all )
 {
	this->m_ui->wUpBarProps->setEnabled( this->getTargetURI().isValid() );
	if ( !this->getTargetURI().isValid() ) {
		return;
	 }
	Request* request_properties = NULL;
	if ( !force_all && this->m_ui->tableView->getTableModel() != NULL && this->m_ui->tableView->getTableModel()->getDataTable() != NULL && this->m_ui->tableView->selectionModel()->hasSelection() ) {
		NEW_List* keys_list = new NEW_List();
		QModelIndexList iList = this->m_ui->tableView->selectionModel()->selectedIndexes();
		unsigned int iH = 0;
		for(int i=0; i<iList.size(); i++) {
			wosh::Variant dataName = this->m_ui->tableView->getItemAsVariant( iList.at(i).row()+iH, 0);
			if ( keys_list->search(&dataName) != -1 )
				continue;
			keys_list->push_back(dataName.clone());
		 }
		if ( keys_list->size() == 1 ) {
			request_properties = new Request( _METHOD_GetProperty, keys_list->itemAt(0)->clone() );
			this->m_ui->wUpBarProps->showMessage( QString(tr("Retrieving property '%1'..")).arg(keys_list->itemAt(0)->toString().c_str()) );
			delete keys_list;
		 }
		else if ( keys_list->size() != 0 && keys_list->size() < this->m_ui->tableView->model()->rowCount() ) {
			request_properties = new Request( _METHOD_GetProperties, keys_list );
			this->m_ui->wUpBarProps->showMessage( QString(tr("Retrieving %1 properties..")).arg(keys_list->size()) );
		 }
	 }

	if ( request_properties == NULL ) {
		request_properties = new Request( _METHOD_ListProperties );
		this->m_ui->wUpBarProps->sayRetrieving();
	 }

	Message* message = new Message(request_properties);
	message->setDestinationBus( _Bus_ANY );
	postMessageForTarget(message);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT WidgetProperties::loadTable( const Table* table )
 {
	this->m_ui->tableView->loadObject(table);

	this->m_ui->tableView->clearSelection();
	this->m_ui->tableView->resizeColumnsToContents();
	this->m_ui->tableView->horizontalHeader()->setResizeMode(1,QHeaderView::Stretch);

	this->m_ui->wUpBarProps->sayReady();
	return WRET_OK;
 }

WRESULT WidgetProperties::updateTable( const Table* table )
 {
	if ( table == NULL )
		return WRET_ERR_PARAM;
	this->m_ui->wUpBarProps->showMessage( QString(tr("Parsing Data [%1 rows]..")).arg(table->getRows()) );

	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void WidgetProperties::busMessage( const Message& message, const Bus* )
 {
	if ( message.isEmpty() ) return;
	if ( !message.getContent()->isResponse() ) return;
	if ( !isMessageForMeOrFromTarget(message) )
		return;

	const wosh::Response* response = message.getContent()->asResponse();
	if ( WFAILED(response->getReturnValue()) ) {
		if ( !URI::pregMatch( this->myURI, message.getDestination() ) )
			return;
		QString message = "Internal Error";

		if ( response->getMethod() == _METHOD_ListProperties )
			message = tr("Error#%1 retrieving Properties (Table)!").arg(response->getReturnValue());
		else if ( response->getMethod() == _METHOD_GetProperty )
			message = tr("Error#%1 retrieving Property!").arg(response->getReturnValue());
		else if ( response->getMethod() == _METHOD_SetProperty )
			message = tr("Error#%1 setting Property!").arg(response->getReturnValue());
		else if ( response->getMethod() == _METHOD_GetProperties )
			message = tr("Error#%1 retrieving Properties!").arg(response->getReturnValue());
		else if ( response->getMethod() == _METHOD_Man )
			message = tr("Error#%1 retrieving Man!").arg(response->getReturnValue());

		if ( response->hasData() && response->getData()->isClass(_Variant_TYPE) ) {
			const wosh::Variant* dataInfo = dynamic_cast<const wosh::Variant*>(response->getData());
			message += QString(" [%1]").arg(dataInfo->toString().c_str());
		 }
		loadTable(NULL);
		this->m_ui->wUpBarProps->showError(message);
		return;
	 }
	if ( !response->hasData() )
		return;

	if ( response->getMethod() == _METHOD_ListProperties ) {
		if ( response->getData()->isClass(_Table_TYPE) ) {
			const wosh::Table* propTable = dynamic_cast<const wosh::Table*>(response->getData());
			loadTable(propTable);
		 }
	 }
	else if ( response->getMethod() == _METHOD_GetProperty || response->getMethod() == _METHOD_SetProperty ) {

		if ( response->getData()->isClass(_Property_TYPE) ) {
			const wosh::Property* prop = dynamic_cast<const wosh::Property*>(response->getData());
			Table* table = new Table(1,3);
			table->set<Variant>( prop->getKey(), 0,0);
			table->set( prop->getValue().clone(), 0,1);
			table->set( prop->getPermission().clone(), 0,2);
			updateTable(table);
			delete table;
			if ( response->getMethod() == _METHOD_SetProperty )
				this->m_ui->wUpBarProps->showMessage( QString(tr("Applied Property '%1'")).arg(prop->getKey().c_str()) );
		 }
	 }
	else if ( response->getMethod() == _METHOD_GetProperties ) {

		if ( response->getData()->isClass(_Table_TYPE) ) {
			const wosh::Table* propTable = dynamic_cast<const wosh::Table*>(response->getData());
			updateTable(propTable);
		 }
	 }
	else if ( response->getMethod() == _METHOD_Man ) {
		if ( !response->hasData() )
			return;
		QMessageBox::information(this, "Man", response->getData()->toString().c_str() );
	 }

 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void WidgetProperties::cell_valueChanged( unsigned int row, unsigned int col, ISerializable* newValue )
 {
	if ( newValue == NULL )
		return;
	wosh::Variant nameData = this->m_ui->tableView->getItemAsVariant(row, 0 );
	Property* new_property = new Property(nameData.toString());

	if ( col == 1 ) {
		if ( !newValue->isClass(_Variant_TYPE) ) {
			return;
		 }
		const wosh::Variant* valueData = dynamic_cast<const wosh::Variant*>(newValue);
		new_property->setValue(*valueData);
		// dont set permission (so it is empty), so it wont be updated!
	 }
	else if ( col == 2 ) {
		if ( !newValue->isClass(_Permission_TYPE) ) {
			return;
		 }
		const wosh::Permission* propPerm = dynamic_cast<const wosh::Permission*>(newValue);
		new_property->setPermission(*propPerm);
		// dont set value (so it is empty), so it wont be updated!
		new_property->getValue().clear();
	 }

	Request* request_properties = new Request( _METHOD_SetProperty, new_property );
	Message* message = new Message(request_properties);
	message->setDestinationBus( _Bus_ANY );
	postMessageForTarget(message);
	this->m_ui->wUpBarProps->showMessage( QString(tr("Applying changes of '%1'..")).arg(nameData.toString().c_str()) );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace gui
}; // namespace wosh
