/** @file    FormBuildingManager.cpp
 *  @author  Alessandro Polo
 *  @version $Id: FormBuildingManager.cpp 1718 2010-01-28 05:54:14Z alex $
 *  @brief
 * File containing methods for the wosh::gui::FormBuildingManager class.
 * The header for this class can be found in FormBuildingManager.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "FormBuildingManager.h"
 #include "ui_FormBuildingManager.h"

 #include <core/MethodsDef.h>
 #include <core/Request.h>
 #include <core/Response.h>
 #include <core/BusManager.h>

 #include <QtGui>
 #include <framework/gui/GuiUtilities.h>


using namespace std;
namespace wosh {
 namespace gui {

 WOSH_REGISTER(wosh::gui::FormBuildingManager, "wosh::gui::IWidgetGeneric", 1.0, _static_FormBuildingManager )
 WOSH_GUI_MAP(wosh::gui::FormBuildingManager, "wosh::interfaces::services::BuildingManager", 0.0, _static_Map_FormBuildingManager )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

FormBuildingManager::FormBuildingManager(QWidget *parent) : IWidgetGeneric(parent)
 {
	this->m_ui = new Ui::FormBuildingManager();
	this->m_ui->setupUi(this);

	this->getWidgetURI().setName( Utilities::format("FormBuildingManager#%10lu", Utilities::randomLong()) );
	this->getTargetURI().clearPathName();
	this->getTargetURI().setKernelAny();
	this->getTargetURI().setName("Building");

	this->setWindowTitle("Building Manager");
	this->ignoreSourceKernel = true;

	this->wTableModel = new WoshTableModel(NULL, this);
	this->wTableModel->setSecurityToken(SecurityManager::generateToken(GuiCache::CurrentUser));
	this->m_ui->tableView->setModel(this->wTableModel);

	this->treeModel = new WoshObjectTreeModel(this->bObjCache.getRootItem(), this);
	this->m_ui->treeView->setModel(this->treeModel);

	connect(&this->bObjCache, SIGNAL(objectAdded(BuildingObjectGraphicsItem*)), this, SLOT(objectAdded(BuildingObjectGraphicsItem*)));
	connect(&this->bObjCache, SIGNAL(objectUpdated(BuildingObjectGraphicsItem*)), this, SLOT(objectUpdated(BuildingObjectGraphicsItem*)));
	connect(&this->bObjCache, SIGNAL(objectEvent(BuildingObjectGraphicsItem*, Message*)), this, SLOT(objectEvent(BuildingObjectGraphicsItem*, Message*)));


	// RENDERER
	this->m_ui->graphicsView->setRenderHint(QPainter::Antialiasing, false);
	this->m_ui->graphicsView->setDragMode(QGraphicsView::RubberBandDrag);
	this->m_ui->graphicsView->setOptimizationFlags(QGraphicsView::DontSavePainterState);
	this->m_ui->graphicsView->setViewportUpdateMode(QGraphicsView::SmartViewportUpdate);
	this->m_ui->graphicsView->setAcceptDrops(true);

	if ( this->m_ui->graphicsView->scene() == NULL) {
		QGraphicsScene* scene = new QGraphicsScene();
		this->m_ui->graphicsView->setScene(scene );
	 }
	QTimer* timer = new QTimer(this);
	QObject::connect(timer, SIGNAL(timeout()), this->m_ui->graphicsView->scene(), SLOT(advance()));
	timer->start(1000 / 5);

	connect(this->m_ui->btnCanvasZoomIn, SIGNAL(clicked()), this, SLOT(gArea_zoomIn()));
	connect(this->m_ui->btnCanvasZoomOut, SIGNAL(clicked()), this, SLOT(gArea_zoomOut()));
    connect(this->m_ui->btnCanvasRotateLeft, SIGNAL(clicked()), this, SLOT(gArea_rotateLeft()));
    connect(this->m_ui->btnCanvasRotateRight, SIGNAL(clicked()), this, SLOT(gArea_rotateRight()));
	connect(this->m_ui->btnCanvasReset, SIGNAL(clicked()), this, SLOT(gArea_reset()));
	connect(this->m_ui->zoomSlider, SIGNAL(valueChanged(int)), this, SLOT(gArea_setupMatrix()));
	connect(this->m_ui->rotateSlider, SIGNAL(valueChanged(int)), this, SLOT(gArea_setupMatrix()));

	QMenu* menuOptions = new QMenu(this);
	QMenu* menuItems = new QMenu(menuOptions);
	menuOptions->addAction( tr("Save Image..") );
	menuOptions->addMenu(menuItems);
	menuItems->setTitle( tr("Set Items") );
	this->actItemsMoveable = new QAction(tr("Moveable"), menuItems);
	this->actItemsMoveable->setCheckable(true);
	menuItems->addAction( this->actItemsMoveable );
	this->actItemsVisible = new QAction(tr("Visible"), menuItems);
	menuItems->addAction( this->actItemsVisible );
	this->m_ui->btnCanvasOptions->setMenu(menuOptions);
	connect(menuOptions, SIGNAL(triggered(QAction*)), this, SLOT(gArea_option_triggered(QAction*)));

	connect(this->m_ui->cmbHouse, SIGNAL(activated(const QString&)), this, SLOT(render_item(const QString&)));
	connect(this->m_ui->cmbFloor, SIGNAL(activated(const QString&)), this, SLOT(render_item(const QString&)));
	connect(this->m_ui->cmbRoom, SIGNAL(activated(const QString&)), this, SLOT(render_item(const QString&)));

	connect(this->m_ui->wUpBarBuildingObjects, SIGNAL(updateClicked()), this, SLOT(refreshAll()));

	connect(this->m_ui->btnReload, SIGNAL(clicked()), this, SLOT(methodReload()));
	connect(this->m_ui->btnRefresh, SIGNAL(clicked()), this, SLOT(refreshAll()));


	// register to BUS
	BusBuilding.setMessageHandler(this);
	WRESULT bus_connected = BusBuilding.connect( _Bus_Building, true );
	(void)bus_connected;
 }

FormBuildingManager::~FormBuildingManager()
{
	delete this->m_ui;
}

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormBuildingManager::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormBuildingManager::refreshAll()
 {
	this->setWindowTitle( QString("Building Manager [%1]").arg(this->getTargetURI().getKernelName().c_str()) );

	executeRemoteMethod( _METHOD_List );
	this->m_ui->wUpBarBuildingObjects->sayRetrieving();
	this->m_ui->progressBar->setValue(1);
 }

void FormBuildingManager::reDrawAll()
 {
	gArea_reset();
	this->m_ui->graphicsView->scene()->update();
 }

WRESULT FormBuildingManager::methodReload() {
	executeRemoteMethod( _BuildingManager_METHOD_reload );
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////// CANVAS

void FormBuildingManager::gArea_setupMatrix()
 {
	qreal scale = qPow(qreal(2), (this->m_ui->zoomSlider->value() - 250) / qreal(50));
	QMatrix matrix;
	matrix.scale(scale, scale);
	matrix.rotate(this->m_ui->rotateSlider->value());
	this->m_ui->graphicsView->setMatrix(matrix);
 }

void FormBuildingManager::gArea_zoomIn() {
	this->m_ui->graphicsView->scale(1.2, 1.2);
 }
void FormBuildingManager::gArea_zoomOut() {
	this->m_ui->graphicsView->scale(1 / 1.2, 1 / 1.2);
 }
void FormBuildingManager::gArea_rotateLeft() {
	this->m_ui->rotateSlider->setValue(this->m_ui->rotateSlider->value() - 10);
 }
void FormBuildingManager::gArea_rotateRight() {
    this->m_ui->rotateSlider->setValue(this->m_ui->rotateSlider->value() + 10);
 }

void FormBuildingManager::gArea_reset()
 {
	this->m_ui->zoomSlider->setValue(250);
	this->m_ui->rotateSlider->setValue(0);
	gArea_setupMatrix();
	this->m_ui->graphicsView->ensureVisible(QRectF(0, 0, 0, 0));
	this->m_ui->graphicsView->scene()->update();
 }

void FormBuildingManager::gArea_save()
 {
	QPixmap pixmap;
	QPainter painter(&pixmap);
	painter.setRenderHint(QPainter::Antialiasing);
	this->m_ui->graphicsView->render(&painter);
	painter.end();
	pixmap.save("scene.png");
 }

void FormBuildingManager::gArea_option_triggered( QAction* action )
 {
	if ( action == NULL ) return;
	if ( action->text() == tr("Save Image..") ) {
		gArea_save();
	 }
 	else if ( action == this->actItemsVisible ) {
		QGraphicsScene* scene = this->m_ui->graphicsView->scene();
		QList<QGraphicsItem*> its = scene->items();
		for(int i=0; i<its.size(); i++)
			its.at(i)->setVisible(true);
	 }
 	else if ( action == this->actItemsMoveable ) {
		QGraphicsScene* scene = this->m_ui->graphicsView->scene();
		QList<QGraphicsItem*> its = scene->items();
		for(int i=0; i<its.size(); i++)
			its.at(i)->setFlag(QGraphicsItem::ItemIsMovable, true);
//			its.at(0)->setFlag(QGraphicsItem::ItemIsMovable, action->isChecked());
	 }

 }

////////////////////////////////////////////////////////////////////////////////////////////////////// CANVAS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormBuildingManager::render_item( const QString& text )
{
	BuildingObjectGraphicsItem* item = this->bObjCache.getItems().find(text.toLatin1().data());
	if ( text == "All Rooms" )
		item = this->bObjCache.getItems().find( this->m_ui->cmbFloor->currentText().toLatin1().data() );

	if ( item == NULL ) {
		QMessageBox::warning(this, "Error", "Object not found" );
		return;
	 }
	QList<BuildingObjectGraphicsItem*> items = this->bObjCache.getItemsByType( item->getRemoteType() );

	for(int i=0; i<items.size(); i++) {
		if ( items.at(i) == NULL ) continue;
		BuildingObjectGraphicsItem* cItem = items.at(i);
		if ( cItem == item ) {
			item->setVisible(true);
			item->update();
			continue;
		 }
		cItem->setVisible(false);
	 }
	this->m_ui->graphicsView->scene()->update();
 }


void FormBuildingManager::objectAdded( BuildingObjectGraphicsItem* item )
 {
	if ( item == NULL ) return;
	QGraphicsScene* scene = this->m_ui->graphicsView->scene();
	scene->addItem(item);
 }

void FormBuildingManager::objectUpdated( BuildingObjectGraphicsItem* item )
 {
	QGraphicsScene* scene = this->m_ui->graphicsView->scene();
	scene->update();

	Request* request_Geometry = new Request( _METHOD_ListProperties );
	Message* msgRequest = new Message(request_Geometry);
	msgRequest->setSource( this->getWidgetURI() );
	msgRequest->setDestination( item->getRemoteURI() );
	msgRequest->setDestinationBus( _Bus_Building );
	msgRequest->setSecurityToken( SecurityManager::generateToken(GuiCache::CurrentUser) );
	BusGui.postMessage(msgRequest);
 }

void FormBuildingManager::objectEvent( BuildingObjectGraphicsItem* item, Message* message )
 {
	message->setSource( this->getWidgetURI() );
	message->setDestination( item->getRemoteURI() );
	message->setDestinationBus( _Bus_Building );
	message->setSecurityToken( SecurityManager::generateToken(GuiCache::CurrentUser) );
	BusGui.postMessage(message);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormBuildingManager::busMessage( const Message& message, const Bus* source )
 {
	(void)source;
 	if ( message.isEmpty() ) return;
	if ( !this->ignoreSourceKernel ) {
		if ( message.getSourceKernel() != this->targetURI.getKernelName() )
			return;
	 }

	if ( message.getContent()->isResponse() ) {
		const wosh::Response* response = message.getContent()->asResponse();
		if ( response->getMethod() == _BuildingManager_METHOD_reload ) {
			QMessageBox::information(this, this->windowTitle(), "RELOAD DB - Done" );
			return;
		 }
		if ( !response->hasData() )
			return;
		if ( response->getMethod() == _METHOD_List ) {
			if ( message.getSource().getName() != "Building" && message.getSource().getPath().empty() ) {
				return;
			 }
			if ( response->getData()->isClass(_Table_TYPE) ) {
				const wosh::Table* bobjTable = dynamic_cast<const wosh::Table*>(response->getData());
				this->wTableModel->loadTable(bobjTable);
				this->m_ui->tableView->clearSelection();
				this->m_ui->tableView->resizeColumnsToContents();

				this->m_ui->wUpBarBuildingObjects->sayReady();

				this->bObjCache.parseObjects( bobjTable, message );

				this->treeModel->updated();
				this->m_ui->progressBar->setValue( this->bObjCache.getItems().size() );

				this->m_ui->cmbHouse->clear();
				this->m_ui->cmbHouse->addItems( this->bObjCache.getItemsNameByType("wosh::building::House") );
				this->m_ui->cmbFloor->clear();
				this->m_ui->cmbFloor->addItems( this->bObjCache.getItemsNameByType("wosh::building::Floor") );
				this->m_ui->cmbRoom->clear();
				this->m_ui->cmbRoom->addItem("All Rooms");
				this->m_ui->cmbRoom->addItems( this->bObjCache.getItemsNameByType("wosh::building::Room") );
				return;
			 }
		 }
		else if ( response->getMethod() == _METHOD_GetProperties || response->getMethod() == _METHOD_ListProperties ) {
			if ( message.getSource().getPath().empty() || message.getSource().getPath().at(0) != "Building" ) {
				return;
			 }
			this->m_ui->progressBar->setValue( this->m_ui->progressBar->value() + 100/this->bObjCache.getItems().size() );

/*
			if ( response->getData()->isClass(_Table_TYPE) ) {
				const wosh::Table* bobjTable = dynamic_cast<const wosh::Table*>(response->getData());
				this->bObjCache.parseProperties( bobjTable, message );
				if ( this->bObjCache.getItems().size_() > 0 && this->m_ui->progressBar->value() > 0 && this->m_ui->progressBar->value() < 100 )
					this->m_ui->progressBar->setValue( this->m_ui->progressBar->value() + 100/this->bObjCache.getItems().size_() );
				return;
			 }
*/
		 }
	 }

	if ( message.getSource().getPath().empty() || message.getSource().getPath().at(0) != "Building" ) {
		return;
	 }

	this->bObjCache.parseMessage( message );

 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace gui
}; // namespace wosh
