/** @file    FormBundleManager.cpp
 *  @author  Alessandro Polo
 *  @version $Id: FormBundleManager.cpp 1710 2010-01-26 23:24:55Z alex $
 *  @brief
 * File containing methods for the wosh::gui::FormBundleManager class.
 * The header for this class can be found in FormBundleManager.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "FormBundleManager.h"
 #include "ui_FormBundleManager.h"
 #include <core/BundleManager.h>
 #include <core/NEW_List.h>

 #include <QtGui/QInputDialog>
 #include <QtGui/QMessageBox>


using namespace std;
namespace wosh {
 namespace gui {

 WOSH_REGISTER(wosh::gui::FormBundleManager, "wosh::gui::IWidgetGeneric", 1.0, _static_FormBundleManager )
 WOSH_GUI_MAP(wosh::gui::FormBundleManager, "wosh::BundleManager", 0.0, _static_Map_FormBundleManager )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

FormBundleManager::FormBundleManager(QWidget *parent) : IWidgetGeneric(parent)
 {
	this->m_ui = new Ui::FormBundleManager();
	this->m_ui->setupUi(this);

	setupFor(_BundleManager_URI);
	buildMenus();

	// tab Installed Bundles [0]
	this->m_ui->tableView->setDataRequest(_METHOD_List);
	this->m_ui->tableView->setResizeStretch(0);
	this->m_ui->tableView->setContextMenuPolicy(Qt::CustomContextMenu);
	connect(this->m_ui->tableView, SIGNAL(customContextMenuRequested(const QPoint&)),this, SLOT(tableView_ContextMenu(const QPoint&)) );
	connect(this->m_ui->tableView, SIGNAL(updated()), this->m_ui->wUpBarBundles, SLOT(sayReady()));
	connect(this->m_ui->wUpBarBundles, SIGNAL(updateClicked()), this, SLOT(refreshAll()));
	connect(this->m_ui->btnBundleStart, SIGNAL(clicked()), this, SLOT(btnBundleStart_clicked()));
	connect(this->m_ui->btnBundleStop, SIGNAL(clicked()), this, SLOT(btnBundleStop_clicked()));
	connect(this->m_ui->btnBundleLoad, SIGNAL(clicked()), this, SLOT(btnBundleLoad_clicked()));

	// tab Supported Bundles [1]

 }

FormBundleManager::~FormBundleManager() {
	delete this->m_ui;
 }

void FormBundleManager::changeEvent(QEvent *e) {
	QWidget::changeEvent(e);
	if ( e->type() == QEvent::LanguageChange )
		this->m_ui->retranslateUi(this);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormBundleManager::refreshAll() {
	this->setWindowTitle( QString("Bundle Manager [%1]").arg(this->getTargetURI().getKernelName().c_str()) );
	refreshBundles();
	refreshSupported();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormBundleManager::refreshBundles() {
	executeRemoteMethod( _METHOD_List );
	this->m_ui->wUpBarBundles->sayRetrieving();
 }

void FormBundleManager::refreshSupported() {

 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormBundleManager::btnBundleStart_clicked() {
	Variant bundleName = this->m_ui->tableView->getCurrentItemAsVariantAtCol(0);
	NEW_List* args = new NEW_List(2);
	args->push_back( bundleName.clone() );
	args->push_back<Variant>( "start" );
	executeRemoteMethod( _BundleManager_METHOD_control_bundle, args );
 }

void FormBundleManager::btnBundleStop_clicked() {
	Variant bundleName = this->m_ui->tableView->getCurrentItemAsVariantAtCol(0);
	NEW_List* args = new NEW_List(2);
	args->push_back( bundleName.clone() );
	args->push_back<Variant>( "stop" );
	executeRemoteMethod( _BundleManager_METHOD_control_bundle, args );
 }

void FormBundleManager::btnBundleLoad_clicked() {
	bool ok;
	QString bundlestr = QInputDialog::getText(this, tr("Load Bundle.."), tr("Enter Bundle-Type [optional Bundle-Name]:"), QLineEdit::Normal, "", &ok);
	if ( !ok || bundlestr.isEmpty() )
		return;
	QStringList params = bundlestr.split(" ");
	if ( params.empty() ) return;
	string bundle_type = params.at(0).toLatin1().data();
	string bundle_name = "";
	if ( params.count() > 1 )
		bundle_name = params.at(1).toLatin1().data();
	if ( params.count() > 2 )
		QMessageBox::warning(this, tr("Warning"), tr("Only two parameters are supported.") );

	NEW_List* args = NEW_List::createFrom<Variant>(bundle_type, bundle_name);
	executeRemoteMethod( _BundleManager_METHOD_LoadBundle, args );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormBundleManager::buildMenus() {
	this->menuBundleTitle = new QAction(this);
	this->menuBundleTitle->setEnabled(false);
	this->menuBundleCreateGui = new QAction(this);
	this->menuBundleCreateGui->setText(tr("&Create Gui.."));
	this->menuBundle = new QMenu(this);
	this->menuBundle->addAction(this->menuBundleTitle);
	this->menuBundle->addSeparator();
	this->menuBundle->addAction(this->menuBundleCreateGui);
 }

void FormBundleManager::tableView_ContextMenu( const QPoint& pos ) {
	QPoint globalPos = this->mapToGlobal(pos);
	QModelIndex index = this->m_ui->tableView->indexAt(pos);
	if ( !index.isValid() )
		return;
	Variant bundleName = this->m_ui->tableView->getCurrentItemAsVariantAtCol(0);
	Variant bundleType = this->m_ui->tableView->getCurrentItemAsVariantAtCol(1);

	this->menuBundleTitle->setText( bundleName.toString().c_str() );
	this->menuBundleTitle->setIcon( GuiUtilities::createIconFor(bundleType.toString()) );

	QAction* selected = this->menuBundle->exec( globalPos );
	if ( selected == this->menuBundleCreateGui ) {
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormBundleManager::busMessage( const Message& message, const Bus* ) {
	if ( message.isEmpty() || !message.getContent()->isResponse() ) return;
	if ( !isMessageForMeOrFromTarget(message) ) return;
	const wosh::Response* response = message.getContent()->asResponse();

	this->m_ui->tableView->evalResponse(response);

	promptMethodResponse( message, false );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace gui
}; // namespace wosh
