/** @file    FormSecurityManager.cpp
 *  @author  Alessandro Polo
 *  @version $Id: FormSecurityManager.cpp 1708 2010-01-26 22:28:01Z alex $
 *  @brief
 * File containing methods for the wosh::gui::FormSecurityManager class.
 * The header for this class can be found in FormSecurityManager.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "FormSecurityManager.h"
 #include "ui_FormSecurityManager.h"
 #include <core/SecurityManager.h>

 #include <QtGui/QCompleter>


using namespace std;
namespace wosh {
 namespace gui {

 WOSH_REGISTER(wosh::gui::FormSecurityManager, "wosh::gui::IWidgetGeneric", 1.0, _static_FormSecurityManager )
 WOSH_GUI_MAP(wosh::gui::FormSecurityManager, "wosh::SecurityManager", 0.0, _static_Map_FormSecurityManager )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

FormSecurityManager::FormSecurityManager(QWidget *parent) : IWidgetGeneric(parent)
 {
	this->m_ui = new Ui::FormSecurityManager();
	this->m_ui->setupUi(this);

	setupFor(_SecurityManager_URI);

	// tab EventsLog [0]
	this->m_ui->tableViewEventsLog->setDataRequest(_SecurityManager_METHOD_eventslog);
	this->m_ui->tableViewEventsLog->setResizeStretch(1);
	connect(this->m_ui->tableViewEventsLog, SIGNAL(updated()), this->m_ui->wUpBarEventsLog, SLOT(sayReady()));
	connect(this->m_ui->wUpBarEventsLog, SIGNAL(updateClicked()), this, SLOT(refreshEventsLog()));
	this->mapMethodButton(_SecurityManager_METHOD_clear_log, this->m_ui->btnClearLog);

	// tab Fake Event [2]
	QCompleter* completerURI = new QCompleter(this);
	completerURI->setCaseSensitivity(Qt::CaseInsensitive);
	completerURI->setModel( this->m_ui->tableViewEventsLog->getTableModel() );
	completerURI->setCompletionColumn(0);
	this->m_ui->txtEmulateSource->setCompleter(completerURI);
	QCompleter* completerEvent = new QCompleter(this);
	completerEvent->setCaseSensitivity(Qt::CaseInsensitive);
	completerEvent->setModel( this->m_ui->tableViewEventsLog->getTableModel() );
	completerEvent->setCompletionColumn(2);
	this->m_ui->txtEmulateEventName->setCompleter(completerEvent);
	QCompleter* completerData = new QCompleter(this);
	completerData->setCaseSensitivity(Qt::CaseInsensitive);
	completerData->setModel( this->m_ui->tableViewEventsLog->getTableModel() );
	completerData->setCompletionColumn(3);
	this->m_ui->txtEmulateData->setCompleter(completerData);
	connect(this->m_ui->btnEmulateEvent, SIGNAL(clicked()), this, SLOT(btnEmulateEvent_clicked()));

	// tab Configuration [3]
	this->propMapperCfg = new PropertiesMapper(this, true);
	connect(this->propMapperCfg, SIGNAL(send_request(const QString&, ISerializable*)), this, SLOT(executeRemoteMethod(const QString&, ISerializable*)));
	this->propMapperCfg->set(_SecurityManager_KEY_EventLogMaxSize, this->m_ui->spinEventLogSize);
	connect(this->m_ui->wUpBarConfiguration, SIGNAL(updateClicked()), this, SLOT(refreshConfiguration()));

}

FormSecurityManager::~FormSecurityManager() {
	delete this->m_ui;
 }

void FormSecurityManager::changeEvent(QEvent *e){
	QWidget::changeEvent(e);
	if ( e->type() == QEvent::LanguageChange )
		this->m_ui->retranslateUi(this);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormSecurityManager::refreshAll() {
	this->setWindowTitle( QString("Security Manager [%1]").arg(this->getTargetURI().getKernelName().c_str()) );
	refreshEventsLog();
	refreshConfiguration();
 }
void FormSecurityManager::refreshEventsLog() {
	executeRemoteMethod( _SecurityManager_METHOD_eventslog );
	this->m_ui->wUpBarEventsLog->sayRetrieving();
 }
void FormSecurityManager::refreshConfiguration() {
	NEW_List* dlProps = this->propMapperCfg->createKeysNEW_List();
	executeRemoteMethod( _METHOD_GetProperties, dlProps );
	this->m_ui->wUpBarConfiguration->sayRetrieving();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormSecurityManager::btnEmulateEvent_clicked() {
	NEW_List* dl = new NEW_List();
	URI uri_src;
	uri_src.fromString(this->m_ui->txtEmulateSource->text().toLatin1().data());
	dl->push_back<Variant>( uri_src.getName() );
	dl->push_back<Variant>( this->m_ui->txtEmulateEventName->text().toLatin1().data() );
	dl->push_back<Variant>( this->m_ui->txtEmulateData->text().toLatin1().data() );
//	executeRemoteMethod( _SecurityManager_METHOD_fake_event, dl );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormSecurityManager::busMessage( const Message& message, const Bus* ) {
	if ( message.isEmpty() || !message.getContent()->isResponse() ) return;
	if ( !isMessageForMeOrFromTarget(message) ) return;
	const wosh::Response* response = message.getContent()->asResponse();

	this->m_ui->tableViewEventsLog->evalResponse(response);

	if ( this->propMapperCfg->evalResponse(response) )
		this->m_ui->wUpBarConfiguration->sayReady();

	if ( response->getMethod() == _SecurityManager_METHOD_clear_log && WSUCCEEDED(response->getReturnValue()) )
		refreshEventsLog();

	promptMethodResponse(message);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace gui
}; // namespace wosh
