/** @file    SocketTcpQt.cpp
 *  @author  Alessandro Polo
 *  @version $Id: SocketTcpQt.cpp 1920 2010-02-22 13:42:51Z alex $
 *  @brief
 * File containing methods for the wosh::network::qt::SocketTcpQt class.
 * The header for this class can be found in SocketTcpQt.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include <framework/network/Qt/SocketTcpQt.h>
 #include <framework/network/Qt/ServerTcpQt.h>

 #include <core/ObjectAllocator.h>
 #include <framework/network/NetworkUtilities.h>
 #include <core/Utilities.h>


using namespace std;
namespace wosh {
 namespace network {
  namespace qt {

  WOSH_REGISTER(wosh::network::qt::SocketTcpQt, "wosh::network::SocketTcp", 1.0, _static_SocketTcpQt )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

SocketTcpQt::SocketTcpQt() : QTcpSocket(), wosh::network::SocketTcp()
 {
#ifdef _SocketTcpQt_DEBUG_ENABLED
	Log.setIndent( 5 );
	Log.setContext( "SocketTcpQt" );
	Log.setLevel( LOG_INFO );
#endif
	QObject::connect(this, SIGNAL(readyRead()), this, SLOT(readClient()));
	QObject::connect(this, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(socket_error(QAbstractSocket::SocketError)));

 }

SocketTcpQt::~SocketTcpQt() {
#ifdef _SocketTcpQt_DEBUG_ENABLED
	Log(LOG_VERBOSE, ":~SocketTcpQt() Destroying.." );
#endif
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT SocketTcpQt::connect( unsigned long timeout )
 {
	if ( this->state() == QAbstractSocket::ConnectedState )
		return WRET_ERR_PARAM;
	QHostAddress address;
	address.setAddress( QString(this->address.c_str()) );
	this->connectToHost ( address, this->port );
	bool connected = this->waitForConnected(timeout);
	if ( !connected ) {
#ifdef _SocketTcpQt_DEBUG_ENABLED
		Log(LOG_CRITICAL, ":connect() FAILED [%s]", this->errorString().toLatin1().data() );
#endif
		return WRET_ERR_INTERNAL;
	 }
	return WRET_OK;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT SocketTcpQt::disconnect( unsigned long timeout ) {

	this->disconnectFromHost();
	if ( this->state() != QAbstractSocket::UnconnectedState ) {
		bool disconnected = this->waitForDisconnected(timeout);
		if ( !disconnected ) {
#ifdef _SocketTcpQt_DEBUG_ENABLED
			Log(LOG_CRITICAL, ":connect() FAILED [%s]", this->errorString().toLatin1().data() );
#endif
			return WRET_ERR_INTERNAL;
		 }
	}
	this->close();
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

int SocketTcpQt::sendDatagram( const char* data, int size )
 {
	if ( data == NULL ) return -1;
#ifdef _SocketTcpQt_DEBUG_ENABLED
	Log(LOG_VERBOSE, ":sendDatagram() %i bytes..", size );
#endif

	size = this->write(data, size);
	this->flush();

	return size;
 }

void SocketTcpQt::readClient()
{
//	QDataStream in(this);
//    in.setVersion(QDataStream::Qt_4_0);
//	in.readRawData

	quint64 ba = bytesAvailable();
#ifdef _SocketTcpQt_DEBUG_ENABLED
	Log(LOG_VERBOSE, ":readClient() %i bytes..", ba );
#endif

	if ( this->listener != NULL )
		this->listener->tcpDataAvailable(ba, this);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool SocketTcpQt::waitWritten( unsigned long timeout )
 {
	return this->waitForBytesWritten(timeout);
 }

bool SocketTcpQt::waitResponse( char* data, int max_size, long& size, unsigned long timeout )
 {
	size = 0;
	bool done = false;
	done = this->waitForReadyRead(timeout);
    if ( !done )
		return false;

	size = this->readData(data, max_size);
	///@bug here
	size = strlen(data);
	return true;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT SocketTcpQt::setSocket( int socket_descriptor )
 {
	 bool done = QTcpSocket::setSocketDescriptor(socket_descriptor);
	 if ( !done ) {
#ifdef _SocketTcpQt_DEBUG_ENABLED
		 Log(LOG_CRITICAL, ":setSocket(%i) : FAILED!", socket_descriptor );
#endif
		return WRET_ERR_INTERNAL;
	  }

#ifdef _SocketTcpQt_DEBUG_ENABLED
	Log(LOG_INFO, ":setSocket(%i) : Applied", socket_descriptor );
#endif

	QHostAddress add = this->peerAddress();
	this->address = std::string(add.toString().toAscii().data());
	this->port = this->peerPort();
	this->address_port = Utilities::format("%s:%i", this->address.c_str(), this->port);

	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

std::string SocketTcpQt::getPeerAddressPort() const {
	return wosh::Utilities::format( "%s:%i", this->peerAddress().toString().toLatin1().data(), this->peerPort() );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SocketTcpQt::socket_error(QAbstractSocket::SocketError socketError)
 {
	switch (socketError) {
		case QAbstractSocket::RemoteHostClosedError:
#ifdef _SocketTcpQt_DEBUG_ENABLED
			Log(LOG_WARNING, ":socket_error() Socket Error 'Remote Host closed Connection'!" );
#endif
			if ( this->listener != NULL )
				this->listener->tcpSocketError(_SocketTcp_ERROR_RemoteHostClosedError, "Remote Host closed Connection", this);
			break;
		case QAbstractSocket::HostNotFoundError:
#ifdef _SocketTcpQt_DEBUG_ENABLED
			Log(LOG_CRITICAL, ":socket_error() Socket Error 'Host not Found' [%s:%i]", this->address.c_str(), this->port );
#endif
			if ( this->listener != NULL )
				this->listener->tcpSocketError(_SocketTcp_ERROR_HostNotFoundError, "Host not Found", this);
			break;
		case QAbstractSocket::ConnectionRefusedError:
#ifdef _SocketTcpQt_DEBUG_ENABLED
			Log(LOG_CRITICAL, ":socket_error() Socket Error 'Connection REFUSED'!" );
#endif
			if ( this->listener != NULL )
				this->listener->tcpSocketError(_SocketTcp_ERROR_ConnectionRefusedError, "Connection REFUSED", this);
			break;
		default:
#ifdef _SocketTcpQt_DEBUG_ENABLED
			Log(LOG_WARNING, ":socket_error() Socket Error '%s'!", this->errorString().toLatin1().data() );
#endif
			if ( this->listener != NULL )
				this->listener->tcpSocketError(0, this->errorString().toLatin1().data(), this);
	}

 }


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

  }; // namespace qt
 }; // namespace network
}; // namespace wosh
