/**
 * @class   wosh::network::SocketUdpQt
 * @brief   The class provides a UDP socket.
 *          UDP (user datagram protocol) is a lightweight, unreliable,
 *          datagram-oriented, connectionless protocol.
 *
 ****************************************************************************
 * @version $Id: SocketUdpQt.h 1917 2010-02-22 12:05:14Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Network_Qt_SocketUdpQt_H__
 #define __WOSH_Network_Qt_SocketUdpQt_H__

 #include <framework/network/SocketUdp.h>

 #include <QThread>
 #include <QtNetwork/QUdpSocket>

namespace wosh {
 namespace network {
  namespace qt {


class SocketUdpQt :		private QThread,
						public wosh::network::SocketUdp
 {
	Q_OBJECT
	WOSH_CLASSNAME(wosh::network::qt::SocketUdpQt)
	WOSH_CLASS_ABSTRACT(false)

	public:
		/**
		 * @brief   Empty constructor, init vars.
		 */
		SocketUdpQt();
		/**
		 * @brief   Empty destructor, may (synch) unbind socket (stop internal thread)
		 */
		virtual ~SocketUdpQt();


	public:

		/**
		 * @brief  Bind the UDP socket.
		 * @return WRET_ERR_WRONG_STATE when already binded; WRET_ERR_INTERNAL if
		 *         the binding failed (or timed-out); WRET_OK else
		 * @note   this call will start synchronously the internal thread
		 * @see    setAddressPort()
		 */
		virtual WRESULT bindSocket( unsigned long timeout = LIMIT_ULONG_MAX );

		/**
		 * @brief  UnBind the UDP socket.
		 * @return WRET_ERR_WRONG_STATE when not binded; WRET_ERR_INTERNAL if
		 *         the binding failed (or timed-out); WRET_OK else
		 * @note   this call will stop synchronously the internal thread
		 * @see    bindSocket()
		 */
		virtual WRESULT unbindSocket( unsigned long timeout = LIMIT_ULONG_MAX );

		/**
		 * @brief  Send an UDP datagram to remote host.
		 * @param [in] destination target address:port (such as '192.168.0.255:8787')
		 * @param [in] data the block of bytes to be sent (no marshalling)
		 * @param [in] size number of bytes of 'data' argument
		 * @return number of sent bytes
		 */
		virtual int sendDatagram( const std::string& destination, const char* data, int size );


		virtual inline bool isBinded() const			{ return (isRunning() && this->ready); }


/** @name Internal vars and methods
 * @{
 ******************************************************************************/
	protected:
		QUdpSocket* udpSocket;		///< real socket

		bool ready;
		bool failure;

		void run();

	private slots:

		/**
		 * @brief  Event handler (slot) called by QT QUdpSocket when one or more datagram are ready.
		 *         The method will iterate for each datagram and raise ISocketUdpQtListener callbacks.
		 * @see    ISocketUdpQtListener::processIncomingDatagram( char * data, int size)
		 * @note   if the parent was not set (NULL), datagrams will be ignored.
		 */
		void processPendingDatagrams();

//@}

}; // class def

  }; // namespace qt
 }; // namespace network
}; // namespace wosh

#endif //__WOSH_Network_Qt_SocketUdpQt_H__
