/**
 * @class   wosh::network::ServerTcp
 * @brief   The class provides a TCP-based server, it handles
 *          incoming TCP connections.
 *
 ****************************************************************************
 * @version $Id: ServerTcp.h 2082 2010-03-11 06:26:42Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Network_ServerTcp_H__
 #define __WOSH_Network_ServerTcp_H__

 #include <core/Object.h>
 #include <core/LogContext.h>
 #include <framework/network/NetworkUtilities.h>


namespace wosh {
 namespace network {

 #define _ServerTcp_TYPE				"wosh::network::ServerTcp"

 #define _ServerTcp_DEBUG_ENABLED

class ServerTcp;
class SocketTcp;

class IServerTcpListener {
	public:
		virtual void serverTcp_incomingConnection( int socket_id, wosh::network::ServerTcp* source ) = 0;

	public:
		virtual ~IServerTcpListener () { }

}; // class def



class ServerTcp : public IReflection
 {
	WOSH_CLASSNAME(wosh::network::ServerTcp)
	WOSH_CLASS_ABSTRACT(true)

	public:

		ServerTcp()  {
			this->listener = NULL;

			this->address = "";
			this->port = 0;
			this->address_port = "";
		 }

		virtual ~ServerTcp() { }

	public:

		virtual bool isListening() const = 0;


		virtual WRESULT bindSocket( unsigned long timeout = LIMIT_ULONG_MAX ) = 0;

		virtual WRESULT unbindSocket( unsigned long timeout = LIMIT_ULONG_MAX ) = 0;

		virtual WRESULT bindSocket( const std::string& address_port, unsigned long timeout = LIMIT_ULONG_MAX ) {
			if ( !setAddressPort(address_port) ) {
#ifdef _ServerTcp_DEBUG_ENABLED
				Log(LOG_CRITICAL, ":bindSocket(%s) FAILED Setting address", address_port.c_str() );
#endif
				return WRET_ERR_PARAM;
			 }
			return bindSocket(timeout);
		 }

		virtual WRESULT bindSocket( const std::string& address, int port, unsigned long timeout = LIMIT_ULONG_MAX ) {
			if ( !setAddressPort(address, port) ) {
#ifdef _ServerTcp_DEBUG_ENABLED
				Log(LOG_CRITICAL, ":bindSocket(%s:%i) FAILED Setting address", address.c_str(), port );
#endif
				return WRET_ERR_PARAM;
			 }
			return bindSocket(timeout);
		 }


	public:

		bool setAddressPort( const std::string& add_value, int port_value ) {
			if ( isListening() ) {
		#ifdef _ServerTcp_DEBUG_ENABLED
				Log(LOG_CRITICAL, ":setAddressPort(%s, %i) Not permitted while Binded!", add_value.c_str(), port_value );
		#endif
				return false;
			 }

			this->address = add_value;
			this->port = port_value;
			this->address_port = Utilities::format("%s:%i", this->address.c_str(), this->port);
#ifdef _ServerTcp_DEBUG_ENABLED
			Log(LOG_INFO, ":setAddressPort('%s')", this->address_port.c_str() );
#endif
			return true;
		 }

		bool setAddressPort( const std::string& value ) {
			if ( isListening() ) {
#ifdef _ServerTcp_DEBUG_ENABLED
				Log(LOG_CRITICAL, ":setAddressPort(%s) Not permitted while Binded!", value.c_str() );
#endif
				return false;
			 }

			this->address = wosh::network::NetworkUtilities::getAddressFromDestination(value);
			this->port = wosh::network::NetworkUtilities::getPortFromDestination(value);
			this->address_port = value;
#ifdef _ServerTcp_DEBUG_ENABLED
			Log(LOG_INFO, ":setAddressPort('%s')", this->address_port.c_str() );
#endif
			return true;
		 }

		inline const std::string& getAddress() const				{ return this->address; }
		inline int getPort() const									{ return this->port; }
		inline const std::string& getAddressPort() const 			{ return this->address_port; }

		wosh::network::IServerTcpListener* getListener() 							{ return this->listener; }
		void setListener( wosh::network::IServerTcpListener* tcp_listener ) 		{ this->listener = tcp_listener; }

	protected:

#ifdef _ServerTcp_DEBUG_ENABLED
		mutable wosh::LogContext Log;
#endif

		std::string address;
		int port;
		std::string address_port; // shadow var

		wosh::network::IServerTcpListener* listener;

		friend class wosh::network::SocketTcp;


}; // class def

 }; // namespace network
}; // namespace wosh

#endif //__WOSH_Network_ServerTcp_H__
