/** @file    persistence/Qt/EncoderReaderXml.cpp
 *  @author  Alessandro Polo
 *  @version $Id: EncoderReaderXml.cpp 1917 2010-02-22 12:05:14Z alex $
 *  @brief
 * File containing methods for the wosh::persistence::EncoderReaderXml class.
 * The header for this class can be found in EncoderReaderXml.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include <framework/persistence/Qt/EncoderReaderXml.h>

 #include <QtXml>

using namespace std;
namespace wosh {
 namespace persistence {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

EncoderReaderXml::EncoderReaderXml() : IEncoderReader()
 {
	this->doc = NULL;
	this->currentNode = NULL;

	this->explicitTypes = true;

	Log.setContext( "EncoderWriterXml" );
	Log.setIndent( 3 );
	Log.setLevel( LOG_INFO );
Log.setLevel( LOG_VERBOSE );

 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT EncoderReaderXml::open( const std::string& filename )
 {
	this->doc = new QDomDocument("none");
	this->currentNode = NULL;
	this->docFilename = filename;

	QFile file(filename.c_str());
	if ( !file.open(QIODevice::ReadOnly) ) {
		Log(LOG_CRITICAL, ":loadXmlDocument() FAILED : Can't open file: %s", filename.c_str() );
		return WRET_ERR_PARAM;
	 }
	if ( !doc->setContent(&file) ) {
		file.close();
		Log(LOG_CRITICAL, ":loadXmlDocument() FAILED : Can't set contents from: %s", filename.c_str() );
		return WRET_ERR_PARAM;
	 }
	file.close();

	QDomElement woshNode = doc->documentElement();
	if ( woshNode.isNull() ) {
		Log(LOG_WARNING, ":loadXmlDocument() FAILED : No root node in %s", filename.c_str() );
		return WRET_ERR_INTERNAL;
	 }

	this->currentNode = new QDomElement();
	*this->currentNode = woshNode;

	return WRET_OK;
 }

WRESULT EncoderReaderXml::close()
 {
	if ( this->doc == NULL )
		return WRET_ERR_ILLEGAL_USE;

	this->currentNode = NULL;

	delete this->doc; this->doc = NULL;

	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////


WRESULT EncoderReaderXml::pop( ISerializable* object, const std::string& field_name, SerializeStrategy strategy )
 {
	if ( object == NULL ) return WRET_ERR_PARAM;
	if ( this->currentNode == NULL )
		return WRET_ERR_INTERNAL;

	switch (strategy) {
		case SERIALIZE_NODE: {

				QDomElement targetNode = this->currentNode->namedItem(field_name.c_str()).toElement();
				if ( targetNode.isNull() ) {
					targetNode = this->currentNode->nextSibling().toElement();
					
				 }

				QDomElement previousCurrentNode = *this->currentNode;
				// recursive
				*this->currentNode = targetNode;
				WRESULT ret = object->retrieve(this);
				*this->currentNode = previousCurrentNode;
				
				break;
			 }
		case SERIALIZE_FIELD: {
				// read from attributes
				if ( !field_name.empty() ) {
					if ( this->currentNode->hasAttribute(field_name.c_str()) ) {
						// NO:
						//object->retrieve(this);
					 }
					else
						return WRET_ERR_PARAM;
				 }
				return WRET_ERR_PARAM;
				break;
			 }
		case SERIALIZE_DEFAULT:
		default: {

				break;
			 }
	 }

	return WRET_OK;
 }



//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT EncoderReaderXml::pop( std::string& obj, const std::string& field_name, SerializeStrategy strategy )
 {
	if ( this->currentNode == NULL )
		return WRET_ERR_INTERNAL;

	switch (strategy) {
		case SERIALIZE_NODE: {
				// create new (string) node

				QDomElement targetNode = this->currentNode->namedItem(field_name.c_str()).toElement();
				if ( targetNode.isNull() )
					return WRET_ERR_PARAM;

				obj = string( targetNode.text().toLatin1().data() );

				break;
			 }
		case SERIALIZE_FIELD:
		case SERIALIZE_DEFAULT:
		default: {
				// read from attributes
				if ( !field_name.empty() ) {
					if ( this->currentNode->hasAttribute(field_name.c_str()) )
						obj = string( this->currentNode->attribute(field_name.c_str()).toLatin1().data() );
					else
						return WRET_ERR_PARAM;
				 }
				return WRET_ERR_PARAM;
				break;
			 }
	 }

	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT EncoderReaderXml::pop( int& obj, const std::string& field_name, SerializeStrategy strategy )
 {
	if ( this->currentNode == NULL )
		return WRET_ERR_INTERNAL;

	switch (strategy) {
		case SERIALIZE_NODE: {
				// create new node

				QDomElement targetNode = this->currentNode->namedItem(field_name.c_str()).toElement();
				if ( targetNode.isNull() )
					return WRET_ERR_PARAM;

				obj = targetNode.text().toInt();

				break;
			 }
		case SERIALIZE_FIELD:
		case SERIALIZE_DEFAULT:
		default: {
				// read from attributes
				if ( !field_name.empty() ) {
					if ( this->currentNode->hasAttribute(field_name.c_str()) )
						obj = this->currentNode->attribute(field_name.c_str()).toInt();
					else
						return WRET_ERR_PARAM;
				 }
				break;
			 }
	 }

	return WRET_OK;
 }


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace persistence
}; // namespace wosh
