/**
 * @class   wosh::security::AlarmEvent
 * @brief   The class defines a generic alarm-system event.
 *          AlarmEvent are generated by AlarmTrigger and saved into AlarmState
 *          class.
 *
 * @see     wosh::security::AlarmTrigger
 ****************************************************************************
 * @version $Id: AlarmEvent.h 2057 2010-03-08 21:32:15Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Security_AlarmEvent_H__
 #define __WOSH_Security_AlarmEvent_H__

 #include <woshDefs.h>
 #include <core/Serialization.h>
 #include <core/Utilities.h>
 #include <core/URI.h>


namespace wosh {
 namespace security {

 #define _AlarmEvent_TYPE							"wosh::security::AlarmEvent"


class AlarmEvent : public virtual ISerializable
 {
	WOSH_SERIALIZABLE(wosh::security::AlarmEvent)
	WOSH_COMPARE_RTTI(wosh::security::AlarmEvent)

	public:

		AlarmEvent() {
			this->uid = Utilities::randomLong();
			this->level = 0;
			this->description = "";
			this->timestamp = Utilities::std_time();
		 }
		/**
		 * @brief  Copy constructor.
		 */
		AlarmEvent( const AlarmEvent& m ) {
			*this = m;
		 }

		virtual ~AlarmEvent() { }


/** @name Setters
 * @{
 ******************************************************************************/
	public:

		void setAlarmLevel(long value)						{ this->level = value; }

		void setDescription( const std::string& value )		{ this->description = value; }

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:

		long getID() const									{ return this->uid; }

		std::string& getDescription()						{ return this->description; }
		const std::string& getDescription() const			{ return this->description; }

		long getTimeStamp() const							{ return this->timestamp; }

		long getAlarmLevel() const							{ return this->level; }

//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:

		virtual AlarmEvent& operator=(const AlarmEvent& m)
		 {
			if (this == &m) return *this; // same object?

			this->uid = m.uid;
			this->level = m.level;
			this->description = m.description;
			this->timestamp = m.timestamp;

			return *this;
		 }

		virtual bool operator==(const AlarmEvent &other) const
		 {
			if ( this->level != other.level )
				return false;

			if ( this->description != other.description )
				return false;

			return true;
		 }

		virtual bool operator!=(const AlarmEvent &other) const	{ return !(*this == other);  }

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const				{ return ""; }

		virtual WRESULT store( IEncoderWriter* serializer ) const {
			if ( serializer == NULL ) return WRET_ERR_PARAM;
			WRESULT ret = WRET_OK;
			return ret;
		 }
		virtual WRESULT retrieve( IEncoderReader* deserializer ) {
			if ( deserializer == NULL ) return WRET_ERR_PARAM;
			WRESULT ret = WRET_OK;
			return ret;
		 }
//@}
	public:

		long uid;				///< unique id of the event
		long level;				///< alarm level of the event

		long timestamp;			///< timestamp of the event

URI source;

		std::string description;		///< description of the event



}; // class def

 }; // namespace security
}; // namespace wosh

#endif //__WOSH_Security_AlarmEvent_H__
