/**
 * @file    test_Properties.cpp
 * @brief   Test classes: wosh::Property, wosh::PropertiesProvider
 *          Indirectly tests also wosh::Data
 *
 * @version $Id: test_Properties.cpp 2067 2010-03-09 23:40:30Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/

 #include "test_Properties.h"

 #include <woshDefs.h>
 #include <core/Variant.h>
 #include <core/Property.h>
 #include <core/PropertiesProvider.h>
 #include <core/LogContext.h>
 #include <core/Utilities.h>


int test_Properties()
 {
	Log::log( "test_Properties() : Starting TEST" );

	test_Property();

	test_PropertiesProvider();

	Log::log( "test_Properties() : Finished TEST" );
	return 0;
 }


void test_Property()
 {
	Log::log( " test_Property() : Starting TEST for class 'Property'" );

	Property prop;
	Property* prop_clone = NULL;

	ASSERT( prop.isEmpty() )

	prop_clone = prop.clone();
	ASSERT( prop_clone->isEmpty() )
	ASSERT( prop == *prop_clone )

	prop.setKey( "string_property" );
	prop.setValue( "a string" );
	prop.setAccess( Property::READ_WRITE );

	ASSERT( prop.getKey() == "string_property" )
	ASSERT( prop.getValue().isString() )
	ASSERT( prop.getValue() == Data("a string") )
	ASSERT( prop.getValue() == "a string" )

	ASSERT( prop != *prop_clone )
	*prop_clone = prop;
	ASSERT( prop == *prop_clone )

	prop_clone->setKey( "double_property" );
	prop_clone->setValue( 15.5 );
	prop_clone->setAccess( Property::READ_ONLY );

	ASSERT( prop != *prop_clone )

	ASSERT( prop_clone->getKey() == "double_property" )
	ASSERT( prop_clone->getValue().isDouble() )
	ASSERT( prop_clone->getValue().getData() == "15.5" )
	ASSERT( prop_clone->getValue() == Data( 15.5 ) )

	delete prop_clone;

	Log::log( " test_Property() : Finished TEST for class 'Property'" );
 }


void test_PropertiesProvider()
 {
	Log::log( "test_PropertiesProvider() : Starting TEST for class 'PropertiesProvider'" );

	PropertiesProvider pp;

	pp.add( Property("first_property", "its value as string") );
	ASSERT( pp.size() == 1 )

	Property* prop_copy1 = pp.getProperty( "first_property" );
	ASSERT( prop_copy1 != NULL )
	ASSERT( prop_copy1->getKey() == "first_property" )
	ASSERT( prop_copy1->getValue() == "its value as string" )
	if ( prop_copy1 != NULL )
		delete prop_copy1;

	ASSERT( pp.getPropertyValue("not_found_property").isEmpty() )
	ASSERT( pp.getPropertyValue("first_property").isString() )

	WRESULT removed_no = pp.remove( "not_found_property" );
	ASSERT( removed_no == WRET_ERR_PARAM )
	ASSERT( pp.size() == 1 )

	WRESULT added_ok = pp.add( Property("second_property", 15.65 ) );
	ASSERT( added_ok == WRET_OK )
	ASSERT( pp.size() == 2 )

	WRESULT added_no = pp.add( Property("second_property", 23.65 ) );
	ASSERT( added_no == WRET_ERR_PARAM )
	ASSERT( pp.size() == 2 )

	WRESULT removed_ok = pp.remove( "first_property" );
	ASSERT( removed_ok == WRET_OK )
	ASSERT( pp.size() == 1 )

	Property* prop_copy = pp.getProperty( "second_property" );

	ASSERT( prop_copy != NULL )
	ASSERT( prop_copy->getKey() == "second_property" )
	ASSERT( prop_copy->getValue() == 15.65 )
	if ( prop_copy != NULL )
		delete prop_copy;

	pp.clear();
	ASSERT( pp.isEmpty() )

	pp.setProperty( "intvalue", 44 );
	ASSERT( pp.size() == 1 )
	ASSERT( pp.getPropertyValue("intvalue") == Data(44) )

	pp.setProperty( "intvalue", 54, Property::READ_ONLY );
	ASSERT( pp.size() == 1 )
	ASSERT( pp.getPropertyValue("intvalue") == 54 )

	ASSERT( pp.find( "not_found" ).isEmpty() )

	ASSERT( pp.find( "intvalue" ).getValue().isInteger() )
	ASSERT( pp.find( "intvalue" ).getValue() == 54 )

	pp["doublevalue"].set(44.55);
	ASSERT( pp.size() == 2 )

	ASSERT( pp.find( "doublevalue" ).getValue() == 44.55 )
	ASSERT( pp.getPropertyValue("doublevalue") == 44.55 )

	pp["doublevalue"] = 55.44;
	ASSERT( pp.find( "doublevalue" ).getValue().isDouble() )
	ASSERT( pp.getPropertyValue("doublevalue") == Data(55.44) )
	
	Log::log( "test_PropertiesProvider() : Finished TEST for class 'PropertiesProvider'" );
 }


